/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
module.exports = AddSupObjMacDlg

/**
 * Boîte de dialogue proposant une liste formée de toutes  les macros auxquelles on peut ajouter
 * ou supprimer des objets.
 * @constructor
 * @param app  L'application MtgApp propriétaire
 * @param bAuMoinsUnElt Si true seules les macro ayant plus d'un objet dans la liste associée sont accessibles.
 * @param title Le titre de la boîte de dialogue
 * @param callBackOK Fonction de callBack à appeler après validation par OK et rpenant comme paramètre la macro sélectionnée.
 */
function AddSupObjMacDlg (app, bAuMoinsUnElt, title, callBackOK) {
  MtgDlg.call(this, app, 'AddSupObjMacDlg', callBackOK)
  var self = this
  var list = app.listePr
  this.inf = list.listeMacrosAvecListeModif(bAuMoinsUnElt)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var label = kernel.ce('label')
  tr.appendChild(label)
  $(label).html(kernel.getStr('Macro' + ':'))
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.select = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:300px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }

  tr.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    option.ondblclick = function () { self.OK() }
    this.select.appendChild(option)
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  label = kernel.ce('label')
  tr.appendChild(label)
  $(label).html(kernel.getStr('Info') + ' :')

  var textarea = kernel.ce('textarea', {
    id: 'ta',
    cols: 60,
    rows: 5,
    disabled: 'true'
  })
  tr.appendChild(textarea)
  this.onSelectChange()
  this.create(title, 500)
}

AddSupObjMacDlg.prototype = new MtgDlg()

AddSupObjMacDlg.prototype.onSelectChange = function () {
  $('#ta').val(this.inf.pointeurs[this.select.selectedIndex].infoHist())
}

AddSupObjMacDlg.prototype.OK = function () {
  this.destroy()
  this.callBackOK(this.inf.pointeurs[this.select.selectedIndex])
}
