/*
 * Created by yvesb on 02/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const EditeurValeurComplexe = require('./EditeurValeurComplexe')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const CValeur = require('../objets/CValeur')
const CValeurComp = require('../objets/CValeurComp')
const MtgInput = require('./MtgInput')

module.exports = CalculDlg

/**
 * Dialogue de création ou modification d'un calcul réel ou complexe
 * @param app La mtgApp propriétaire
 * @param calc Le calcul dont la formule doit être modifiée
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier un calcul déjà existant
 * @param bReel {boolean} : true pour un calcul réel et false pour un calcul complexe
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annulation
 * @constructor
 */
function CalculDlg (app, calc, modification, bReel, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'calculdlg', callBackOK, callBackCancel)
  var self = this
  this.calc = calc
  this.modification = modification
  this.bReel = bReel
  var list = app.listePr
  // La formule obtenue pour le calcul sera renvoyée dans un CValeur
  this.valeur = bReel ? new CValeur(list) : new CValeurComp(list)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  this.inputName = new MtgInput()
  td.appendChild(this.inputName)
  tr.appendChild(td)
  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Formule') + ' : ')
  td.appendChild(label)
  tr.appendChild(td)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputFormula = new MtgInput({
    size: 50
  })
  td.appendChild(this.inputFormula)
  this.inputFormula.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  this.inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      if ($(self.inputFormula).val() === '') self.inputFormula.focus()
      else {
        $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
        self.OK()
      }
    }
  }
  if (modification) {
    $(this.inputName).val(calc.nomCalcul)
    $(this.inputName).attr('disabled', true)
    $(this.inputFormula).val(calc.chaineCalcul)
  }
  var indmax = modification ? list.indexOf(calc) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputFormula, bReel, indmax)
  tab.appendChild(paneBoutonsValFonc)
  this.editeurName = new EditeurNomCalcul(app, !modification, this.inputName)
  if (bReel) this.editeurFormule = new EditeurvaleurReelle(app, this.inputFormula, indmax, this.valeur, null)
  else this.editeurFormule = new EditeurValeurComplexe(app, this.inputFormula, indmax, this.valeur, null)
  this.create(this.bReel ? 'CalculReel' : 'CalculComp', 680)
}

CalculDlg.prototype = new MtgDlg()

CalculDlg.prototype.OK = function () {
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (this.editeurName.validate()) {
    if (this.editeurFormule.validate($(this.inputFormula).val())) {
      if (!this.modification) this.calc.nomCalcul = $(this.inputName).val()
      this.calc.calcul = this.valeur.calcul
      this.calc.chaineCalcul = this.calc.calcul.chaineCalculSansPar(null)
      if (!this.modification) this.app.ajouteElement(this.calc)
      this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj'
        : this.bReel ? 'CalculReel' : 'CalculComp')
      this.destroy()
      if (this.callBackOK !== null) this.callBackOK()
    } else this.inputFormula.focus()
  }
}

CalculDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $(self.modification ? self.inputFormula : self.inputName).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}
