/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CMathGraphDoc = require('../objets/CMathGraphDoc')
const $ = require('jquery')
// attention, jstree fait un require jquery et ajoute des choses dessus sans le renvoyer
// => il faut un resolve.alias.jquery dans webpack.config.js pour imposer le module à prendre,
//    histoire que tout le monde récupère la même instance, car il pourrait y avoir plusieurs
//    jquery dans node_modules
// => $.jstree.plugins.types.get_node veut du jQuery en global, on pourrait le mettre via un
//    new webpack.ProvidePlugin({'jQuery': 'jquery'})
//    dans la conf webpack, mais ça ajouterait jquery à tous les builds, on le fait ici
window.jQuery = $
require('jstree')
require('jstree/dist/themes/default/style.min.css')

module.exports = ChoixOutilsDlg

/**
 * Boîte de dialogue de choix des outils disponibles pour la figure en cours
 * @constructor
 * @param app L'application propriétaire
 * @param doc Un document dans lequel les choix d'outils seront enregistrés après validation de la boîte de dialogue
 * @param niv Le niveau de filtarge des outils proposés
 */
function ChoixOutilsDlg (app, doc, niv) {
  var tr, div, label, radio, tab, i, cb
  MtgDlg.call(this, app, 'OutilAddDlg')
  this.doc = doc // Servira lors de la validation
  this.niv = niv
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  /// //////////////////////////////////////////////////////////////////////////////////////////////////////////////
  // En bas, un label et deux boutons radio pour savoir si les outils sohés sont les outils permis ou interdits  //
  /// //////////////////////////////////////////////////////////////////////////////////////////////////////////////
  if (app.electron) {
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cb'
    })
    div.appendChild(cb)
    label = kernel.ce('label', {
      for: 'cb'
    })
    $(label).html(kernel.getStr('ToolsDem'))
    div.appendChild(label)
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)

  label = kernel.ce('label')
  $(label).html(kernel.getStr('ChoixOutDlg1'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  radio = kernel.ce('input', {
    id: 'rad1',
    type: 'radio',
    name: 'rad'
  })
  div.appendChild(radio)

  label = kernel.ce('label', {
    for: 'rad1'
  })
  $(label).html(kernel.getStr('ChoixOutDlg2'))
  div.appendChild(label)
  radio = kernel.ce('input', {
    id: 'rad2',
    type: 'radio',
    name: 'rad',
    style: 'margin-left:20px;'
  })
  div.appendChild(radio)
  label = kernel.ce('label', {
    for: 'rad2'
  })
  $(label).html(kernel.getStr('ChoixOutDlg3'))
  div.appendChild(label)
  if (doc.itemsCochesInterdits) $('#rad2').attr('checked', 'checked')
  else $('#rad1').attr('checked', 'checked')
  // Dessous l'arbre dans un div
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div', {
    id: 'div',
    style: 'display:inline-block;width:550px;height:350px;vertical-align:top;overflow:auto;border:1px solid black;margin-top: 5px;'
  })
  tr.appendChild(div)
  $(div).jstree({
    plugins: ['wholerow', 'checkbox', 'types'],
    core: { // types pour pouvoir avoir des user defined icones
      check_callback: true
    }
  })
  this.tree = $(div).jstree(true)
  this.idroot = this.tree.create_node('#', kernel.getStr('ChoixOutils'))
  this.addToolsTopBar()
  tab = ['Points', 'Droites', 'Segments', 'Cercles', 'Polys', 'Marques', 'Lieux', 'Transf', 'Mes', 'Disp', 'Calculs', 'Surfaces']
  for (i = 0; i < tab.length; i++) this.addTools(tab[i])
  this.addTools('Divers', true)

  // Création de la boîte de dialogue par jqueryui
  this.create('ChoixOutils', 600)
  this.tree.open_node(this.idroot)
}

ChoixOutilsDlg.prototype = new MtgDlg()

/**
 * Fonction rajoutant à la racine de l'arbre une nouvelle branche avec comme texte le name de app.expandableBarName
 * et comme noeuds enfants les noms des outils de cette ExpandableBar
 * @param expandableBarName Le name de la barre d'outils
 * @param {boolean} bDivers  true pour le dernier outil proposant des outils divers
 */
ChoixOutilsDlg.prototype.addTools = function (expandableBarName, bDivers = false) {
  var id, id2, id3, b, tool, tool2
  var self = this
  var app = this.app
  var eb = app['bar' + expandableBarName]
  var tab = eb.tab
  id = this.tree.create_node(this.idroot, kernel.getStr(expandableBarName))
  var size = tab.length
  var itemsCochesInterdits = this.doc.itemsCochesInterdits
  for (var i = 0; i < size; i++) {
    tool = app['outil' + tab[i]]
    if (tool.toolName.indexOf('Add') === 0) {
      if (this.hasTools(tool)) {
        if (!bDivers) id2 = this.tree.create_node(id, { text: ' ' + kernel.getStr('Add') })
        for (var j = 0; j < tool.toolArray.length; j++) {
          tool2 = app['outil' + tool.toolArray[j]]
          if (app.levels[this.niv].toolDispo(tool2.toolIndex)) {
            id3 = this.tree.create_node(bDivers ? id : id2, kernel.getStr(tool2.toolName))
            b = this.doc.toolDispo(tool2.toolIndex)
            if ((!itemsCochesInterdits && b) || (itemsCochesInterdits && !b)) { this.tree.select_node(id3) } else this.tree.deselect_node(id3)
            self[id3] = tool2.toolIndex
          }
        }
      }
    } else
    if (app.levels[this.niv].toolDispo(tool.toolIndex)) {
      var hasIcon = app['button' + tab[i]]
      tool2 = app['outil' + tab[i]]
      id2 = hasIcon
        ? this.tree.create_node(id, {
          text: ' ' + kernel.getStr(tool2.toolName),
          icon: app['button' + tab[i]].url
        })
        : this.tree.create_node(id, kernel.getStr(tool2.toolName))
      b = this.doc.toolDispo(tool2.toolIndex)
      if ((!itemsCochesInterdits && b) || (itemsCochesInterdits && !b)) { this.tree.select_node(id2) } else this.tree.deselect_node(id2)
      self[id2] = tool2.toolIndex // Pour pouvoir associer à chaque branche le nom de l'outil correspondant
    }
  }
}

ChoixOutilsDlg.prototype.addToolsTopBar = function () {
  var app = this.app
  var self = this
  var id = this.tree.create_node(this.idroot, kernel.getStr('ToolsBarHor'))
  var tab = ['ModifObjGraph', 'ZoomPlus', 'ZoomMoins', 'ModifObjNum', 'Palette', 'ModeTrace', 'Recalculer', 'Gomme', 'Rideau',
    'TranslationFigure', 'ExecutionMacro', 'Protocole', 'ToggleToolsAdd']
  var size = tab.length
  var itemsCochesInterdits = this.doc.itemsCochesInterdits
  for (var i = 0; i < size; i++) {
    var tool2 = app['outil' + tab[i]]
    var id2 = this.tree.create_node(id, {
      text: ' ' + kernel.getStr(tool2.toolName),
      icon: app['button' + tab[i]].url
    })
    var b = this.doc.toolDispo(tool2.toolIndex)
    if ((!itemsCochesInterdits && b) || (itemsCochesInterdits && !b)) { this.tree.select_node(id2) } else this.tree.deselect_node(id2)
    self[id2] = tool2.toolIndex // Pour pouvoir associer à chaque branche le nom de l'outil correspondant
  }
}
/**
 * Fonction renvoyant true si l'outil addTool de type OutilAdd a au moins un outil à mettre dans la liste
 * @param addTool
 * @returns {boolean}
 */
ChoixOutilsDlg.prototype.hasTools = function (addTool) {
  var app = this.app
  for (var j = 0; j < addTool.toolArray.length; j++) {
    var tool2 = app['outil' + addTool.toolArray[j]]
    if (app.levels[this.niv].toolDispo(tool2.toolIndex)) return true
  }
  return false
}

ChoixOutilsDlg.prototype.OK = function () {
  var app = this.app
  var self = this
  var b; var doc; var checked
  var selectedData = []
  var selectedIndexes = this.tree.get_selected(true)

  $.each(selectedIndexes, function (index) {
    b = self[selectedIndexes[index].id] // b est undefined pour les noeuds racines des branches
    if (b) selectedData.push(self[selectedIndexes[index].id])
  })
  checked = $('#rad2').prop('checked')
  this.doc.setIdMenus(app, checked, selectedData, this.niv)
  // Si on est en mode électron et si la case est cocheé, on appelle setLevel de la page index.html
  if (app.electron) {
    if ($('#cb').prop('checked')) {
      doc = new CMathGraphDoc('', false, false, 1)
      doc.dimf = app.dimf
      doc.setIdMenus(app, checked, selectedData, this.niv)
      setLevel(doc.getBase64Code()) // eslint-disable-line no-undef
      // setLevel est défini dans index.html
      doc = null
    }
  }
  this.destroy()
}
