/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const ListeObjNumPrConst = require('./ListeObjNumPrConst')
const $ = require('jquery')

module.exports = ChoixSrcNumDlg

/**
 * Dialogue de choix des objets sources numériques pour une construction
 * @constructor
 * @param app La mtgApp propriétaire
 */
function ChoixSrcNumDlg (app) {
  MtgDlg.call(this, app, 'ChoixSrcNumDlg')
  var self = this
  // On crée les deux listes de gauche et de droite.
  // La liste de droite doit être créée en premier car la liste de gauche dépend des objets dépend des objets
  // présents à droite.
  this.listRight = new ListeObjNumPrConst(app, false)
  this.listLeft = new ListeObjNumPrConst(app, true, this.listRight, function () { self.onBtnAddFin() })
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  // Le panneau principal est forme de deux colonnes.
  // A gauche une colonne pour la liste des éléments numériques disponibles avec en dessous des boutons
  var td = kernel.ce('td', {
    style: 'vertical-align:top;'
  })
  tabPrincipal.appendChild(td)
  var tr = kernel.ce('tr')
  td.appendChild(tr)
  var label = kernel.ce('label')
  tr.appendChild(label)
  $(label).html(kernel.getStr('SrcDlg2'))
  tr = kernel.ce('tr')
  td.appendChild(tr)
  tr.appendChild(this.listLeft.select)
  tr = kernel.ce('tr')
  td.appendChild(tr)
  // Sous la liste de gauche un tableau contenant les trois boutons
  var tabBtnLeft = kernel.ce('table', {
    cellspacing: 5
  })
  tr.appendChild(tabBtnLeft)
  var btns = ['Ins', 'AddDeb', 'AddFin']
  for (var i = 0; i < 3; i++) {
    tr = kernel.ce('tr')
    tabBtnLeft.appendChild(tr)
    var btn = kernel.ce('button', {
      tabindex: -1,
      style: 'width:220px;font-size:13px'
    })
    btn.owner = this
    btn.onclick = self['onBtn' + btns[i]]
    $(btn).html(kernel.getStr(btns[i]))
    tr.appendChild(btn)
  }

  // A droite une colonne pour la liste des éléments numériques choisi avec en dessous des boutons
  var tdright = kernel.ce('td')
  tabPrincipal.appendChild(tdright)
  tr = kernel.ce('tr')
  tdright.appendChild(tr)
  tr = kernel.ce('tr')
  tdright.appendChild(tr)
  var tabListRight = kernel.ce('table', {
    cellspacing: 0
  })
  tr.appendChild(tabListRight)
  tr = kernel.ce('tr')
  tabListRight.appendChild(tr)
  td = kernel.ce('td', {
    style: 'text-align:center;'
  })
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SrcDlg3'))
  td.appendChild(label)
  tr = kernel.ce('tr')
  tabListRight.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  td.appendChild(this.listRight.select)
  // A droite de la liste de droite deux boutons avec image pour remonter et descendre un élément
  td = kernel.ce('td', {
    style: 'vertical-align:middle;'
  })
  tr.appendChild(td)
  var tabIcones = kernel.ce('table')
  td.appendChild(tabIcones)
  var ar = ['up', 'down']
  var af = [this.up, this.down]
  for (i = 0; i < 2; i++) {
    tr = kernel.ce('tr')
    tabIcones.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    td.owner = this
    $(td).css('padding', '1px')
    $(td).html(app.local
      ? "<img src='" + app.localPath + 'images/' + ar[i] + ".png' width='22px' height='22px' />"
      : "<img src='" + app.imagePath + ar[i] + ".png' width='22px' height='22px' crossorigin='anonymous' />"
    )
    td.onclick = af[i]
  }

  // Sous la liste deux boutons
  tr = kernel.ce('tr')
  tdright.appendChild(tr)
  var tabBtnRight = kernel.ce('table', {
    cellspacing: 5
  })
  tr.appendChild(tabBtnRight)
  td = kernel.ce('td')
  tabBtnRight.appendChild(td)
  btns = ['Ret', 'RetAll']
  for (i = 0; i < 2; i++) {
    tr = kernel.ce('tr')
    td.appendChild(tr)
    btn = kernel.ce('button', {
      tabindex: -1,
      style: 'width:220px;font-size:13px'
    })
    btn.owner = this
    btn.onclick = self['onBtn' + btns[i]]
    $(btn).html(kernel.getStr(btns[i]))
    tr.appendChild(btn)
  }
  this.create('ChoixSrcNumConst', 600)
}

ChoixSrcNumDlg.prototype = new MtgDlg()

ChoixSrcNumDlg.prototype.onBtnIns = function () {
  var owner = this.owner
  owner.listRight.insert(owner.listLeft.getSelectedItem())
  owner.updateLeftList()
}

ChoixSrcNumDlg.prototype.onBtnAddDeb = function () {
  var owner = this.owner
  owner.listRight.insertDeb(owner.listLeft.getSelectedItem())
  owner.updateLeftList()
}

ChoixSrcNumDlg.prototype.onBtnAddFin = function () {
  var owner = this.owner || this // Le || car peut aussi être appelé via un double click sur un item de la liste de gauche
  owner.listRight.add(owner.listLeft.getSelectedItem())
  owner.updateLeftList()
}

ChoixSrcNumDlg.prototype.updateLeftList = function () {
  this.listLeft.metAJourListe(this.listRight)
}

ChoixSrcNumDlg.prototype.onBtnRet = function () {
  var owner = this.owner
  owner.listRight.ret()
  owner.updateLeftList()
}

ChoixSrcNumDlg.prototype.onBtnRetAll = function () {
  var owner = this.owner
  owner.listRight.retAll()
  owner.updateLeftList()
}

ChoixSrcNumDlg.prototype.up = function () {
  this.owner.listRight.up()
}

ChoixSrcNumDlg.prototype.down = function () {
  this.owner.listRight.down()
}

ChoixSrcNumDlg.prototype.OK = function () {
  // On vide la liste app.listSrcNG et on lui ajoute tous les éléments de la liste de droite
  var listeNG = this.app.listeSrcNG
  listeNG.retireTout()
  var pointeurs = this.listRight.pointeurs
  for (var i = 0; i < pointeurs.length; i++) {
    listeNG.add(pointeurs[i])
  }
  this.destroy()
  this.app.activeOutilCapt()
}

ChoixSrcNumDlg.prototype.onCancel = function () {
  this.destroy()
  this.app.activeOutilCapt()
}
