/*
 * Created by yvesb on 18/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const PaneAlignement = require('./PaneAlignement')
const PaneTaillePolice = require('./PaneTaillePolice')
const PaneModeAffichage = require('./PaneModeAffichage')
const TextButton = require('./TextButton')
const CCommentaire = require('../objets/CCommentaire')
const StyleEncadrement = require('../types/StyleEncadrement')
const InsertionDynamiqueDlg = require('./InsertionDynamiqueDlg')
const InsertionFormuleDlg = require('./InsertionFormuleDlg')
const MtgTextAreaWithCharSpe = require('./MtgTextAreaWithCharSpe')
const AvertDlg = require('./AvertDlg')
module.exports = CommentaireDlg

/**
 * Dialogue de création ou modification d'un affichage affichage de texte
 * @constructor
 * @param app L'application propriétaire
 * @param comm Le CCommentaire à modifier
 * @param modification true si on modifie un CCommentaire déjà présent dans la figure
 * @param callBackOK Fonction de callBack à appeler si on valide par OK
 * @param callBackCancel Fonction de callBack à appeler si on referme sans valider
 */
function CommentaireDlg (app, comm, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'CommentaireDlg', callBackOK, callBackCancel)
  this.comm = comm
  this.modification = modification
  var self = this
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBoutons = kernel.ce('table')
  tr.appendChild(tabBoutons)
  /*
  this.textarea = kernel.ce("textarea", {
    cols : 40,
    rows : 12,
    wrap : "off"
  });
  */
  this.textarea = new MtgTextAreaWithCharSpe(app, { cols: 45, rows: 12, wrap: 'off' }, ['grec', 'math', 'arrows'], false, this)
  this.textarea.owner = this
  var jqta = $(this.textarea)
  jqta.css('overflow-x', 'scroll')
  // On ne peut pas utiliser de style css pour le textarea car sinon il est écrasé par un autre style de jquery
  jqta.css('font-size', '14px')
  jqta.keyup(this.onkeyup)

  // Le tableau des boutons est formé de 3 lignes de 9 icônes
  var files =
    ['styleTexteExposant', 'styleTexteIndice', 'styleTexteNormal', 'styleTexteGras', 'styleTexteItalique', 'styleTexteSouligne']
  var codes =
    ['#H()', '#L()', '#N', '#G', '#I', '#U']
  var decalage = [3, 3, 2, 2, 2, 2]
  tr = kernel.ce('tr')
  for (var j = 0; j < 6; j++) {
    tabBoutons.appendChild(tr)
    var textButton = new TextButton(app, files[j], codes[j], this.textarea, decalage[j], function () { self.updatePreview() })
    tr.appendChild(textButton.container)
  }
  // Au centre un tableau d'une seule ligne contenant le textarea d'édition du code et à droite un svg pour l'aperçu temps réel
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)
  tr = kernel.ce('tr')
  var th = kernel.ce('th', {
    class: 'mtgcaption'
  })
  $(th).html(kernel.getStr('TexteAAff'))
  tabCentre.appendChild(th)
  th = kernel.ce('th', {
    class: 'mtgcation'
  })
  $(th).html(kernel.getStr('Apercu'))
  tabCentre.appendChild(th)
  tabCentre.appendChild(tr)
  let td = kernel.ce('td')
  tr.appendChild(td)
  td.appendChild(this.textarea)
  td = kernel.ce('td')
  $(td).css('overflow', 'scroll')
  $(td).css('border', '1px solid lightgray')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  var div = kernel.ce('div', {
    // style : "position:relative;margin-left:auto;margin-right:auto;top:0px;width:300px;height:150px;"
    style: 'position:relative;width:400px;height:150px;font-weight:normal;font-family: "Times New Roman", Times, serif;'
  })
  td.appendChild(div)
  this.svg = kernel.cens('svg', {
    width: '600px',
    height: '600px'
  })
  div.appendChild(this.svg)
  /* Ajout version 6.4.1 pour un rendu correct de l'aperçu à cause d'un bug MathJax */
  const rootFontFamily = getComputedStyle(document.body).fontFamily
  const containerFontFamily = getComputedStyle(this.svg.parentNode).fontFamily
  if (rootFontFamily && rootFontFamily !== containerFontFamily) {
    this.svg.parentNode.style.fontFamily = rootFontFamily
  }
  // Un ligne contenant un bouton pour insertion de valeur dynamique
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var btnIns = kernel.ce('button', {
    tabindex: -1
  })
  btnIns.onclick = function () { self.onBtnIns() }
  $(btnIns).html(kernel.getStr('InsVal'))
  tr.appendChild(btnIns)

  var btnFor = kernel.ce('button', {
    tabindex: -1,
    style: 'margin-left:10px;'
  })
  btnFor.onclick = function () { self.onBtnFor() }
  $(btnFor).html(kernel.getStr('InsForDlg1'))
  tr.appendChild(btnFor)

  // Dans la troisième ligne du tableau principal, un tableau de trois colonnes contenant un panneau de choix de
  // mode d'affichage, un panneau de choix d'alignement et un manneua de choix de taille de police
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.paneAlign = new PaneAlignement(app, comm.alignementHorizontal, comm.alignementVertical)
  var tableBas = kernel.ce('table')
  tr.appendChild(tableBas)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  td.appendChild(this.paneAlign.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  this.paneTaillePolice = new PaneTaillePolice(comm.taillePolice, function () { self.updatePreview() })
  tableBas.appendChild(this.paneTaillePolice.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  var list = app.listePr
  var indmax = modification ? list.indexOf(comm) - 1 : list.longueur() - 1
  this.paneModeAffichage = new PaneModeAffichage(this, false, comm.effacementFond,
    comm.encadrement, comm.couleurFond, comm.angText.chaineInfo(), indmax, function () { self.updatePreview() })
  td.appendChild(this.paneModeAffichage.container)
  if (modification) $(this.textarea).val(comm.chaineCommentaire)
  this.create('Commentaire', 800)
}

CommentaireDlg.prototype = new MtgDlg()

CommentaireDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  this.paneModeAffichage.init()
  this.updatePreview()
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    self.textarea.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

CommentaireDlg.prototype.OK = function () {
  // const time = +new Date();
  // if (time - this.timeStart < MtgDlg.delay) return; // Moins d'une seconde dans doute mauvais endroit de clic sur un bouton
  // if (this.app.lastDlgId() !== this.id) return;
  var ch = $(this.textarea).val()
  var self = this
  if (ch === '') {
    new AvertDlg(this.app, 'Vide', function () { self.textarea.focus() })
  } else {
    var comm = this.comm
    var pma = this.paneModeAffichage
    if (!pma.validate()) {
      new AvertDlg(this.app, 'Incorrect')
      return
    } // Faute dans l'angle de rotation du texte

    comm.encadrement = pma.getStyleEnc()
    comm.effacementFond = pma.getEffFond()
    comm.couleurFond = pma.getColor()
    comm.angText = pma.getAng()
    // Si l'alignement vertical ou horizontal est changé on annule tout décalage manuel précédent
    var newHorAlign = this.paneAlign.getHorAlign()
    var newVerAlign = this.paneAlign.getVerAlign()
    if ((comm.alignementHorizontal !== newHorAlign) || (comm.alignementVertical !== newVerAlign)) {
      comm.decX = 0
      comm.decY = 0
    }
    //
    comm.alignementHorizontal = newHorAlign
    comm.alignementVertical = newVerAlign
    comm.taillePolice = this.paneTaillePolice.getTaille()
    // comm.chaineCommentaire = CLatex.traiteAccents(ch); Inutile avec MathJax 3
    comm.chaineCommentaire = ch
    comm.determineDependances()
    comm.positionne(false, this.app.dimf)
    comm.setReady4MathJax()
    // Les lignes suivantes avaient été supprimées version 6.4.1 mais sont nécessaires cer sinon si on change la
    // d'affichage la nouvelle taille n'est pas appliquée après validation mais elles ne doivent pas être appelées
    // si cette boîte de dialogue est appelée depuis la boîte de dialogue de protocole
    if (this.modification && self.app.dlg[0] !== 'ProtocoleDlg') {
      // Il faut récréer l'affichage complètement
      MathJax.Hub.Queue(function () {
        comm.reCreateDisplay(self.app.svgFigure)
      })
    }
    if (this.callBackOK !== null) this.callBackOK()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
  }
}

CommentaireDlg.prototype.onkeyup = function (evt) {
  // Non traité si c'est la touche entrée ou une touche de déplacement de curseur
  var kc = evt.keyCode
  // On sort si touche déplacement à gauche ou à droite mais pas si touch Entrée qui permet de passer à la ligne
  if ((evt.charCode === 0) && ((kc === 37) || (kc === 39))) return
  var self = this.owner
  self.updatePreview()
}

CommentaireDlg.prototype.updatePreview = function () {
  var app = this.app
  var list = app.listePr
  var pma = this.paneModeAffichage
  var comm = new CCommentaire(list, null, false, this.comm.couleur, 0, 0, 0, 0, false, null,
    this.paneTaillePolice.getTaille(), pma.getStyleEnc(),
    pma.getEffFond(), pma.getColor(), 0, 0, $(this.textarea).val())
  comm.determineDependances(list.indexOf(this.comm))
  comm.positionne(false, app.dimf)
  comm.setReady4MathJax()
  var self = this
  MathJax.Hub.Queue(function () {
    while (self.svg.childNodes.length !== 0) self.svg.removeChild(self.svg.childNodes[0])
    var g = comm.createg()
    if (comm.encadrement !== StyleEncadrement.Sans) $(g).attr('transform', 'translate(2,2)')
    self.svg.appendChild(g)
  })
}

CommentaireDlg.prototype.onBtnIns = function () {
  var app = this.app
  var list = app.listePr
  new InsertionDynamiqueDlg(app, this.textarea, this, this.modification ? list.indexOf(this.comm) - 1 : -1, false)
}

CommentaireDlg.prototype.onBtnFor = function () {
  var app = this.app
  var list = app.listePr
  new InsertionFormuleDlg(app, this.textarea, this, this.modification ? list.indexOf(this.comm) - 1 : -1, false)
}
