/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const PaneListeReperes = require('./PaneListeReperes')
const $ = require('./../global/jQuery-ui')
module.exports = CourbeAvecTanDlg

/**
 * Boîte de dialogue pour créer une courbe définie par des points (de 2 à 9)
 * et des coefficients directeurs de tangentes dans un repère
 * @param app L'application proprétaire
 * @param callBackOK Fonction de callBack prenant deux paramètres nbPts et rep qui sont le nomebre de points
 * choisis par l'utilisateur et le repère choisi
 * @param callBackCancel Fonction de callBack à appeler si l'utilisateur annule
 * @constructor
 */
function CourbeAvecTanDlg (app, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'CourbeAvecTanDlg', callBackOK, callBackCancel)
  var list = app.listePr
  var tabPrincipal = kernel.ce('table')
  var tr = kernel.ce('tr')
  var label = kernel.ce('label')
  var tabBas = kernel.ce('table')
  var td = kernel.ce('td', {
    style: 'vertical-align:top;'
  })
  var tabBasDroit = kernel.ce('table')
  this.appendChild(tabPrincipal)
  tabPrincipal.appendChild(tr)
  $(label).html(kernel.getStr('CourbeAvecTanDlg1'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('CourbeAvecTanDlg2'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  tr.appendChild(tabBas)
  tabBas.appendChild(td)
  this.paneListeRep = new PaneListeReperes(app, list.longueur() - 1)
  td.appendChild(this.paneListeRep.getTab())
  td = kernel.ce('td')
  tabBas.appendChild(td)
  td.appendChild(tabBasDroit)
  td = kernel.ce('td', {
    style: 'vertical-align:middle;'
  })
  tabBasDroit.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('NbPts') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td', {
    style: 'vertical-align:top;'
  })
  tabBasDroit.appendChild(td)
  this.select = kernel.ce('select', {
    id: 'nbPts',
    size: 6 // Le nombre de lignes visibles par défaut
  })
  td.appendChild(this.select)
  for (var i = 2; i < 11; i++) {
    var option = kernel.ce('option', {
      class: 'mtgOption'
    })
    if (i === 3) option.setAttribute('selected', 'selected')
    $(option).html(i)
    this.select.appendChild(option)
  }
  this.create('CourbeAvecTan', 500)
}

CourbeAvecTanDlg.prototype = new MtgDlg()

CourbeAvecTanDlg.prototype.OK = function () {
  this.destroy()
  this.callBackOK(this.select.selectedIndex + 2, this.paneListeRep.getSelectedRep())
}
