/*
 * Created by yvesb on 07/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const NatCal = require('../types/NatCal')
const NatObj = require('../types/NatObj')
const MotifPoint = require('../types/MotifPoint')
const CImplementationProto = require('../objets/CImplementationProto')
const PaneListeReperes = require('./PaneListeReperes')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const AvertDlg = require('./AvertDlg')
const MtgInput = require('./MtgInput')
const StyleTrait = require('../types/StyleTrait')
module.exports = CourbeFoncAvecCrochetsDlg

function CourbeFoncAvecCrochetsDlg (app, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'CourbeDlg', callBackOK, callBackCancel)
  var self = this
  var list = app.listePr
  this.inf = list.listeParNatCal(app, NatCal.NFoncR1Var, -1)
  // 3 CValeur pour stocker les valeurs pour a, b et r (le rayon des croches et pixel).
  this.valeura = new CValeur(list)
  this.valeurb = new CValeur(list)
  this.valeurr = new CValeur(list)

  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // Un premier tableau en haut pour contenir la liste des fonctions et la liste des repères
  var tabListes = kernel.ce('table')
  $(tabListes).css('margin', ' 0 auto')
  tr.appendChild(tabListes)
  var td = kernel.ce('td')
  tabListes.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:220px'
  })
  $(caption).html(kernel.getStr('InsDynDlg1'))
  td.appendChild(caption)
  this.select = kernel.ce('SELECT', {
    size: 5, // Le nombre de lignes visibles par défaut
    style: 'width:220px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabListes.appendChild(td)
  this.paneListeRep = new PaneListeReperes(app, list.longueur() - 1)
  td.appendChild(this.paneListeRep.getTab())
  // Un tableau pour contenir les boutons radio de choix d'intervalle pour la courbe
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabRadio = kernel.ce('table')
  tr.appendChild(tabRadio)
  tr = kernel.ce('tr')
  tabRadio.appendChild(tr)
  var label = kernel.ce('label', {
    for: 'radio1'
  })
  $(label).html(kernel.getStr('courbeDlg1'))
  tr.appendChild(label)
  // Tableau formé des fonctions a appeler quand on coche une des quatre cases
  var tab = [']a;b[', '[a;b[', ']a;b]', ']a;+&infin;[', ']-&infin;;a[']
  // Pour j variant de 1 à 5 : Courbe sur ]a;b[, sur [a;b[, sur ]a;b], sur ]a; + infini[, sur ]- infini; a[
  for (let j = 1; j < 6; j++) {
    var radio = kernel.ce('input', {
      type: 'radio',
      id: 'radio' + j,
      name: 'interv'
    })
    tr.appendChild(radio)
    if (j === 1) radio.setAttribute('checked', 'checked')
    radio.onchange = function () {
      (j < 4) ? self.showInputab() : self.showInputa()
    }
    label = kernel.ce('label', {
      for: 'radio' + j
    })
    $(label).html(tab[j - 1])
    tr.appendChild(label)
  }

  var trValeursab = kernel.ce('tr')
  tabPrincipal.appendChild(trValeursab)
  var paneValeurs = kernel.ce('table', {
    id: 'paneVal'
  })
  trValeursab.appendChild(paneValeurs)
  td = kernel.ce('td')
  paneValeurs.appendChild(td)
  var paneValeursGauche = kernel.ce('table', {
    id: 'paneVala'
  })
  td.appendChild(paneValeursGauche)
  tr = kernel.ce('tr')
  paneValeursGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    id: 'labela'
  })
  $(label).html(kernel.getStr('Valeura') + ' :')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de a
  this.inputa = new MtgInput()
  $(this.inputa).attr('size', 15)
  td.appendChild(this.inputa)
  var paneBoutonsValFonca = new PaneBoutonsValFonc(this.app, this, this.inputa, true, -1)
  paneValeursGauche.appendChild(paneBoutonsValFonca)

  td = kernel.ce('td')
  paneValeurs.appendChild(td)
  var paneValeursDroit = kernel.ce('table', {
    id: 'paneValb'
  })
  td.appendChild(paneValeursDroit)
  tr = kernel.ce('tr')
  paneValeursDroit.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    id: 'labelb'
  })
  $(label).html(kernel.getStr('Valeurb') + ' :')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de b
  this.inputb = new MtgInput()
  $(this.inputb).attr('size', 15)
  td.appendChild(this.inputb)
  var paneBoutonsValFoncb = new PaneBoutonsValFonc(this.app, this, this.inputb, true, -1)
  paneValeursDroit.appendChild(paneBoutonsValFoncb)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneValr = kernel.ce('table')
  tr.appendChild(paneValr)
  tr = kernel.ce('tr')
  paneValr.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label', {
    id: 'labelb'
  })
  $(label).html(kernel.getStr('RayPix') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // Editeur pour la valeur de b
  this.inputr = new MtgInput()
  $(this.inputr).attr('size', 15)
  td.appendChild(this.inputr)
  $(this.inputr).val('6') // 6 pixels comme rayon proposé par défaut pour les crochets
  var paneBoutonsValFoncr = new PaneBoutonsValFonc(this.app, this, this.inputr, true, -1)
  paneValr.appendChild(paneBoutonsValFoncr)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneInfo = kernel.ce('table')
  tr.appendChild(paneInfo)
  tr = kernel.ce('tr')
  paneInfo.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  $(td).css('vertical-align', 'top')
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var textarea = kernel.ce('textarea', {
    id: 'info',
    cols: 35,
    rows: 2,
    disabled: 'true'
  })
  tr.appendChild(textarea)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // Un tableau contenant l'éditeur du nombre de points
  var paneInput = kernel.ce('table')
  tr.appendChild(paneInput)
  tr = kernel.ce('tr')
  paneInput.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('courbeDlg10'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var inputnb = new MtgInput({
    id: 'inputnb',
    size: 3
  })
  $(inputnb).val('500')
  td.appendChild(inputnb)

  var indf = list.longueur() - 1
  this.editeura = new EditeurValeurReelle(app, this.inputa, indf, this.valeura, null)
  this.editeurb = new EditeurValeurReelle(app, this.inputb, indf, this.valeurb, null)
  this.editeurr = new EditeurValeurReelle(app, this.inputr, indf, this.valeurr, null)
  this.onSelectChange()
  this.create('CourbeFoncCr', 650)
}

CourbeFoncAvecCrochetsDlg.prototype = new MtgDlg()

CourbeFoncAvecCrochetsDlg.prototype.showInputab = function () {
  $('#paneVala').css('visibility', 'visible')
  $('#paneValb').css('visibility', 'visible')
}

CourbeFoncAvecCrochetsDlg.prototype.showInputa = function () {
  $('#paneVala').css('visibility', 'visible')
  $('#paneValb').css('visibility', 'hidden')
}

CourbeFoncAvecCrochetsDlg.prototype.onSelectChange = function () {
  $('#info').val(this.inf.pointeurs[this.select.selectedIndex].infoHist())
}

CourbeFoncAvecCrochetsDlg.prototype.OK = function () {
  // On cherche quel est l'indice du bouton radio coché
  for (var selectrad = 1; selectrad < 6; selectrad++) {
    if ($('#radio' + selectrad).prop('checked')) break
  }
  var avecEditb = selectrad < 4
  if (this.editeura.validate() && (avecEditb ? this.editeurb.validate() : true) && this.editeurr.validate()) {
    var app = this.app
    var nbPts = parseInt($('#inputnb').val())
    if (nbPts < 4 || nbPts > 5000 || isNaN(nbPts)) {
      new AvertDlg(app, 'Incorrect', function () {
        $('#inputnb').focus()
      })
    } else {
      var list = app.listePr
      // Array formé des noms des prototypes à utiliser
      var tab = ['CourbeSurabCrochetab', 'CourbeSurabCrochetb', 'CourbeSurabCrocheta',
        'CourbeSuraInfCrocheta', 'CourbeSurInfaCrocheta']
      var proto = app.docConsAv.getPrototype(tab[selectrad - 1])

      // Si les valeurs choisies pour le minimum et le maximum ne sont pas des références à des valeurs exsitantes
      // on crée de nouveux calculs qu'on ajoute à la figure
      var valeura = this.valeura.getCalcForImpProto(app, 'a', true)
      if (avecEditb) var valeurb = this.valeurb.getCalcForImpProto(app, 'b', true)
      var valeurr = this.valeurr.getCalcForImpProto(app, 'rpix', true)
      // On affecte les éléments sources au prototype
      var i = 0
      proto.get(i++).elementAssocie = this.paneListeRep.getSelectedRep()
      proto.get(i++).elementAssocie = this.inf.pointeurs[this.select.selectedIndex]
      proto.get(i++).elementAssocie = valeura
      if (selectrad < 4) proto.get(i++).elementAssocie = valeurb
      proto.get(i).elementAssocie = valeurr
      var impProto = new CImplementationProto(list, proto)
      impProto.implemente(app.dimf, proto)
      impProto.nomProto = kernel.getStr('CourbeFonc')
      var styleTrait = app.getStyleTrait()
      var coul = app.getCouleur()
      var lieu = impProto.premierFinal(NatObj.NLieu)
      lieu.donneCouleur(coul)
      lieu.donneStyle(styleTrait)
      lieu.nombrePoints = parseInt($('#inputnb').val())
      var arc = impProto.premierFinal(NatObj.NArc)
      var sw = styleTrait.strokeWidth
      arc.donneStyle(new StyleTrait(list, StyleTrait.styleTraitContinu, sw))
      arc.donneCouleur(coul)
      if (avecEditb) {
        arc = impProto.dernierFinal(NatObj.NArc)
        arc.donneStyle(new StyleTrait(list, StyleTrait.styleTraitContinu, sw))
        arc.donneCouleur(coul)
      }
      if (selectrad === 2 || selectrad === 3) {
        var pt = impProto.dernierFinal(NatObj.NPointDansRepere)
        pt.donneMotif(MotifPoint.rond)
        pt.donneCouleur(coul)
      }

      var indImpProto = list.indexOf(impProto)
      list.positionne(false, app.dimf)
      list.setReady4MathJax()
      list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
      app.gestionnaire.enregistreFigureEnCours('CourbeFoncCr')
      this.destroy()
      app.activeOutilCapt()
    }
  }
}
