/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const NatObj = require('../types/NatObj')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const CImplementationProto = require('../objets/CImplementationProto')
const MtgInputWithCharSpe = require('./MtgInputWithCharSpe')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const PaneNombreDecimales = require('./PaneNombreDecimales')
const AvertDlg = require('./AvertDlg')
const MtgInput = require('./MtgInput')
const MotifPoint = require('../types/MotifPoint')

module.exports = CurseurDlg

/**
 * Dialogue de création d'un curseur
 * @constructor
 * @param app La mtgApp propriétaire
 * @param pt Le point qui est destiné à être l'extrémité du curseur
 * @param bptCree true si le point pt a été créé en clqiaunt à un endroit vide de la figure, false sinon
 */
function CurseurDlg (app, pt, bptCree) {
  MtgDlg.call(this, app, 'curseurdlg')
  this.pt = pt
  this.bptCree = bptCree
  var self = this
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie pour l'abscisse avant vérification
  this.valMin = new CValeur(list, 0)
  // Un CValeur pour stocker la valeur choisie pour l'ordonnée avant vérification
  this.valMax = new CValeur(list, 0)
  // Un CValeur pour stocker la valeur choisie pour le pas avant vérification
  this.valPas = new CValeur(list, 0)
  // Un CValeur pour stocker la valeur choisie pour le pas de graduation avant vérification
  this.valPasGrad = new CValeur(list, 0)

  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  // En haut un tableau pour contenir les quatre boutons radios de choix de type de curseur et la
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneButtons = kernel.ce('table')
  tr.appendChild(paneButtons)
  tr = kernel.ce('tr')
  paneButtons.appendChild(tr)
  // Bouton radio de choix de curseur horizontal
  var td = kernel.ce('td')
  tr.appendChild(td)
  var rad = kernel.ce('input', {
    id: 'rad1',
    type: 'radio',
    name: 'orient'
  })
  td.appendChild(rad)
  $(rad).attr('checked', 'true')
  var label = kernel.ce('label', {
    for: 'rad1'
  })
  $(label).html(kernel.getStr('CurHor'))
  td.appendChild(label)
  // Bouton radio de choix de curseur vertical
  td = kernel.ce('td')
  tr.appendChild(td)
  rad = kernel.ce('input', {
    id: 'rad2',
    type: 'radio',
    name: 'orient'
  })
  td.appendChild(rad)
  label = kernel.ce('label', {
    for: 'rad2'
  })
  $(label).html(kernel.getStr('CurVer'))
  td.appendChild(label)
  // Ligne suivante.
  tr = kernel.ce('tr')
  paneButtons.appendChild(tr)
  // Bouton radio de choix de curseur de petite taille
  td = kernel.ce('td')
  tr.appendChild(td)
  rad = kernel.ce('input', {
    id: 'rad3',
    type: 'radio',
    name: 'size'
  })
  td.appendChild(rad)
  $(rad).attr('checked', 'true')
  label = kernel.ce('label', {
    for: 'rad3'
  })
  $(label).html(kernel.getStr('CurSmall'))
  td.appendChild(label)

  // Bouton radio de choix de curseur de grande taille
  td = kernel.ce('td')
  tr.appendChild(td)
  rad = kernel.ce('input', {
    id: 'rad4',
    type: 'radio',
    name: 'size'
  })
  td.appendChild(rad)
  label = kernel.ce('label', {
    for: 'rad4'
  })
  $(label).html(kernel.getStr('CurBig'))
  td.appendChild(label)

  // Une ligne pour la checkBox de curseur entier
  tr = kernel.ce('tr')
  paneButtons.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var cb = kernel.ce('input', {
    id: 'cbEnt',
    type: 'checkbox'
  })
  td.appendChild(cb)
  cb.onclick = function () {
    self.onCheckPas()
  }
  label = kernel.ce('label', {
    for: 'cbEnt'
  })
  $(label).html(kernel.getStr('CurEnt'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  cb = kernel.ce('input', {
    id: 'cbGrad',
    type: 'checkbox'
  })
  td.appendChild(cb)
  cb.onclick = function () {
    self.onCheckPasGrad()
  }
  label = kernel.ce('label', {
    for: 'cbGrad'
  })
  $(label).html(kernel.getStr('AvecGrad'))
  td.appendChild(label)

  // Au centre un tableau qui va contenir deux colonnes. Dans la colonne de gauche un tableue pour contenir
  // les éditeurs de valeurs mini et maxi et à droite un panneau de choix de nombre de décimales.
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneCentre = kernel.ce('table')
  tr.appendChild(paneCentre)
  td = kernel.ce('td')
  paneCentre.appendChild(td)
  var paneCentreGauche = kernel.ce('table')
  td.appendChild(paneCentreGauche)
  tr = kernel.ce('tr')
  paneCentreGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ValMin'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputMin = new MtgInput()
  td.appendChild(this.inputMin)
  var paneBoutonsValFoncMin = new PaneBoutonsValFonc(this.app, this, this.inputMin, true, -1)
  paneCentreGauche.appendChild(paneBoutonsValFoncMin)
  this.editeurMin = new EditeurvaleurReelle(app, this.inputMin, list.longueur() - 1, this.valMin, null)

  tr = kernel.ce('tr')
  paneCentreGauche.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ValMax'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputMax = new MtgInput()
  td.appendChild(this.inputMax)
  var paneBoutonsValFoncMax = new PaneBoutonsValFonc(this.app, this, this.inputMax, true, -1)
  paneCentreGauche.appendChild(paneBoutonsValFoncMax)
  this.editeurMax = new EditeurvaleurReelle(app, this.inputMax, list.longueur() - 1, this.valMax, null)

  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  paneCentre.appendChild(td)
  this.paneDecimales = new PaneNombreDecimales(2)
  td.appendChild(this.paneDecimales.container)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // Un tableau pour contenir les composants permettant d'éditeur la valeur du pas si
  // la checkBox correspondante est cochée
  var panePas = kernel.ce('table', {
    id: 'panepas'
  })
  tr.appendChild(panePas)
  tr = kernel.ce('tr')
  panePas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('PasCur'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputPas = new MtgInput()
  td.appendChild(this.inputPas)
  this.paneBoutonsValPas = new PaneBoutonsValFonc(this.app, this, this.inputPas, true, -1)
  panePas.appendChild(this.paneBoutonsValPas)
  this.editeurPas = new EditeurvaleurReelle(app, this.inputPas, list.longueur() - 1, this.valPas, null)

  // Un tableau pour contenir les composants permettant d'éditeur la valeur du pas de graduation si
  // la checkBox correspondante est cochée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneGrad = kernel.ce('table', {
    id: 'panegrad'
  })
  tr.appendChild(paneGrad)
  tr = kernel.ce('tr')
  paneGrad.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('PasGrad'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputPasGrad = new MtgInput()
  td.appendChild(this.inputPasGrad)
  this.paneBoutonsValGrad = new PaneBoutonsValFonc(this.app, this, this.inputPasGrad, true, -1)
  paneGrad.appendChild(this.paneBoutonsValGrad)
  this.editeurPasGrad = new EditeurvaleurReelle(app, this.inputPasGrad, list.longueur() - 1, this.valPasGrad, null)

  // En bas un tableau de quatre lignes pour contenir 3 champs d'édition et un checkBox sur quatre lignes.
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneBas = kernel.ce('table')
  tr.appendChild(paneBas)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('CurNom'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputNom = new MtgInput()
  td.appendChild(this.inputNom)
  this.editeurNom = new EditeurNomCalcul(app, true, this.inputNom)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('EnTete'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // this.inputEnTete = kernel.ce("input");
  this.inputEnTete = new MtgInputWithCharSpe(app, {}, ['grec', 'math', 'arrows'], false, this)

  td.appendChild(this.inputEnTete)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('SuiviDe'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  // this.inputSuiviDe = kernel.ce("input");
  this.inputSuiviDe = new MtgInputWithCharSpe(app, {}, ['grec', 'math', 'arrows'], false, this)
  td.appendChild(this.inputSuiviDe)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  cb = kernel.ce('input', {
    id: 'cbPunaiser',
    type: 'checkbox'
  })
  td.appendChild(cb)
  $(cb).attr('checked', 'true')
  label = kernel.ce('label', {
    for: 'cbPunaiser'
  })
  $(label).html(kernel.getStr('CurPun'))
  td.appendChild(label)
  this.onCheckPas()
  this.onCheckPasGrad()
  this.create('Curseur', 550)
}

CurseurDlg.prototype = new MtgDlg()

CurseurDlg.prototype.onCheckPas = function () {
  var checked = $('#cbEnt').prop('checked')
  $('#panepas').css('display', checked ? 'inline' : 'none')
  $(this.paneDecimales.container).css('visibility', checked ? 'hidden' : 'visible')
}

CurseurDlg.prototype.onCheckPasGrad = function () {
  var checked = $('#cbGrad').prop('checked')
  $('#panegrad').css('display', checked ? 'inline' : 'none')
}

CurseurDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    self.inputMin.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

CurseurDlg.prototype.OK = function () {
  var valaff, seg, pt, ptb
  var discret = $('#cbEnt').prop('checked')
  var gradue = $('#cbGrad').prop('checked')
  if (this.editeurNom.validate() && this.editeurMin.validate() && this.editeurMax.validate() &&
      (discret ? this.editeurPas.validate() : true) &&
      (gradue ? this.editeurPasGrad.validate() : true)) {
    var app = this.app
    var list = app.listePr
    var self = this
    var min = this.valMin.valeur
    var max = this.valMax.valeur
    var hor = $('#rad1').prop('checked')
    var petit = $('#rad3').prop('checked')

    var punaise = $('#cbPunaiser').prop('checked')
    if ((min === max) || (discret && min > max)) {
      new AvertDlg(app, 'ErrMinMax', function () {
        self.inputMin.focus()
      })
      return
    }
    if (discret && ((min !== Math.floor(min)) || (max !== Math.floor(max)))) {
      new AvertDlg(app, 'Incorrect', function () {
        self.inputMin.focus()
      })
      return
    }
    if (discret) {
      var pas = this.valPas.valeur
      var qpas = (max - min) / pas
      if (!kernel.zero(qpas - Math.round(qpas))) {
        new AvertDlg(app, kernel.getStr('ErrPas'), function () {
          self.inputPas.focus()
        })
        return
      }
    }
    if (gradue) {
      var pasGrad = this.valPasGrad.valeur
      var qpasGrad = (max - min) / pasGrad
      if (!kernel.zero(qpasGrad - Math.round(qpasGrad))) {
        new AvertDlg(app, kernel.getStr('ErrPas'), function () {
          self.inputPasGrad.focus()
        })
        return
      }
    }
    var nomProto = 'Curseur'
    if (discret) nomProto += 'Discret'
    nomProto += hor ? 'Hor' : 'Ver'
    if (gradue) nomProto += 'Gradue'

    // Si les valeurs choisies pour le minimum et le maximum ne sont pas des références à des valeurs exsitantes
    // on crée de nouveux calculs qu'on ajoute à la figure
    var proto = app.docCons.getPrototype(nomProto)
    var i = 0
    proto.get(i++).elementAssocie = this.valMin.getCalcForImpProto(app, 'mini', true)
    proto.get(i++).elementAssocie = this.valMax.getCalcForImpProto(app, 'maxi', true)
    if (discret) { proto.get(i++).elementAssocie = this.valPas.getCalcForImpProto(app, kernel.getStr('step'), true) }

    if (gradue) { proto.get(i++).elementAssocie = this.valPasGrad.getCalcForImpProto(app, kernel.getStr('stepGrid'), true) }
    proto.get(i).elementAssocie = this.pt
    var impProto = new CImplementationProto(list, proto)
    impProto.implemente(app.dimf, proto)
    impProto.nomProto = kernel.getStr('Curseur')

    // On modifie le dernier objet implémenté qui est un affichage de valeur lié à un point
    // suivant les choix de la boîte de dialogue
    var lon = list.longueur()
    // var affval = list.get(lon - 1);
    var affval = impProto.dernierFinal(NatObj.NValeurAffichee)
    affval.enTete = $(this.inputEnTete).val()
    affval.postChaine = $(this.inputSuiviDe).val()
    affval.couleurFond = app.doc.couleurFond
    affval.effacementFond = true
    affval.taillePolice = 15
    // On donne au calcul représentant le curseur le nom choisi
    var calc = list.get(lon - 2)
    calc.donneNom($(this.inputNom).val())
    // On modifie le style de trait et de couleur des objets finaux
    var col = app.getCouleur()
    var styleTrait = app.getStyleTrait()
    var indImpProto = list.indexOf(impProto)
    if (punaise) {
      if (this.pt.estDeNature(NatObj.NPointMobile)) { this.pt.pointLibre = false }
    }

    // pt = list.get(indImpProto + 2);
    pt = impProto.premierFinal(NatObj.NPointLie)
    pt.donneCouleur(col)
    // pt.tailleNom = frame.pref_indiceTaillePoliceNom; // A revoir
    pt.tailleNom = 16
    pt.pointLibre = !punaise
    // seg = list.get(indImpProto + 3);
    seg = impProto.premierFinal(NatObj.NSegment)
    seg.donneCouleur(col)
    seg.donneStyle(styleTrait)
    // ptb = list.get(indImpProto + 11);
    ptb = impProto.dernierFinal(discret ? NatObj.NPointBaseEnt : NatObj.NPointLie)
    ptb.donneCouleur(col)
    ptb.donneMotif(MotifPoint.rond)
    // ptb.tailleNom = frame.pref_indiceTaillePoliceNom; // Ajout version 4.9.1.1
    ptb.tailleNom = 16
    // valaff = list.get(indImpProto + 14);
    valaff = impProto.dernierFinal(NatObj.NValeurAffichee)
    valaff.donneCouleur(col)

    // Si on a demandé un curseur de grande taille, on multiplie l'abscisse du point lié par 2
    if (!petit) {
      pt.donneAbscisse(pt.abscisse * 2)
    }
    if (!discret) affval.nombreDecimales = this.paneDecimales.getDigits()
    list.positionne(false, app.dimf)
    list.setReady4MathJax()
    list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
    this.destroy()
    app.outilCurseur.saveFig()
    app.activeOutilCapt()
  }
}

CurseurDlg.prototype.onCancel = function () {
  var app = this.app
  if (this.bptCree) {
    var list = app.listePr
    list.get(list.longueur() - 1).removegElement(app.svgFigure)
    app.detruitDernierElement()
  }
  this.destroy()
  app.activeOutilCapt()
}
