/*
 * Created by yvesb on 01/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
//
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const MtgInput = require('./MtgInput')

module.exports = DeriveeDlg

/**
 * Boîte de dialogue permettant de créer la dérivée d'une fonction pour laquelle la dérivée n'a pas
 * été déjà calculée
 * @param app
 * @param der La dérivée à modifier
 * @param modification true si on modifie une dérivée déjà présente dans la figure
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annulation
 * @constructor
 */
function DeriveeDlg (app, der, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'DeriveeDlg', callBackOK, callBackCancel)
  this.der = der
  this.modification = modification
  var self = this
  var list = app.listePr
  var indmax = modification ? list.indexOf(der) - 1 : -1
  this.inf = list.listeFonctionsReellesSansDerivees(indmax, modification ? der.fonctionAssociee : null)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('DerDlg1'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  td.appendChild(input)
  if (modification) {
    $(input).val(der.nomCalcul)
    $(input).attr('disabled', true)
  }
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('DerDlg2'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.select = kernel.ce('SELECT', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:180px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.inf.pointeurs[i] === der.fonctionAssociee) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  // En bas du tableau un champ d'info sur la fonction sélectionnée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 40,
    rows: 4
  })
  tr.appendChild(inputinf)
  this.editeurName = new EditeurNomCalcul(app, !this.modification, input)
  this.onSelectChange()
  this.create('Der', 450)
}

DeriveeDlg.prototype = new MtgDlg()

DeriveeDlg.prototype.onSelectChange = function () {
  var ind, el
  ind = this.select.selectedIndex
  if (ind === -1) return
  el = this.inf.pointeurs[ind]
  $('#mtginfo').text(el.infoHist())
}

DeriveeDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  if (!this.modification) {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  } else {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.select).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  }
}

DeriveeDlg.prototype.OK = function () {
  var app = this.app
  // if (app.lastDlgId() !== this.id) return;
  if (this.editeurName.validate()) {
    if (!this.modification) this.der.nomCalcul = $('#mtginput').val()
    this.der.fonctionAssociee = this.inf.pointeurs[this.select.selectedIndex]
    if (!this.modification) app.ajouteElement(this.der)
    app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'Der')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  }
}
