/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Pointeur = require('../types/Pointeur')
const $ = require('jquery')
require('jquery-textrange')
const CalcR = require('../kernel/CalcR')
const AvertDlg = require('./AvertDlg')

module.exports = EditeurConst

/**
 * Objet associé à un input d'une boîte de dialogue et chargé de regarder si
 * la syntaxe du contenu de l'éditeur est correcte.
 * L'éditeur doit contenir une constante ne dépendant d'aucune valeur de la figure
 * @param app L'application mtgApp propriétaire
 * @param input L'éditeur associé
 * @param {int} min  La vameur mini permise pour la constante
 * @param {int} max  La valeur maxi permise pour la constante
 * @param {boolean} bint true si on attend un résultat entier
 * @constructor
 */
function EditeurConst (app, input, min, max, bint = true) {
  this.app = app
  this.input = input
  this.min = min
  this.max = max
  this.bint = bint
}

EditeurConst.prototype.validate = function () {
  var app = this.app
  var list = app.listePr
  var inderr = new Pointeur(0)
  var self = this
  var ch = $(this.input).val()
  if (CalcR.verifieSyntaxe(app.listePourConst, ch, inderr, -1, null)) {
    // var calc = new CalcR.ccb(ch, list, 0, ch.length - 1, null) // Corrigé version 6.4.1
    var calc = CalcR.ccb(ch, list, 0, ch.length - 1, null)
    try {
      // list.initialiseNombreIterations();
      var valeur = calc.resultat(false) // Pour pouvoir rajouter une condition sur la veleur entée
      if (valeur < this.min || valeur > this.max || (this.bint ? (valeur !== Math.round(valeur)) : false)) {
        new AvertDlg(app, 'Incorrect', function () {
          self.input.focus()
          // Test ligne suivante des fois que j'aie oublié de déclarer un éditeur comme MtgInput
          if (self.input.marquePourErreur) self.input.marquePourErreur()
        })
        return false
      } else return true
    } catch (e) {
      inderr.setValue(ch.length)
      $(this.input).textrange('setcursor', ch.length)
      new AvertDlg(app, 'ErrCalculNonExist', function () {
        self.input.focus()
        // Test ligne suivante des fois que j'aie oublié de déclarer un éditeur comme MtgInput
        if (self.input.marquePourErreur) self.input.marquePourErreur()
      })
      return false
    }
  } else {
    var col = inderr.getValue()
    $(this.input).textrange('setcursor', col)
    new AvertDlg(app, 'ErrSyntaxe', function () {
      self.input.focus()
      // Test ligne suivante des fois que j'aie oublié de déclarer un éditeur comme MtgInput
      if (self.input.marquePourErreur) self.input.marquePourErreur()
    })
    return false
  }
}
