/*
 * Created by yvesb on 01/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const ListeTailles = require('./ListeTailles')
const EditeurNomPtOuDte = require('./EditeurNomPtOuDte')
const MtgInputWithCharSpe = require('./MtgInputWithCharSpe')
const AvertDlg = require('./AvertDlg')
const NatObj = require('../types/NatObj')
module.exports = EntreeNomDlg

/**
 * Boîte de dialogue permettant d'entrer ou modifier le nom d'un point ou une droite
 * @param app L'application propriétaire
 * @param obj Le point ou la droite dont le nom doit être modifié
 * @constructor
 */
function EntreeNomDlg (app, obj) {
  MtgDlg.call(this, app, 'nomDlg')
  this.obj = obj
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 10
  })
  this.appendChild(tabPrincipal)
  var trPrincipal = kernel.ce('tr')
  tabPrincipal.appendChild(trPrincipal)
  var td = kernel.ce('td')
  trPrincipal.appendChild(td)
  // Un tableau représentant la partie gauche du la boîte de dialogue
  var tabGauche = kernel.ce('table', {
    cellspacing: 10
  })
  td.appendChild(tabGauche)
  var tr = kernel.ce('tr')
  tabGauche.appendChild(tr)

  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Nom') + ' : ')
  tr.appendChild(label)
  this.inputName = new MtgInputWithCharSpe(app, {}, ['grec'], false, this, true)
  $(this.inputName).val(obj.nom)
  this.inputName.size = 4
  // Pour valider par la touche OK
  var self = this
  this.inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }

  tr.appendChild(this.inputName)
  tr = kernel.ce('tr')
  tabGauche.appendChild(tr)
  label = kernel.ce('label', {
    for: 'mtgcb'
  })
  $(label).html(kernel.getStr('NomMasque') + ' : ')
  tr.appendChild(label)
  var inp = kernel.ce('input', {
    id: 'mtgcb',
    type: 'checkbox'
  })
  if (obj.nomMasque) inp.setAttribute('checked', 'checked')
  tr.appendChild(inp)
  // Dans la partie de droite on met la liste des tailles de polices disponibles
  td = kernel.ce('tr')
  trPrincipal.appendChild(td)
  this.listeTailles = new ListeTailles(obj.tailleNom)
  td.appendChild(this.listeTailles.getComponent())
  this.editeurNom = new EditeurNomPtOuDte(app, this.inputName)
  this.create('EntreeNom', 320)
}

EntreeNomDlg.prototype = new MtgDlg()

EntreeNomDlg.prototype.OK = function () {
  var i; var el
  var app = this.app
  var self = this
  var list = app.listePr
  var nomMasque = $('#mtgcb').prop('checked')
  // Modification version 6.3.5 : Pour les macs ou Ipad on remplace les apostrophes courbes qui
  // sont générées par le clavier Français par des apostrophes droites
  var rep = $(this.inputName).val().replace(/’/g, "'")
  const valide = this.editeurNom.validate(this.obj)

  if (valide) {
    // Il faut annuler le clignotement dun point ou de la droite pour qu'il ne soit pas considéré comme masqué
    app.outilActif.annuleClignotement()
    if (rep === '' && app.listePr.nomIndispensable(this.obj)) {
      new AvertDlg(app, 'ErrNomInd', function () {
        self.inputName.marquePourErreur()
        self.inputName.focus()
      })
    } else {
      this.obj.nomMasque = nomMasque
      this.obj.nom = rep
      var taille = this.listeTailles.getTaille()
      this.obj.tailleNom = taille
      app.pref_TaillePoliceNom = taille
      this.obj.updateName(app.svgFigure, true)
      this.obj.updateRectName()
      // Si il existe des objets dupliqués de cet objet il faut mettre à jout leur affichage de nom
      for (i = 0; i < list.longueur(); i++) {
        el = list.get(i)
        if (el.estDeNature(NatObj.NObjetDuplique) && (el.elementDuplique === this.obj)) { el.update(app.svgFigure, app.doc.couleurFond, app.doc.opacity) }
      }
      app.outilNommer.reselect()
      app.outilNommer.saveFig()
      this.destroy()
    }
  }
}

EntreeNomDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    self.inputName.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

EntreeNomDlg.prototype.onCancel = function () {
  // const time = +new Date();
  // if (time - this.timeStart < MtgDlg.delay) return; // Moins d'une seconde dans doute mauvais endroit de clic sur un bouton
  this.app.outilNommer.deselect()
  this.app.outilNommer.select()
  this.destroy()
}
