/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const exportHTML = require('../kernel/exportHTML')
const $ = require('jquery')

module.exports = ExportHTMLDlg

/**
 * Dialogue d'exportation de la figure en html
 * @constructor
 * @param app La mtgApp propriétaire
 * @param {function} callBack Fonction de callBack à appler après validation
 */
function ExportHTMLDlg (app, callBack) {
  MtgDlg.call(this, app, 'tikzDlg')
  this.callBack = callBack
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Title'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = kernel.ce('input', {
    id: 'inp1',
    type: 'text'
  })
  td.appendChild(input)

  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Aut'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  input = kernel.ce('input', {
    id: 'inp2',
    type: 'text'
  })
  td.appendChild(input)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas1 = kernel.ce('table')
  var caption = kernel.ce('caption', {
    class: 'mtgcaption'
  })
  $(caption).html(kernel.getStr('TextHtm1'))
  tabBas1.appendChild(caption)
  tr.appendChild(tabBas1)
  td = kernel.ce('td')
  tabBas1.appendChild(td)
  var textarea = kernel.ce('textarea', {
    id: 'ta1',
    cols: 60,
    rows: 4
  })
  td.appendChild(textarea)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas2 = kernel.ce('table')
  caption = kernel.ce('caption', {
    class: 'mtgcaption'
  })
  $(caption).html(kernel.getStr('TextHtm2'))
  tabBas2.appendChild(caption)
  tr.appendChild(tabBas2)
  td = kernel.ce('td')
  tabBas2.appendChild(td)
  textarea = kernel.ce('textarea', {
    id: 'ta2',
    cols: 60,
    rows: 4
  })
  td.appendChild(textarea)
  // Création de la boîte de dialogue par jqueryui
  this.create('ExportHTML', 550)
}

ExportHTMLDlg.prototype = new MtgDlg()

ExportHTMLDlg.prototype.OK = function () {
  var app = this.app
  var text = exportHTML.text
  var title = $('#inp1').val()
  var aut = $('#inp2').val()
  var txt1 = $('#ta1').val()
  var txt2 = $('#ta2').val()
  var dimf = app.dimf
  var width = parseInt(app.cadre ? app.widthCadre : dimf.x)
  var height = parseInt(app.cadre ? app.heightCadre : dimf.y)
  text = text.replace('*width', width)
  text = text.replace('*height', height)
  text = text.replace('<title>', '<title>' + kernel.getStr('HTML'))
  text = text.replace('*title', title)
  text = text.replace('*texte1', txt1)
  text = text.replace('*texte2', txt2)
  text = text.replace('*creeAvec', kernel.getStr('CrWith'))
  text = text.replace('*auteur', aut)
  text = text.replace('*figureData', app.getBase64Code())
  this.destroy()
  this.callBack(text)
}
