/*
 * Created by yvesb on 10/05/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const AvertDlg = require('./AvertDlg')
const DataOutputStream = require('../entreesSorties/DataOutputStream')
const PaneAlignement = require('./PaneAlignement')
const PaneModeAffichage = require('./PaneModeAffichage')
const $ = require('./../global/jQuery-ui')
module.exports = ImageDlg

function ImageDlg (app, image, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'imageDlg', callBackOK, callBackCancel)
  this.image = image
  this.modification = modification
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  if (!modification) {
    var tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    this.input = kernel.ce('input', {
      type: 'file',
      id: 'file',
      accept: 'image/*',
      class: 'inputfile'
    })
    var self = this
    this.input.addEventListener('change', function () {
      $('#labelinf').html(' ' + self.input.files[0].name)
    })
    tr.appendChild(this.input)
    var label = kernel.ce('label', {
      for: 'file'
    })
    $(label).html(kernel.getStr('Parcourir'))
    tr.appendChild(label)
    // Un label à droite pour contenir le chemin d'accès du fichier choisi
    var labelinf = kernel.ce('label', {
      id: 'labelinf',
      style: 'margin-left: 10px;'
    })
    tr.appendChild(labelinf)
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.paneAlign = new PaneAlignement(app, image.alignementHorizontal, image.alignementVertical)
  $(this.paneAlign.container).css('margin-top', '100px')
  var tableBas = kernel.ce('table')
  tr.appendChild(tableBas)
  var td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  td.appendChild(this.paneAlign.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  var list = app.listePr
  var indmax = modification ? list.indexOf(image) - 1 : list.longueur() - 1
  this.paneModeAffichage = new PaneModeAffichage(this, false, image.effacementFond,
    image.encadrement, image.couleurFond, image.angText.chaineInfo(), indmax)
  td.appendChild(this.paneModeAffichage.container)
  $(this.paneModeAffichage.container).css('margin-top', '100px')

  this.create('Image', 500)
  $(this.input).blur()
}

ImageDlg.prototype = new MtgDlg()

ImageDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  this.paneModeAffichage.init()
}

ImageDlg.prototype.OK = function () {
  const self = this
  const app = this.app
  var im = this.image
  var pma = this.paneModeAffichage
  if (!pma.validate()) {
    new AvertDlg(app, 'Incorrect')
    return
  } // Faute dans l'angle de rotation du texte
  im.encadrement = pma.getStyleEnc()
  im.effacementFond = pma.getEffFond()
  im.couleurFond = pma.getColor()
  // Si l'alignement vertical ou horizontal est changé on annule tout décalage manuel précédent
  var newHorAlign = this.paneAlign.getHorAlign()
  var newVerAlign = this.paneAlign.getVerAlign()
  if ((im.alignementHorizontal !== newHorAlign) || (im.alignementVertical !== newVerAlign)) {
    im.decX = 0
    im.decY = 0
  }
  //
  im.alignementHorizontal = this.paneAlign.getHorAlign()
  im.alignementVertical = this.paneAlign.getVerAlign()
  im.angText = pma.getAng()
  const allowedTypes = ['png', 'jpg', 'jpeg', 'gif']
  if (this.modification) {
    this.destroy()
    this.image.reCreateDisplay(app.svgFigure)
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    if (this.callBackOK !== null) this.callBackOK()
  } else {
    const file = this.input.files[0]
    if (file) {
      let imgType = file.name.split('.')
      imgType = imgType[imgType.length - 1].toLowerCase() // On utilise toLowerCase() pour éviter les extensions en majuscules
      if (allowedTypes.indexOf(imgType) !== -1) {
        const natImage = DataOutputStream.getNatImage(imgType)
        const reader = new FileReader() // eslint-disable-line no-undef
        reader.readAsDataURL(file)
        reader.addEventListener('load', function () {
          self.callBackOK(natImage, reader.result) // reader.result contient la chaîne Base64 représentant l'image
        }, false)
        this.destroy()
        return
      }
    }
    new AvertDlg(app, 'FichierErr')
  }
}
