/*
 * Created by yvesb on 16/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const NatCal = require('../types/NatCal')
const Nat = require('../types/Nat')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
module.exports = InsertionFormuleDlg

/**
 * Dialogue appelé par les boîtes de dialogue de création d'un affichageLaTeX ou d'un commentaire
 * quand on clique sur le bouton d'insertion
 * @constructor
 * @param app L'application propriétaire
 * @param textarea Le textarea dans lequel sera copié le code lors de la validation
 * @param dlg Le dialogue propriétaire du textarea (dialogue d'adition de commentiare ou d'affichage Latex)
 * @param indmax L'indice maximum des objets à résenter dans la liste (-1 pour avoir tous les objets).
 * @param blatex true si on édite un CLaTeX, false pour un CCommentaire
 */
function InsertionFormuleDlg (app, textarea, dlg, indmax, blatex) {
  MtgDlg.call(this, app, 'InsDynDlg')
  this.textarea = textarea
  this.dlg = dlg
  this.blatex = blatex
  var self = this
  var list = this.app.listePr
  this.inf = list.listeParNatCal(app, Nat.or(NatCal.NCalcouFoncParFormule, NatCal.NMatrice), indmax)
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 5
  })
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  // En haut un panneau contenant à gauche la liste des valeurs compatibles et à droite une liste de choix
  // pour le nombre d décimales.
  var tabHaut = kernel.ce('table', {
    cellspacing: 5
  })
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:200px'
  })
  $(caption).html(kernel.getStr('InsForDlg2'))
  td.appendChild(caption)
  this.select = kernel.ce('select', {
    size: 6, // Le nombre de lignes visibles par défaut
    style: 'width:200px'
  })
  td.appendChild(this.select)
  this.select.onchange = function () {
    self.updateCode()
  }
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  td = kernel.ce('td')
  tr.appendChild(td)
  var tabHautDroit = kernel.ce('table')
  td.appendChild(tabHautDroit)
  if (this.blatex) {
    tr = kernel.ce('tr')
    tabHautDroit.appendChild(tr)
    var input = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbformsimp'
    })
    $(input).attr('checked', 'checked')
    tr.appendChild(input)
    input.onchange = function () {
      const sel = $(this).prop('checked')
      $('#cbfrac').css('visibility', sel ? 'visible' : 'hidden')
      $('#lblfrac').css('visibility', sel ? 'visible' : 'hidden')
      self.updateCode()
    }
    var label = kernel.ce('label', {
      for: 'cbformsimp'
    })
    $(label).html(kernel.getStr('InsForDlg3'))
    tr.appendChild(label)

    tr = kernel.ce('tr')
    tabHautDroit.appendChild(tr)
    input = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbfrac'
    })
    // $(input).attr('checked', 'checked')
    tr.appendChild(input)
    input.onchange = function () {
      self.updateCode()
    }
    label = kernel.ce('label', {
      for: 'cbfrac',
      id: 'lblfrac'
    })
    $(label).html(kernel.getStr('FracRat'))
    tr.appendChild(label)
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ChoixValeurDlg3'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.textareainfo = kernel.ce('textarea', {
    id: 'info',
    cols: 50,
    rows: 5,
    disabled: 'true'
  })
  tr.appendChild(this.textareainfo)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InsDynDlg3'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.infoCode = kernel.ce('input', {
    type: 'text',
    disabled: 'true',
    size: 45
  })
  tr.appendChild(this.infoCode)
  this.updateCode()
  this.create('InsForDlg1', 500)
}
InsertionFormuleDlg.prototype = new MtgDlg()

InsertionFormuleDlg.prototype.updateCode = function () {
  var codeAInserer
  var ind = this.select.selectedIndex
  if (ind === -1) return
  var el = this.inf.pointeurs[ind]
  $(this.textareainfo).text(el.infoHist())
  var nomValeurSelectionnee = el.getNom()
  codeAInserer = this.debutCode() + nomValeurSelectionnee
  codeAInserer += this.blatex ? '}' : ')'
  $(this.infoCode).val(codeAInserer)
}

InsertionFormuleDlg.prototype.debutCode = function () {
  var deb = this.blatex ? '\\' : '#'
  var parouv = this.blatex ? '{' : '('
  var simp = this.blatex ? $('#cbformsimp').prop('checked') : false
  var frac = $('#cbfrac').prop('checked')
  return deb + (simp ? (frac ? 'ForSimpFrac' : 'ForSimp') : 'For') + parouv
}

InsertionFormuleDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  $(this.select).focus()
}

InsertionFormuleDlg.prototype.OK = function () {
  var ind = this.select.selectedIndex
  if (ind !== -1) {
    var st = $(this.textarea).val()
    var deb = $(this.textarea).textrange('get', 'start')
    var fin = $(this.textarea).textrange('get', 'end')
    var code = $(this.infoCode).val()
    $(this.textarea).val(st.substring(0, deb) + code + st.substring(fin))
    $(this.textarea).textrange('setcursor', deb + code.length)
  }
  this.dlg.updatePreview()
  this.destroy()
}
