/*
 * Created by yvesb on 13/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const PaneAlignement = require('./PaneAlignement')
const PaneTaillePolice = require('./PaneTaillePolice')
const PaneModeAffichage = require('./PaneModeAffichage')
const TextButton = require('./TextButton')
const CLatex = require('../objets/CLatex')
const StyleEncadrement = require('../types/StyleEncadrement')
const InsertionDynamiqueDlg = require('./InsertionDynamiqueDlg')
const InsertionFormuleDlg = require('./InsertionFormuleDlg')
const MtgTextAreaWithCharSpe = require('./MtgTextAreaWithCharSpe')
const AvertDlg = require('./AvertDlg')
module.exports = LatexDlg

/**
 * Dialogue de création ou modification d'un affichage LaTeX
 * @constructor
 * @param app L'application propriétaire
 * @param latex Le CLatex à modifier
 * @param modification true si on modifie un CLaTeX déjà présent dans la figure
 * @param callBackOK Fonction de callBack à appeler si on valide par OK
 * @param callBackCancel Fonction de callBack à appeler si on referme sans valider
 */
function LatexDlg (app, latex, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'LatexDlg', callBackOK, callBackCancel)
  this.latex = latex
  this.modification = modification
  var self = this
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBoutons = kernel.ce('table')
  tr.appendChild(tabBoutons)
  /*
  this.textarea = kernel.ce("textarea", {
    cols : 40,
    rows : 12,
    wrap : "off"
  });
  */
  this.textarea = new MtgTextAreaWithCharSpe(app, { cols: 45, rows: 12, wrap: 'off' }, ['grec', 'math', 'arrows'], true, this)
  this.textarea.owner = this
  var jqta = $(this.textarea)
  jqta.css('overflow-x', 'scroll')
  /*
  $(this.textarea).css("overflow-y", "auto");
  */
  // On ne peut pas utiliser de style css pour le textarea car sinon il est écrasé par un autre style de jquery
  jqta.css('font-size', '14px')
  jqta.keyup(this.onkeyup)

  // Le tableau des boutons est formé de 3 lignes de 9 icônes
  var files = [
    ['LatexQuotient', 'LatexRacine', 'LatexRacineN', 'LatexVecteur', 'LatexMesureAlgebrique', 'LatexSouligne',
      'LatexIntegrale', 'LatexSigma', 'LatexTexte'],
    ['LatexPar', 'LatexCrochet', 'LatexAccolade', 'LatexAccoladeDouble', 'LatexValeurAbsolue',
      'LatexNorme', 'LatexMatrice21', 'LatexMatrice12',
      'LatexMatrice22'],
    ['LatexMatrice31', 'LatexMatrice13',
      'LatexAngle', 'LatexLimite', 'LatexSin', 'LatexCos', 'LatexTan', 'LatexExp', 'LatexLn']
  ]
  // Version 6.4.8 : On utilise dfrac au lieu de frac suite au passage à MathJax3
  var codes = [
    ['\\dfrac{}{}', '\\sqrt{}', '\\sqrt [] {}', '\\overrightarrow {}', '\\overline {}', '\\underline {}',
      '\\int_{}^{} ', '\\sum_{}^{}', '\\text{}'],
    ['\\left(  \\right)', '\\left[  \\right]', '\\left\\{  \\right.', '\\left\\{  \\right\\}', '\\left|  \\right|',
      '\\left\\|  \\right\\|', '\\begin{array}{l}\n \n\\\\ \n\\end{array}', '\\begin{matrix}\n & \n\\end{matrix}',
      '\\begin{matrix}\n & \n\\\\  & \n\\end{matrix}'],
    ['\\begin{array}{l}\n \n\\\\ \n\\\\ \n\\end{array}', '\\begin{matrix}\n &  & \n\\end{matrix}',
      '\\widehat{}', '\\lim_{x \\to {}}', '\\sin ', '\\cos ', '\\tan ', '\\mathrm{e}^{}', '\\ln ']
  ]
  var decalage = [
    [7, 6, 7, 17, 11, 12, 6, 6, 6],
    [7, 7, 8, 8, 7, 8, 17, 15, 15],
    [17, 15, 9, 13, 5, 5, 5, 12, 4]
  ]
  for (var i = 0; i < 3; i++) {
    tr = kernel.ce('tr')
    for (var j = 0; j < 9; j++) {
      tabBoutons.appendChild(tr)
      var textButton = new TextButton(app, files[i][j], codes[i][j], this.textarea, decalage[i][j], function () { self.updatePreview() })
      tr.appendChild(textButton.container)
    }
  }
  // Au centre un tableau d'une seule ligne contenant le textarea d'édition du code et à droite un svg pour l'aperçu temps réel
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)
  tr = kernel.ce('tr')
  var th = kernel.ce('th', {
    class: 'mtgcaption'
  })
  $(th).html(kernel.getStr('CodeLatex'))
  tabCentre.appendChild(th)
  th = kernel.ce('th', {
    class: 'mtgcation'
  })
  $(th).html(kernel.getStr('Apercu'))
  tabCentre.appendChild(th)
  tabCentre.appendChild(tr)
  let td = kernel.ce('td')
  tr.appendChild(td)
  td.appendChild(this.textarea)
  td = kernel.ce('td')
  $(td).css('overflow', 'scroll').css('border', '1px solid lightgray').css('vertical-align', 'top')
  tr.appendChild(td)
  var div = kernel.ce('div', {
    style: 'position:relative;margin-left:auto;margin-right:auto;width:400px;height:150px;font-weight:normal;font-family: "Times New Roman", Times, serif;'
  })
  td.appendChild(div)
  this.svg = kernel.cens('svg', {
    width: '600',
    height: '600'
  })
  div.appendChild(this.svg)
  /* Ajout version 6.4.1 pour un rendu correct de l'aperçu à cause d'un bug MathJax */
  const rootFontFamily = getComputedStyle(document.body).fontFamily
  const containerFontFamily = getComputedStyle(this.svg.parentNode).fontFamily
  if (rootFontFamily && rootFontFamily !== containerFontFamily) {
    this.svg.parentNode.style.fontFamily = rootFontFamily
  }
  // Un ligne contenant un bouton pour insertion de valeur dynamique
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var btnIns = kernel.ce('button', {
    tabindex: -1
  })
  btnIns.onclick = function () { self.onBtnIns() }
  $(btnIns).html(kernel.getStr('InsVal'))
  tr.appendChild(btnIns)
  var btnFor = kernel.ce('button', {
    tabindex: -1,
    style: 'margin-left:10px;'
  })
  btnFor.onclick = function () { self.onBtnFor() }
  $(btnFor).html(kernel.getStr('InsForDlg1'))
  tr.appendChild(btnFor)

  // Dans la troisième ligne du tableau principal, un tableau de trois colonnes contenant un panneau de choix de
  // mode d'affichage, un panneau de choix d'alignement et un manneua de choix de taille de police
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.paneAlign = new PaneAlignement(app, latex.alignementHorizontal, latex.alignementVertical)
  var tableBas = kernel.ce('table')
  tr.appendChild(tableBas)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  td.appendChild(this.paneAlign.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  this.paneTaillePolice = new PaneTaillePolice(latex.taillePolice, function () { self.updatePreview() })
  tableBas.appendChild(this.paneTaillePolice.container)
  td = kernel.ce('td', {
    valign: 'top'
  })
  tableBas.appendChild(td)
  var list = app.listePr
  var indmax = modification ? list.indexOf(latex) - 1 : list.longueur() - 1
  this.paneModeAffichage = new PaneModeAffichage(this, false, latex.effacementFond,
    latex.encadrement, latex.couleurFond, latex.angText.chaineInfo(), indmax, function () { self.updatePreview() })
  td.appendChild(this.paneModeAffichage.container)
  if (modification) $(this.textarea).val(latex.chaineCommentaire)
  this.create('AffLatex', 800)
}

LatexDlg.prototype = new MtgDlg()

LatexDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  this.paneModeAffichage.init()
  // this.updatePreview(); // Inutile version 6.4.0
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    self.textarea.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

LatexDlg.prototype.OK = function () {
  // const time = +new Date();
  // if (time - this.timeStart < MtgDlg.delay) return; // Moins d'une seconde dans doute mauvais endroit de clic sur un bouton
  // if (this.app.lastDlgId() !== this.id) return;
  var self = this
  var ch = $(this.textarea).val()
  if (ch === '') {
    new AvertDlg(this.app, 'Vide', function () { self.textarea.focus() })
  } else {
    var lat = this.latex
    var pma = this.paneModeAffichage
    if (!pma.validate()) {
      new AvertDlg(this.app, 'Incorrect')
      return
    } // Faute dans l'angle de rotation du texte
    lat.encadrement = pma.getStyleEnc()
    lat.effacementFond = pma.getEffFond()
    lat.couleurFond = pma.getColor()
    lat.angText = pma.getAng()
    // Si l'alignement vertical ou horizontal est changé on annule tout décalage manuel précédent
    var newHorAlign = this.paneAlign.getHorAlign()
    var newVerAlign = this.paneAlign.getVerAlign()
    if ((lat.alignementHorizontal !== newHorAlign) || (lat.alignementVertical !== newVerAlign)) {
      lat.decX = 0
      lat.decY = 0
    }
    //
    lat.alignementHorizontal = newHorAlign
    lat.alignementVertical = newVerAlign
    lat.taillePolice = this.paneTaillePolice.getTaille()
    // lat.chaineCommentaire = CLatex.traiteAccents(ch); // INutu=ile avec MathJax 3
    lat.chaineCommentaire = ch
    // $(lat).css("font-size", Fonte.tailleLatex(lat.taillePolice))
    lat.determineDependances()
    lat.positionne(false, this.app.dimf)
    lat.setReady4MathJax()
    /* Supprimé version 6.4.1
    if (this.modification) {
      // Il faut récréer l'affichage complètement
      MathJax.Hub.Queue(function () {
        lat.reCreateDisplay(self.app.svgFigure)
      })
    }
    */
    if (this.callBackOK !== null) this.callBackOK()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
  }
}

LatexDlg.prototype.onkeyup = function (evt) {
  // Non traité si c'est la touche entrée ou une touche de déplacement de curseur
  var kc = evt.keyCode
  // On sort si touche entrée ou déplacement à gauche ou à droite
  if ((evt.charCode === 0) && ((kc === 13) || (kc === 37) || (kc === 39))) return
  var self = this.owner
  MathJax.Hub.Queue(function () {
    while (self.svg.childNodes.length !== 0) self.svg.removeChild(self.svg.childNodes[0])
  })
  MathJax.Hub.Queue(function () {
    self.updatePreview()
  })
}

LatexDlg.prototype.updatePreview = function () {
  var app = this.app
  var list = app.listePr
  var pma = this.paneModeAffichage
  var lat = new CLatex(list, null, false, this.latex.couleur, 0, 0, 0, 0, false, null,
    this.paneTaillePolice.getTaille(), pma.getStyleEnc(),
    pma.getEffFond(), pma.getColor(), 0, 0, $(this.textarea).val())
  lat.determineDependances(list.indexOf(this.latex))
  lat.positionne(false, app.dimf)
  lat.setReady4MathJax()
  var self = this
  MathJax.Hub.Queue(function () {
    while (self.svg.childNodes.length !== 0) self.svg.removeChild(self.svg.childNodes[0])
    var g = lat.createg()
    if (lat.encadrement !== StyleEncadrement.Sans) $(g).attr('transform', 'translate(3,2)')
    self.svg.appendChild(g)
  })
}

LatexDlg.prototype.onBtnIns = function () {
  var app = this.app
  var list = app.listePr
  new InsertionDynamiqueDlg(app, this.textarea, this, this.modification ? list.indexOf(this.latex) - 1 : -1, true)
}

LatexDlg.prototype.onBtnFor = function () {
  var app = this.app
  var list = app.listePr
  new InsertionFormuleDlg(app, this.textarea, this, this.modification ? list.indexOf(this.latex) - 1 : -1, true)
}
