/*
 * Created by yvesb on 12/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const MtgInput = require('./MtgInput')
const EditeurConst = require('./EditeurConst')
const NatObj = require('../types/NatObj')
module.exports = LieuDlg

/**
 * Dialogue de création ou modification d'un lieu de points généré par point lié (lieu continu ou discret)
 * @constructor
 * @param app La mtgApp propriétaire
 * @param lieu Le lieu de points qui doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annumlation
 */
function LieuDlg (app, lieu, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'LieuDlg', callBackOK, callBackCancel)
  var self = this
  this.lieu = lieu
  this.modification = modification
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NbPts') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)

  var input = new MtgInput({
    id: 'mtginput',
    size: 4
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  if (lieu.estDeNature(NatObj.NLieu)) {
    tr = kernel.ce('tr')
    tab.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    label = kernel.ce('label', {
      for: 'cbLieuFerme'
    })
    $(label).html(kernel.getStr('LieuFerme'))
    td.appendChild(label)
    td = kernel.ce('td')
    tr.appendChild(td)
    var cbLieuFerme = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbLieuFerme'
    })
    td.appendChild(cbLieuFerme)

    tr = kernel.ce('tr')
    tab.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    label = kernel.ce('label', {
      for: 'cbGestAuto'
    })
    $(label).html(kernel.getStr('courbeDlg9'))
    td.appendChild(label)
    td = kernel.ce('td')
    tr.appendChild(td)
    var cbGestAuto = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbGestAuto'
    })
    td.appendChild(cbGestAuto)
    if (lieu.infoLieu.fermeOuNon) cbLieuFerme.setAttribute('checked', 'checked')
    if (lieu.infoLieu.gestionDiscontinuite) cbGestAuto.setAttribute('checked', 'checked')
    $(input).val(lieu.infoLieu.nombreDePoints)
  } else $(input).val(lieu.nombreDePoints)
  this.editeur = new EditeurConst(app, input, 4, 5000)
  this.create('LieuPt', 400)
}

LieuDlg.prototype = new MtgDlg()

LieuDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

LieuDlg.prototype.OK = function () {
  var bnbptsdif
  var nbPts = parseInt($('#mtginput').val())
  if (this.editeur.validate()) {
    if (this.lieu.estDeNature(NatObj.NLieu)) {
      var info = this.lieu.infoLieu
      bnbptsdif = info.nombreDePoints !== nbPts
      if (bnbptsdif) {
        info.nombreDePoints = nbPts
        this.lieu.changeCaracteristiques()
        this.app.listePr.metAJourSurfacesAyantPourBord(this.lieu)
      }
      info.fermeOuNon = $('#cbLieuFerme').prop('checked')
      info.gestionDiscontinuite = $('#cbGestAuto').prop('checked')
    } else {
      bnbptsdif = this.lieu.nombreDePoints !== nbPts
      if (bnbptsdif) {
        this.lieu.nombreDePoints = nbPts
        this.lieu.changeCaracteristiques()
      }
    }
    this.lieu.metAJour()
    if (this.callBackOK !== null) this.callBackOK()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
  }
}
