/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgInput = require('./MtgInput')
const MacroDlg = require('./MacroDlg')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
var PaneChoixVar = require('./PaneChoixVar')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const $ = require('jquery')

module.exports = MacAffValVarDlg

/**
 * Boîte de dialogue servant à créer ou modifier une macro affectant une valeur à une variable
 * @constructor
 * @param app L'application propriétaire
 * @param {CMacroAffectationValeurVariable} mac la macro
 * @param {boolean} modification true si on modifie une macro déjà existante
 * @param callBackOK Fonction de callBack à appeler une fois qu'on a cliqué sur OK
 * @param callBackCancel Fonction de callBack à appeler une fois qu'on a cliqué sur Cancel
 */
function MacAffValVarDlg (app, mac, modification, callBackOK, callBackCancel) {
  MacroDlg.call(this, app, mac, modification, callBackOK, callBackCancel)
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie avant vérification
  this.valeur = new CValeur(list, 0)
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 2
  })
  this.appendChild(tabPrincipal)
  // Ajout de l'éditeur d'intitulé /////////////////////////////////
  this.addEditInt(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane d'information sur la macro //////////////////////
  this.addPaneInfo(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Au centre un panneau avec à gauche l'éditeur pour la valeur  //
  // à affecter à la variable et à droite une liste des variables //
  // présentes                                                    //
  /// ///////////////////////////////////////////////////////////////
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneCentreHaut = kernel.ce('table', {
    cellspacing: 2
  })
  tr.appendChild(paneCentreHaut)
  tr = kernel.ce('tr')
  paneCentreHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Valeur') + ':')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  td.appendChild(input)
  var indmax = modification ? list.indexOf(mac) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  paneCentreHaut.appendChild(paneBoutonsValFonc)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneCentreBas = kernel.ce('table', {
    cellspacing: 2
  })
  tr.appendChild(paneCentreBas)
  tr = kernel.ce('tr')
  paneCentreBas.appendChild(tr)
  this.paneChoixVar = new PaneChoixVar(app, true, 'AffectA')
  tr.appendChild(this.paneChoixVar.container)
  /// ////////////////////////////////////////////////////////////////////////////////////////////////////
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeur, null)
  if (modification) {
    $(input).val(mac.valeurAssociee.calcul.chaineCalculSansPar())
    this.paneChoixVar.selectVar(this.mac.variableAssociee)
  }
  this.addPaneBas(tabPrincipal)
  this.create('MacAffValVar', 550)
}

MacAffValVarDlg.prototype = new MacroDlg()

MacAffValVarDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  if ($('#inputint').val() !== '') {
    if (this.editor.validate($('#mtginput').val())) {
      this.saveBasic()
      this.mac.valeurAssociee = this.valeur
      this.mac.variableAssociee = this.paneChoixVar.getVar()
      this.destroy()
      if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
      if (this.callBackOK !== null) this.callBackOK()
    }
  } else this.avertIncorrect()
}
