/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgInput = require('./MtgInput')
const MacroDlg = require('./MacroDlg')
const PaneAnimation = require('./PaneAnimation')
const InfoAnim = require('../types/InfoAnim')
const $ = require('jquery')

const AvertDlg = require('./AvertDlg')

module.exports = MacAnimDlg

/**
 * Dialogue de création ou modification d'une macro d'apparition d'objets
 * @constructor
 * @param app La mtgApp propriétaire
 * @param bAvecTraces true pour une macro d'apparition avec trace d'objets, false sinon
 * @param mac La macro qui doit être modifiée
 * @param modification {boolean} : true si le dialogue est ouvert pour modifier une macro déjà existante
 */
function MacAnimDlg (app, bAvecTraces, mac, modification, callBackOK, callBackCancel) {
  MacroDlg.call(this, app, mac, modification, callBackOK, callBackCancel)
  var tr, div, label, cb, radio1, radio2
  this.bAvecTraces = bAvecTraces
  var self = this
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 2
  })
  this.appendChild(tabPrincipal)
  // Ajout de l'éditeur d'intitulé /////////////////////////////////
  this.addEditInt(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane d'information sur la macro //////////////////////
  this.addPaneInfo(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane pour options de transition et fréquence /////////
  this.paneAnimation = new PaneAnimation(app, modification ? new InfoAnim(mac) : app.pref_Anim, true, true, false)
  tabPrincipal.appendChild(this.paneAnimation.container)
  /// /////////////////////////////////////////////////////////////////////////
  // Deux checkBox pour savoir si l'animation se fait sur un seul cycle //////
  // et si le point lié revient à sa position de départ en fin d'animation  //
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  cb = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbuncyc'
  })
  cb.onchange = function () {
    self.onchange()
  }
  div.appendChild(cb)
  if (mac.unCycle) $(cb).attr('checked', 'checked')
  label = kernel.ce('label', {
    for: 'cbuncyc'
  })
  $(label).html(kernel.getStr('UnCyc') + ' ')
  div.appendChild(label)
  cb = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbretfin'
  })
  div.appendChild(cb)
  if (mac.retourDepart) $(cb).attr('checked', 'checked')
  label = kernel.ce('label', {
    for: 'cbretfin'
  })
  $(label).html(kernel.getStr('RetFinCyc'))
  div.appendChild(label)
  /// ///////////////////////////////////////////////////////////////////////////////////////////////
  // Si la macro est avec traces, un checkBox pour savoir si on efface les traces en fin de cycle //
  if (bAvecTraces) {
    tr = kernel.ce('tr', {
      id: 'trtra'
    })
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbeff'
    })
    div.appendChild(cb)
    if (mac.effacementFinCycle) $(cb).attr('checked', 'checked')
    label = kernel.ce('label', {
      for: 'cbeff'
    })
    $(label).html(kernel.getStr('EffFinCyc'))
    div.appendChild(label)
  }
  /// /////////////////////////////////////////////////////////////////////////////////////
  // Deux boutons radio pour savoir si l'animation est arrêtée par un clic souris ou si //
  // Elle est de durée donnée, avec un champ d'édition pour la durée.                   //
  tr = kernel.ce('tr', {
    id: 'trclic'
  })
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  radio1 = kernel.ce('input', {
    type: 'radio',
    id: 'stopclic',
    name: 'opanim'
  })
  radio1.onchange = function () {
    $('#inputdur').css('visibility', $('#stopclic').prop('checked') ? 'hidden' : 'visible')
  }
  div.appendChild(radio1)
  label = kernel.ce('label', {
    for: 'stopclic'
  })
  $(label).html(kernel.getStr('StopClic'))
  div.appendChild(label)
  tr = kernel.ce('tr', {
    id: 'trdur'
  })
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  radio2 = kernel.ce('input', {
    type: 'radio',
    id: 'durdon',
    name: 'opanim'
  })
  radio2.onchange = function () {
    self.onradio2change()
  }
  div.appendChild(radio2)
  label = kernel.ce('label', {
    for: 'durdon'
  })
  $(label).html(kernel.getStr('DureeDon'))
  div.appendChild(label)
  this.inputDur = new MtgInput({
    id: 'inputdur',
    size: 6
  })
  div.appendChild(this.inputDur)
  if (mac.dureeAnimation === 0) {
    radio1.setAttribute('checked', 'checked')
    $('#inputdur').css('visibility', 'hidden')
  } else {
    radio2.setAttribute('checked', 'checked')
    $('#inputdur').val(mac.dureeAnimation)
  }
  this.inputDur.onkeyup = function () {
    this.demarquePourErreur()
  }
  /// ///////////////////////////////////////////////////////////////
  // En bas, ajout d'un tableau de trois colonnes contenant un
  // panneau de choix de mode d'alignement et un panneau de choix
  // de taille de police
  /// ///////////////////////////////////////////////////////////////
  this.addPaneBas(tabPrincipal)
  this.onchange()
  this.create(bAvecTraces ? 'MacAnimTr' : 'MacAnim', 500)
}

MacAnimDlg.prototype = new MacroDlg()

MacAnimDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var mac = this.mac
  var self = this
  if ($('#inputint').val() !== '') {
    if (this.paneAnimation.getinfoAnim()) {
      if (!$('#stopclic').prop('checked')) {
        var dur = $(this.inputDur).val()
        var duree = parseInt(dur)
        if ((dur === '') || (duree < 0) || (duree > 3000)) {
          new AvertDlg(this.app, 'Incorrect', function () {
            self.inputDur.marquePourErreur()
            self.inputDur.focus()
          })
          return
        } else mac.dureeAnimation = duree
      } else mac.dureeAnimation = 0
      this.saveBasic()
      var infoAnim = this.paneAnimation.infoAnim
      mac.nombrePointsPourAnimation = infoAnim.nbPts
      mac.frequenceAnimationMacro = infoAnim.freq
      mac.animationCyclique = infoAnim.animCycl
      mac.inverserSens = infoAnim.invSens
      if (this.bAvecTraces) mac.effacementFinCycle = $('#cbeff').prop('checked')
      mac.unCycle = $('#cbuncyc').prop('checked')
      if (mac.unCycle) mac.dureeAnimation = 50
      mac.retourDepart = $('#cbretfin').prop('checked')
      if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
      this.destroy()
      if (this.callBackOK !== null) this.callBackOK()
    }
  } else this.avertIncorrect()
}

MacAnimDlg.prototype.onchange = function () {
  var b = $('#cbuncyc').prop('checked')
  $('#trtra').css('visibility', b ? 'hidden' : 'visible')
  $('#trclic').css('visibility', b ? 'hidden' : 'visible')
  $('#trdur').css('visibility', b ? 'hidden' : 'visible')
  // $("#inputdur").css("visibility", b ? "hidden" : "visible");
  if (!b) $('#stopclic').attr('checked', 'checked')
  this.onradio2change()
}

MacAnimDlg.prototype.onradio2change = function () {
  var sel = $('#inputdur')
  var b = $('#cbuncyc').prop('checked')
  sel.css('visibility', $('#durdon').prop('checked') && !b ? 'visible' : 'hidden')
  sel.val('50')
}
