/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MacroDlg = require('./MacroDlg')
const MtgInput = require('./MtgInput')
const $ = require('jquery')

const AvertDlg = require('./AvertDlg')

module.exports = MacClignDlg

/**
 * Dialogue de création ou modification d'une macro de clignotement d'objets
 * @constructor
 * @param app La mtgApp propriétaire
 * @param mac La macro qui doit être modifiée
 * @param {boolean} modification  true si le dialogue est ouvert pour modifier une macro déjà existante
 */
function MacClignDlg (app, mac, modification, callBackOK, callBackCancel) {
  MacroDlg.call(this, app, mac, modification, callBackOK, callBackCancel)
  var tr, div, label, radio1, radio2
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  // Ajout de l'éditeur d'intitulé //////////////////////////////
  this.addEditInt(tabPrincipal)
  /// ///////////////////////////////////////////////////////////////
  // Ajout du pane d'information sur la macro //////////////////////
  this.addPaneInfo(tabPrincipal)
  /// /////////////////////////////////////////////////////////////////////////////////////
  // Deux boutons radio pour savoir si l'animation est arrêtée par un clic souris ou si //
  // Elle est de durée donnée, avec un champ d'édition pour la durée.                   //
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  radio1 = kernel.ce('input', {
    type: 'radio',
    id: 'stopclic',
    name: 'opanim'
  })
  radio1.onchange = function () {
    $('#inputdur').css('visibility', $('#stopclic').prop('checked') ? 'hidden' : 'visible')
  }
  div.appendChild(radio1)
  label = kernel.ce('label', {
    for: 'stopclic'
  })
  $(label).html(kernel.getStr('StopClic'))
  div.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  radio2 = kernel.ce('input', {
    type: 'radio',
    id: 'durdon',
    name: 'opanim'
  })
  radio2.onchange = function () {
    var sel = $('#inputdur')
    sel.css('visibility', $('#durdon').prop('checked') ? 'visible' : 'hidden')
    if (mac.dureeAnimation === 0) sel.val('50')
  }
  div.appendChild(radio2)
  label = kernel.ce('label', {
    for: 'durdon'
  })
  $(label).html(kernel.getStr('DureeDon'))
  div.appendChild(label)
  this.inputDur = new MtgInput({
    id: 'inputdur',
    size: 6
  })
  div.appendChild(this.inputDur)
  if (mac.dureeAnimation === 0) {
    radio1.setAttribute('checked', 'checked')
    $('#inputdur').css('visibility', 'hidden')
  } else {
    radio2.setAttribute('checked', 'checked')
    $('#inputdur').val(mac.dureeAnimation)
  }
  this.inputDur.onkeyup = function () {
    this.demarquePourErreur()
  }
  /// ///////////////////////////////////////////////////////////////
  // En bas, ajout d'un tableau de trois colonnes contenant un
  // panneau de choix de mode d'alignement et un panneau de choix
  // de taille de police
  /// ///////////////////////////////////////////////////////////////
  this.addPaneBas(tabPrincipal)
  this.create('MacClign', 480)
}

MacClignDlg.prototype = new MacroDlg()

MacClignDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var mac = this.mac
  var self = this
  if ($('#inputint').val() !== '') {
    if (!$('#stopclic').prop('checked')) {
      var dur = $(this.inputDur).val()
      var duree = parseInt(dur)
      if ((dur === '') || (duree < 0) || (duree > 3000)) {
        new AvertDlg(this.app, 'Incorrect', function () {
          self.inputDur.marquePourErreur()
          self.inputDur.focus()
        })
        return
      } else mac.dureeAnimation = duree
    } else mac.dureeAnimation = 0

    this.saveBasic()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  } else this.avertIncorrect()
}
