/*
 * Created by yvesb on 03/06/2021.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const AvertDlg = require('./AvertDlg')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')

module.exports = MatriceParFormDlg

/**
 * @constructor
 * @param app
 * @param mat
 * @param modification
 * @param callBackOK
 * @param callBackCancel
 * @constructor
 */
function MatriceParFormDlg (app, mat, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'matDlg', callBackOK, callBackCancel)
  let td, tr, label
  const self = this
  this.mat = mat
  this.modification = modification
  const list = app.listePr
  this.valn = new CValeur(list, this.mat.n)
  this.valp = new CValeur(list, this.mat.p)
  const indmax = modification ? list.indexOf(mat) - 1 : list.longueur() - 1
  this.valeur = new CValeur(list) // Sera chargé de contenir la formule proposée
  const tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  const tabNom = kernel.ce('table')
  tr.appendChild(tabNom)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  const tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabNom.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('NomMat') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  this.inputName = new MtgInput()
  td.appendChild(this.inputName)
  tr.appendChild(td)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  const tabDim = kernel.ce('table')
  tr.appendChild(tabDim)
  tr = kernel.ce('table')
  tabDim.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('NbLig') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  this.inputn = new MtgInput({
    id: 'mtginputn',
    size: '4'
  })
  td.appendChild(this.inputn)
  tr.appendChild(td)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('NbCol') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  this.inputp = new MtgInput({
    id: 'mtginputp',
    size: 4
  })
  td.appendChild(this.inputp)

  tr.appendChild(td)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  const tabVal = kernel.ce('table')
  tr.appendChild(tabVal)
  tr = kernel.ce('tr')
  tabVal.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  const tab = kernel.ce('table')
  // this.appendChild(tab);
  td.appendChild(tab)
  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Formule') + '(i,j) : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputFormula = new MtgInput({
    size: 30
  })
  td.appendChild(this.inputFormula)
  this.inputFormula.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputFormula, true, indmax)
  tab.appendChild(paneBoutonsValFonc)
  if (modification) {
    $(this.inputName).val(this.mat.nomCalcul)
    $(this.inputName).attr('disabled', true)
    $(this.inputn).val(this.mat.n)
    $(this.inputp).val(this.mat.p)
  }
  $(this.inputn).val(this.mat.n)
  $(this.inputp).val(this.mat.p)
  $(this.inputFormula).val(this.mat.calcul.chaineCalculSansPar(['i', 'j']))

  // Deux éditeurs de valeur réelle pour le nombre de lignes et de colonnes sans avertissement si faute de syntaxe
  this.editeurn = new EditeurvaleurReelle(app, this.inputn, -1, this.valn, null)
  this.editeurp = new EditeurvaleurReelle(app, this.inputp, -1, this.valp, null)
  // Un éditeur de formule pour la fonction de deux varia bles (i,j)
  this.editeurFormule = new EditeurvaleurReelle(app, this.inputFormula, indmax, this.valeur, ['i', 'j'])
  this.editeurName = new EditeurNomCalcul(app, !modification, this.inputName)

  this.create('MatriceParForm', 550)
}

MatriceParFormDlg.prototype = new MtgDlg()

MatriceParFormDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  const self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    if (self.modification) {
      $('#mtginput1').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    } else {
      $(self.inputName).focus()
    }
  }, 100)
}

MatriceParFormDlg.prototype.OK = function () {
  if (this.editeurName.validate()) {
    const app = this.app
    if (!this.modification) {
      this.mat.nomCalcul = $(this.inputName).val()
    }
    // On regarde si la chaîne entrée est non-vide et ne contient que des caractères ascii
    if (!this.editeurn.validate($('#mtginputn').val())) {
      return
    } else {
      const nblig = this.valn.rendValeur()
      if (nblig < 0) {
        new AvertDlg(app, 'Invalide')
        $('#mtginputn').focus()
      } else this.mat.n = nblig
    }
    if (!this.editeurp.validate($('#mtginputp').val())) {
      return
    } else {
      const nbcol = this.valp.rendValeur()
      if (nbcol < 0) {
        new AvertDlg(app, 'Invalide')
        $('#mtginputp').focus()
      } else this.mat.p = nbcol
    }
    if (this.editeurFormule.validate($(this.inputFormula).val())) {
      this.mat.calcul = this.valeur.calcul
      // Il faut recréer les calculs clones du calcul principal
      this.mat.creeCalculs()
      if (!this.modification) this.app.ajouteElement(this.mat)
      this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'MatriceParForm')
      if (this.callBackOK !== null) this.callBackOK()
      this.destroy()
    }
  }
}
