/*
 * Created by yvesb on 06/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCalAdd')
const MtgDlg = require('./MtgDlg')
const ConfirmDlg = require('./ConfirmDlg')
const NatObj = require('../types/NatObj')
const AvertDlg = require('./AvertDlg')
const $ = require('./../global/jQuery-ui')
module.exports = ModificationObjNumDlg

/**
 * Boîte de dialogue proposant une liste formé de tous les objets de type calcul
 * dont on peut modifier la formule.
 * Des boutons permettent de modifier, supprimer l'objet.
 * @param app L'application MtgApp propriétaire
 * @param callBackOK Fonction de callBack à appeler après validation par OK.
 * @constructor
 */
function ModificationObjNumDlg (app, callBackOK) {
  var tabPrincipal, tabHaut, tabBoutons, tr, td, label, btnModifier, btnSupprimer, self, btnReclassDeb,
    btnReclassFin
  MtgDlg.call(this, app, 'ModificationObjNumDlg', callBackOK)
  self = this
  this.inf = this.app.listePr.listeParNatCal(app, NatCal.NTtObjNum, -1)
  tabPrincipal = kernel.ce('table', {
    cellspacing: 10
  })
  this.appendChild(tabPrincipal)
  tr = kernel.ce('TR')
  tabPrincipal.appendChild(tr)
  tabHaut = kernel.ce('table', {
    cellspacing: 10
  })
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.select = kernel.ce('SELECT', {
    size: 12, // Le nombre de lignes visilbles par défaut
    style: 'width:200px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  td = kernel.ce('td', {
    style: 'vertical-align:middle;'
  })
  tr.appendChild(td)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  this.updateList()

  tabBoutons = kernel.ce('table', {
    cellspacing: 10
  })
  td.appendChild(tabBoutons)
  tr = kernel.ce('tr')
  tabBoutons.appendChild(tr)
  btnModifier = kernel.ce('button', {
    tabindex: -1,
    style: 'width:280px'
  })
  this.btnModifier = btnModifier
  btnModifier.onclick = function () { self.onBtnModifier() }
  $(btnModifier).html(kernel.getStr('Modifier'))
  tr.appendChild(btnModifier)
  tr = kernel.ce('tr')
  tabBoutons.appendChild(tr)
  btnSupprimer = kernel.ce('button', {
    tabindex: -1,
    style: 'width:280px'
  })

  this.btnSupprimer = btnSupprimer
  btnSupprimer.onclick = function () { self.onBtnSupprimer() }
  $(btnSupprimer).html(kernel.getStr('Supprimer'))
  tr.appendChild(btnSupprimer)
  if (!this.app.estExercice) {
    tr = kernel.ce('tr')
    tabBoutons.appendChild(tr)
    btnReclassDeb = kernel.ce('button', {
      tabindex: -1,
      style: 'width:280px'
    })
    this.btnReclassDeb = btnReclassDeb
    btnReclassDeb.onclick = function () {
      self.onBtnReclass(true)
    }
    $(btnReclassDeb).html(kernel.getStr('ReclassDebObjGra'))
    tr.appendChild(btnReclassDeb)

    tr = kernel.ce('tr')
    tabBoutons.appendChild(tr)
    btnReclassFin = kernel.ce('button', {
      tabindex: -1,
      style: 'width:280px'
    })
    this.btnReclassFin = btnReclassFin
    btnReclassFin.onclick = function () {
      self.onBtnReclass(false)
    }
    $(btnReclassFin).html(kernel.getStr('ReclassFinObjGra'))
    tr.appendChild(btnReclassFin)
  }

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('ChoixValeurDlg3'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  this.textarea = kernel.ce('textarea', {
    cols: 50,
    rows: 6,
    disabled: 'true'
  })
  tr.appendChild(this.textarea)

  this.onSelectChange()
  // Attention : Pour cette boîte de dialogue ne pas appeler create
  var buttons = {}
  buttons[kernel.getStr('Fermer')] = function (ev) {
    self.destroy()
    self.stopEvent(ev)
    self.callBackOK()
  }

  $('#' + self.id).dialog({
    modal: true,
    title: kernel.getStr('ObjNum'),
    buttons: buttons,
    close: function (ev) {
      self.destroy()
      self.stopEvent(ev)
      self.callBackOK()
    },
    width: 570,
    closeOnEscape: false,
    // On centre la boîte de dialogue sur le div parent de l'application
    position: { my: 'center', at: 'center', of: '#divMtgDlg' }
  })
}

ModificationObjNumDlg.prototype = new MtgDlg()

ModificationObjNumDlg.prototype.onSelectChange = function () {
  var ind = this.select.selectedIndex
  var el = this.inf.pointeurs[ind]
  let ch = el.infoHist()
  if (el.estElementFinal) ch += '\n' + kernel.getStr('ObjFin') + ' ' + el.impProto.nomProto
  $(this.textarea).text(ch)
  $(this.btnModifier).css('visibility', el.modifiableParMenu() ? 'visible' : 'hidden')
  $(this.btnSupprimer).css('visibility', el.estDeNatureCalcul(NatCal.NCalculReelConstant) ? 'hidden' : 'visible')
  var reclassable = el.estDeNatureCalcul(NatCal.NTtObjNumSaufConst)
  $(this.btnReclassDeb).css('visibility', reclassable ? 'visible' : 'hidden')
  $(this.btnReclassFin).css('visibility', reclassable ? 'visible' : 'hidden')
}

/**
 * Fonction reclassant si c'est possible l'objet en cours vers le début si bdebut est true, vers la fin sinon
 * @param {boolean} bdebut
 */
ModificationObjNumDlg.prototype.onBtnReclass = function (bdebut) {
  var self = this
  var app = this.app
  var list = app.listePr
  var ind = this.select.selectedIndex
  var el = this.inf.pointeurs[ind]
  if (bdebut ? list.reclasseVersDebutAvecDependants(el) : list.reclasseVersFinAvecDependants(el)) {
    list.positionneDependantsDe(false, app.dimf, el)
    list.metAJourParNatureObjet(NatObj.NComouLatex) // Pour que les dépendances de commentaires et laTeX dynamiques soient remis à jour
    // Des commentaires ou LaTeX dynamiques peuvent avoir à être mis à jours
    list.positionneParNat(NatObj.NComouLatex, false, app.dimf)
    // Réafficher l'élément ne suffit pas car sa place dans le Dom ne chagera pas.
    // Il faut tout réafficher
    app.reCreateDisplay()
    //
    app.gestionnaire.enregistreFigureEnCours(bdebut ? 'ReclassDebObjGra' : 'ReclassFinObjGra')
    new AvertDlg(app, bdebut ? 'AvertRecDeb' : 'AvertRecFin', function () { self.updateList() })
  } else new AvertDlg(app, 'AvertRecImp')
}

ModificationObjNumDlg.prototype.onBtnModifier = function () {
  var app = this.app
  var ind = this.select.selectedIndex
  var el = this.inf.pointeurs[ind]
  var self = this
  if (el.modifiableParMenu()) {
    el.modifDlg(app, function () { // La fonction utilisée quand l'utilisateur valide la boîte de dialogue
      // app.listePr.metAJour(); // Pour mettre à jour les calculs de dérivées et le lieu d'objets
      // A revoir
      // Il est nécessaire d'appler positionneDependantsDe car par exemple le nombre d'objets d'un lieu
      // d'objets peut avoir été modifié et il faut alors le mettre à jour mais quand on le met à jour
      // sa liste de copies d'objets n'est plus positionnée et pour chacune de ses copies existe est undefined
      var list = app.listePr
      list.initialiseDependances()
      list.positionneDependantsDe(false, app.dimf, el)
      list.metAJourObjetsDependantDe(el)
      list.positionneDependantsDe(false, app.dimf, el) // Il faut rappeler positionneDependantsDe pour par exemple recalculer les dérivées
      list.setDependantLatexToBeUpdated(el) // Le deuxième positionnement des CLaTeX dépendant de el a mis leur membre isToBeUpdated
      // à false et il faut le remettre à true pour qu'ils soient réaffichés
      // car ces dérivées n'ont été mises à jour que dans metAJourObjetsDependantDe
      list.updateDependants(el, app.svgFigure, app.doc.couleurFond, app.doc.opacity, true)
      if (el.estDeNatureCalcul(NatCal.NVariable)) el.updateDisplay()
      self.updateList()
      self.select.selectedIndex = ind
      self.onSelectChange()
    }, null)
  }
}

ModificationObjNumDlg.prototype.onBtnSupprimer = function () {
  var ind = this.select.selectedIndex
  var el = this.inf.pointeurs[ind]
  var app = this.app
  var list = app.listePr
  var self = this
  if (list.nombreDependants(el) >= 2) {
    new ConfirmDlg(app, 'ch37', function () { self.detruitObjet(el) }, function () {})
  } else this.detruitObjet(el)
}

ModificationObjNumDlg.prototype.detruitObjet = function (el) {
  var app = this.app
  var bdlgAss = el.estDeNatureCalcul(NatCal.NVariable) && el.dialogueAssocie
  if (bdlgAss) app.removePaneVariables()
  app.detruitDependants(el)
  if (bdlgAss) app.listePr.creePaneVariables()
  app.gestionnaire.enregistreFigureEnCours('Supprimer')
  this.updateList()
  this.onSelectChange()
  app.reInitConst() // Pour réinitialiser une éventuelle construction en cours
}

ModificationObjNumDlg.prototype.updateList = function () {
  var i, liste, option, self
  self = this
  liste = this.app.listePr
  $(this.select).empty()
  this.inf = liste.listeParNatCal(this.app, NatCal.NTtObjNum, -1)
  for (i = 0; i < this.inf.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    option.ondblclick = function () { self.onBtnModifier() }
    this.select.appendChild(option)
  }
}
