/*
 * Created by yvesb on 02/05/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
// const $ = require('./jQuery-ui')
const $ = require('jquery')
const kernel = require('../kernel/kernel')
const AvertDlg = require('./AvertDlg')
const PaneFreq = require('./PaneFreq')
module.exports = PaneAnimation

/**
 * Classe représentant un panneau permettant de choisir des options pour une animation animation
 * @param app L'application mtgApp propriétaire
 * @param infoAnim objet du type infoAnim où
 *     nbPts : est le nombre de points à calculer pour l'animation
 *     freq : est la fréquence d'animation en millièmes de secondes
 *     animCycl : est un boolean qui vaut true si l'animation est cyclique
 *     invSens : est un boolean qui vaut true si on doit inverser le sens d'animation
 * @param {boolean} binvSens true si le chechBox pour inverser le sens doit être présent
 * @param {boolean} bAnimaCyc true si le chackBox pour choix d'animation cyclique doit présent
 * @param {boolean} bFreqMax true si on veut qu'une fréquance max soit dans la liste
 * @constructor
 */
function PaneAnimation (app, infoAnim, binvSens, banimCyc, bFreqMax) {
  this.app = app
  this.infoAnim = infoAnim
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 10
  }) // Le tableau principal qui sera this.container
  this.container = tabPrincipal
  $(tabPrincipal).css('border', '1px solid lightgray')
  var td = kernel.ce('td') // La colonne de gauche
  $(td).css('vertical-align', 'top')
  tabPrincipal.appendChild(td)
  var tabgauche = kernel.ce('table')
  td.appendChild(tabgauche)
  var tr = kernel.ce('tr')
  tabgauche.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NbPtsPourAnim'))
  div.appendChild(label)
  var input = kernel.ce('input', {
    id: 'mtginput',
    type: 'text'
  })
  input.size = 4
  $(input).val(infoAnim.nbPts)
  div.appendChild(input)
  if (banimCyc) {
    tr = kernel.ce('tr')
    tabgauche.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    var cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbAnimCycl'
    })
    div.appendChild(cb)
    if (infoAnim.animCycl) $(cb).attr('checked', 'checked')
    label = kernel.ce('label', {
      for: 'cbAnimCycl'
    })
    $(label).html(kernel.getStr('AnimCycl'))
    div.appendChild(label)
  }
  if (binvSens) {
    tr = kernel.ce('tr')
    tabgauche.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    cb = kernel.ce('input', {
      type: 'checkbox',
      id: 'cbInv'
    })
    div.appendChild(cb)
    if (infoAnim.invSens) $(cb).attr('checked', 'checked')
    label = kernel.ce('label', {
      for: 'cbInv'
    })
    $(label).html(kernel.getStr('InvSens'))
    div.appendChild(label)
  }
  this.paneFreq = new PaneFreq(infoAnim.freq, bFreqMax)
  tabPrincipal.appendChild(this.paneFreq.container)
  this.container = tabPrincipal
}

PaneAnimation.prototype.getinfoAnim = function () {
  var nbPts = parseInt($('#mtginput').val())
  if (nbPts < 2 || nbPts > 10000 || isNaN(nbPts)) {
    new AvertDlg(this.app, 'Incorrect', function () {
      $('#mtginput').focus()
      return false
    })
  } else {
    this.infoAnim.set(nbPts, this.paneFreq.getFreq(), $('#cbAnimCycl').prop('checked'),
      $('#cbInv').prop('checked'))
    return true
  }
}
