/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const CarButton = require('./CarButton')
const Fonte = require('../types/Fonte')
module.exports = PaneInsChar

/**
 * Fonction créant un tableau formé de boutons de caractères associés soit au caractère lui-même soit au
 * code LaTeX de ce caractère
 * @param {string} family  "grec", "mathjs" ou "arrows"
 * @param editor L'éditeur associe dans lequel le code est inséré quand on clique sur le boutons
 * @param dlg Le dialogue propriétaire de l'éditeur
 * @param bLatex true si on insére le code LaTeX du caractère, false si on insère le caractère lui-même
 * @param dlg Le dialogue propriétaire de l'éditeur
 * @constructor
 */
function PaneInsChar (family, editor, dlg, bLatex) {
  var tabChar, tabCode
  switch (family) {
    case 'grec' :
      tabChar = Fonte.grec
      tabCode = bLatex ? Fonte.grecLatex : tabChar
      break
    case 'math' :
      tabChar = Fonte.math
      tabCode = bLatex ? Fonte.mathLatex : tabChar
      break
    case 'arrows' :
      tabChar = Fonte.arrows
      tabCode = bLatex ? Fonte.arrowsLatex : tabChar
  }
  var nbCarParLigne = 6
  var tab = kernel.ce('table', {
    cellspacing: 1,
    style: 'width:60px;height:100px'
  })
  for (var i = 0; i < tabChar.length; i++) {
    var btn = new CarButton(tabChar[i], tabCode[i], editor, dlg)
    if (i % nbCarParLigne === 0) {
      var tr = kernel.ce('tr')
      tab.appendChild(tr)
    }
    var td = kernel.ce('td', {
      style: 'text-align:center;' // Pour que les bouton soient centrés
    })
    tr.appendChild(td)
    td.appendChild(btn.container)
  }
  this.container = tab
}
