/*
 * Created by yvesb on 26/05/2017.
 */

/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const constantes = require('../kernel/constantes')
const MtgDlg = require('./MtgDlg')
const CListeObjets = require('../objets/CListeObjets')
const NatObj = require('../types/NatObj')
const NatCal = require('../types/NatCal')
const kernelAdd = require('../kernel/kernelAdd')
const $ = require('./../global/jQuery-ui')
const ConfirmDlg = require('./ConfirmDlg')
const AvertDlg = require('../dialogs/AvertDlg')
const TagDlg = require('./TagDlg')
module.exports = ProtocoleDlg

/**
 * Boîte de dialogue montrant l'historique de tous les objets de la figure.
 * Des boutons permettent de modifier, supprimer, reclasser l'objet.
 * @param app L'application MtgApp propriétaire
 * @param callBackOK Fonction de callBack à appeler après validation par OK.
 * @constructor
 */
function ProtocoleDlg (app, callBackOK) {
  MtgDlg.call(this, app, 'ProtocoleDlg', callBackOK)
  // Modification version 6.3.0. Quand le dialogue est ouvert, on met app.isRunningProtocol à true.
  // Ainis quand on modifie un objet dont le dialogue de modification appelle la sauvegarde de la figure, cette sauvegarde
  // ne sera pas effectuée par l'objet Gestionnaire
  app.isRunningProtocol = true
  var estExercice = app.estExercice
  // this.indPremierObjet représente l'inidce du premier objet à ne pas afficher (dans le cas d'exercice ce n'est pas 0)
  this.indPremierObjet = estExercice ? app.nbObjInit : 0
  this.modified = false // Sera mis à true si l'utilisateur a modifié ou reclassé un objet
  this.interm = false // Sera true si les objets intermédiaires de construction doivent être affichés
  // Le reclassement d'objets peut être lent pour une figure mourde.
  // Il ne faut pas qu'on puisse lancer un reclassement alors qu'un recassement est déjà en cours
  this.isReclassing = false
  var self = this
  var list = this.app.listePr
  // On retire tous les g Elements de la figure pour les recréer ensuite une fois nommés les points non nommés nommés
  list.removegElements(app.svgFigure, false, this.indPremierObjet)
  // list.deleteComponents(app.svgFigure);
  // On restraint le nombre d'objets à présenter dans la liste aux 3000 premiers objets
  this.indObjMax = Math.min(list.longueur() - 1, constantes.nbObjMaxProto)
  this.infInit = this.listePourHist(this.indObjMax)
  this.listeObjetsNommes = new CListeObjets()
  // list.genereNoms(this.listeObjetsNommes, false);
  list.genereNoms(estExercice ? app.nbObjInit : 0, this.listeObjetsNommes)
  // On recrée les gElements associés aux objets graphiques
  this.prepareForProtocol(true, false)
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 2
  })
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  var td = kernel.ce('td')
  tabHaut.appendChild(td)
  this.select = kernel.ce('select', {
    size: 12, // Le nombre de lignes visibles par défaut
    style: 'width:220px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  this.updateSelect(0)
  td = kernel.ce('td')
  tabHaut.appendChild(td)
  $(td).css('vertical-align', 'top')
  var tabIcones = kernel.ce('table')
  // Un tableau pour les icônes permettant de naviguer dans la liste et de reclasser les objets.
  var ar = estExercice
    ? ['upMaxArrow', 'upArrow', 'downArrow', 'downMaxArrow',
      'outilGomme', 'outilRideau']
    : ['upMaxArrow', 'upMax', 'upArrow', 'up', 'downArrow', 'down', 'downMaxArrow', 'downMax',
      'outilSup', 'supEnd', 'outilGomme', 'outilRideau', 'outilModifObjGraph']

  var af = estExercice
    ? [this.upMax, this.up, this.down, this.downMax,
      this.hide, this.show]
    : [this.upMax, this.reclassUpMax, this.up, this.reclassUp, this.down, this.reclassDown, this.downMax, this.reclassDownMax,
      this.sup, this.supEnd, this.hide, this.show, this.modify]
  td.appendChild(tabIcones)
  for (var j = 0; j < (estExercice ? 6 : 13); j++) {
    if (estExercice && (j < 4)) tr = kernel.ce('tr')
    else if (j % 2 === 0) tr = kernel.ce('tr')
    tabIcones.appendChild(tr)
    td = kernel.ce('td', {
      id: ar[j]
    })
    td.owner = this
    tr.appendChild(td)
    $(td).css('padding', '1px')
    $(td).html(app.local
      ? "<img src='" + app.localPath + 'images/' + ar[j] + ".png' width='22px' height='22px' />"
      : "<img src='" + app.imagePath + ar[j] + ".png' width='22px' height='22px' crossorigin='anonymous' />"
    )
    td.onclick = af[j]
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var info = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 40,
    rows: 4
  })
  $(info).css('font-size', '13px')
  tr.appendChild(info)
  if (!estExercice) {
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    var div = kernel.ce('div')
    tr.appendChild(div)
    var tabTag = kernel.ce('table',
      {
        id: 'tabtag'
      })
    div.appendChild(tabTag)
    tr = kernel.ce('tr')
    tabTag.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    var label = kernel.ce('label')
    $(label).text(kernel.getStr('tag') + ' : ')
    td.appendChild(label)
    td = kernel.ce('td')
    tr.appendChild(td)
    var tagInput = kernel.ce('input', {
      id: 'taginput',
      size: 16,
      disabled: 'true'
    })
    td.appendChild(tagInput)
    tr = kernel.ce('tr')
    tabTag.appendChild(tr)
    td = kernel.ce('td')
    tr.appendChild(td)
    td = kernel.ce('td')
    tr.appendChild(td)
    var btnTag = kernel.ce('button')
    btnTag.owner = this
    $(btnTag).css('font-size', '13px')
    $(btnTag).html(kernel.getStr('changeTag'))
    btnTag.onclick = function () {
      var owner = this.owner
      var ind = owner.select.selectedIndex
      var el = owner.inf.pointeurs[ind]
      new TagDlg(self.app, el, function () {
        $('#taginput').val(el.tag)
        self.modified = true
      })
    }
    td.appendChild(btnTag)
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    var cbinterm = kernel.ce('input', {
      id: 'cbinterm',
      type: 'checkbox'
    })
    div.appendChild(cbinterm)
    cbinterm.onclick = function () {
      self.interm = $('#cbinterm').prop('checked')
      self.removegElements(!self.interm)
      self.updateList(!self.interm, 0)
      self.updateDisplay()
    }
    label = kernel.ce('label', {
      for: 'cbinterm'
    })
    div.appendChild(label)
    $(label).html(kernel.getStr('ObjInterm') + ' (*)')

    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    div = kernel.ce('div')
    tr.appendChild(div)
    var cbObjMasVis = kernel.ce('input', {
      id: 'cbobjmasvis',
      type: 'checkbox'
    })
    cbObjMasVis.setAttribute('checked', 'checked')
    div.appendChild(cbObjMasVis)
    this.objMasVis = true
    cbObjMasVis.onclick = function () {
      self.objMasVis = $('#cbobjmasvis').prop('checked')
      self.removegElements(self.interm)
      self.prepareForProtocol(self.objMasVis, self.interm)
      self.updateList(self.interm, 0)
      self.updateDisplay()
    }
    label = kernel.ce('label', {
      for: 'cbobjmasvis'
    })
    div.appendChild(label)
    $(label).html(kernel.getStr('ObjMasVis'))
  }
  // Correction version 6.2.0 : Il faut attendre que tous les g elements soient créés et certains peuvent être créés
  // de façon asynchrone
  MathJax.Hub.Queue(function () {
    self.onSelectChange()
  })
  // Attention : Pour cette boîte de dialogue ne pas appeler create
  var buttons = {}
  buttons[kernel.getStr('Fermer')] = function (ev) {
    self.stopEvent(ev)
    self.onClose(ev)
  }

  $('#' + self.id).dialog({
    modal: true,
    title: kernel.getStr((list.longueur() > constantes.nbObjMaxProto) ? 'ProtoNbMax' : 'ProtoFig'),
    buttons: buttons,
    close: function (ev) {
      self.onClose(ev)
    },
    width: 320,
    closeOnEscape: false,
    // On affiche la boîte de dialogue sur le div parent de l'application en haut et à droite
    position: { my: 'right top', at: 'right top', of: '#divMtgDlg' }
  })
}

ProtocoleDlg.prototype = new MtgDlg()

ProtocoleDlg.prototype.upMax = function () {
  this.owner.select.selectedIndex = 0
  this.owner.onSelectChange()
}

ProtocoleDlg.prototype.up = function () {
  var ind = this.owner.select.selectedIndex
  if (ind > 0) this.owner.select.selectedIndex--
  this.owner.onSelectChange()
}

ProtocoleDlg.prototype.downMax = function () {
  this.owner.select.selectedIndex = this.owner.select.length - 1
  this.owner.onSelectChange()
}

ProtocoleDlg.prototype.down = function () {
  var ind = this.owner.select.selectedIndex
  if (ind < this.owner.select.length - 1) this.owner.select.selectedIndex++
  this.owner.onSelectChange()
}

ProtocoleDlg.prototype.getIndex = function (el) {
  for (var i = 0; i < this.inf.noms.length; i++) {
    if (this.inf.pointeurs[i] === el) return i
  }
  return -1
}

ProtocoleDlg.prototype.prepareForProtocol = function (objMasquesVisibles, bInterm) {
  var app = this.app
  var list = app.listePr
  var doc = app.doc
  list.prepareForProtocol(app.svgFigure, doc.couleurFond, doc.opacity, objMasquesVisibles, bInterm,
    app.estExercice ? app.nbObjInit : 0, this.indObjMax)
}

ProtocoleDlg.prototype.reclassUp = function () {
  var owner = this.owner
  var app = owner.app
  var list = app.listePr
  var ind = owner.select.selectedIndex
  if (owner.isReclassing || (ind === -1)) return // Si on a appuyé sur un bouton alors qu'un reclassement est en cours
  var el = owner.inf.pointeurs[ind]
  owner.isReclassing = true
  owner.removegElements(owner.interm)
  if (owner.app.listePr.reclassUpOneStep(el)) {
    // Modification version 6.3.0
    // list.metAJourObjetsDependantDe(el);
    list.metAJourParNatureObjet(NatObj.NComouLatex)
    // Des commentaires ou LaTeX dynamiques peuvent avoir à être mis à jours
    list.positionneParNat(NatObj.NComouLatex, false, app.dimf)
    owner.updateSelect(-1)
    owner.select.selectedIndex = owner.getIndex(el)
    owner.modified = true
  } else new AvertDlg(app, 'AvertRecImp')
  owner.prepareForProtocol(owner.objMasVis, owner.interm)
  owner.updateDisplay()
  $(owner.select).focus() // Pour les périphériques mobiles
  owner.isReclassing = false
}

ProtocoleDlg.prototype.reclassDown = function () {
  var owner = this.owner
  var app = owner.app
  var list = app.listePr
  var ind = owner.select.selectedIndex
  if (owner.isReclassing || (ind === -1)) return // Si on a appuyé sur un bouton alors qu'un reclassement est en cours
  owner.isReclassing = true
  if (ind === 0) return // Pas de reclassement pour la constante pi
  var el = owner.inf.pointeurs[ind]
  owner.removegElements(owner.interm)
  if (owner.app.listePr.reclassDownOneStep(el)) {
    // Modification version 6.3.0
    // list.metAJourObjetsDependantDe(el);
    list.metAJourParNatureObjet(NatObj.NComouLatex)
    // Des commentaires ou LaTeX dynamiques peuvent avoir à être mis à jours
    list.positionneParNat(NatObj.NComouLatex, false, app.dimf)
    owner.updateSelect(-1)
    owner.select.selectedIndex = owner.getIndex(el)
    owner.modified = true
  } else new AvertDlg(app, 'AvertRecImp')
  owner.prepareForProtocol(owner.objMasVis, owner.interm)
  owner.updateDisplay()
  $(owner.select).focus() // Pour les périphériques mobiles
  owner.isReclassing = false
}

ProtocoleDlg.prototype.reclassUpMax = function () {
  var owner = this.owner
  var app = owner.app
  var list = app.listePr
  var ind = owner.select.selectedIndex
  if (owner.isReclassing || (ind === -1)) return // Si on a appuyé sur un bouton alors qu'un reclassement est en cours
  var el = owner.inf.pointeurs[ind]
  owner.isReclassing = true
  owner.removegElements(owner.interm)
  if (list.reclasseVersDebutAvecDependants(el)) {
    // Modifié verion 6.3.0
    // list.metAJourObjetsDependantDe(el);
    list.metAJourParNatureObjet(NatObj.NComouLatex)
    // Des commentaires ou LaTeX dynamiques peuvent avoir à être mis à jours
    list.positionneParNat(NatObj.NComouLatex, false, app.dimf)

    new AvertDlg(app, 'AvertRecDeb')
    owner.updateSelect(-1)
    owner.select.selectedIndex = owner.getIndex(el)
    owner.modified = true
  } else new AvertDlg(app, 'AvertRecImp')
  owner.prepareForProtocol(owner.objMasVis, owner.interm)
  owner.updateDisplay()
  $(owner.select).focus() // Pour les périphériques mobiles
  owner.isReclassing = false
}

ProtocoleDlg.prototype.reclassDownMax = function () {
  var owner = this.owner
  var app = owner.app
  var list = app.listePr
  var ind = owner.select.selectedIndex
  if (owner.isReclassing || (ind === -1)) return // Si on a appuyé sur un bouton alors qu'un reclassement est en cours
  var el = owner.inf.pointeurs[ind]
  owner.isReclassing = true
  owner.removegElements(owner.interm)
  if (list.reclasseVersFinAvecDependants(el)) {
    // Modifiction version 6.2.0
    // list.metAJourObjetsDependantDe(el);
    list.metAJourParNatureObjet(NatObj.NComouLatex)
    // Des commentaires ou LaTeX dynamiques peuvent avoir à être mis à jours
    list.positionneParNat(NatObj.NComouLatex, false, app.dimf)
    new AvertDlg(app, 'AvertRecFin')
    owner.updateSelect(-1)
    owner.select.selectedIndex = owner.getIndex(el)
    owner.modified = true
  } else new AvertDlg(app, 'AvertRecImp')
  owner.prepareForProtocol(owner.objMasVis, owner.interm)
  owner.updateDisplay()
  $(owner.select).focus() // Pour les périphériques mobiles
  owner.isReclassing = false
}

/**
 * Fonction montrant ou cachant les boutons de reclassemnt d'objet suivant la valeur de bshow
 * @param bshow Si ture on montre les boutons sinon on les cache
 */
ProtocoleDlg.prototype.montreBoutonsRecl = function (bshow) {
  var at = bshow ? 'visible' : 'hidden'
  $('#upMax').css('visibility', at)
  $('#up').css('visibility', at)
  $('#down').css('visibility', at)
  $('#downMax').css('visibility', at)
}

ProtocoleDlg.prototype.detruitObjet = function (el) {
  var app = this.app
  app.outilProtocole.annuleClignotement()
  var bdlgAss = el.estDeNatureCalcul(NatCal.NVariable) && el.dialogueAssocie
  if (bdlgAss) app.removePaneVariables()
  this.removegElements(this.interm)
  app.detruitDependants(el.antecedentDirect(), false) // false car on a déjà détruit les gElements
  if (bdlgAss) app.listePr.creePaneVariables()
  this.updateList(this.interm, this.select.selectedIndex)
  this.updateDisplay()
  this.onSelectChange()
  this.modified = true
}

ProtocoleDlg.prototype.sup = function () {
  var owner = this.owner
  var ind = owner.select.selectedIndex
  if (ind !== -1) {
    var el = owner.inf.pointeurs[ind]
    var app = owner.app
    var list = app.listePr
    if (list.nombreDependants(el) >= 2) {
      new ConfirmDlg(app, 'ch37', function () {
        owner.detruitObjet(el)
      }, function () {
      })
    } else owner.detruitObjet(el)
  }
}

ProtocoleDlg.prototype.supEnd = function () {
  var owner = this.owner
  var app = owner.app
  app.outilProtocole.annuleClignotement()
  var ind = owner.select.selectedIndex
  if (ind !== -1) {
    var el = owner.inf.pointeurs[ind].antecedentDirect()
    var list = app.listePr
    owner.removegElements(this.interm)
    var indel = el.index
    var nb = list.longueur() - indel
    for (var i = 0; i < nb; i++) app.detruitDernierElement()
    owner.updateList(owner.interm, owner.select.selectedIndex - 1)
    owner.updateDisplay()
    owner.onSelectChange()
    owner.modified = true
  }
}

ProtocoleDlg.prototype.show = function () {
  var owner = this.owner
  var ind = owner.select.selectedIndex
  var el = owner.inf.pointeurs[ind]
  el.masque = false
  owner.modified = true
  owner.onSelectChange()
}

ProtocoleDlg.prototype.hide = function () {
  var owner = this.owner
  var ind = owner.select.selectedIndex
  var el = owner.inf.pointeurs[ind]
  el.masque = true
  owner.modified = true
  owner.onSelectChange()
}
// Quand modify est appelé, this est le bouton de modification
ProtocoleDlg.prototype.modify = function () {
  this.owner.modif()
}

ProtocoleDlg.prototype.modif = function () {
  var self = this
  var app = this.app
  var ind = this.select.selectedIndex
  var el = this.inf.pointeurs[ind]
  var ant = el.antecedentDirect()
  if (ant.estDeNature(NatObj.NTransformation)) {
    ant.modificationImageForProtocol(app, el, false, function () { // La fonction utilisée quand l'utilisateur valide la boîte de dialogue
      self.removegElements(self.interm)
      app.listePr.metAJourObjetsDependantDe(el.transformation) // Pour mettre à jour les calculs de dérivées et les lieux d'objets Modifié version 6.4.1
      app.listePr.positionneDependantsDe(false, app.dimf, el.transformation, true) // Version 6.4.1 : Ajout de full = true et el.transformation
      self.updateList(self.interm, self.select.selectedIndex)
      // self.updateDisplay() // onSelectChange appelle updateDisplay
      self.onSelectChange()
      self.modified = true
    })
  } else {
    el.modifDlg(app, function () { // La fonction utilisée quand l'utilisateur valide la boîte de dialogue
      self.removegElements(self.interm)
      app.listePr.metAJourObjetsDependantDe(el) // Pour mettre à jour les calculs de dérivées et les lieux d'objets
      app.listePr.positionneDependantsDe(false, app.dimf, el, true) // Version 6.4.1 : Ajout de full = true
      self.updateList(self.interm, self.select.selectedIndex)
      // self.updateDisplay()
      self.onSelectChange() // onSelectChange appelle updateDisplay
      self.modified = true
      if (el.estDeNatureCalcul(NatCal.NVariable)) el.updateDisplay() // Pour mettre à jour un éventuel dialogue associé à une variable
    }, null)
  }
}

ProtocoleDlg.prototype.onClose = function (ev) {
  var app = this.app
  var list = app.listePr
  // var inf = this.inf;
  app.isRunningProtocol = false // Ajout version 6.3.0
  var op = app.outilProtocole
  op.annuleClignotement()
  list.removegElements(app.svgFigure, this.interm, 0, this.indObjMax)
  // On redonne aux objets leur statut masque ou non qui a été mémorisé
  this.retablitMasquageObjs()
  list.positionne(false, app.dimf)
  this.destroy()
  this.stopEvent(ev)
  this.listeObjetsNommes.effaceNoms(this.app.pref_TaillePoliceNom)
  // Ligne suivante modifié version 6.4.1 avec dernier paramètre immédiat à true
  list.afficheTout(0, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity, true)
  if (this.modified) app.gestionnaire.enregistreFigureEnCours('Modifier')
  this.callBackOK()
}

ProtocoleDlg.prototype.retablitMasquageObjs = function () {
  var inf = this.infInit
  for (var i = 0; i < inf.noms.length; i++) {
    var el = inf.pointeurs[i]
    if (el.estDeNature(NatObj.NTtObj)) {
      // el.masque = inf.maskstate[i];
      if (el.estDeNature(NatObj.NObjNommable)) el.nomMasque = inf.namemaskstate[i]
    }
  }
}

ProtocoleDlg.prototype.onSelectChange = function () {
  var ind, el
  ind = this.select.selectedIndex
  if (ind === -1) return
  el = this.inf.pointeurs[ind]
  $('#tabtag').css('display', el.estElementIntermediaire() || !el.estDeNature(NatObj.NTtObj) ? 'none' : 'block')
  $('#taginput').val(el.tag)
  this.montreBoutonsRecl(!el.estElementIntermediaire() && !el.estDeNature(NatObj.NImpProto))
  // var initialMaskedState = this.inf.maskstate[ind];
  if (el.estDeNature(NatObj.NTtObj)) {
    $('#outilGomme').css('visibility', el.masque ? 'hidden' : 'visible')
    $('#outilRideau').css('visibility', el.masque ? 'visible' : 'hidden')
  } else {
    $('#outilGomme').css('visibility', 'hidden')
    $('#outilRideau').css('visibility', 'hidden')
  }
  $('#supEnd').css('visibility', (el.antecedentDirect() === el) && (ind !== 0) ? 'visible' : 'hidden')
  if (!this.app.estExercice) {
    $('#outilModifObjGraph').css('visibility', el.modifiableParProtocole() ? 'visible' : 'hidden')
    $('#outilSup').css('visibility', el.className !== 'CCalcConst' ? 'visible' : 'hidden') // Pas permis de détruire la constante pi
  }
  var ch = el.infoHist()
  if (el.estElementIntermediaire()) ch = '*' + ch
  if (el.estDeNature(NatObj.NTtObj)) {
    if (!el.existe) ch += '\n' + kernel.getStr('ch18')
    if (el.masque) ch += '\n' + kernel.getStr('ObjMas')
  }
  if (el.estElementFinal) ch += '\n' + kernel.getStr('ObjFin') + ' ' + el.impProto.nomProto
  else if (el.estElementIntermediaire()) {
    ch += '\n' + kernel.getStr('ObjInter')
  }
  ch += '\nN°html : ' + el.index
  $('#mtginfo').text(ch)
  this.updateDisplay()
}

ProtocoleDlg.prototype.removegElements = function (interm) {
  var app = this.app
  var list = app.listePr
  list.removegElements(app.svgFigure, interm, this.indPremierObjet, this.indObjMax)
}

ProtocoleDlg.prototype.updateDisplay = function () {
  const list = this.app.listePr
  const app = this.app
  const op = app.outilProtocole
  let ind = this.select.selectedIndex
  if (ind === -1) {
    ind = this.select.length - 1
    this.select.selectedIndex = ind
  }
  const el = this.inf.pointeurs[ind]
  // Version 6.4.1 : La suite doit être dans la pile d'appels MathJax car les éléments affichés ne sont peut-être pas encore prêts
  const self = this
  MathJax.Hub.Queue(function () {
    op.annuleClignotement()
    list.displayForProtocol(self.indPremierObjet, el.index, self.objMasVis, self.interm, self.indObjMax)
    op.ajouteClignotementDe(el)
    // Le deuxième paramètre ci-dessous ne sert que pour les objets images par une transformation et que pour les
    // translations par point origine et extrémité
    el.ajouteAntecedents(app.listeClignotante, app)
    op.resetClignotement()
  })
}

ProtocoleDlg.prototype.listePourHist = function (nbObj) {
  var app = this.app
  var list = app.listePr
  return list.listePourHist(app.estExercice ? app.nbObjInit : 0, this.interm, nbObj)
}

ProtocoleDlg.prototype.updateList = function (bInterm, indSelect) {
  var app = this.app
  var list = app.listePr
  this.retablitMasquageObjs()
  // this.listeObjetsNommes.effaceNoms(this.app.pref_TaillePoliceNom);
  // this.listeObjetsNommes = new CListeObjets();
  // list.genereNoms(this.listeObjetsNommes, this.interm);
  // On recrée les gElements associés aux objets graphiques
  var doc = app.doc
  // On restraint le nombre d'objets à présenter dans la liste aux 2000 premiers objest
  this.indObjMax = Math.min(list.longueur() - 1, constantes.nbObjMaxProto)
  this.inf = this.listePourHist(this.indObjMax)
  list.prepareForProtocol(app.svgFigure, doc.couleurFond, doc.opacity, true, this.interm,
    app.estExercice ? app.nbObjInit : 0, this.indObjMax)
  this.updateSelect(indSelect)
}

ProtocoleDlg.prototype.updateSelect = function (indsel) {
  var self = this
  this.inf = this.listePourHist(this.indObjMax)
  for (var i = this.select.options.length - 1; i >= 0; i--) this.select.remove(i)
  for (i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === indsel) option.setAttribute('selected', 'selected')
    var el = this.inf.pointeurs[i]
    var ch = this.inf.noms[i]
    if (el.estElementIntermediaire()) ch = '*' + ch
    $(option).html(kernelAdd.replaceIneg(ch)) // Le replaceIneg pour que les caractères < et > soient bien rendus
    this.select.appendChild(option)
    option.ondblclick = function () {
      var e = self.inf.pointeurs[self.select.selectedIndex]
      if (e.modifiableParProtocole()) self.modif()
    }
  }
  // Si rien n'a été sélectionné c'est qu'on a détruit le dernier objet de la liste
  if (this.select.selectedIndex === -1) this.select.selectedIndex = indsel - 1
}
