/*
 * Created by yvesb on 25/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')

module.exports = PtParAbsDlg

/**
 * Dialogue de choix d'abscisse d'un point relativement à deux autres
 * @constructor
 * @param app La mtgApp propriétaire
 * @param pt L'arc dont l'angle doit être changé
 * @param modification boolean : true si on modifie l'abscisse d'un point déjà créé
 * @param callBackOK null ou fonction de callBack à appeler après OK
 * @param callBackCancel null ou fonction de callBack à appeler après annulation
 */
function PtParAbsDlg (app, pt, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'angleArcDlg', callBackOK, callBackCancel)
  var self = this
  this.pt = pt
  this.modification = modification
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie avant vérification
  this.valeur = new CValeur(list, 1)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Abscisse') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var indmax = modification ? list.indexOf(this.pt) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  tab.appendChild(paneBoutonsValFonc)
  //
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeur, null)
  if (modification) $(input).val(pt.abscisse.calcul.chaineCalculSansPar())
  this.create('PtParAbs', 400)
}

PtParAbsDlg.prototype = new MtgDlg()

PtParAbsDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  var valid = this.editor.validate($('#mtginput').val())
  if (valid) {
    this.pt.abscisse = this.valeur
    if (this.callBackOK !== null) this.callBackOK()
    if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
    this.destroy()
  }
}

PtParAbsDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}
