/*
 * Created by yvesb on 29/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const PaneListeReperes = require('./PaneListeReperes')
const MtgInput = require('./MtgInput')

module.exports = PtParCoordDlg

/**
 * Dialogue de choix de coordonnées d'un point dans un repère
 * @constructor
 * @param app La mtgApp propriétaire
 * @param point Le point dont les coordonnées doivent être changées
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 * @param callBackOK Fonction de callBack à appeler quand l'utilisateur clique sur OK
 * Dans le cas où sbutton vaut "Implem" ou ", cette fonction prend un paramètre proto qui est le prototype de la figure sélectionné
 * @param callBackCancel Fonction de callBack à appeler si l'utilisateur annule

 */
function PtParCoordDlg (app, point, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'ptParCoordDlg', callBackOK, callBackCancel)
  var self = this
  this.point = point
  this.modification = modification
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie pour l'abscisse avant vérification
  this.valeurAbs = new CValeur(list, 0)
  // Un CValeur pour stocker la valeur choisie pour l'ordonnée avant vérification
  this.valeurOrd = new CValeur(list, 0)
  var tabprincipal = kernel.ce('table')
  this.appendChild(tabprincipal)
  var trprincipal = kernel.ce('tr')
  tabprincipal.appendChild(trprincipal)
  var td = kernel.ce('td')
  trprincipal.appendChild(td)

  var tab = kernel.ce('table')
  // this.appendChild(tab);
  td.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Abscisse') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputAbs = new MtgInput({
  })
  this.inputAbs.onkeyup = function (ev) {
    this.demarquePourErreur()
    if ($(self.inputOrd).val().length !== 0) {
      if (ev.keyCode === 13) {
        $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
        self.OK()
      }
    }
  }

  td.appendChild(this.inputAbs)
  var indmax = modification ? list.indexOf(point) - 1 : list.longueur() - 1
  var paneBoutonsValFoncAbs = new PaneBoutonsValFonc(this.app, this, this.inputAbs, true, indmax)
  tab.appendChild(paneBoutonsValFoncAbs)
  this.editorAbs = new EditeurvaleurReelle(app, this.inputAbs, indmax, this.valeurAbs, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Ordonnee') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputOrd = new MtgInput({
  })
  this.inputOrd.onkeyup = function (ev) {
    this.demarquePourErreur()
    if ($(self.inputAbs).val().length !== 0) {
      if (ev.keyCode === 13) {
        $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
        self.OK()
      }
    }
  }

  td.appendChild(this.inputOrd)
  var paneBoutonsValFoncOrd = new PaneBoutonsValFonc(this.app, this, this.inputOrd, true, indmax)
  tab.appendChild(paneBoutonsValFoncOrd)
  this.editorOrd = new EditeurvaleurReelle(app, this.inputOrd, indmax, this.valeurOrd, null)
  if (modification) {
    $(this.inputAbs).val(point.abs.calcul.chaineCalculSansPar())
    $(this.inputOrd).val(point.ord.calcul.chaineCalculSansPar())
  }
  // On rajoute à droite un panneau pour le choix du repère
  td = kernel.ce('td')
  trprincipal.appendChild(td)
  this.paneListeReperes = new PaneListeReperes(this.app, indmax)
  td.appendChild(this.paneListeReperes.getTab())
  if (modification) this.paneListeReperes.selectRep(point.rep)
  this.create('PtParCoord', 550)
}

PtParCoordDlg.prototype = new MtgDlg()

PtParCoordDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    self.inputAbs.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

PtParCoordDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (this.editorAbs.validate($(this.inputAbs).val())) {
    if (this.editorOrd.validate($(this.inputOrd).val())) {
      this.point.abs = this.valeurAbs
      this.point.ord = this.valeurOrd
      this.point.rep = this.paneListeReperes.getSelectedRep()
      // this.app.outilPtParCoord.actionApresDlg();
      if (this.modification) this.app.gestionnaire.enregistreFigureEnCours('ModifObj')
      if (this.callBackOK !== null) this.callBackOK()
      this.destroy()
    } else $(this.inputOrd).focus()
  } else $(this.inputAbs).focus()
}
