/*
 * Created by yvesb on 15/05/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const NatCal = require('../types/NatCalAdd')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const MtgInput = require('./MtgInput')
module.exports = RenommerCalculDlg

function RenommerCalculDlg (app, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'DeriveeDlg', callBackOK, callBackCancel)
  var self = this
  // Un seul bouton Fermer pour cette boîte de dialogue
  var buttons = {}
  buttons[kernel.getStr('Fermer')] = function (ev) {
    self.stopEvent(ev)
    self.destroy()
    self.callBackOK()
  }

  var list = app.listePr
  this.inf = list.listeParNatCal(app, NatCal.NTtCalcNommeSaufConst, -1)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tr.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:180px'
  })
  $(caption).html(kernel.getStr('EditForDlg1'))
  td.appendChild(caption)
  this.selectCalc = kernel.ce('SELECT', {
    size: 12, // Le nombre de lignes visibles par défaut
    style: 'width:180px'
  })
  this.selectCalc.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.selectCalc)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.selectCalc.appendChild(option)
  }
  td = kernel.ce('td')
  tr.appendChild(td)
  var tabDroit = kernel.ce('table')
  tr.appendChild(tabDroit)
  tr = kernel.ce('tr')
  tabDroit.appendChild(tr)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('RenCalcDlg'))
  tr.appendChild(label)
  var inputName = new MtgInput({
    id: 'inputname'
  })
  tr = kernel.ce('tr')
  tabDroit.appendChild(tr)
  inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.onBtnRenommer()
    }
  }
  tr.appendChild(inputName)
  tr = kernel.ce('tr')
  tabDroit.appendChild(tr)
  var btnRenommer = kernel.ce('button', {
    tabindex: -1
  })
  btnRenommer.onclick = function () { self.onBtnRenommer() }
  $(btnRenommer).html(kernel.getStr('Renommer'))
  tr.appendChild(btnRenommer)

  // En bas du tableau un champ d'info sur la fonction sélectionnée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  tr.appendChild(label)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var inputinf = kernel.ce('textarea', {
    id: 'mtginfo',
    disabled: 'true',
    cols: 60,
    rows: 3
  })
  tr.appendChild(inputinf)
  this.editeurName = new EditeurNomCalcul(app, true, inputName)
  this.onSelectChange()

  // Attention : Pour cette boîte de dialogue ne pas appeler create
  $('#' + self.id).dialog({
    modal: true,
    title: kernel.getStr('RenommerCalcul'),
    buttons: buttons,
    close: function (ev) {
      self.stopEvent(ev)
      self.destroy()
    },
    width: 520,
    closeOnEscape: false,
    // On centre la boîte de dialogue sur le div parent de l'application
    position: { my: 'center', at: 'center', of: '#divMtgDlg' }
  })
}

RenommerCalculDlg.prototype = new MtgDlg()

RenommerCalculDlg.prototype.onSelectChange = function () {
  var ind, el
  ind = this.selectCalc.selectedIndex
  el = this.inf.pointeurs[ind]
  $('#mtginfo').text(el.infoHist())
  $('#inputname').val(el.nomCalcul)
}

RenommerCalculDlg.prototype.onBtnRenommer = function () {
  if (this.editeurName.validate()) {
    var ind = this.selectCalc.selectedIndex
    var el = this.inf.pointeurs[ind]
    var chNom = $('#inputname').val()
    var app = this.app
    var list = app.listePr
    list.remplaceNomValeurDynamiqueDansCommentairesOuLatexDepDe(el.nomCalcul, chNom, el)
    el.nomCalcul = chNom
    list.reconstruitChainesCalculDepDe(el)
    list.determineDependancesCommentaires()
    if (el.getNatureCalcul() === NatCal.NVariable) {
      app.removePaneVariables()
      list.creePaneVariables()
    }
    this.updateList()
    this.selectCalc.selectedIndex = ind
    this.app.outilRenommerCalcul.saveFig()
  }
}

RenommerCalculDlg.prototype.updateList = function () {
  var i, liste, option
  liste = this.app.listePr
  for (i = this.selectCalc.options.length - 1; i >= 0; i--) this.selectCalc.remove(i)
  this.inf = liste.listeParNatCal(this.app, NatCal.NTtCalcNommeSaufConst, -1)
  for (i = 0; i < this.inf.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.selectCalc.appendChild(option)
  }
}
