/*
 * Created by yvesb on 22/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const CValeur = require('../objets/CValeur')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const CDroiteAB = require('../objets/CDroiteAB')
const LineStylePanel = require('../interface/LineStylePanel')
const ColorButton = require('../interface/ColorButton')
const constantes = require('../kernel/constantes')
const StyleTrait = require('../types/StyleTrait')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')

const Color = require('../types/Color')

module.exports = RepDlg

function RepDlg (app, rep, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'RepDlg', callBackOK, callBackCancel)
  this.rep = rep
  this.modification = modification
  this.thickness = rep.style.strokeWidth
  var list = app.listePr
  this.absor = new CValeur(list, 0)
  this.ordor = new CValeur(list, 0)
  this.unitex = new CValeur(list, 1)
  this.unitey = new CValeur(list, 1)
  // On utilise un cellspacing important pour que la boîte de dialogue soit assez grande pour que
  // la boîte de dialogue de chois de couleur tienne dedans
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 25
  })
  this.appendChild(tabPrincipal)
  var tr1 = kernel.ce('tr')
  tabPrincipal.appendChild(tr1)
  var tabHaut = kernel.ce('table')
  tr1.appendChild(tabHaut)
  var tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var tabHautGauche = kernel.ce('table')
  td.appendChild(tabHautGauche)
  var tr2 = kernel.ce('tr')
  tabHautGauche.appendChild(tr2)
  td = kernel.ce('td')
  tr2.appendChild(td)
  var input = kernel.ce('input', {
    id: 'rdquadver',
    type: 'checkBox'
  })
  td.appendChild(input)
  if (rep.quadrillageVertical) input.setAttribute('checked', 'checked')
  td = kernel.ce('td')
  tr2.appendChild(td)
  var label = kernel.ce('label', {
    for: 'rdquadver'
  })
  $(label).html(kernel.getStr('NewRepDlg5'))
  td.appendChild(label)
  tr2 = kernel.ce('tr')
  tabHautGauche.appendChild(tr2)
  td = kernel.ce('td')
  tr2.appendChild(td)
  input = kernel.ce('input', {
    id: 'rdquadhor',
    type: 'checkBox'
  })
  td.appendChild(input)
  if (rep.quadrillageHorizontal) input.setAttribute('checked', 'checked')
  td = kernel.ce('td')
  tr2.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdquadhor'
  })
  $(label).html(kernel.getStr('NewRepDlg6'))
  td.appendChild(label)
  tr2 = kernel.ce('tr')
  tabHautGauche.appendChild(tr2)
  td = kernel.ce('td')
  tr2.appendChild(td)
  input = kernel.ce('input', {
    id: 'rdquadpoint',
    type: 'checkBox'
  })
  td.appendChild(input)
  if (rep.pointilles) input.setAttribute('checked', 'checked')
  td = kernel.ce('td')
  tr2.appendChild(td)
  label = kernel.ce('label', {
    for: 'rdquadpoint'
  })
  $(label).html(kernel.getStr('NewRepDlg7'))
  td.appendChild(label)
  // En haut et à droite un svg va contenir un bouton de choix de couleur et un lineStylePanel
  td = kernel.ce('td')
  tr.appendChild(td)
  var svg = kernel.cens('svg', {
    width: Math.floor(2 * constantes.lineStyleWidth + 5),
    height: String(6 * constantes.lineStyleButtonHeight)
  })
  td.appendChild(svg)
  this.lineStylePanel = new LineStylePanel(app, rep.style.style)
  svg.appendChild(this.lineStylePanel.g)
  this.colorButton = new ColorButton(this, rep.couleur)
  this.colorButton.container.setAttribute('transform', 'translate(' + String(constantes.lineStyleWidth + 5) + ',0)')
  svg.appendChild(this.colorButton.container)
  // En troisième colonne de la première ligne un select pour le choix de l'épaisseur
  td = kernel.ce('td')
  tr.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:80px;'
  })
  $(caption).html(kernel.getStr('Epaisseur'))
  td.appendChild(caption)
  this.select = kernel.ce('select', {
    size: 4, // Le nombre de lignes visibles par défaut
    style: 'width:80px;'
  })
  var self = this
  this.select.onchange = function () {
    self.thickness = self.select.selectedIndex + 1
  }
  td.appendChild(this.select)
  for (var i = 0; i < 5; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === this.thickness - 1) option.setAttribute('selected', 'selected')
    $(option).html(String(i + 1))
    this.select.appendChild(option)
  }
  // Quatre éditeurs de formule pour l'abscisse à l'origine, l'ordonnée à l'origine et les unités
  var indmax = modification ? list.indexOf(rep) - 1 : list.longueur() - 1
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('OrAbs'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputabsor = new MtgInput({
    id: 'inputabsor'
  })
  td.appendChild(this.inputabsor)
  $(this.inputabsor).val(rep.abscisseOrigine.chaineInfo())
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputabsor, true, indmax)
  tabBas.appendChild(paneBoutonsValFonc)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('OrOrd'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputordor = new MtgInput({
    id: 'inputordor'
  })
  td.appendChild(this.inputordor)
  $(this.inputordor).val(rep.ordonneeOrigine.chaineInfo())
  paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputordor, true, indmax)
  tabBas.appendChild(paneBoutonsValFonc)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Unitex'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputunitex = new MtgInput({
    id: 'inputunitex'
  })
  td.appendChild(this.inputunitex)
  $(this.inputunitex).val(rep.uniteX.chaineInfo())
  paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputunitex, true, indmax)
  tabBas.appendChild(paneBoutonsValFonc)

  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Unitey'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputunitey = new MtgInput({
    id: 'inputunitey'
  })
  td.appendChild(this.inputunitey)
  $(this.inputunitey).val(rep.uniteY.chaineInfo())
  paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputunitey, true, indmax)
  tabBas.appendChild(paneBoutonsValFonc)

  if (!modification) {
    tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    label = kernel.ce('label')
    tr.appendChild(label)
    $(label).html(kernel.getStr('NewRepDlg8'))
    input = kernel.ce('input', {
      type: 'checkbox',
      id: 'creerax'
    })
    label.appendChild(input)
    $(input).attr('checked', 'checked')
  }

  this.editororx = new EditeurValeurReelle(app, this.inputabsor, indmax, this.absor, null)
  this.editorory = new EditeurValeurReelle(app, this.inputordor, indmax, this.ordor, null)
  this.editorux = new EditeurValeurReelle(app, this.inputunitex, indmax, this.unitex, null)
  this.editoruy = new EditeurValeurReelle(app, this.inputunitey, indmax, this.unitey, null)

  // Création de la boîte de dialogue par jqueryui
  this.create('Repere', 550)
}

RepDlg.prototype = new MtgDlg()

RepDlg.prototype.OK = function () {
  var axe; var app = this.app
  if (this.editororx.validate($('#inputabsor').val())) {
    if (this.editorory.validate($('#inputordor').val())) {
      if (this.editorux.validate($('#inputunitex').val())) {
        if (this.editoruy.validate($('#inputunitey').val())) {
          var rep = this.rep
          rep.donneCouleur(this.colorButton.color)
          rep.style = new StyleTrait(this.app.listePr, this.lineStylePanel.style, this.thickness)
          rep.quadrillageHorizontal = $('#rdquadver').prop('checked')
          rep.quadrillageVertical = $('#rdquadhor').prop('checked')
          rep.pointilles = $('#rdquadpoint').prop('checked')
          rep.abscisseOrigine = this.absor
          rep.ordonneeOrigine = this.ordor
          rep.uniteX = this.unitex
          rep.uniteY = this.unitey
          if (this.modification) app.gestionnaire.enregistreFigureEnCours('ModifObj')
          else {
            if ($('#creerax').prop('checked')) {
              var taille = app.getTaillePoliceNom()
              var list = app.listePr
              var stfc = StyleTrait.stfc(list)
              if (!list.existeDroiteABDefiniePar(rep.o, rep.i)) {
                axe = new CDroiteAB(list, null, false, Color.black, false, 0, 0, false, '', taille, stfc, 0.9, rep.o, rep.i)
                app.ajouteElement(axe, false)
                axe.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
              }
              if (!list.existeDroiteABDefiniePar(rep.o, rep.j)) {
                axe = new CDroiteAB(list, null, false, Color.black, false, 0, 0, false, '', taille, stfc, 0.9, rep.o, rep.j)
                app.ajouteElement(axe, false)
                axe.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
              }
            }
          }
          if (this.callBackOK !== null) this.callBackOK()
          this.destroy()
        } else this.inputunitey.focus()
      } else this.inputunitex.focus()
    } else this.inputordor.focus()
  } else this.inputabsor.focus()
}
