/*
 * Created by yvesb on 02/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeurAngle = require('../objets/CValeurAngle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')
const ChoixAngleUsuelDlg = require('./ChoixAngleUsuelDlg')

module.exports = RotationDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param rot La rotation dont l'angle doit être changé
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 */
function RotationDlg (app, rot, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'rotDlg', callBackOK, callBackCancel)
  var self = this
  this.rot = rot
  this.modification = modification
  var list = app.listePr
  // Un CValeurAng pour stocker la valeur choisie pour l'angle avant vérification
  this.valeur = new CValeurAngle(list, 0)
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Angle') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  /*
  var input = new MtgInput({
    id: "mtginput",
    list : "mtgAngleList"
  });
  */
  var input = new MtgInput({
    id: 'mtginput',
    list: 'mtgAngleList'
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  td = kernel.ce('td')
  tr.appendChild(td)
  $(td).css('padding', '1px')
  $(td).html(app.local
    ? "<img src='" + app.localPath + "images/dropdown.png' width='22px' height='22px' />"
    : "<img src='" + app.imagePath + "dropdown.png' width='22px' height='22px' crossorigin='anonymous' />"
  )
  td.onclick = function () {
    new ChoixAngleUsuelDlg(app, input, list.uniteAngle)
  }

  var indmax = modification ? list.indexOf(rot) - 1 : list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indmax)
  tab.appendChild(paneBoutonsValFonc)
  this.editor = new EditeurvaleurReelle(app, input, indmax, this.valeur, null)
  if (modification) $(input).val(rot.angle.calcul.chaineCalculSansPar())
  this.create('AngleRot', 400)
}

RotationDlg.prototype = new MtgDlg()

RotationDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

RotationDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (this.editor.validate()) {
    this.rot.angle = this.valeur
    if (this.callBackOK !== null) this.callBackOK()
    this.destroy()
    // this.app.outilActif.actionApresDlg();
  }
}
