/*
 * Created by yvesb on 18/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const MtgInput = require('./MtgInput')
const AvertDlg = require('./AvertDlg')
const DataOutputStream = require('../entreesSorties/DataOutputStream')
const FileSaver = require('file-saver')
const $ = require('./../global/jQuery-ui')
const Dimf = require('../types/Dimf')
module.exports = SaveDlg

function SaveDlg (app) {
  MtgDlg.call(this, app, 'SaveDlg')
  var self = this
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 10
  })
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('saveDlg1'))
  div.appendChild(label)
  this.input = new MtgInput({
    id: 'mtginput'
  })
  div.appendChild(this.input)
  this.input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  this.create('Save', 550)
}

SaveDlg.prototype = new MtgDlg()

SaveDlg.prototype.OK = function () {
  var ch = $('#mtginput').val()
  // On regarde si la chaîne entrée est non-vide et ne contient que des caractères ascii
  var app = this.app
  if ((ch === '') || ch.match(/\W/)) {
    this.input.marquePourErreur()
    new AvertDlg(app, 'NomFichierErr', function () {
      $('#mtginput').focus()
    })
  }
  var doc = app.doc
  // Si un cadre a été choisi dans la figure et si la case à cocher est cochée, on enregistre la figure
  // avec les dimensions du cadre
  var olddimf = doc.dimf
  if (app.cadre !== null) doc.dimf = new Dimf(app.widthCadre, app.heightCadre)
  var oups = new DataOutputStream()
  doc.write(oups)
  var ba = oups.ba
  var len = ba.length
  var buffer = new ArrayBuffer(ba.length)
  var dataView = new DataView(buffer)
  for (var i = 0; i < len; i++) {
    dataView.setUint8(i, ba[i])
  }
  FileSaver.saveAs(new Blob([buffer], { type: 'application-x/MathGrap32-File' }), ch + '.mgj') // eslint-disable-line no-undef
  this.destroy()
  doc.dimf = olddimf
}

SaveDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}
