/*
 * Created by yvesb on 18/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const MtgDlg = require('./MtgDlg')
const MtgInput = require('./MtgInput')
const AvertDlg = require('./AvertDlg')
const FileSaver = require('file-saver')
const $ = require('./../global/jQuery-ui')
module.exports = SaveImageDlg

/**
 * Dialogue d'exportation dans une image
 * @constructor
 * @param app L'application propriétaire
 * @param {string} imageType "png" ou "jpeg"
 */
function SaveImageDlg (app, imageType) {
  MtgDlg.call(this, app, 'SaveImDlg')
  this.imageType = imageType
  var self = this
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 10
  })
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('saveDlg1'))
  div.appendChild(label)
  this.input = new MtgInput({
    id: 'mtginput'
  })
  div.appendChild(this.input)
  this.input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var title = imageType === 'png' ? 'SavePNG' : (imageType === 'jpeg' ? 'SaveJPG' : 'SaveSVG')
  this.create(title, 550)
}

SaveImageDlg.prototype = new MtgDlg()

SaveImageDlg.prototype.OK = function () {
  var self = this
  var ch = $('#mtginput').val()
  // On regarde si la chaîne entrée est non-vide et ne contient que des caractères ascii
  var app = this.app
  if ((ch === '') || ch.match(/\W/)) {
    this.input.marquePourErreur()
    new AvertDlg(app, 'NomFichierErr', function () {
      $('#mtginput').focus()
    })
    return
  }

  var suffix = self.imageType === 'jpeg' ? 'jpg' : self.imageType
  var fileName = ch + '.' + suffix
  if (suffix === 'svg') {
    app.getBase64ImageData(suffix, function (data) {
      FileSaver.saveAs(new Blob([data]), fileName) // eslint-disable-line no-undef
      self.destroy()
    }, 1, 1)
  } else {
    app.getBase64ImageData('png', function (data) {
      self.callBack(fileName, kernelAdd.base64toBlob(data, self.imageType))
    }, app.pref_coefMult, 1)
  }
}

SaveImageDlg.prototype.callBack = function (name, blob) {
  FileSaver.saveAs(blob, name)
  this.destroy()
}
