/*
 * Created by yvesb on 18/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const MtgInput = require('./MtgInput')
const AvertDlg = require('./AvertDlg')
const FileSaver = require('file-saver')
const $ = require('./../global/jQuery-ui')
const kernelAdd = require('../kernel/kernelAdd')
module.exports = SavePNGWithUnityDlg

/**
 * Dialogue d'exportation dans une image avec longueur unité
 * @constructor
 * @param app L'application propriétaire
 * @param {boolean} bSaveInFile true si après cette boîte de dialogue on enregistre dans un fichier
 * et false si on copie le résultat dans le presse-papier
 */
function SavePNGWithUnityDlg (app, bSaveInFile) {
  MtgDlg.call(this, app, 'SaveImUnDlg')
  this.bSaveInFile = bSaveInFile
  var self = this
  var tabPrincipal = kernel.ce('table', {
    cellspacing: 10
  })
  this.appendChild(tabPrincipal)
  // Seulement pour la version non electron on demande le nom du fichier
  if (!app.electron) {
    var tr = kernel.ce('tr')
    tabPrincipal.appendChild(tr)
    var div = kernel.ce('div')
    tr.appendChild(div)
    var label = kernel.ce('label')
    $(label).html(kernel.getStr('saveDlg1'))
    div.appendChild(label)
    this.input = new MtgInput({
      id: 'mtginput'
    })
    div.appendChild(this.input)
    this.input.onkeyup = function (ev) {
      this.demarquePourErreur()
      if (ev.keyCode === 13) {
        $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
        self.OK()
      }
    }
  }
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Unit'))
  div.appendChild(label)
  var inputUnity = kernel.ce('input', {
    type: 'text',
    id: 'inputUn'
  })
  div.appendChild(inputUnity)
  inputUnity.onkeyup = function (ev) {
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    } else self.displayDim()
  }

  $(inputUnity).val('1')

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('DimFig'))
  div.appendChild(label)
  // On ajoute un label pour afficher les dimensions de la figure
  var labelDim = kernel.ce('label', {
    id: 'dim'
  })
  div.appendChild(labelDim)
  this.displayDim()
  this.create(bSaveInFile ? 'SavePNGWithUnity' : 'CopyWithUnity', 550)
}

SavePNGWithUnityDlg.prototype = new MtgDlg()

SavePNGWithUnityDlg.prototype.OK = function () {
  var app = this.app
  var listePr = app.listePr
  var self = this
  if (!app.electron) {
    var ch = $('#mtginput').val()
    // On regarde si la chaîne entrée est non-vide et ne contient que des caractères ascii
    if ((ch === '') || ch.match(/\W/)) {
      this.input.marquePourErreur()
      new AvertDlg(app, 'NomFichierErr', function () {
        $('#mtginput').focus()
      })
      return
    }
  }
  var unity = parseFloat($('#inputUn').val())
  if (isNaN(unity) || unity === 0) {
    new AvertDlg(app, 'Incorrect', function () {
      $('#inputUn').focus()
    })
    return
  }
  // var coef = 96/2.54/listePr.pointeurLongueurUnite.rendLongueur()*unity;
  var coef = 96 / 2.54 / listePr.pointeurLongueurUnite.rendLongueur() * unity / app.pref_coefMult
  if (app.electron) { // savePNG est une fonction globale de la page index.html pour la version electron
    this.destroy()
    if (this.bSaveInFile) {
      app.getBase64ImageData('png', function (data) {
        savePNG(data) // eslint-disable-line no-undef
        // savePng est défini dans index.html
      }, app.pref_coefMult, coef)
    } else {
      app.getBase64ImageData('png', function (data) {
        copy(data) // eslint-disable-line no-undef
        // copy est défini dans index.html
        new AvertDlg(app, kernel.getStr('CopyOK'))
      }, app.pref_coefMult, coef)
    }
  } else {
    var fileName = ch + '.png'
    app.getBase64ImageData('png', function (data) {
      self.callBack(fileName, kernelAdd.base64toBlob(data, 'png'))
    }, app.pref_coefMult, coef)
  }
}

SavePNGWithUnityDlg.prototype.callBack = function (name, blob) {
  FileSaver.saveAs(blob, name)
  this.destroy()
}

SavePNGWithUnityDlg.prototype.displayDim = function () {
  var app = this.app
  var dimf = app.dimf
  var ch = $('#inputUn').val()
  if (ch !== '') {
    var unity = parseFloat(ch)
    var long = app.listePr.pointeurLongueurUnite.rendLongueur()
    var avecCadre = app.cadre !== null
    var w = avecCadre ? app.widthCadre : dimf.x
    var h = avecCadre ? app.heightCadre : dimf.y
    w = Math.round(w / long * unity * 100) / 100
    h = Math.round(h / long * unity * 100) / 100
    $('#dim').html('(' + w + ' cm, ' + h + ' cm)')
  } else $('#dim').html('')
}
