/*
 * Created by yvesb on 27/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const CValeur = require('../objets/CValeur')
const CImplementationProto = require('../objets/CImplementationProto')
const MtgInput = require('./MtgInput')
const AvertDlg = require('./AvertDlg')
module.exports = SegmentParLongDlg

function SegmentParLongDlg (app, pt, bptCree) {
  MtgDlg.call(this, app, 'segparlongdlg')
  this.pt = pt
  this.bptCree = bptCree
  var list = app.listePr
  this.valeur = new CValeur(list, 1)
  var self = this
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('LongSeg'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = new MtgInput({
    id: 'mtginput'
  })
  input.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  td.appendChild(input)
  var indder = list.longueur() - 1
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, input, true, indder)
  tabPrincipal.appendChild(paneBoutonsValFonc)
  this.editeur = new EditeurvaleurReelle(app, input, indder, this.valeur, null)
  this.create('SegmentParLong', 500)
}

SegmentParLongDlg.prototype = new MtgDlg()

SegmentParLongDlg.prototype.OK = function () {
  var app = this.app
  var list = app.listePr
  // if (this.app.lastDlgId() !== this.id) return;
  var valid = this.editeur.validate($('#mtginput').val())
  if (valid) {
    if (this.valeur.valeur <= 0) { new AvertDlg(app, 'Incorrect', function () { $('#mtginput').focus() }) } else {
      // Si la valeur choisie n'est pas une référence à une valeur exsitante
      // on crée un nouveau calcul qu'on ajoute à la figure
      var val = this.valeur.getCalcForImpProto(app, kernel.getStr('long'), true)
      var proto = app.docCons.getPrototype('SegmentParLongueur')
      proto.get(0).elementAssocie = val // Pointe sur le repère
      proto.get(1).elementAssocie = this.pt
      var impProto = new CImplementationProto(list, proto)
      impProto.implemente(app.dimf, proto)
      impProto.nomProto = kernel.getStr('SegmentParLong')
      // On modifie le style de trait et de couleur des objets finaux
      var col = app.getCouleur()
      var affval = list.get(list.longueur() - 1)
      affval.couleurFond = app.doc.couleurFond
      affval.effacementFond = false
      affval.donneCouleur(col)
      affval.taillePolice = 16
      var styleTrait = app.getStyleTrait()
      var indImpProto = list.indexOf(impProto)
      var pt = list.get(indImpProto + 2)
      pt.donneCouleur(col)
      pt.donneMotif(app.getStylePoint())
      pt.decaleNom(3, 0)
      // pt.donneNom(app.genereNomPourPoint()); // Ajout version 4.7.0.1
      pt.tailleNom = 16
      var seg = list.get(indImpProto + 3)
      seg.donneCouleur(col)
      seg.donneStyle(styleTrait)
      list.positionne(false, app.dimf)
      // list.setReady4MathJax(); // Inutile car pas de CLatex
      list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
      app.outilSegmentParLong.saveFig()
      app.activeOutilCapt()
      app.listePourConst = app.listePourConstruction()
      this.destroy()
      // On édite le nom du nouveau point créé
      var nameEditor = app.nameEditor
      if (this.pt.nom === '') {
        nameEditor.associeA(this.pt)
        nameEditor.setPosition()
        nameEditor.montre(true)
      } else {
        nameEditor.associeA(pt)
        nameEditor.setPosition()
        nameEditor.montre(true)
      }
    }
  }
}

SegmentParLongDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#mtginput').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

SegmentParLongDlg.prototype.onCancel = function () {
  if (this.bptCree) {
    var app = this.app
    var list = app.listePr
    list.get(list.longueur() - 1).removegElement(app.svgFigure)
    app.detruitDernierElement()
  }
  this.destroy()
  this.app.activeOutilCapt()
}
