/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCal')
const MtgDlg = require('./MtgDlg')
//
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const CValeur = require('../objets/CValeur')
const MtgInput = require('./MtgInput')
module.exports = SomProdDlg

/**
 * Boîte de dialogue servant à créer une somme inidcée ou un produit indicé
 * @param app L'application associée
 * @param {CTestEquivalence} somme Le test d'équivalence ou de factorisation associé
 * @param {boolean} modification true si on modifie un objet déjà créé
 * @param {boolean} bSomme true si on modifie une somme indicée, false pour un produit indicé
 * @param {function} callBackOK function appelée si l'utilisateur valide ou null
 * @param {function} callBackCancel  Fonction appelée si l'utilisateur ferme la boîte de dialogue ou null
 * @constructor
 */
function SomProdDlg (app, somme, modification, bSomme, callBackOK, callBackCancel) {
  var option
  MtgDlg.call(this, app, 'SomInd', callBackOK, callBackCancel)
  this.somme = somme
  this.modification = modification
  this.bSomme = bSomme
  var list = app.listePr
  // La formule obtenue pour le calcul de l'indice mini
  this.valeurMin = new CValeur(list)
  // La formule obtenue pour le calcul de l'indice maxi
  this.valeurMax = new CValeur(list)

  var indmax = modification ? list.indexOf(somme) - 1 : list.longueur() - 1
  this.infleft = list.listeParNatCal(app, NatCal.NTteValR, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  div.appendChild(label)
  var inputName = new MtgInput({
    id: 'inputName'
  })
  div.appendChild(inputName)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)

  var tdleft = kernel.ce('td')
  tabCentre.appendChild(tdleft)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:250px'
  })
  $(caption).html(kernel.getStr(bSomme ? 'ValSom' : 'valProd'))
  tdleft.appendChild(caption)

  this.selectleft = kernel.ce('select', {
    size: 7, // Le nombre de lignes visibles par défaut
    style: 'width:250px'
  })
  tdleft.appendChild(this.selectleft)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.infleft.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.infleft.pointeurs[i] === somme.valeur) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.infleft.noms[i])
    this.selectleft.appendChild(option)
  }
  var tabLeft = kernel.ce('table')
  tdleft.appendChild(tabLeft)
  tr = kernel.ce('tr')
  tabLeft.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('chinfo117'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputMin = new MtgInput()
  td.appendChild(this.inputMin)
  var paneBoutonsValFoncLeft = new PaneBoutonsValFonc(this.app, this, this.inputMin, true, indmax)
  tabLeft.appendChild(paneBoutonsValFoncLeft)

  this.infright = list.listeParNatCal(app, NatCal.NVariable, indmax)
  var tdright = kernel.ce('td')
  tabCentre.appendChild(tdright)
  caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:250px'
  })
  $(caption).html(kernel.getStr('VarInd'))
  tdright.appendChild(caption)

  this.selectright = kernel.ce('select', {
    size: 7, // Le nombre de lignes visibles par défaut
    style: 'width:250px'
  })
  tdright.appendChild(this.selectright)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (i = 0; i < this.infright.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.infright.pointeurs[i] === somme.indice) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.infright.noms[i])
    this.selectright.appendChild(option)
  }

  var tabRight = kernel.ce('table')
  tdright.appendChild(tabRight)
  tr = kernel.ce('tr')
  tabRight.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('chinfo118'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputMax = new MtgInput()
  td.appendChild(this.inputMax)
  var paneBoutonsValFoncRight = new PaneBoutonsValFonc(this.app, this, this.inputMax, true, indmax)
  tabRight.appendChild(paneBoutonsValFoncRight)

  this.editeurName = new EditeurNomCalcul(app, !modification, inputName)
  this.editeurFormuleMin = new EditeurvaleurReelle(app, this.inputMin, indmax, this.valeurMin, null)
  this.editeurFormuleMax = new EditeurvaleurReelle(app, this.inputMax, indmax, this.valeurMax, null)
  if (modification) {
    $(inputName).val(somme.nomCalcul)
    $(inputName).attr('disabled', true)
    $(this.inputMin).val(somme.indiceMin.chaineInfo())
    $(this.inputMax).val(somme.indiceMax.chaineInfo())
  }

  this.create(bSomme ? 'SomInd' : 'ProdInd', 550)
}

SomProdDlg.prototype = new MtgDlg()

SomProdDlg.prototype.OK = function () {
  if (this.editeurName.validate()) {
    if (this.editeurFormuleMin.validate($(this.inputMin).val())) {
      if (this.editeurFormuleMax.validate($(this.inputMax).val())) {
        if (!this.modification) this.somme.nomCalcul = $('#inputName').val()
        this.somme.valeur = this.infleft.pointeurs[this.selectleft.selectedIndex]
        this.somme.indice = this.infright.pointeurs[this.selectright.selectedIndex]
        this.somme.indiceMin = this.valeurMin
        this.somme.indiceMax = this.valeurMax
        this.somme.metAJour() // Nécessaire pour que this.listeElementsAncetres soit établi avant d'appeler positionne pour la première fois
        if (!this.modification) this.app.ajouteElement(this.somme)
        this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : this.bSomme ? 'SomInd' : 'ProdInd')
        this.destroy()
        if (this.callBackOK !== null) this.callBackOK()
      } else this.inputMax.focus()
    } else this.inputMin.focus()
  }
}
