/*
 * Created by yvesb on 12/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
const AvertDlg = require('./AvertDlg')
module.exports = TagDlg

/**
 * Dialogue de création ou modification d'un lieu de points généré par point lié (lieu continu ou discret)
 * @constructor
 * @param app La mtgApp propriétaire
 * @param {CElementGraphique} el Objet dont le tag doit être changé
 * @param {function} callBackOK Fonction de callBack à appeler quand on valide par OK
 */
function TagDlg (app, el, callBackOK) {
  MtgDlg.call(this, app, 'TagDlg', callBackOK)
  var self = this
  this.el = el
  var tab = kernel.ce('table')
  this.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).text(kernel.getStr('tag') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var input = kernel.ce('input', {
    id: 'inputtag',
    size: 16
  })
  td.appendChild(input)
  input.onkeyup = function (ev) {
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  $(input).val(el.tag)
  this.create('LieuPt', 400)
}

TagDlg.prototype = new MtgDlg()

TagDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $('#inputtag').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

TagDlg.prototype.OK = function () {
  var app = this.app
  var newTag = $('#inputtag').val()
  if (newTag !== '') {
    var doc = this.app.doc
    var list = doc.listePr
    for (var i = 0; i < list.longueur(); i++) {
      var elt = list.get(i)
      if (elt !== this.el && !elt.estElementIntermediaire() && elt.tag === newTag) {
        new AvertDlg(app, kernel.getStr('tagErr'))
        $('#taginput').focus()
        return
      }
    }
  }
  this.el.tag = newTag
  this.callBackOK()
  this.destroy()
}
