/*
 * Created by yvesb on 13/04/2017.
 */
/*
 * Created by yvesb on 01/04/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const NatCal = require('../types/NatCal')
const CImplementationProto = require('../objets/CImplementationProto')
const PaneListeReperes = require('./PaneListeReperes')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurValeurReelle = require('./EditeurValeurReelle')
const EditeurNomCalcul = require('../dialogs/EditeurNomCalcul')
const MtgInput = require('./MtgInput')
module.exports = TangenteDlg

function TangenteDlg (app, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'CourbeDlg', callBackOK, callBackCancel)
  var self = this
  var list = app.listePr
  this.inf = list.listeParNatCal(app, NatCal.NFoncR1Var, -1)
  // Un CValeur pour stocker les valeurs pour l'équation, a, b et c.
  this.valAbs = new CValeur(list)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabListes = kernel.ce('table')
  tr.appendChild(tabListes)
  tr = kernel.ce('tr')
  tabListes.appendChild(tr)
  var td = kernel.ce('td')
  tabListes.appendChild(td)
  var caption = kernel.ce('caption', {
    class: 'mtgcaption',
    style: 'width:220px'
  })
  $(caption).html(kernel.getStr('TgteDlg1'))
  td.appendChild(caption)
  this.select = kernel.ce('SELECT', {
    size: 5, // Le nombre de lignes visibles par défaut
    style: 'width:220px'
  })
  this.select.onchange = function () {
    self.onSelectChange()
  }
  td.appendChild(this.select)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.inf.noms.length; i++) {
    var option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.inf.noms[i])
    this.select.appendChild(option)
  }
  td = kernel.ce('td')
  $(td).css('vertical-align', 'top')
  tabListes.appendChild(td)
  this.paneListeRep = new PaneListeReperes(app, list.longueur() - 1)
  td.appendChild(this.paneListeRep.getTab())
  // Un ligne contenant un tableau donnant des infos sur la fonction sélectionnée
  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var paneInfo = kernel.ce('table')
  tr.appendChild(paneInfo)
  tr = kernel.ce('tr')
  paneInfo.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  $(td).css('vertical-align', 'top')
  let label = kernel.ce('label')
  $(label).html(kernel.getStr('InfoFonc'))
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  var textarea = kernel.ce('textarea', {
    id: 'info',
    cols: 35,
    rows: 2,
    disabled: 'true'
  })
  tr.appendChild(textarea)
  // En bas un tableau pour contenir les deux éditeurs
  tr = kernel.ce('table')
  tabPrincipal.appendChild(tr)
  var paneBas = kernel.ce('table')
  tr.appendChild(paneBas)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  td.appendChild(label)
  $(label).html(kernel.getStr('TgteDlg2'))
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputAbs = new MtgInput()
  td.appendChild(this.inputAbs)
  this.inputAbs.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputAbs, true, -1)
  paneBas.appendChild(paneBoutonsValFonc)
  tr = kernel.ce('tr')
  paneBas.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  td.appendChild(label)
  $(label).html(kernel.getStr('TgteDlg3'))
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputName = new MtgInput()
  $(this.inputName).val(list.genereNomPourCalcul('m', true))
  td.appendChild(this.inputName)
  this.editeurAbs = new EditeurValeurReelle(app, this.inputAbs, list.longueur() - 1, this.valAbs, null)
  this.editeurNom = new EditeurNomCalcul(app, true, this.inputName)
  this.onSelectChange()
  this.create('Tangente', 500)
}

TangenteDlg.prototype = new MtgDlg()

TangenteDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    $(self.inputAbs).focus() // Pour que le focus soit donné à l'éditeur au départ
  }, 500)
}

TangenteDlg.prototype.onSelectChange = function () {
  $('#info').val(this.inf.pointeurs[this.select.selectedIndex].infoHist())
}

TangenteDlg.prototype.OK = function () {
  var app = this.app
  var list = app.listePr
  if (this.editeurAbs.validate() && this.editeurNom.validate()) {
    // Si la valeur choisie pour l'abscisse n'est pas une références à une valeur existante
    // on crée un nouveau calcul qu'on ajoute à la figure
    var valeurAbs = this.valAbs.getCalcForImpProto(app, 'abs', true)
    var proto = app.docCons.getPrototype('Tangente')
    proto.get(0).elementAssocie = this.inf.pointeurs[this.select.selectedIndex] // Pointe sur la fonction
    proto.get(1).elementAssocie = valeurAbs
    proto.get(2).elementAssocie = this.paneListeRep.getSelectedRep()
    var impProto = new CImplementationProto(list, proto)
    impProto.implemente(app.dimf, proto)
    impProto.nomProto = kernel.getStr('Tangente')
    var lon = list.longueur()
    var indImpProto = list.indexOf(impProto)
    // On donne au calcul final le nom choisi pour l'abscisse
    list.get(lon - 2).donneNom($(this.inputName).val())
    // On modifie le nom de l'implémentation (pour internationalisation)
    impProto.nomProto = kernel.getStr('Tangente')
    // On modifie le dernier objet implémenté qui est un affichage de valeur lié à un point
    var droite = list.get(lon - 1)
    droite.donneStyle(app.getStyleTrait())
    droite.donneCouleur(app.getCouleur())
    list.positionne(false, app.dimf)
    list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
    app.gestionnaire.enregistreFigureEnCours('Tangente')
    this.destroy()
    app.activeOutilCapt()
  }
}
