/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCal')
const MtgDlg = require('./MtgDlg')
//
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const CValeur = require('../objets/CValeur')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const MtgInput = require('./MtgInput')
const AvertDlg = require('./AvertDlg')
module.exports = TestEqFactDlg

/**
 * Boîte de dialogue servant à créer un test d'équivalence ou un test de factorisation
 * @param app L'application associée
 * @param {CTestEquivalence} test Le test d'équivalence ou de factorisation associé
 * @param {boolean} modification true si on modifie un objet déjà créé
 * @param {boolean} bTestEq true si on modifie un test d'équivalnece, false pour un test de factorisation
 * @param {function} callBackOK function appelée si l'utilisateur valide ou null
 * @param {function} callBackCancel  Fonction appelée si l'utilisateur ferme la boîte de dialogue ou null
 * @constructor
 */
function TestEqFactDlg (app, test, modification, bTestEq, callBackOK, callBackCancel) {
  var option
  MtgDlg.call(this, app, 'TestEq', callBackOK, callBackCancel)
  this.test = test
  this.modification = modification
  var list = app.listePr
  // La formule obtenue pour le calcul de la condition d'existence
  this.valeur = new CValeur(list)
  var self = this
  var indmax = modification ? list.indexOf(test) - 1 : list.longueur() - 1
  this.indmax = indmax
  this.infleft = list.listeParNatCal(app, NatCal.NCalcouFoncParFormule, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  div.appendChild(label)
  var inputName = new MtgInput({
    id: 'inputName'
  })
  inputName.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }

  div.appendChild(inputName)
  if (modification) {
    $(inputName).val(test.nomCalcul)
    $(inputName).attr('disabled', true)
  }
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr(bTestEq ? 'AComp' : 'FacComp'))
  tr.appendChild(label)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)

  var tdleft = kernel.ce('td')
  tabCentre.appendChild(tdleft)

  this.selectleft = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:250px'
  })
  this.selectleft.onchange = function () {
    self.onSelectChange()
  }
  tdleft.appendChild(this.selectleft)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.infleft.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.infleft.pointeurs[i] === test.calcul1) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.infleft.noms[i])
    this.selectleft.appendChild(option)
  }
  div = kernel.ce('div')
  tdleft.appendChild(div)
  var cbleft = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbleft'
  })
  div.appendChild(cbleft)
  label = kernel.ce('label', {
    for: 'cbleft'
  })
  $(label).html(kernel.getStr('RempVal'))
  div.appendChild(label)
  if (test.remplacementValeurs1) $(cbleft).attr('checked', 'checked')

  var sel = this.infleft.pointeurs[this.selectleft.selectedIndex]
  this.infright = list.listeParNatCal(app, sel.getNatureCalcul(), indmax, sel)
  var tdright = kernel.ce('td')
  tabCentre.appendChild(tdright)

  this.selectright = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:250px'
  })
  tdright.appendChild(this.selectright)

  div = kernel.ce('div')
  tdright.appendChild(div)
  var cbright = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbright'
  })
  div.appendChild(cbright)
  label = kernel.ce('label', {
    for: 'cbright'
  })
  $(label).html(kernel.getStr('RempVal'))
  div.appendChild(label)
  if (test.remplacementValeurs2) $(cbright).attr('checked', 'checked')

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  div = kernel.ce('div')
  tr.appendChild(div)
  var cbeq = kernel.ce('input', {
    type: 'checkbox',
    id: 'cbeq'
  })
  div.appendChild(cbeq)
  label = kernel.ce('label', {
    for: 'cbeq'
  })
  $(label).html(kernel.getStr('EqDecFr'))
  div.appendChild(label)
  if (test.equivalenceDecimalFracIrr) $(cbeq).attr('checked', 'checked')

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)

  var tabBas = kernel.ce('table')
  tr.appendChild(tabBas)
  tr = kernel.ce('tr')
  tabBas.appendChild(tr)
  var td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('CondEx') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputCond = new MtgInput()
  td.appendChild(this.inputCond)
  $(this.inputCond).val(test.test.chaineInfo())
  var paneBoutonsValFonc = new PaneBoutonsValFonc(this.app, this, this.inputCond, true, indmax)
  tabBas.appendChild(paneBoutonsValFonc)

  this.editeurName = new EditeurNomCalcul(app, !modification, inputName)
  this.editeurFormule = new EditeurvaleurReelle(app, this.inputCond, indmax, this.valeur, null)
  this.onSelectChange(modification ? test.calcul2 : null) // Met à jour la liste de droite en fonction de l'élément sélectionné à gauche
  this.create(bTestEq ? 'TestEq' : 'TestFact', 550)
}

TestEqFactDlg.prototype = new MtgDlg()

TestEqFactDlg.prototype.onSelectChange = function (calc = null) {
  var i, list, option
  list = this.app.listePr
  for (i = this.selectright.options.length - 1; i >= 0; i--) this.selectright.remove(i)
  var sel = this.infleft.pointeurs[this.selectleft.selectedIndex]
  this.infright = list.listeParNatCal(this.app, sel.getNatureCalcul(), this.indmax, sel)
  for (i = 0; i < this.infright.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if ((calc === null && i === 0) || calc === this.infright.pointeurs[i]) { option.setAttribute('selected', 'selected') }
    $(option).html(this.infright.noms[i])
    this.selectright.appendChild(option)
  }
}

TestEqFactDlg.prototype.OK = function () {
  if (this.editeurName.validate()) {
    if (this.editeurFormule.validate($(this.inputFormula).val())) {
      if (!this.modification) this.test.nomCalcul = $('#inputName').val()
      if (this.selectright.selectedIndex === -1) {
        // Pas d'autres calculs de même nature que le calcul sélectionné à gauche
        new AvertDlg(this.app, 'Incorrect')
        return
      }
      this.test.test = this.valeur
      this.test.calcul1 = this.infleft.pointeurs[this.selectleft.selectedIndex]
      this.test.calcul2 = this.infright.pointeurs[this.selectright.selectedIndex]
      this.test.remplacementValeurs1 = $('#cbleft').prop('checked')
      this.test.remplacementValeurs2 = $('#cbright').prop('checked')
      this.test.equivalenceDecimalFracIrr = $('#cbeq').prop('checked')
      if (!this.modification) this.app.ajouteElement(this.test)
      this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'TestEq')
      this.destroy()
      if (this.callBackOK !== null) this.callBackOK()
    } else this.inputCond.focus()
  }
}

TestEqFactDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  if (!this.modification) {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $('#inputName').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  } else {
    var self = this
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.selectleft).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  }
}
