/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCal')
const MtgDlg = require('./MtgDlg')
const $ = require('./../global/jQuery-ui')
require('jquery-textrange')
const MtgInput = require('./MtgInput')
const EditeurNomCalcul = require('./EditeurNomCalcul')
const CValeur = require('../objets/CValeur')

module.exports = TestEqNatOpDlg

/**
 * Boîte de dialogue servant à créer un test de nature d'opérateur
 * @param app L'application associée
 * @param {CTestEquivalence} test Le test d'équivalence ou de factorisation associé
 * @param {boolean} modification true si on modifie un objet déjà créé
 * @param {function} callBackOK function appelée si l'utilisateur valide ou null
 * @param {function} callBackCancel  Fonction appelée si l'utilisateur ferme la boîte de dialogue ou null
 * @constructor
 */
function TestEqNatOpDlg (app, test, modification, callBackOK, callBackCancel) {
  var option
  MtgDlg.call(this, app, 'TestEqNatOp', callBackOK, callBackCancel)
  this.test = test
  this.modification = modification
  var list = app.listePr
  // La formule obtenue pour le calcul de la condition d'existence
  this.valeur = new CValeur(list)
  var indmax = modification ? list.indexOf(test) - 1 : list.longueur() - 1
  this.infleft = list.listeParNatCal(app, NatCal.NCalcouFoncParFormule, indmax)
  var tabPrincipal = kernel.ce('table')
  this.appendChild(tabPrincipal)
  var tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabHaut = kernel.ce('table')
  tr.appendChild(tabHaut)
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  var div = kernel.ce('div')
  tr.appendChild(div)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('NomCalcul') + ' : ')
  div.appendChild(label)
  var inputName = new MtgInput({
    id: 'inputName'
  })
  div.appendChild(inputName)
  if (modification) {
    $(inputName).val(test.nomCalcul)
    $(inputName).attr('disabled', true)
  }
  tr = kernel.ce('tr')
  tabHaut.appendChild(tr)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('AComp'))
  tr.appendChild(label)

  tr = kernel.ce('tr')
  tabPrincipal.appendChild(tr)
  var tabCentre = kernel.ce('table')
  tr.appendChild(tabCentre)

  var tdleft = kernel.ce('td')
  tabCentre.appendChild(tdleft)

  this.selectleft = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:250px'
  })
  tdleft.appendChild(this.selectleft)
  // C'est là qu'on ajoute les noms des valeurs numériques disponibles dans la liste déroulante
  for (var i = 0; i < this.infleft.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.infleft.pointeurs[i] === test.calcul1) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.infleft.noms[i])
    this.selectleft.appendChild(option)
  }

  this.infright = list.listeParNatCal(app, NatCal.NCalcouFoncParFormule, indmax)
  var tdright = kernel.ce('td')
  tabCentre.appendChild(tdright)

  this.selectright = kernel.ce('select', {
    size: 8, // Le nombre de lignes visibles par défaut
    style: 'width:250px'
  })
  tdright.appendChild(this.selectright)
  for (i = 0; i < this.infright.noms.length; i++) {
    option = kernel.ce('Option', {
      class: 'mtgOption'
    })
    if (modification ? (this.infright.pointeurs[i] === test.calcul2) : i === 0) option.setAttribute('selected', 'selected')
    $(option).html(this.infright.noms[i])
    this.selectright.appendChild(option)
  }

  this.editeurName = new EditeurNomCalcul(app, !modification, inputName)
  this.create('TestEqNatOp', 550)
}

TestEqNatOpDlg.prototype = new MtgDlg()

TestEqNatOpDlg.prototype.OK = function () {
  if (this.editeurName.validate()) {
    if (!this.modification) this.test.nomCalcul = $('#inputName').val()
    this.test.calcul1 = this.infleft.pointeurs[this.selectleft.selectedIndex]
    this.test.calcul2 = this.infright.pointeurs[this.selectright.selectedIndex]
    if (!this.modification) this.app.ajouteElement(this.test)
    this.app.gestionnaire.enregistreFigureEnCours(this.modification ? 'ModifObj' : 'TestEqNatOp')
    this.destroy()
    if (this.callBackOK !== null) this.callBackOK()
  }
}

TestEqNatOpDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  if (!this.modification) {
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $('#inputName').focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  } else {
    var self = this
    setTimeout(function () { // Ne marche pas sans un setTimeOut
      $(self.selectleft).focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
    }, 100)
  }
}
