/*
 * Created by yvesb on 10/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const MtgDlg = require('./MtgDlg')
const CValeur = require('../objets/CValeur')
const EditeurvaleurReelle = require('./EditeurValeurReelle')
const $ = require('./../global/jQuery-ui')
const PaneBoutonsValFonc = require('./PaneBoutonsValFonc')
const PaneListeReperes = require('./PaneListeReperes')
const MtgInput = require('./MtgInput')

module.exports = TransParCoordDlg

/**
 * Dialogue de choix d'angle de rotation
 * @constructor
 * @param app La mtgApp propriétaire
 * @param trans La translation dont les coordonnées doivent être changées
 * @param modification boolean : true si l'angle de la rotation est un angle déjà existant à modifier
 */
function TransParCoordDlg (app, trans, modification, callBackOK, callBackCancel) {
  MtgDlg.call(this, app, 'transParCoordDlg', callBackOK, callBackCancel)
  var self = this
  this.trans = trans
  this.modification = modification
  var list = app.listePr
  // Un CValeur pour stocker la valeur choisie pour la première coordonnée avant vérification
  this.valeurx = new CValeur(list, 0)
  // Un CValeur pour stocker la valeur choisie pour la deuxième coordonnée avant vérification
  this.valeury = new CValeur(list, 0)
  var tabprincipal = kernel.ce('table')
  this.appendChild(tabprincipal)
  var trprincipal = kernel.ce('tr')
  tabprincipal.appendChild(trprincipal)
  var td = kernel.ce('td')
  trprincipal.appendChild(td)

  var tab = kernel.ce('table')
  // this.appendChild(tab);
  td.appendChild(tab)
  var tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  var label = kernel.ce('label')
  $(label).html(kernel.getStr('Valeura') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputX = new MtgInput()
  td.appendChild(this.inputX)
  var indmax = modification ? list.indexOf(trans) - 1 : list.longueur() - 1
  var paneBoutonsValFoncAbs = new PaneBoutonsValFonc(this.app, this, this.inputX, true, indmax)
  tab.appendChild(paneBoutonsValFoncAbs)
  this.editorX = new EditeurvaleurReelle(app, this.inputX, indmax, this.valeurx, null)

  tr = kernel.ce('tr')
  tab.appendChild(tr)
  td = kernel.ce('td')
  tr.appendChild(td)
  label = kernel.ce('label')
  $(label).html(kernel.getStr('Valeurb') + ' : ')
  td.appendChild(label)
  td = kernel.ce('td')
  tr.appendChild(td)
  this.inputY = new MtgInput()
  td.appendChild(this.inputY)
  this.inputY.onkeyup = function (ev) {
    this.demarquePourErreur()
    if (ev.keyCode === 13) {
      $(this).blur() // Nécéssaire sur ipad pour une raison que j'ignore
      self.OK()
    }
  }
  var paneBoutonsValFoncOrd = new PaneBoutonsValFonc(this.app, this, this.inputY, true, indmax)
  tab.appendChild(paneBoutonsValFoncOrd)
  this.editorY = new EditeurvaleurReelle(app, this.inputY, indmax, this.valeury, null)
  if (modification) {
    $(this.inputX).val(trans.x.calcul.chaineCalculSansPar())
    $(this.inputY).val(trans.y.calcul.chaineCalculSansPar())
  }
  // On rajoute à droite un panneau pour le choix du repère
  td = kernel.ce('td')
  trprincipal.appendChild(td)
  this.paneListeReperes = new PaneListeReperes(this.app, indmax)
  td.appendChild(this.paneListeReperes.getTab())
  if (modification) this.paneListeReperes.selectRep(trans.rep)
  this.create('TransParCoord', 550)
}

TransParCoordDlg.prototype = new MtgDlg()

TransParCoordDlg.prototype.onOpen = function () {
  MtgDlg.prototype.onOpen.call(this)
  var self = this
  setTimeout(function () { // Ne marche pas sans un setTimeOut
    self.inputX.focus() // Pour que le focus soit donné à l'éditeur au départ ou quand un  message d'erreur est affiché
  }, 100)
}

// Pour cet outil il faut redéfinir onCancel car on actibe l'outil de capture si abandon
TransParCoordDlg.prototype.onCancel = function () {
  if (this.callBackCancel !== null) this.callBackCancel()
  this.destroy()
  this.app.activeOutilCapt()
}

TransParCoordDlg.prototype.OK = function () {
  // if (this.app.lastDlgId() !== this.id) return;
  // On regarde si l'entrée est correcte sur le plan syntaxique
  if (this.editorX.validate($(this.inputX).val())) {
    if (this.editorY.validate($(this.inputY).val())) {
      this.trans.x = this.valeurx
      this.trans.y = this.valeury
      this.trans.rep = this.paneListeReperes.getSelectedRep()
      if (this.callBackOK !== null) this.callBackOK()
      this.destroy()
    } else $(this.inputY).focus()
  } else $(this.inputX).focus()
}
