/*
 * Created by yvesb on 04/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Button = require('./Button')
const constantes = require('../kernel/constantes')
const kernel = require('../kernel/kernel')

module.exports = ButtonTool

/**
 * Création d'un objet ButtonTool qui représente un bouton associé à un outil
 * @param app L'appliation propriétaire
 * @param toolName Le nom de l'outil
 * @param target "left" pour un outil de barre flottante, "top" pour un outil de la barre d'outils supérieure
 * @param bIsCaptTool true pour l'outil de capture
 * @param row Le numéro de colonne du bouton ou -1 pour un bouton qui n'est pas affiché (il faut quand même
 * créer le bouton car son icône doit être disponible lorsqu'on choisit les outils actilfs pour la figure)
 * @param tip l'indication à afficher au survol du bouton
 * @constructor
 */
function ButtonTool (app, toolName, target, bIsCaptTool = false, row, tip) {
  if (arguments.length === 0) return // Pour ButtonToolAdd
  var left = target === 'left'
  var top = target === 'top'
  var float = target === 'float'
  var zoomFactor = app.zoomFactor
  var dim = left ? constantes.leftIconSize : constantes.topIconSize
  Button.call(this, app, 'outil' + toolName, arguments.length === 6 ? tip : toolName, dim, dim) // Icônes des outils carrées
  this.toolName = toolName
  this.expandableBar = null // Modifié quand on crée les expandableBar
  this.isActivated = false // Changé quand l'outil correspondant est activé
  app.buttons.push(this)
  this.tipDisplayed = false
  // this.tool = app["outil" + toolName];
  var isCaptTool = bIsCaptTool
  this.target = target
  var tool = app['outil' + toolName]
  if (!app.estExercice || (app.estExercice && app['outil' + toolName] && (tool.always || app.doc.toolDispo(tool.toolIndex)))) {
    this.build()
    var g = this.container // Crée dans le constructeur de Button
    g.setAttribute('id', toolName)
    // Nécessaire pour chaque bouton de lui affecter un transform qui est une translation
    g.setAttribute('transform', left ? 'translate(0,2)' : 'translate(' +
      String(row * ((constantes.topIconSize + 2) * zoomFactor)) + ',0)')
    if (row !== -1) {
      // Initialement les boutons de la barre de gauche ne sont pas visibles.
      // Ils sont rendus visibles quand on crée les expandableBar
      g.setAttribute('visibility', (top || float || isCaptTool) ? 'visible' : 'hidden')
      if (left) app.svgPanel.appendChild(g); else app.toolBar.appendChild(g)
    }
  } else {
    this.container = kernel.cens('g')
    this.container.setAttribute('visibility', 'hidden')
  }
}
ButtonTool.prototype = new Button()
ButtonTool.prototype.constructor = ButtonTool // Utile car ButtonToolAdd est un descendant
/**
 * Fonction appelée lors d'un simple clic sur la fonction
 */
ButtonTool.prototype.singleClickAction = function () {
  const nameEditor = this.app.nameEditor
  // Si un éditeur de nom de point ou droite est en cours, on le masque à ce niveau
  if (nameEditor.isVisible) nameEditor.montre(false)
  var eb = this.expandableBar
  if (eb !== null) {
    if (eb && eb.expanded) {
      eb.activeToolName = this.toolName
      eb.updateActiveIcon(eb.row)
    } else {
      if (this.app.hasBarExpanded()) {
        this.app.unFoldExpandableBars()
        this.app.updateActiveIcons()
      }
    }
  } else this.app.unFoldExpandableBars()
  this.app.selectTool(this.toolName)
}
/**
 * Fonction appelée lors d'un double clic sur le bouton
 */
ButtonTool.prototype.doubleClickAction = function () {
  var bar = this.expandableBar
  if (bar === null) {
    this.singleClickAction()
    return
  }
  if (bar.expanded) { bar.updateActiveIcon(bar.row) } else {
    this.app.unFoldExpandableBars()
    this.expandableBar.expand()
  }
}
