/*
 * Created by yvesb on 13/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Color = require('../types/Color')
const kernel = require('../kernel/kernel')
const constantes = require('../kernel/constantes')

const $ = require('../global/jQuery-ui')
require('../../outilsExternes/spectrum/spectrum.js')
require('../../outilsExternes/spectrum/spectrum.css')

module.exports = ColorChoicePanel

// y est l'ordonnée réelle du coin supérieur gauche du bouton relativement à la barre d'outils de droite
function ColorChoicePanel (app, y) {
  var zf = app.zoomFactor
  var self = this
  this.app = app
  this.y = y // Pour le tip
  this.target = 'right'
  this.tip = kernel.getStr('CouleurPerso')
  var g = kernel.cens('g', {
    transform: 'translate(0,' + y + ')'
  })
  this.container = g
  var col = app.getCouleur().rgb()
  var rect = kernel.cens('rect', {
    x: 0,
    y: 0,
    width: constantes.rightPanelWidth * zf,
    height: constantes.colorChoicePanelHeight * zf,
    stroke: 'none',
    fill: constantes.buttonBackGroundColor
  })
  g.appendChild(rect)
  var cx = constantes.rightPanelWidth / 4 * zf
  var cy = constantes.colorChoicePanelHeight / 2 * zf
  var rx = cx - 2
  var ry = cy - 2
  this.btnLeft = kernel.cens('ellipse', {
    cx: cx,
    cy: cy,
    rx: rx,
    ry: ry,
    color: 'black',
    fill: col
  })
  g.appendChild(this.btnLeft)
  this.btnLeft.y = y
  this.btnLeft.addEventListener('mouseover', function () {
    if (!self.tipDisplayed) {
      self.app.cacheTip() // Sans argument pour effacer l'ancien tip quel qu'il soit
      self.tipDisplayed = false
      self.app.setTip(self)
      setTimeout(function () {
        self.app.cacheTip(self)
      }, 2500)
    }
  })
  this.btnRight = kernel.cens('ellipse', {
    cx: 3 * cx,
    cy: cy,
    rx: rx,
    ry: ry,
    color: 'black',
    fill: col,
    opacity: app.doc.opacity
  })
  g.appendChild(this.btnRight)
  this.btnLeft.addEventListener('mousedown', function () {
    if (document.getElementById('colorPicker') !== null) return // Déjà affiché
    var app = self.app
    var x = app.dimf.x - constantes.rightPanelWidth - 200
    var y = self.y + constantes.toolbarHeight - 200
    var div = kernel.ce('div', {
      // class : "cp-default",
      id: 'colorPickerDiv',
      style: 'left:' + x + 'px;top:' + y + 'px;witdh=200px;height:230px;position:absolute;margin:0px'
    })
    app.colorPickerDiv = div
    var parentDiv = app.svg.parentNode // Pointe sur le div parent de l'appli
    parentDiv.appendChild(div)
    var input = kernel.ce('input', {
      type: 'text',
      id: 'colorPicker'
    })
    div.appendChild(input)
    $('#colorPicker').spectrum({
      color: self.app.getCouleur().rgb(),
      cancelText: kernel.getStr('Cancel'),
      chooseText: kernel.getStr('OK'),
      flat: true,
      showInput: true,
      showPalette: true,
      preferredFormat: 'hex',
      palette: constantes.colorPalette,
      clickoutFiresChange: false,
      change: function (color) {
        var app = self.app
        app.couleurActive = new Color(Math.round(color._r), Math.round(color._g), Math.round(color._b))
        $('#colorPicker').spectrum('destroy').remove()
        // $("#colorPicker").remove();
        $('#colorPickerDiv').remove()
        app.colorChoicePanel.setColor(app.couleurActive)
      }
    })
  })
}
ColorChoicePanel.prototype.setColor = function (col) {
  this.btnLeft.setAttribute('fill', col.rgb())
  this.btnRight.setAttribute('fill', col.rgb())
  this.btnRight.setAttribute('opacity', this.app.doc.opacity) // Modifé version 6.2.0
}

ColorChoicePanel.prototype.update = function () {
  this.setColor(this.app.couleurActive)
}
