/*
 * Created by yvesb on 16/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
// on importe kernel
const kernel = require('./kernel')
const Point = require('../types/Point')
const NatObj = require('../types/NatObj')
const NatCal = require('../types/NatCal')
const constantes = require('../kernel/constantes')
const $ = require('jquery')

const kernelAdd = {
  listeAngleDeg: ['90', '-90', '60', '-60', '30', '-30', '45', '-45', '120', '-120', '135', '-135'],
  listeAngleRad: ['pi/2', '-pi/2', 'pi/3', '-pi/3', 'pi/6', '-pi/6', 'pi/4', '-pi/4', '2*pi/3', '-2*pi/3', '3*pi/4', '-3*pi/4'],
  /**
   * Fonction renvoyant un Point formé des coordonnées d'un événement touch relatives
   * à un svg dans lequel travaille une mtgApp
   * @param svg Le svg de l'application
   * @param evt l'événement
   * @returns {Point}
   */
  touchPosition: function (svg, evt, zoomFactor = 1) {
    // Modifié version 6.5.1
    /*
    var tab = kernel.getTouchPosition(evt)
    var par = svg.parentNode
    var offs = kernel.getPosition2(par) // Ne pas appeler getOffset sur le svg mais sur le div parent
    return new Point(tab[0] - offs[0] - constantes.svgPanelWidth * zoomFactor,
      tab[1] - offs[1] - constantes.topIconSize * zoomFactor)
     */
    const par = svg.parentNode // Dans le cas de l'appli le parent du svg de la figure est un autre svg
    var tab = kernel.getTouchPositionToParent(evt, par, constantes.svgPanelWidth * zoomFactor, constantes.topIconSize * zoomFactor)
    return new Point(tab[0], tab[1])
  },
  /**
   * Fonction renvoyant un Point formé des coordonnées d'un événement souris relatives
   * à un svg dans lequel travaille une mtgApp
   * @param svg
   * @param evt
   * @returns {Point}
   */
  mousePosition: function (svg, evt, zoomFactor = 1) {
    // Modifié version 6.5.1
    /*
    var tab = kernel.getMousePosition(evt)
    var par = svg.parentNode
    var posdiv = kernel.getPosition2(par)
    return new Point(tab[0] - posdiv[0] - constantes.svgPanelWidth * zoomFactor,
      tab[1] - posdiv[1] - constantes.topIconSize * zoomFactor)
     */
    const par = svg.parentNode // Dans le cas de l'appli le parent du svg de la figure est un autre svg
    var tab = kernel.getMousePositionToParent(evt, par, constantes.svgPanelWidth * zoomFactor, constantes.topIconSize * zoomFactor)
    return new Point(tab[0], tab[1])
  },
  /**
   * Fonction sélectionnant les caractères d'indice selectionStart à selectionEnd
   * dans le input input
   * @param input
   * @param selectionStart
   * @param selectionEnd
   */
  setSelectionRange: function (input, selectionStart, selectionEnd) {
    if (input.setSelectionRange) {
      input.focus()
      input.setSelectionRange(selectionStart, selectionEnd)
    } else if (input.createTextRange) {
      var range = input.createTextRange()
      range.collapse(true)
      range.moveEnd('character', selectionEnd)
      range.moveStart('character', selectionStart)
      range.select()
    }
  },
  insertAfter: function (newNode, referenceNode) {
    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling)
  },
  distancePointDroite: function (xp, yp, xd, yd, u) {
    return Math.abs(u.y * (xd - xp) - u.x * (yd - yp)) / Math.sqrt(u.x * u.x + u.y * u.y)
  },
  getButtonArrow: function (app, id) {
    var btn = kernel.ce('input', {
      class: 'buttonarrow',
      type: 'image',
      src: app.local ? app.localPath + 'images/menuArrow.png' : app.imagePath + 'menuArrow.png',
      id: id
    })
    $(btn).css('width', '25px')
    $(btn).css('height', '18px')
    return btn
  },
  // Function remplaçant dans une chaîne les caractères < et > par leur équivalent html
  replaceIneg: function (ch) {
    var st = ch.replace(/</g, '&lt;')
    st = st.replace(/>/g, '&gt;')
    return st
  },
  // Fonction remplaçant dabns ch les caractères % non précédés d'un \ par \% pour code Tikz
  adaptStringTikz: function (ch) {
    var s = ch.replace(/([^\\]){1}%{1}/, '$1\\%')
    if (s.charAt(0) === '%') s = '\\' + s
    return s
  },
  /**
   * Fonction qui à partir d'une chaine Base64 décrivant une image fournit un blob
   * de façon à pouvoir petre utiisé dans FileSaver
   * @param base64Data
   * @param imageType
   * @returns {Blob}
   */
  base64toBlob: function (base64Data, imageType) {
    var byteCharacters = atob(base64Data) // eslint-disable-line no-undef
    var byteNumbers = new Array(byteCharacters.length)
    for (var i = 0; i < byteCharacters.length; i++) {
      byteNumbers[i] = byteCharacters.charCodeAt(i)
    }
    var byteArray = new Uint8Array(byteNumbers)
    return new Blob([byteArray], { type: 'image/' + imageType }) // eslint-disable-line no-undef
  },
  /**
   * Fonction renvoyant la chaîne de caractères représentant le type d'objet grahique dont la présence est nécessaire
   * pour un objet source de prototype de nature nat.
   * Aussi utilisé dans les exercices de construction.
   * @param nat
   * @returns {*}
   */
  chaineNatGraphPourProto: function (nat) {
    var ar1 = ['NDroite', 'NDemiDroite', 'NCercle', 'NSegment', 'NArc', 'NPointLie', 'NTtPoint', 'NPolygone', 'NLigneBrisee', 'NLieu']
    var ar2 = ['Dte', 'Ddte', 'Cerc', 'Seg', 'Arc', 'PointLie', 'Point', 'Polygone', 'LigneBrisee', 'LieuPt']
    for (var i = 0; i < ar1.length; i++) {
      if (nat.isOfNature(NatObj[ar1[i]])) return kernel.getStr(ar2[i])
    }
    return '' // Par précaution
  },
  /**
   * Fonction renvoyant la chaîne de caractères représentant le type d'objet grahique dont la présence est nécessaire
   * pour un objet source de prototype de nature calcul nat.
   * Aussi utilisé dans les exercices de construction.
   * @param nat
   * @returns {*}
   */
  chaineNatCalcPourProto: function (nat) {
    var ar1 = ['NVariable', 'NTteValR', 'NTteValC', 'NRepere', 'NFonction', 'NFonction2Var', 'NFonction3Var', 'NFonctionComplexe',
      'NFonctionComplexe2Var', 'NFonctionComplexe3Var', 'NSuiteRecurrenteReelle', 'NSuiteRecurrenteComplexe',
      'NSuiteRecurrenteReelle2', 'NSuiteRecurrenteReelle3', 'NSuiteRecurrenteComplexe2', 'NSuiteRecurrenteComplexe3',
      'NTteMatrice']
    var ar2 = ['Variable', 'ValR', 'ValC', 'Repere', 'Fonc', 'Fonc2Var', 'Fonc3Var', 'FoncComp',
      'FoncComp2Var', 'FoncComp3Var', 'SuiteRec', 'lieu', 'SuiteRec2', 'SuiteRec3', 'SuiteRecComplexe2', 'SuiteRecComplexe3',
      'Matrice']

    for (var i = 0; i < ar1.length; i++) {
      if (nat.isOfNature(NatCal[ar1[i]])) return kernel.getStr(ar2[i])
    }
    return '' // Par précaution
  },
  /**
   * Fonction qui renvoie pour une implémentation de prototype quel est la nature d'objet numérique qui est compatible
   * pour l'implémentation avec la nature nat
   * @param {NatCal} nat  la nature de l'objet testé qui est un objet source pour une construction
   * @returns {NatCal}
   */
  natObjCalPourProto: function (nat) {
    if (nat.isOfNature(NatCal.NCalculReel)) return NatCal.NTteValR
    if (nat.isOfNature(NatCal.NCalculComplexe)) return NatCal.NTteValC
    // Ligne suivante ajoutée version 6.7 pour les constructions ayant des objest sources de type matrice
    if (nat.isOfNature(NatCal.NTteMatrice)) return NatCal.NTteMatrice
    // Dans les autres cas même nature attendue
    return nat
  },
  /**
   * Fonction qui renvoie pour une implémentation de prototype quel est la nature d'objet graphique qui est compatible
   * pour l'implémentation avec la nature nat
   * @param {NatObj} nat  la nature de l'objet testé qui est un objet source pour une construction
   * @returns {NatObj}
   */
  natObjGraphPourProto: function (nat) {
    if (nat.isOfNature(NatObj.NTtPoint) && !nat.isOfNature(NatObj.NPointLie)) return NatObj.NTtPoint
    else return nat
  }
}

module.exports = kernelAdd
