/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Complexe = require('../types/Complexe')
const CValDyn = require('./CValDyn')

module.exports = CCalculAncetre

/**
 * @constructor
 * @extends CValDyn
 * Classe ancêtre de tous les objets de type calcul
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou pointeur sur la construction propriétaire.
 * @param {boolean} estElementFinal  true si élément finla de construction.
 * @param {String} nomCalcul  Le nom du calcul.
 * @returns {undefined}
 */
function CCalculAncetre (listeProprietaire, impProto, estElementFinal, nomCalcul) {
  if (arguments.length === 0) CValDyn.call(this) // Ajout version WebPack
  else {
    if (arguments.length === 1) CValDyn.call(this, listeProprietaire)
    else CValDyn.call(this, listeProprietaire, impProto, estElementFinal)
    if (arguments.length >= 4) this.nomCalcul = nomCalcul
  }
}
CCalculAncetre.prototype = new CValDyn()
CCalculAncetre.prototype.constructor = CCalculAncetre
CCalculAncetre.prototype.superClass = 'CValDyn'
CCalculAncetre.prototype.className = 'CCalculAncetre'

/**
 * Renvoie le nom du calcul
 * @returns {String}
 */
CCalculAncetre.prototype.getNom = function () {
  return this.nomCalcul
}
/**
 * Donne un nom au calcul
 * @param {String} nouveauNom
 * @returns {undefined}
 */
CCalculAncetre.prototype.donneNom = function (nouveauNom) {
  this.nomCalcul = nouveauNom
}
/**
 * Nombre de variables dans le cas d'une fonction, 0 sinon
 * @returns {Number}
 */
CCalculAncetre.prototype.nombreVariables = function () {
  return 0
}
// A revoir javascript : sans doute inutile
/**
 * Dans le cas d'une fonction, renvoie l'image de parametre par la fonction
 * @param {boolean} infoRandom  si true les calculs aléatoires par rand sont relancés
 * @param {type} parametre
 * @returns {Number}
 */
CCalculAncetre.prototype.rendValeurFonction = function (infoRandom, parametre) {
  return this.rendValeur()
}
// A revoir javascript : sans doute inutile
/**
 * Dans le cas d'une fonction, renvoie l'image de parametre par la fonction
 * @param {boolean} infoRandom  si true les calculs aléatoires par rand sont relancés
 * @param {Array} parametre  Tableau contenant le nom des paramètres.
 * @param {Complexe} zRes  Contient le résultat de f(parametre)
 * @returns {undefined}
 */
CCalculAncetre.prototype.rendValeurFonctionComplexe = function (infoRandom, parametre, zRes) {
}

CCalculAncetre.prototype.read = function (inps, list) {
  CValDyn.prototype.read.call(this, inps, list)
  this.nomCalcul = inps.readUTF()
}

CCalculAncetre.prototype.write = function (oups, list) {
  CValDyn.prototype.write.call(this, oups, list)
  oups.writeUTF(this.nomCalcul)
}
/**
 * Spécial JavaScript
 * Renvoie la valeur du calcul
 * @returns {Number}
 */
CCalculAncetre.prototype.valeur = function () {
  return this.rendValeur()
}
/**
 * Spécial JavaScript
 * Renvoie la valeur complexe du calcul
 * @returns {Number}
 */
CCalculAncetre.prototype.valeurComplexe = function () {
  var res = new Complexe()
  this.rendValeurComplexe(res)
  return res
}
