/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const Complexe = require('../types/Complexe')
const CCalculAncetre = require('./CCalculAncetre')
const CalcC = require('../kernel/CalcC')
const CCalcul = require('./CCalcul')

module.exports = CCalculComplexe

/**
 * Classe représentant un calcul complexe
 * @constructor
 * @extends CCalcul
 * @param {CListeObjets} listeProprietaire  La liste propriétaire.
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire.
 * @param {boolean} estElementFinal  true si l'objet est un objet intermédiaire de construction.
 * @param {String} nomCalcul  Le nom du calcul
 * @param {String} chaineCalcul  La chaîne repréentant le calcul
 * @param {CCb} calcul  L'arbre binaire contenant le calcul proprement dit
 * @returns {undefined}
 */
function CCalculComplexe (listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, calcul) {
  if (arguments.length === 1) CCalcul.call(this, listeProprietaire)
  else {
    if (arguments.length === 3) CCalcul.call(this, listeProprietaire, impProto, estElementFinal)
    else {
      CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul)
      this.chaineCalcul = chaineCalcul
      if (arguments.length === 5) this.calcul = CalcC.ccbComp(chaineCalcul, this.listeProprietaire, 0, chaineCalcul.length - 1, null)
      else this.calcul = calcul
    }
  }
  this.resultatComplexe = new Complexe()
}

CCalculComplexe.prototype = new CCalcul()
CCalculComplexe.prototype.constructor = CCalculComplexe
CCalculComplexe.prototype.superClass = 'CCalcul'
CCalculComplexe.prototype.className = 'CCalculComplexe'

CCalculComplexe.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var calculClone = this.calcul.getClone(listeSource, listeCible)
  return new CCalculComplexe(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.chaineCalcul, calculClone)
}

CCalculComplexe.prototype.getNatureCalcul = function () {
  return NatCal.NCalculComplexe
}

CCalculComplexe.prototype.positionne = function (infoRandom, dimfen) {
  this.existe = this.calcul.existe(infoRandom)
  if (!this.existe) return
  try {
    this.listeProprietaire.initialiseNombreIterations()
    this.calcul.resultatComplexe(infoRandom, this.resultatComplexe)
  } catch (e) {
    this.existe = false
    // Ajout version 6.0
    this.resultatComplexe = new Complexe(-1, 0)
  }
  if (!isFinite(this.resultatComplexe.x) || !isFinite(this.resultatComplexe.y)) this.existe = false
}

CCalculComplexe.prototype.rendValeurComplexe = function (zRes) {
  zRes.set(this.resultatComplexe)
}

CCalculComplexe.prototype.rendChaineValeurPourCommentaire = function (nbDecimales) {
  return this.rendChaineValeurComplexe(nbDecimales)
}
/**
 * Spécial JavaScript : Renvoie la chaîne LaTeX de la formule du calcul
 */
CCalculComplexe.prototype.chaineLatex = function () {
  return this.calcul.chaineLatexSansPar(null)
}
