/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const COb = require('./COb')
const Ope = require('../types/Ope')
const Opef = require('../types/Opef')
const CCbGlob = require('../kernel/CCbGlob')

// parmi toutes ces dépendances, on ne peut pas charger ici tous les objetTruc qui ont du
// objetTruc.prototype = new CCb()
// on commente ici et redéclare dans les méthodes qui les utilisent
// et qui pourraient peut-être passer dans un module séparé

// const CAppelFonction = require('./CAppelFonction')
// const CAppelFonctionComplexe = require('./CAppelFonctionComplexe')
// const CAppelFonctionNVar = require('./CAppelFonctionNVar')
// const CAppelFonctionComplexeNVar = require('./CAppelFonctionComplexeNVar')
// const CAppelFonctionInterne = require('./CAppelFonctionInterne')
// const CAppelFonctionInterneNVar = require('./CAppelFonctionInterneNVar')
// const CConstante = require('./CConstante')
// const CConstantei = require('./CConstantei')
// const CFonction = require('./CFonction')
// const CFonction2Var = require('./CFonction2Var')
// const CFonction3Var = require('./CFonction3Var')
// const CIntegraleDansFormule = require('./CIntegraleDansFormule')
// const CMoinsUnaire = require('./CMoinsUnaire')
// const COp = require('./COperation')
// const CProduitDansFormule = require('./CProduitDansFormule')
// const CPuissance = require('./CPuissance')
// const CResultatValeur = require('./CResultatValeur')
// const CResultatValeurComplexe = require('./CResultatValeurComplexe')
// const CVariableFormelle = require('./CVariableFormelle')
// const CSommeDansFormule = require('./CSommeDansFormule')
// const CCbNull = require('./CCbNull')

module.exports = CCb

/*
 * Classe anctre de tous les objets de type calcul pouvant être utilisés dans un
 * arbre binaire de calcul.
 * @constructor
 * @extends COb
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet
 * @returns {CCb}
 */
function CCb (listeProprietaire) {
  if (arguments.length === 0) COb.call(this) // Ajout version WebPack
  else {
    COb.call(this, listeProprietaire)
    // Ajout version 4.8 pour optimiser la fonction depDe
    this.elementTestePourDependDe = null
    this.dependDeElementTeste = false
  }
}
CCb.prototype = new COb()
CCb.prototype.constructor = CCb
CCb.prototype.superClass = 'COb'
CCb.prototype.className = 'CCb'

// Ajout version 4.8
/**
 * Fonction mémorisant la dépendance de l'objet par rapport à une objet déjà testé
 * @param {type} resultat
 * @returns {boolean}
 */
CCb.prototype.memDep = function (resultat) {
  this.dependDeElementTeste = resultat
  return resultat
}

/**
 * Fonction rendant une chaîne de caractères représentant le calcul représenté
 * par this qui est un arbre binaire de calcul.
 * A redéfinir pour les desdendants.
 * Utiliser plutôt chaineCalculSansPar car le résultat peut contenir des parenthèses.
 * @param {Array} varFor  Un tableau représentant les variables formelles
 * pour une fonction de plusieurs variables.
 * @returns {String}
 */
CCb.prototype.chaineCalcul = function (varFor) {
  return ''
}
/**
 * Fonction rendant une chaîne de caractères sans parenthèses externes
 * représentant le calcul représenté  * par this qui est un arbre binaire de calcul.
 * A redéfinir pour les desdendants.
 * @param {Array} varFor  Un tableau représentant les variables formelles
 * pour une fonction de plusieurs variables.
 * @returns {String}
 */
CCb.prototype.chaineCalculSansPar = function (varFor) {
  return this.chaineCalcul(varFor)
}
/**
 * Sert à reconstituer une formule LaTeX à partir de son arbre binaire
 * conjointement à chaineLatexSansPar A redéfinir pour les descendants
 */
/**
 * Fonction rendant une chaîne de caractères représentant en laTeX le calcul représenté
 * par this qui est un arbre binaire de calcul.
 * A redéfinir pour les desdendants.
 * Utiliser plutôt chaineLatexSansPar car le résultat peut contenir des parenthèses.
 * @param {Array} varFor  Un tableau représentant les variables formelles
 * pour une fonction de plusieurs variables.
 * @returns {String}
 */
CCb.prototype.chaineLatex = function (varFor) {
  return this.chaineCalcul(varFor)
}
/**
 * Sert à reconstituer une formule à partir de son arbre binaire conjointement
 * à chaineCalcul A redéfinir pour les descendants
 */
/**
 * Fonction rendant une chaîne de caractères sans parenthèses externes
 * représentant en laTeX le calcul représenté
 * par this qui est un arbre binaire de calcul.
 * A redéfinir pour les desdendants.
 * @param {Array} varFor  Un tableau représentant les variables formelles
 * pour une fonction de plusieurs variables.
 * @returns {String}
 */
CCb.prototype.chaineLatexSansPar = function (varFor) {
  return this.chaineLatex(varFor)
}
/**
 * Renvoie true si le calcul est constant (c'est-à-dire nn'utilise pas de valeurs
 * dynamiques) sans faire appel à la fonction Rand().
 * @returns {boolean}
 */
CCb.prototype.estConstant = function () {
  return false
}
/**
 * Fonction renvoyant true si this est un calcul constant renvoyant la valeur 0.
 * Redéfinit dans CConstante.
 * @returns {boolean}
 */
CCb.prototype.estConstante0 = function () {
  return false
}
/**
 * Fonction renvoyant true si le calcul existe.
 * A redéfinir pour les descendants.
 * existe prend un paramètre boolean infoRandom
 * C'est dû à l'optimisation de la fonction de 3 variables si(test,
 * valeurSiVrai, valeurSiFaux) qui nécessite de connaître la valeur
 * du premier argument test pour savoir si le résultat existe ou non. Si test
 * renvoie 1, existe renvoie valeurSiVrai.existe(inforRandom)
 * et sinon ValeurSiFaux.existe(infoRandom)
 * @returns {boolean}
 */
CCb.prototype.existe = function () {
  return true
}
/**
 * Fonction à redéfinir pour tous les descendants.
 * Renvoie un clone de l'objet, listeSource étant la liste propriétaire de l'objet
 * et listeCible la liste propriétaire du clone à créer.
 * @param {CListeObjets} listeSource  La liste propriétaire de l'objet
 * @param {CListeObjets} listeCible  La liste propriétaire de l'objet clone à céer.
 * @returns {CCb}
 */
CCb.prototype.getClone = function (listeSource, listeCible) {
  return null
}
/**
  * Fonction à redéfinir pour tous les descendants.
  * Renvoie une copie conforme de l'objet dans sa liste propriétaire.
  * Utilisé dans les calculs de dérivées.
 * @returns {CCb}
 */
CCb.prototype.getCopie = function () {
  return null
}
// Revu par rapport à la version Java : Paramètre infoRandom supprimé car inutilisé
/**
 * Fonction utilisée pour savoir si une fonction utilisateur existe.
 * Est différent de existe à cause de la fonction de 3 variables si(test, valeurSiVrai, valeurSiFaux)
 * @returns {boolean}
 */
CCb.prototype.existePourFonc = function () {
  return this.existe()
}
/**
 * Fonction renvoyant dans le cas d'un opérateur le membre de gauche et sinon this.
 * A redéfinir pour certians descendants.
 * @returns {CCb}
 */
CCb.prototype.membreGauche = function () {
  return this
}
/**
 * Fonction renvoyant dans le cas d'un opérateur le membre de droite et sinon this.
 * A redéfinir pour certians descendants.
 * @returns {CCb}
 */
CCb.prototype.membreDroit = function () {
  return this
}
/**
 * Fonction servant pour les fonctions à savoir quel est le nombre de variables.
 * @returns {number}
 */
CCb.prototype.nombreVariables = function () {
  return 0
}
/**
 * Fonction renvoyant le résultat du calcul.
 * A redéfinir pour les descendants.
 * @param {Boolean } infoRandom  true si les calculas aléatoires avec rand
 * @returns {number}
 */
CCb.prototype.resultat = function (infoRandom) {
  throw new Error('Appel de Ccb.resultat')
}
// Ajout version 6.7
/**
 * Renvoie un résultat qui est soit un nombre soit soir une matrice réelle sous forme de tableau
 * @param infoRandom
 * @return {number}
 */
CCb.prototype.resultatMat = function (infoRandom) {
  return this.resultat(infoRandom)
}
// Ajout pour la version 2.0 calculs sur les complexes.
// Le résultat est renvoyé dans z
/**
 * Fonction renvoyant dans zRes le résultat d'un calcul complexe.
 * A redéfinir pour les descendnats.
 * @param {boolean} infoRandom  true si les calculs aléatoires avec rand
 * @param {Complexe} zRes  Le résultat complexe du calcul
 * @returns {undefined}
 */
CCb.prototype.resultatComplexe = function (infoRandom, zRes) {
  zRes.x = this.resultat(infoRandom)// Par défaut on suppose le calcul réel
  zRes.y = 0
}
/**
 * Appelé lors des appels de fonctions pour fournir le résultat du calcul.
 * A redéfinir pour les descendants.
 * @param {boolean} infoRandom  true si les calculas aléatoires avec rand
 * @param {Array} valeurparametre  un tableau donnant les valeurs des paramètres pour une
 * fonction de une ou plusieurs variabels.
 * @returns {Number}
 */
CCb.prototype.resultatFonction = function (infoRandom, valeurparametre) {
  return this.resultat(infoRandom)
}
/**
 * Appelé lors des appels de fonctions pour fournir le résultat complexe du calcul.
 * A redéfinir pour les descendants.
* @param {boolean} infoRandom  true si les calculas aléatoires avec rand
 * @param {Array} valeurParametre  un tableau donnant les valeurs des paramètres pour une
 * @param {type} zRes  Le complexe contenant le résultat.
 * @returns {undefined}
 */
CCb.prototype.resultatFonctionComplexe = function (infoRandom, valeurParametre,
  zRes) {
  this.resultatComplexe(infoRandom, zRes)
}

CCb.prototype.initialisePourDependance = function () {
  this.elementTestePourDependDe = null
  this.elementTestePourDependDePourRec = null
}

/**
 * Fonction renvoyant true si le calcul dépend de p.
 * A redéfinir pour les descendants.
 * @param {CElementBase} p
 * @returns {boolean}
 */
CCb.prototype.depDe = function (p) {
  this.elementTestePourDependDe = p
  return false
}
/**
 * La fonction suivante renverra le même résultat que DepDe sauf dans le
 * cas où le calcul fait réferrence à la fonction Rand de génération d'un
 * nombre aléatoire
 * @param {CElementBase} p
 * @returns {boolean}
 */
CCb.prototype.dependDePourBoucle = function (p) {
  return false
}
/**
 * Fonction renvoyant true si l'arbre pointé par this contient un appel de variable
 * formelle, ind étant l'indice de la variable pour une fonction de plusieurs variables ou -1 si
 * on teste la dépendance par rapport à toute variable.
 * A redéfinir pour tous les descendants.
 * @param {number} ind  Indice de la variable pour les fonctions à plusieurs variables.
 * @returns {boolean}
 */
// La possibilité que ind soit égalà à -1 a été rajoutée version 6.4.8.
// Sert à tester la dépendance par rapprot à toute variable formelle
CCb.prototype.dependDeVariable = function (ind) {
  return this.operande.dependDeVariable(ind)
}

// Ajout version 6.3.4 pour traiter de la même façon la commutativité des opérateurs +,* & et |
/**
 * Fonction ajoutant à liste tous les operandes d'opérateur de nature natop
 * Est utilisé pour les opérateurs commutatifs: addition, multiplication, ou et et logique
 * @param liste
 * @param natop
 */
CCb.prototype.ajouteOperateursParNat = function (liste, natop) {
  var nat = this.nature()
  if (nat === CCbGlob.natOperation) {
    if (this.ope === natop) {
      this.operande1.ajouteOperateursParNat(liste, natop)
      this.operande2.ajouteOperateursParNat(liste, natop)
    } else { liste.push(this) }
  } else { liste.push(this) }
}

/**
 * Fonction fournissant un calcul dans lequel tous les CResultatValeur
 * et CResultatValeurComplexes sont remplacés par une constante dont la valeur
 * est le résultat du CResultatValeur ou CResultatValeurComplexes.
 * Versionn 6.7 : On rajoute un paramètre booléen bfrac. Si celui-ci est faux, les valeurs non
 * entières seront remplacées par un quotient correspondant à la fraction continue équivalente
 * à 10^(-12) près.
 * @param {boolean} bfrac
 * @returns {CCb}
 */
CCb.prototype.calculAvecValeursRemplacees = function (bfrac) {
  return this
}
/**
 * Renvoie un calcul normalisé équivalent au calcul this.
 * Si rempval est true les appels à des CResultatValeur ou CResultatValeurComplexe
 * sont remplacés par une constante.
 * @param {boolean} rempval
 * @param {boolean} sousdivnorm  Si true, les soustractions sont remplacées par des additions d'opposés
 * @param {boolean} rempDecParFrac  Si, true, les nombres décimaux (avec au plus 9 décimales) et la fraction irréductible
 * qui les représentent sont considérés comme équivalents.
 * et les divisions pas des multiplications par l'inverse.
 * @returns {CCb}
 */
CCb.prototype.calculNormalise = function (rempval, sousdivnorm, rempDecParFrac) {
  // Dans les autres cas on renvoie le même calcul pour les objets ne redéfinissant pas cette fonction (version 6.4.7)
  return this
}

/**
 * Fonction renvayant true si this est équivalent à calc
 * Deux calculs sont équivalents s'ils contiennent les mêmes opérateurs
 * sans tenier compte de l'odre des additions algébriques ou des multiplications.
 * Avant appel, this et calc doivent avoir été normalisés de façon
 * que this et calc ne contiennent que des additions et divisions.
 * @param {CCb} calc  le calcul dont on teste l'équivalence avec this.
 * @returns {boolean}
 */
CCb.prototype.estEquivalentA = function (calc) {
  var nat1 = this.nature()
  var nat2 = calc.nature()
  if (nat1 !== nat2) { return false }
  if (nat1 === CCbGlob.natConstante) {
    return this.valeur === calc.valeur
  }
  if (nat1 === CCbGlob.natMoinsUnaire) {
    return this.operande.estEquivalentA(calc.operande)
  }
  if (nat1 === CCbGlob.natFonction) {
    return (Opef.estEquivalent(this.opef, calc.opef)) && this.operande.estEquivalentA(calc.operande)
  }
  if (nat1 === CCbGlob.natAppelFonction) {
    // Correction version 6.4.8
    // Si on compare deux CintegraleDansFormule, ce sont les formules des fonctions à inétgerer qu'il faut comparer
    // car la fonction interne a été dupliquée.
    // return (this.fonctionAssociee === calc.fonctionAssociee) && this.operande.estEquivalentA(calc.operande)
    if ((this.fonctionAssociee === calc.fonctionAssociee) && this.operande.estEquivalentA(calc.operande)) return true
    else return (this.fonctionAssociee.calcul.estEquivalentA(calc.fonctionAssociee.calcul)) && this.operande.estEquivalentA(calc.operande)
  }
  if (nat1 === CCbGlob.natFonction2Var) {
    return (this.opef === calc.opef) && this.operande1.estEquivalentA(calc.operande1) &&
        this.operande2.estEquivalentA(calc.operande2)
  }
  if (nat1 === CCbGlob.natFonction3Var) {
    return (this.opef === calc.opef) && this.operande1.estEquivalentA(calc.operande1) &&
        this.operande2.estEquivalentA(calc.operande2) &&
        this.operande3.estEquivalentA(calc.operande3)
  }
  if (nat1 === CCbGlob.natAppelFonctionNVar) {
    if (this.fonctionAssociee !== calc.fonctionAssociee) return false
    // Correction version 6.3.3
    /*
    var op1 = new Array(this.nbVar);
    for (var i = 0; i < this.nbVar; i++)
      op1[i] = this.operande[i];
    var op2 = new CCb[calc.nbVar];
    for (i = 0; i < calc.nbVar; i++)
      op2[i] = calc.operande[i];
    var res = true;
    for (i = 0; i < this.nbVar; i++)
      res = res && op1[i].estEquivalentA(op2[i]);
    */
    var res = true
    // Ligne suivante optimisée version 6.4.1
    // for (var i = 0; i < this.nbVar; i++) { res = res && this.operande[i].estEquivalentA(calc.operande[i]) }
    for (var i = 0; (i < this.nbVar) && res; i++) { res = res && this.operande[i].estEquivalentA(calc.operande[i]) }
    return res
  }
  if (nat1 === CCbGlob.natVariableFormelle) {
    // CVariableFormelle for1 = (CVariableFormelle) this;
    // CVariableFormelle for2 = (CVariableFormelle) calc;
    return this.indvar === calc.indvar
  }
  if (nat1 === CCbGlob.natOperation) {
    // COperation op1 = (COperation) this;
    // COperation op2 = (COperation) calc;
    var natop1 = this.ope
    var natop2 = calc.ope
    if (natop1 !== natop2) {
      if (natop1 === Ope.InfOuEgal) {
        // a <= b équivalent à b>= a)
        if (natop2 === Ope.SupOuEgal) { return (this.operande1.estEquivalentA(calc.operande2) && this.operande2.estEquivalentA(calc.operande1)) } else return false // Rajouté version 4.7.2
      } else {
        if (natop1 === Ope.Inf) {
        // a < b équivalent à b > a)
          if (natop2 === Ope.Sup) { return (this.operande1.estEquivalentA(calc.operande2) && this.operande2.estEquivalentA(calc.operande1)) } else return false // Rajouté version 4.7.2
        } else {
          if (natop1 === Ope.SupOuEgal) {
            // a >= b équivalent à b <= a)
            if (natop2 === Ope.InfOuEgal) { return (this.operande1.estEquivalentA(calc.operande2) && this.operande2.estEquivalentA(calc.operande1)) } else return false // Rajouté version 4.7.2
          } else {
            if (natop1 === Ope.Sup) {
              // a > b équivalent à b < a)
              if (natop2 === Ope.Inf) { return (this.operande1.estEquivalentA(calc.operande2) && this.operande2.estEquivalentA(calc.operande1)) } else return false // Rajouté version 4.7.2
            } else return false
          }
        }
      }
    }
    // Les deux opérateurs sont les mêmes
    // Modification version 6.3.4 pour traiter de la même façon la commutativité des opérateurs +,* & et |
    if (natop1 === Ope.Plus || natop1 === Ope.Mult || natop1 === Ope.Or || natop1 === Ope.And) {
      // Si l'opérateur est une addition, multiplication, ou ou et logique, il faut vérifier que tous les opérandes sous-jascents sont
      // équivalents pour tenir compte de la commutativité de ces opérateurs
      var list1 = []
      var list2 = []
      this.ajouteOperateursParNat(list1, natop1)
      calc.ajouteOperateursParNat(list2, natop1)
      if (list1.length !== list2.length) return false
      for (i = 0; i < list1.length; i++) {
        var c1 = list1[i]
        var trouve = false
        for (var j = 0; j < list2.length; j++) {
          var c2 = list2[j]
          if (c1.estEquivalentA(c2)) {
            trouve = true
            list2.splice(j, 1)
            break
          }
        }
        if (!trouve) { return false } else {
          list1.splice(i, 1)
          i-- // Car le terme d'indice i a été retiré
        }
      }
      return true
    }
    // a = b équivalent à b = a
    if ((natop1 === Ope.Egalite) || (natop1 === Ope.And) || (natop1 === Ope.Or)) {
      return (this.operande1.estEquivalentA(calc.operande1) && this.operande2.estEquivalentA(calc.operande2)) ||
          (this.operande1.estEquivalentA(calc.operande2) && this.operande2.estEquivalentA(calc.operande1))
    }
    // Dans les autres cas (division comprise, pour être équivalent les
    // opérandes respectifs
    // doivent être équivalents
    return (this.operande1.estEquivalentA(calc.operande1)) &&
        (this.operande2.estEquivalentA(calc.operande2))
  }
  if (nat1 === CCbGlob.natPuissance) {
    return (this.operande.estEquivalentA(calc.operande) && this.exposant.estEquivalentA(calc.exposant))
  }
  if (nat1 === CCbGlob.natConstantei) return true
  if (nat1 === CCbGlob.natResultatValeur) {
    return this.valeurAssociee === calc.valeurAssociee // Attention test d'identité en JavaScript
  }
  if (nat1 === CCbGlob.natResultatValeurComplexe) {
    return this.valeurAssociee === calc.valeurAssociee
  }
  if (nat1 === CCbGlob.natIntegraleDansFormule) {
    return this.calculASommer.estEquivalentA(calc.calculASommer) && this.bornea.estEquivalentA(calc.bornea) &&
      this.borneb.estEquivalentA(calc.borneb)
  }
  return false
}
/**
 * Renvoie true si this est un produit factorisé par calc ou simplement équivalent à calc
 * ou à son opposé.
 * Avant appel, this et calc doivent être rendus normalisés pour pouvoir reconnaître
 * les réductions au même dénominateur.
 * @param {CCb} calc  Le calcul dont on etste si c'est un facteur commun à this.
 * @returns {boolean}
 */
CCb.prototype.estFactorisePar = function (calc) {
  var nat1, nat2, natop1, natop2, list1, list2, i, j, c1, c2, trouve
  nat1 = this.nature()
  if (nat1 === CCbGlob.natMoinsUnaire) {
    return this.operande.estFactorisePar(calc)
  } else {
    if (nat1 === CCbGlob.natOperation) {
      // COperation op1 = (COperation) this;
      natop1 = this.ope
      if ((natop1 === Ope.InfOuEgal) || (natop1 === Ope.Inf) || (natop1 === Ope.SupOuEgal) || (natop1 === Ope.Sup) ||
          (natop1 === Ope.Egalite) || (natop1 === Ope.Diff)) {
        return ((this.operande1.estFactorisePar(calc) && this.operande2.estConstante0()) || (this.operande2.estFactorisePar(calc) && this.operande1.estConstante0()))
      }
      if (natop1 === Ope.Mult) {
        // Il faut vérifier si un des facteurs du produit et des facteurs sous-jascents est équivalent à calc
        list1 = []
        list2 = []
        this.ajouteOperateursParNat(list1, Ope.Mult)
        nat2 = calc.nature()
        if (nat2 !== CCbGlob.natOperation) list2.push(calc)
        else {
          natop2 = calc.ope
          if (natop2 === Ope.Mult) calc.ajouteOperateursParNat(list2, natop2)
          else list2.push(calc)
        }
        if (list1.length < list2.length) return false
        for (i = 0; i < list2.length; i++) {
          c2 = list2[i]
          trouve = false
          for (j = 0; j < list1.length; j++) {
            c1 = list1[j]
            if (c1.estEquivalentA(c2)) {
              trouve = true
              list1.splice(j, 1)
              break
            }
          }
          if (!trouve) return false
          else {
            list2.splice(i, 1)
            i-- // Car le terme d'indice i a été retiré
          }
        }
        return true
      }
    }
    return this.estEquivalentA(calc)
  }
}
/**
 * Fonction renvoyant true si le calcul est dérivable.
 * A redéfinir pour les descendants pour lesquels ce n'est pas dérivabel.
 * @returns {boolean}
 */
CCb.prototype.deriveePossible = function () {
  return true
}

/**
 * Fontion à redéfinir pout les descendants qui renvoie true si le calcul est bien un calcul acceptable
 * pour un calcul sur les vecteurs
 * @param tabNames Un tableau de string contenant les noms de calculs complexes assimilés à des points ou des vecteurs
 * @return {boolean}
 */
CCb.prototype.isCalcVect = function (tabNames) {
  return false
}

/**
 * Fontion à redéfinir pout les descendants qui renvoie true si le calcul peut être correct pour un calcul vectoriel
 * @param tabNames Un tableau de string contenant les noms de calculs complexes assimilés à des points ou des vecteurs
 * @return {boolean}
 */
CCb.prototype.isCalcOK4Vect = function (tabNames) {
  return true
}

/**
 * Fonction renvoyant true si this représente un résultat d'un calcul complexe nommé vect0
 * Sert pour les exercices de calcul sur les vecteurs ou de calcul de produit scalaire
 * Sera redéfini seulement pour CResultatValeurComplexe
 * @return {boolean}
 */
CCb.prototype.isVectNul = function () {
  return false
}
