/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const CCalcul = require('./CCalcul')
const CalcR = require('../kernel/CalcR')

module.exports = CFonc

/**
 * Fonction utilisateur réelle de une variable.
 * @constructor
 * @extends CCalcul
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire de l'objet
 * @param {boolean} estElementFinal  true si l'objet est un objet inal de construction
 * @param {String} nomCalcul  Le nom de la fonction.
 * @param {String} chaineCalcul  La formule définissnt la focntion
 * @param {String} chaineVariable  La varaible formelle utilisée pour la formule.
 * @param {CCb} calcul  Arbre binaire repsentant le calcul.
 * @returns {CFonc}
 */
// Attention : Ordre des deux derniers paramètres inversé par rapport version Java
function CFonc (listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, chaineVariable, calcul) {
  if (arguments.length !== 0) { // 0 pour appel dans prototype
    if (arguments.length === 1) CCalcul.call(this, listeProprietaire)
    else {
      if (arguments.length === 3) CCalcul.call(this, listeProprietaire, impProto, estElementFinal)
      else {
        this.chaineVariable = chaineVariable
        if (arguments.length <= 6) this.calcul = null
        if (calcul === null) {
          CCalcul.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul,
            CalcR.ccb(chaineCalcul, listeProprietaire, 0, chaineCalcul.length - 1,
              [chaineVariable]))
        } else CCalcul.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, calcul)
      }
    }
  }
}
CFonc.prototype = new CCalcul()
CFonc.prototype.constructor = CFonc
CFonc.prototype.superClass = 'CCalcul'
CFonc.prototype.className = 'CFonc'

CFonc.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var calculClone = this.calcul.getClone(listeSource, listeCible)
  return new CFonc(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.chaineCalcul, this.chaineVariable, calculClone)
}
CFonc.prototype.getNatureCalcul = function () {
  return NatCal.NFonction
}
CFonc.prototype.rendValeurFonctionComplexe = function (infoRandom, parametre, zRes) {
  this.calcul.resultatFonctionComplexe(infoRandom, parametre, zRes)
}
// Spécial JavaScript. Lors de la création d'une fontion on crée automatiquement ici son calcul
/**
 * Crée l'arbre binaire de calcul associé et le met dans this.calcul.
 * @returns {undefined}
 */
CFonc.prototype.creeCalcul = function () {
  var va = new Array(1)
  va[0] = this.chaineVariable
  this.calcul = CalcR.ccb(this.chaineCalcul, this.listeProprietaire, 0,
    this.chaineCalcul.length - 1, va)
}
/**
 * Le nombre de variables de la fonction.
 * Est redéfini pour les fonctions de plusieurs variables.
 * @returns {Number}
 */
CFonc.prototype.nombreVariables = function () {
  return 1
}
CFonc.prototype.rendValeurFonction = function (infoRandom, parametre) {
  return this.calcul.resultatFonction(infoRandom, parametre)
}
CFonc.prototype.positionne = function (infoRandom, dimfen) {
  this.existe = this.calcul.existePourFonc()
}
CFonc.prototype.reconstruitChaineCalcul = function () {
  var ch = new Array(1)
  ch[0] = this.chaineVariable
  this.chaineCalcul = this.calcul.chaineCalculSansPar(ch)
}
CFonc.prototype.read = function (inps, list) {
  CCalcul.prototype.read.call(this, inps, list)
  this.chaineVariable = inps.readUTF()
}
CFonc.prototype.write = function (oups, list) {
  CCalcul.prototype.write.call(this, oups, list)
  oups.writeUTF(this.chaineVariable)
}
CFonc.prototype.readSansCalcul = function (inps, list) {
  CCalcul.prototype.readSansCalcul.call(this, inps, list)
}
CFonc.prototype.writeSansCalcul = function (oups, list) {
  CCalcul.prototype.writeSansCalcul.call(this, oups, list)
}
CFonc.prototype.estFonctionOuSuite = function () {
  return true
}
/**
 * Spécial JavaScript : Renvoie la chaîen LaTeX de la formule du calcul
 */
CFonc.prototype.chaineLatex = function () {
  return this.calcul.chaineLatexSansPar(this.chaineVariable)
}
