/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatCal = require('../types/NatCal')
const CalcR = require('../kernel/CalcR')
const CCalcul = require('./CCalcul')
module.exports = CFoncNVar

// Attention : Ordre des deux derniers paramètres inverses par rapport à version Java
/**
 * Fonction réelle de n variables.
 * @constructor
 * @extends CCalcul
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire de l'objet
 * @param {boolean} estElementFinal  true si l'objet est un objet inal de construction
 * @param {String} nomCalcul  Le nom de la fonction.
 * @param {String} chaineCalcul  La formule définissnt la fonction
 * @param {Arrray} chaineVariable  Un array de String donnant les variables formelles.
 * @param {CCb} calcul  Un arbre binaire de calcul représentant la fonction.
 * @returns {undefined}
 */
function CFoncNVar (listeProprietaire, impProto, estElementFinal, nomCalcul,
  chaineCalcul, chaineVariable, calcul) {
  var calc
  if (arguments.length === 0) return
  if (arguments.length === 1) CCalcul.call(this, listeProprietaire)
  else {
    if (arguments.length <= 6) calc = null; else calc = calcul
    if (typeof chaineVariable === 'number') {
      // Constructeur sécial pour CDeriveePartielle
      this.nbVar = chaineVariable
      this.chaineVariable = null
      CCalcul.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, calc)
    } else {
      this.chaineVariable = chaineVariable // ChaineVariable est un Array de String
      this.nbVar = chaineVariable.length
      if ((calc === null) && (chaineCalcul.length !== 0)) {
        CCalcul.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul,
          CalcR.ccb(chaineCalcul, listeProprietaire, 0, chaineCalcul.length - 1, chaineVariable))
      } else CCalcul.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul, chaineCalcul, calc)
    }
  }
}
CFoncNVar.prototype = new CCalcul()
CFoncNVar.prototype.constructor = CFoncNVar
CFoncNVar.prototype.superClass = 'CCalcul'
CFoncNVar.prototype.className = 'CFoncNVar'

/**
 * Fonction renvoyant un pointeur sur le tableau de chaînes représentant les variables de la fonction
 * Devra être redéfini pour CDeriveePartielle
 * @returns {Arrray|Array}
 */
CFoncNVar.prototype.chaineVariable = function () {
  return this.chaineVariable
}
CFoncNVar.prototype.getClone = function (listeSource, listeCible) {
  var ind1 = listeSource.indexOf(this.impProto)
  var calculClone = this.calcul.getClone(listeSource, listeCible)
  return new CFoncNVar(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.chaineCalcul, this.chaineVariable, calculClone)
}
CFoncNVar.prototype.getNatureCalcul = function () {
  switch (this.nbVar) {
    case 2 :
      return NatCal.NFonction2Var
    case 3 :
      return NatCal.NFonction3Var
    default :
      return NatCal.NFoncPlusieursVar
  }
}
CFoncNVar.prototype.nombreVariables = function () {
  return this.nbVar
}
CFoncNVar.prototype.rendValeurFonction = function (infoRandom, parametre) {
  return this.calcul.resultatFonction(infoRandom, parametre)
}
CFoncNVar.prototype.positionne = function (infoRandom, dimfen) {
  this.existe = this.calcul.existePourFonc()
}
CFoncNVar.prototype.reconstruitChaineCalcul = function () {
  this.chaineCalcul = this.calcul.chaineCalculSansPar(this.chaineVariable)
}
CFoncNVar.prototype.read = function (inps, list) {
  CCalcul.prototype.read.call(this, inps, list)
  this.nbVar = inps.readInt()
  this.chaineVariable = new Array(this.nbVar)
  for (var i = 0; i < this.nbVar; i++) this.chaineVariable[i] = inps.readUTF()
}
CFoncNVar.prototype.write = function (oups, list) {
  CCalcul.prototype.write.call(this, oups, list)
  oups.writeInt(this.nbVar)
  for (var i = 0; i < this.nbVar; i++) oups.writeUTF(this.chaineVariable[i])
}
CFoncNVar.prototype.estFonctionOuSuite = function () {
  return true
}
CFoncNVar.prototype.rendValeurFonctionComplexe = function (infoRandom, parametre, zRes) {
  this.calcul.resultatFonctionComplexe(infoRandom, parametre, zRes)
}
