// Ajout le 3/5/2021 par Yves Biton
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
import mathjs from '../kernel/mathjs'
const CCalculAncetre = require('./CCalculAncetre')
const CCalcul = require('./CCalcul')
const kernel = require('../kernel/kernel')
const NatCal = require('../types/NatCal')
// const matrix = require('matrix-js')
// module.exports = CMatriceParForm
export default CMatriceParForm

/**
 * Classe représentant une matrice de n lignes et p colonnes dont les valerus sont données
 * par une fonction de deux variables (i, j) où i est le n° de ligne et j celui de colonne
 * @param {CListeObjets} listeProprietaire : La liste propriétaire
 * @param {CImplementationProto} impProto  null ou a construction propriétaire.
 * @param {boolean} estElementFinal  true si élément final de construction
 * @param {String} nomCalcul  Le nom donné au calcul.
 * @param {int} n Le nombre de lignes de la matrice
 * @param {int} p Le nombre de colonnes de la matrice
 * @param {CCb} calcul formule de la  fonction de deux variables (i, j)
 * @constructor
 */
// Attention : cet objet n'utilise qu'un calcul pour s'enregistrer dans le flux mais pour se positionner
// il a besoin que chaque cellule ait pour se calculer un clone de cette formule car sinon quand on appelle
// positionne avec un paramètre false tous les calculs vont renvoyer la même valeur puisque qu'une fois positionné
// avec un paramètre infoRandom à false un calcul renvoie le mêm résultat.
function CMatriceParForm (listeProprietaire, impProto, estElementFinal, nomCalcul, n, p, calcul) {
  if (arguments.length === 0) CCalculAncetre.call(this)
  else {
    if (arguments.length === 1) CCalculAncetre.call(this, listeProprietaire)
    else {
      CCalculAncetre.call(this, listeProprietaire, impProto, estElementFinal, nomCalcul)
      this.n = n // Le nombre de lignes
      this.p = p // Le nombre de colonnes
      this.calcul = calcul // Formule de la fonction de deux variables (i, j)
      this.creeCalculs()
    }
  }
}

CMatriceParForm.prototype = new CCalcul()
CMatriceParForm.prototype.constructor = CMatriceParForm
CMatriceParForm.prototype.superClass = 'CCalcul'
CMatriceParForm.prototype.className = 'CMatriceParForm'

CMatriceParForm.prototype.getClone = function (listeSource, listeCible) {
  const ind1 = listeSource.indexOf(this.impProto)
  var calculClone = this.calcul.getClone(listeSource, listeCible)
  return new CMatriceParForm(listeCible, listeCible.get(ind1, 'CImplementationProto'),
    this.estElementFinal, this.nomCalcul, this.n, this.p, calculClone)
}

CMatriceParForm.prototype.getNatureCalcul = function () {
  return NatCal.NMatriceParForm
}

/**
 * Fonction créant chaque clone du calcul principal associé à chaque terme de la matrice
 * A appeler si on modifie la matrice
 */
CMatriceParForm.prototype.creeCalculs = function () {
  this.calculs = []
  const list = this.listeProprietaire
  for (let i = 0; i < this.n; i++) {
    const lig = []
    this.calculs.push(lig)
    for (let j = 0; j < this.p; j++) {
      lig.push(this.calcul.getClone(list, list))
    }
  }
}

CMatriceParForm.prototype.positionne = function (infoRandom) {
  const mat = []
  for (let i = 0; i < this.n; i++) {
    const lig = []
    mat.push(lig)
    for (let j = 0; j < this.p; j++) {
      // On applique la formule de fonction à chaque coupe (i, j)
      try {
        this.listeProprietaire.initialiseNombreIterations()
        const res = this.calculs[i][j].resultatFonction(infoRandom, [i + 1, j + 1])
        lig.push(res)
      } catch (e) {
        this.existe = false
        return
      }
    }
  }
  this.existe = true
  this.mat = mathjs.matrix(mat) // Matrice contenant les valeurs aléatoires
}

CMatriceParForm.prototype.depDe = function (p) {
  if (this.elementTestePourDependDe === p) return this.dependDeElementTeste
  return this.memDep(CCalculAncetre.prototype.depDe.call(this, p) || this.calcul.depDe(p))
}

CMatriceParForm.prototype.dependDePourBoucle = function (p) {
  return (p === this) || this.calcul.dependDePourBoucle(p)
}

// Inutiile de redéfinir depDe et depDePourBoucle qui sont définies dans CCalcul dont le prototype descend

CMatriceParForm.prototype.read = function (inps, list) {
  CCalculAncetre.prototype.read.call(this, inps, list)
  this.n = inps.readInt()
  this.p = inps.readInt()
  this.calcul = inps.readObject(list)
  this.creeCalculs()
}

CMatriceParForm.prototype.write = function (oups, list) {
  CCalculAncetre.prototype.write.call(this, oups, list)
  oups.writeInt(this.n)
  oups.writeInt(this.p)
  oups.writeObject(this.calcul)
}

/**
 * Pour l'interprétation syntaxique, les matrices sont considérées comme des fonctions à deu variables
 * @return {number}
 */
CMatriceParForm.prototype.nombreVariables = function () {
  return 2
}

/**
 * Fonction utilisée dans CalcR. CMatrice, CMatriceParForm sont les seuls objets
 * descendant de CValDyn renvoyant true pour cette fonction (matrices ne résultant pas d'un calcul)
 * @return {boolean}
 */
CMatriceParForm.prototype.estMatriceBase = function () {
  return true
}

/**
 * Fonction utilisée dans CalcR. CMatrice est le seul objet avec CCalcMat descendant de CValDyn renvpyant true pour cette fonction
 * @return {boolean}
 */
CMatriceParForm.prototype.estMatrice = function () {
  return true
}

/**
 * Fonction renvoyant la représentation LaTeX du résultat de la matrice avec n décimales
 * @param {int} nbdec Le nombre de décimales
 * @return {String}
 */
CMatriceParForm.prototype.latexMat = function (nbdec) {
  return kernel.latexMat(this.mat, nbdec)
}

/**
 * Fonction renvoyant la représentation LaTeX du résultat de la matrice avec appriximation
 * des valeurs par des fractions rationnelles à 10^(-12) près
 * @return {String}
 */
CMatriceParForm.prototype.latexMatFrac = function () {
  return kernel.latexMatFrac(this.mat)
}
