/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CPt = require('./CPt')
module.exports = CPointAncetrePointsMobiles

/**
 * Classe ancêtre de tous les points mobiles., point libres ou liés.
 * @constructor
 * @extends CPt
 * @param {CListeObjets} listeProprietaire  La liste propriétaire de l'objet
 * @param {CImplementationProto} impProto  null ou pointe sur la construction propriétaire de l'objet.
 * @param {boolean} estElementFinal  true si l'objet est un objet final de construction
 * @param {Color} couleur  La couleur de l'objet
 * @param {boolean} nomMasque  true si le nom de l'objet est masqué
 * @param {number} decX  Decalage horizontal du nom
 * @param {number} decY  Decalage vertical du nom
 * @param {boolean} masque  true si l'objet est masque
 * @param {String} nom  Le nom eventuel de l'objet
 * @param {number} tailleNom  Indice de la taille du nom (voir Fonte)
 * @param {MotifPoint} motif  Le motif du point
 * @param {boolean} marquePourTrace  true si le point est marqué pour la trace
 * @param {boolean} pointLibre  true si le point n'est pas punaisé
 * @returns {undefined}
 */
function CPointAncetrePointsMobiles (listeProprietaire, impProto, estElementFinal, couleur, nomMasque,
  decX, decY, masque, nom, tailleNom, motif, marquePourTrace, pointLibre) {
  if (arguments.length === 0) return // Ajout version 4.9.9.4
  if (arguments.length === 1) CPt.call(this, listeProprietaire)
  else {
    CPt.call(this, listeProprietaire, impProto, estElementFinal, couleur, nomMasque,
      decX, decY, masque, nom, tailleNom, motif, marquePourTrace)
    this.pointLibre = pointLibre
  }
}
CPointAncetrePointsMobiles.prototype = new CPt()
CPointAncetrePointsMobiles.prototype.constructor = CPointAncetrePointsMobiles
CPointAncetrePointsMobiles.prototype.superClass = 'CPt'
CPointAncetrePointsMobiles.prototype.className = 'CPointAncetrePointsMobiles'
/**
 * Renvoie this.pointLibre qui est true si le point n'est pas punaisé.
 * @returns {boolean}
 */
CPointAncetrePointsMobiles.prototype.estLibre = function () { return this.pointLibre }
/** @inheritedDoc */
CPointAncetrePointsMobiles.prototype.read = function (inps, list) {
  CPt.prototype.read.call(this, inps, list)
  this.pointLibre = inps.readBoolean()
}
/** @inheritedDoc */
CPointAncetrePointsMobiles.prototype.write = function (oups, list) {
  CPt.prototype.write.call(this, oups, list)
  oups.writeBoolean(this.pointLibre)
}
