/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const CCercle = require('../objets/CCercle')
const NatObj = require('../types/NatObj')

module.exports = CCercle

CCercle.prototype.distancePoint = function (xp, yp, masquage) {
  if (!(this.existe) || (masquage && this.masque)) return -1
  else return this.distancePointPourSurface(xp, yp)
}

CCercle.prototype.distancePointPourSurface = function (xp, yp) {
  if (!this.existe) return -1
  else {
    var distance = kernel.distancePointPoint(this.centreX, this.centreY, xp, yp)
    return Math.abs(distance - this.rayon)
  }
}

/**
 * Fonction servant à savoir si un point est par définition sur un cercle.
 * @param po Le point testé
 * @returns {boolean}
 */
CCercle.prototype.contientParDefinition = function () {
  return false
}

/**
 * Fonction utilisée pour générer un nom pour le protocole de la figure
 * @returns : Le nom généré
 */
CCercle.prototype.genereNom = function () {
  CCercle.ind++
  return kernel.getStr('rcerc') + CCercle.ind
}

CCercle.prototype.coincideAvec = function (p) {
  if (p.getNature() !== NatObj.NCercle) return false
  return kernel.zero(this.centreX - p.centreX) && kernel.zero(this.centreY - p.centreY) &&
    kernel.zero(this.rayon - p.rayon)
}

CCercle.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  if (this.horsFenetre) return ''
  var list = this.listeProprietaire
  var x1 = list.tikzXcoord(this.centreX, dimf, bu)
  var y1 = list.tikzYcoord(this.centreY, dimf, bu)
  var ray = list.tikzLongueur(this.rayon, dimf, bu)
  return '\\draw ' + this.tikzLineStyle(dimf, coefmult) + '(' + kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ') circle ' +
    '(' + kernel.chaineNombre(ray, 3) + ');'
}

/**
 * Fonction servant uniquement pour les exercices de constructions
 * Pour que this soit considéré comme isométrique à un cercle, this et p doivent être de classe CCercleOA, CCercleOR ou CCercleOAB,
 * leurs centres doivent avoir le même nom et leur rayons presque égaux
 * @param p
 * @returns {boolean}
 */
CCercle.prototype.isIsomTo = function (p) {
  if (!this.estCercleParCentre() || !p.estCercleParCentre()) return false
  return (this.o.nom === p.o.nom) && kernel.zero(p.rayon - this.rayon)
}
