/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const kernel = require('../kernel/kernel')
const CDroiteAncetre = require('../objets/CDroiteAncetre')
const CTransformation = require('../objets/CTransformation')
const NatObj = require('../types/NatObj')
const Vect = require('../types/Vect')

module.exports = CDroiteAncetre

/**
 * Fonction utilisée par le protocole de la figure et renvoyant soit le nom de la droite (donnée lors de la boîte
 * de dialogue de protocole soit la chaîne de caractères décriavnt la doite ((OA) par exemple).
 * @returns string
 */
CDroiteAncetre.prototype.getNom = function () {
  return this.nom
}

CDroiteAncetre.prototype.distancePointPourDemiPlan = function (xp, yp) {
  if (!this.existe) return -1
  else {
    kernel.projetteOrtho(xp, yp, this.point_x, this.point_y, this.vect, this.pointr)
    return kernel.distancePointPoint(xp, yp, this.pointr.x, this.pointr.y)
  }
}

CDroiteAncetre.prototype.distancePoint = function (xp, yp, masquage) {
  var dis
  if (!(this.existe) || (masquage && this.masque)) return -1
  else {
    kernel.projetteOrtho(xp, yp, this.point_x, this.point_y, this.vect, this.pointr)
    dis = kernel.distancePointPoint(xp, yp, this.pointr.x, this.pointr.y)
    return dis
  }
}

/**
 * Fonction  servant à savoir si un point
 * est par définition sur une droite. Renverra true par exemple
 * si la droite est définie par deux points et le point est un de ces deux points
 * ou si le point est lié à la droite
 **/
CDroiteAncetre.prototype.contientParDefinition = function () {
  return false
}

/**
 * Fonction renvoyant true si this est une droite perpendicuaire à la droite dte par définition
 * L'argument doit pointer sur une droite
 * A redéfinir pour les descendants
 * @returns {boolean}
 */
CDroiteAncetre.prototype.estParalleleParDefinition = function () {
  return (this.dte === this)
}

/**
 * Fonction renvoyant true si this est une droite perpendicuaire à la droite dte par définition
 * L'argument doit pointer sur une droite
 * A redéfinir pour les descendants
 * @returns {boolean}
 */
CDroiteAncetre.prototype.estPerpendiculaireParDefinition = function () {
  return false
}

/**
 * Fonction renvoyant true si pt1 et pt2 forment une droite parallèle à this.
 * Cela arrive quand les deux points sont des points images par une même translation, homothétie
 * ou symétrie centrale et quand leurs antécédents sont sur une droite parallèle à this
 * ou quand pt2 est l'image de pt1 par une translation de vecteur dont l'origine et l'extrémité
 * sont tous les deux sur une droite parallèel à D ou vice-versa
 * @param pt1
 * @param pt2
 * @returns {boolean}
 */
CDroiteAncetre.prototype.formentDroiteParallele = function (pt1, pt2) {
  var trans, vect
  if (pt1.estPointImage() && pt2.estPointImage()) {
    if ((pt1.transformation === pt2.transformation) &&
      ((pt1.transformation.natureTransformation() & CTransformation.tteTransImDtePar) !== 0)) { return this.listeProprietaire.sontParDefinitionSurParallele(pt1.antecedent, pt2.antecedent, this) } else return false
  } else {
    // On regarde d'abord si pt2 est l'image d'un point de this par une translation  dont l'origine est sur this
    // et l'extrémité est pt1 ou si pt2 est l'image de pt1 par une translation dont le vecteur a son origine et son extrémité
    // qui sont sur this
    if (pt2.estPointImage()) {
      if (pt2.transformation.natureTransformation() === CTransformation.translation) {
        trans = pt2.transformation
        return ((trans.ex === pt1) && (trans.or.appartientDroiteParDefinition(this) && pt2.antecedent.appartientDroiteParDefinition(this))) ||
          ((pt2.antecedent === pt1) && this.listeProprietaire.sontParDefinitionSurParallele(trans.or, trans.ex, this))
      } else {
        if (pt2.transformation.natureTransformation() === CTransformation.translationParVect) {
          vect = pt2.transformation.vect
          return ((vect.point2 === pt1) && (vect.point1.appartientDroiteParDefinition(this) && pt2.antecedent.appartientDroiteParDefinition(this))) ||
            ((pt2.antecedent === pt1) && this.listeProprietaire.sontParDefinitionSurParallele(vect.point1, vect.point2, this))
        } else return false
      }
    }
    // On regarde ensuite si pt1 est l'image d'un point de this par une translation dont l'origine est sur this
    // et l'extrémité est pt2 ou si pt1 est l'image de pt2 par une translation dont le vecteur a son origine et son extrémité
    // qui sont sur this
    if (pt1.estPointImage()) {
      if (pt1.transformation.natureTransformation() === CTransformation.translation) {
        trans = pt1.transformation
        return ((trans.ex === pt2) && (trans.or.appartientDroiteParDefinition(this) && pt1.antecedent.appartientDroiteParDefinition(this))) ||
          ((pt1.antecedent === pt2) && this.listeProprietaire.sontParDefinitionSurParallele(trans.or, trans.ex, this))
      } else {
        if (pt2.transformation.natureTransformation() === CTransformation.translationParVect) {
          vect = pt2.transformation.vect
          return ((vect.point2 === pt2) && (vect.point1.appartientDroiteParDefinition(this) && pt1.antecedent.appartientDroiteParDefinition(this))) ||
            ((pt1.antecedent === pt2) && this.listeProprietaire.sontParDefinitionSurParallele(vect.point1, vect.point2, this))
        } else return false
      }
    } else return false
  }
}

/**
 * Fonction utilisée pour générer un nom pour le protocole de la figure
 * @returns : Le nom généré
 */
CDroiteAncetre.prototype.genereNom = function (ch) {
  if (arguments.length === 0) return this.listeProprietaire.genereNomPourPointOuDroite('d', true)
  CDroiteAncetre.ind++
  return ch + CDroiteAncetre.ind
}

CDroiteAncetre.prototype.coincideAvec = function (p) {
  if (p.getNature() !== NatObj.NDroite) return false
  var vec = new Vect(p.point_x, p.point_y, this.point_x, this.point_y)
  return (kernel.colineaires(this.vect, p.vect) && kernel.colineaires(vec, this.vect))
}

CDroiteAncetre.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  if (this.horsFenetre) return '' // Par exemple pour un segment aux extrémités confondues
  var list = this.listeProprietaire
  var x1 = list.tikzXcoord(this.xext1, dimf, bu)
  var y1 = list.tikzYcoord(this.yext1, dimf, bu)
  var x2 = list.tikzXcoord(this.xext2, dimf, bu)
  var y2 = list.tikzYcoord(this.yext2, dimf, bu)
  var ch = '\\draw ' + this.tikzLineStyle(dimf, coefmult) + '(' + kernel.chaineNombre(x1, 3) + ',' + kernel.chaineNombre(y1, 3) + ')--' +
    '(' + kernel.chaineNombre(x2, 3) + ',' + kernel.chaineNombre(y2, 3) + ');'
  if (nomaff) {
    if (this.nomMasque || this.nom === '') return ch
    ch = ch + '\n'
    var style = 'below right,' + this.tikzCouleur() + ','
    x1 = list.tikzXcoord(this.xNom + this.decX - 7, dimf, bu)
    y1 = list.tikzYcoord(this.yNom + this.decY + 4, dimf, bu)
    ch += '\\node at ' + '(' + kernel.chaineNombre(x1, 3) + ', ' + kernel.chaineNombre(y1, 3) +
      ') [align=left,inner sep = 0pt, outer sep = 0pt,' + style +
      'font= \\sf ' + this.tikzFont(dimf, this.tailleNom, coefmult, bu) + '] {' + this.tikzNom() + '};'
  }
  return ch
}
