/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const CElementLigne = require('../objets/CElementLigne')
const CElementGraphique = require('../objets/CElementGraphique')
const StyleTrait = require('../types/StyleTrait')
const kernel = require('../kernel/kernel')
module.exports = CElementLigne

CElementLigne.prototype.tikzLineStyle = function (dimf, coefmult) {
  var ep // Epaisseur
  var style = this.style
  var tikzCoul = this.tikzCouleur()
  // Si on réduit la figure on réduit l'épaisseur de trait mais si on l'agrandit on garde la même
  var coef = (coefmult > 1) ? 1 : coefmult
  switch (style.style) {
    case StyleTrait.styleTraitContinu:
      ep = 0.4 * coef * style.strokeWidth
      return '[color=' + tikzCoul + ' , ' + 'line width = ' + kernel.chaineNombre(ep, 2) + ']'
    case StyleTrait.styleTraitPointille:
      ep = 0.4 * coef * style.strokeWidth
      return '[color=' + tikzCoul + ' , dotted, line width = ' + kernel.chaineNombre(ep, 2) + ']'
    case StyleTrait.styleTraitTrait:
      ep = 0.4 * coef * style.strokeWidth
      return '[color=' + tikzCoul + ' , dashed, line width = ' + kernel.chaineNombre(ep, 2) + ']'
    case StyleTrait.styleTraitPointPoint:
      ep = 0.4 * coef * style.strokeWidth
      return '[color=' + tikzCoul + ' , ' +
        'dash pattern=on 3pt off 1pt on 1pt off 1pt on 1pt off 1pt, line width = ' + kernel.chaineNombre(ep, 2) + ']'
    case StyleTrait.styleTraitPointPointPoint:
      ep = 0.4 * coef * style.strokeWidth
      return '[color=' + tikzCoul + ' , ' +
        'dash pattern=on 3pt off 1pt on 1pt off 1pt on 1pt off 1pt on 1pt off 1pt, line width = ' + kernel.chaineNombre(ep, 2) + ']'
    case StyleTrait.styleTraitTraitLongTrait:
      ep = 0.4 * coef * style.strokeWidth
      return '[color=' + tikzCoul + ' , ' +
        'dash pattern=on 3pt off 0.9pt on 1.5pt off 0.9pt, line width = ' + kernel.chaineNombre(ep, 2) + ']'
    default:
      return ''
  }
}

CElementLigne.prototype.tikzLineStyleContinu = function (dimf, coefmult) {
  var ep // Epaisseur
  // Si on réduit la figure on réduit l'épaisseur de trait mais si on l'agrandit on garde la même
  var coef = (coefmult > 1) ? 1 : coefmult
  ep = 0.4 * coef * this.style.strokeWidth
  return '[color=' + this.tikzCouleur() + ' , ' + 'line width = ' + kernel.chaineNombre(ep, 2) + ']'
}

CElementLigne.prototype.tikzdrawFillStyle = function (dimf, coefmult) {
  var ep // Epaisseur
  var tikzCoul = this.tikzCouleur()
  var style = this.style
  switch (this.style.style) {
    case StyleTrait.styleTraitContinu:
      ep = 0.4 * coefmult * style.strokeWidth
      return '[draw=' + tikzCoul + ' , ' + 'fill=' + tikzCoul + ',line width = ' + kernel.chaineNombre(ep, 3) + ']'
    case StyleTrait.styleTraitPointille:
      ep = 0.4 * coefmult * style.strokeWidth
      return '[draw=' + tikzCoul + ' , ' + 'fill=' + tikzCoul + ', dotted, ' + 'line width = ' + kernel.chaineNombre(ep, 3) + ']'
    case StyleTrait.styleTraitTrait:
      ep = 0.4 * coefmult * style.strokeWidth
      return '[draw=' + tikzCoul + ' , ' + 'fill=' + tikzCoul + ', dashed, ' + 'line width = ' + kernel.chaineNombre(ep, 3) + ']'
    case StyleTrait.styleTraitPointPoint:
      ep = 0.4 * coefmult * style.strokeWidth
      return '[draw=' + tikzCoul + ' , ' + 'fill=' + tikzCoul + ', dash pattern=on 3pt off 1pt on 1pt off 1pt on 1pt off 1pt, ' +
        'line width = ' + kernel.chaineNombre(ep, 3) + ']'
    case StyleTrait.styleTraitTraitLongTrait:
      ep = 0.4 * coefmult * style.strokeWidth
      return '[draw=' + tikzCoul + ' , ' + 'fill=' + tikzCoul + ', dash pattern=on 9pt off 3pt on 5pt off 3pt, ' +
        'line width = ' + kernel.chaineNombre(ep, 3) + ']'
    default:
      return ''
  }
}

CElementLigne.prototype.adaptRes = function (coef) {
  CElementGraphique.prototype.adaptRes.call(this, coef)
  this.style.strokeWidth *= coef
  this.style.stroke = this.style.getStrokeByCoef(coef)
}
