/*
 * Created by yvesb on 16/12/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CLatex = require('../objets/CLatex')
const kernel = require('../kernel/kernel')
const LatexDlg = require('../dialogs/LatexDlg')
const ModifAffFinalDlg = require('../dialogs/ModifAffFinalDlg')
const CAffLiePt = require('../objets/CAffLiePt')
const StyleEncadrement = require('../types/StyleEncadrement')
const TikzColor = require('../types/TikzColor')

module.exports = CLatex

CLatex.prototype.info = function () {
  return this.nom + ' : ' + kernel.getStr('chinfo167')
}

CLatex.prototype.infoHist = function () {
  var ch = this.info()
  if (this.pointLie !== null) ch += ' ' + kernel.getStr('liea') + ' ' + this.pointLie.nom
  return ch + this.infoAngle()
}

CLatex.prototype.modifDlg = function (app, callBack1, callBack2) {
  this.estElementFinal ? new ModifAffFinalDlg(app, this, callBack1, callBack2) : new LatexDlg(app, this, true, callBack1, callBack2)
}

CLatex.prototype.setReady4MathJaxEvenMasked = function () {
  // Modifié version 5.0.4 pour tenir compte des objts dupliqués d'un CLaTeX
  // qui est masqué
  /*
  this.div = kernel.ce("div",{
    // style : this.styleLatex,
    style : this.getStyleLatex(),
    id : this.id+"prov",
    class : "text2jax_ignore"
  });
  var ch  = "$$"+ this.prelatex() + this.chaineLatex+"$$";
  this.div.appendChild(document.createTextNode(ch));
  document.body.appendChild(this.div);
  MathJax.Hub.Queue(["Typeset",MathJax.Hub,this.div]);
  */
  // Modifié version 4.6.0
  this.typeset()
}

/**
 * Fonction utilisée pour générer un nom pour le protocole de la figure
 * @returns : Le nom généré
 */
// Modifié par rapport à la version Java
CLatex.prototype.genereNom = function () {
  CLatex.ind++
  return kernel.getStr('rlat') + CLatex.ind
}

CLatex.prototype.setCloneTikz = function (ptel) {
  this.setClone(ptel)
  this.chaineLatex = ptel.chaineLatex // Ajout version 4.8.1 pour exportation Tikz
}

/*
CLatex.prototype.tikz = function(dimf, nomaff, coefmult, bu) {
  var list = this.listeProprietaire;
  var ang = String(this.angText.rendValeurRadian()*kernel.ConvRadDeg);
  var style = "";
  switch (this.alignementVertical) {
    case CAffLiePt.alignVerLow :
      style = "above ";
      break;
    case CAffLiePt.alignVerTop :
      style = "below ";
  }
  switch (this.alignementHorizontal) {
    case CAffLiePt.alignHorLeft :
      style += "right";
      break;
    case CAffLiePt.alignHorRight :
      style += "left,";
  }
  if (style !== "") style += ",";
  var tikzCoul = this.tikzCouleur();
  style += tikzCoul + ",";
  if (this.encadrement !== StyleEncadrement.Sans) style +=  "draw="+ tikzCoul + ",";
  if (this.effacementFond) style += "fill=" + TikzColor.colorString(this.couleurFond) +",";
  var dinsepx = list.tikzLongueur(4, dimf, bu)*coefmult;
  var dinsepy = list.tikzLongueur(4, dimf, bu)*coefmult;
  var insepx = kernel.chaineNombre(dinsepx,2);
  var insepy = kernel.chaineNombre(dinsepy,2);
  style += "inner xsep ="  + insepx + "cm,inner ysep =" + insepy + "cm";
  var x = list.tikzXcoord(this.xNom + this.decX - 7, dimf, bu);
  var y = list.tikzYcoord(this.yNom + this.decY -2, dimf, bu);
  return  "\\node at " + "(" + kernel.chaineNombre(x, 3) +", " + kernel.chaineNombre(y, 3) +
    ") [align=left," +"rotate=" + ang+ "," + style + ",font= \\sf " + this.tikzFont(dimf, this.taillePolice, coefmult, bu) + "] {$" + this.chaineLatex + "$};";
}
*/

CLatex.prototype.tikz = function (dimf, nomaff, coefmult, bu) {
  var list = this.listeProprietaire
  var ang = String(this.angText.rendValeurRadian() * kernel.ConvRadDeg)
  var x0 = this.xNom + this.decX
  var y0 = this.yNom + this.decY
  var style = ''
  switch (this.alignementVertical) {
    case CAffLiePt.alignVerLow :
      style = 'above '
      y0 += 2
      break
    case CAffLiePt.alignVerTop :
      style = 'below '
      y0 -= 2
  }
  switch (this.alignementHorizontal) {
    case CAffLiePt.alignHorLeft :
      style += 'right '
      x0 -= 2
      break
    case CAffLiePt.alignHorRight :
      x0 += 2
      style += 'left '
  }
  var x = list.tikzXcoord(x0, dimf, bu)
  var y = list.tikzYcoord(y0, dimf, bu)

  style += ',' + this.tikzCouleur() + ','
  if (this.encadrement !== StyleEncadrement.Sans) {
    style += 'draw=' + this.tikzCouleur() + ','
    var dinsepx = list.tikzLongueur(2, dimf, bu)
    var dinsepy = list.tikzLongueur(4, dimf, bu)
    var insepx = kernel.chaineNombre(dinsepx, 2)
    var insepy = kernel.chaineNombre(dinsepy, 2)
    style += 'inner xsep =' + insepx + 'cm,inner ysep =' + insepy + 'cm,'
  }
  if (this.effacementFond) style += 'fill=' + TikzColor.colorString(this.couleurFond) + ','
  var ch = '$' + this.chaineLatex + '$'
  if (ang === '0') {
    return '\\node at ' + '(' + kernel.chaineNombre(x, 3) + ', ' + kernel.chaineNombre(y, 3) +
      ') [' + style + ',' + style + 'font= \\sf ' + this.tikzFont(dimf, this.taillePolice, coefmult, bu) + '] {' + ch + '};'
  } else {
    return '\\node at ' + '(' + kernel.chaineNombre(x, 3) + ', ' + kernel.chaineNombre(y, 3) + ') [rotate=' + ang + ',' + style + 'font= \\sf ' +
      this.tikzFont(dimf, this.taillePolice, coefmult, bu) + '] {' + ch + '};'
  }
}
