/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CLieuDePoints = require('../objets/CLieuDePoints')
const CLieuDeBase = require('../objets/CLieuDeBase')
const kernel = require('../kernel/kernel')
module.exports = CLieuDePoints

CLieuDePoints.prototype.infoHist = function () {
  return this.nom + ' : ' + kernel.getStr('LieuPt') + ' ' + kernel.getStr('de') + ' ' +
    this.pointATracer.nom + ' ' + kernel.getStr('chinfo207') + ' ' + this.pointLieGenerateur.nom
}

CLieuDePoints.prototype.estGenereParPointLie = function (pointlie) {
  return this.pointLieGenerateur === pointlie
}

CLieuDePoints.prototype.positionneTikz = function (infoRandom, dimfen) {
  var abscisseMini, abscisseMaxi, abscisse, abscisseInitiale, pas, abscisseFinale, nbValeurs
  var plg = this.pointLieGenerateur
  var pointATracer = this.pointATracer
  var infoLieu = this.infoLieu
  if (!plg.existe) {
    this.existe = false
    return
  }
  CLieuDeBase.prototype.positionneTikz.call(this, infoRandom, dimfen)
  abscisseMini = plg.abscisseMinimale()
  abscisseMaxi = plg.abscisseMaximale()
  if (abscisseMini === abscisseMaxi) {
    this.existe = false
    return
  } else {
    // On ordonne l'abscisse minimale et maximale
    if (abscisseMini > abscisseMaxi) {
      var w = abscisseMini
      abscisseMini = abscisseMaxi
      abscisseMaxi = w
    }
  }
  abscisseInitiale = plg.abscisse
  // Modification pour la version 1.9.5
  if (plg.lieALigneFermee()) {
    nbValeurs = infoLieu.nombreDePoints
    pas = (abscisseMaxi - abscisseMini) / nbValeurs
    abscisseFinale = abscisseMaxi - pas
  } else {
    nbValeurs = infoLieu.nombreDePoints - 1
    pas = (abscisseMaxi - abscisseMini) / nbValeurs
    abscisseFinale = abscisseMaxi
  }

  abscisse = abscisseMini

  var ligneEnCours = false
  var indiceLigneEnCours = -1

  var nb = infoLieu.nombreDePoints - 1
  for (var i = 0; i < infoLieu.nombreDePoints; i++) {
    abscisse = (i === 0) ? abscisseMini : ((i < nb) ? abscisse + pas : abscisseFinale)
    plg.donneAbscisse(abscisse)
    this.listeElementsAncetres.positionne(infoRandom, dimfen)
    if (!pointATracer.existe) {
      this.xCoord[i] = kernel.MAX_VALUE
      this.yCoord[i] = kernel.MAX_VALUE
      if (ligneEnCours) {
        this.infoLignes[indiceLigneEnCours].nombrePoints = i - this.infoLignes[indiceLigneEnCours].indicePremierPoint
        ligneEnCours = false
      }
    } else {
      if (!(kernel.testToile(pointATracer.x, pointATracer.y))) {
        if (ligneEnCours) {
          this.infoLignes[indiceLigneEnCours].nombrePoints = i - this.infoLignes[indiceLigneEnCours].indicePremierPoint
          ligneEnCours = false
        }
      } else {
        if (ligneEnCours) {
          if (infoLieu.gestionDiscontinuite) {
            if ((Math.abs(pointATracer.x - this.xCoord[i - 1]) > dimfen.x) ||
              (Math.abs(pointATracer.y - this.yCoord[i - 1]) > dimfen.y)) {
              this.infoLignes[indiceLigneEnCours].nombrePoints = i - this.infoLignes[indiceLigneEnCours].indicePremierPoint
              ligneEnCours = false
            } else {
              this.xCoord[i] = pointATracer.x
              this.yCoord[i] = pointATracer.y
              this.infoLignes[indiceLigneEnCours].nombrePoints++
            }
          } else {
            this.xCoord[i] = pointATracer.x
            this.yCoord[i] = pointATracer.y
            this.infoLignes[indiceLigneEnCours].nombrePoints++
          }
        } else {
          this.xCoord[i] = pointATracer.x
          this.yCoord[i] = pointATracer.y
          indiceLigneEnCours++
          ligneEnCours = true
          this.infoLignes[indiceLigneEnCours].indicePremierPoint = i
          this.infoLignes[indiceLigneEnCours].nombrePoints = 1
        }
      }
    }
  }

  // On  remet le point ancêtre à sa position initiale
  plg.donneAbscisse(abscisseInitiale)
  this.listeElementsAncetres.positionneTikz(infoRandom, dimfen)
  this.nombreLignes = indiceLigneEnCours + 1
  // Si le lieu n'est formé d'aucune ligne ou s'il est formé
  // de lignes avec un seul point, il n'existe pas
  if (this.nombreLignes === 0) {
    this.existe = false
    return
  }
  var auMoinsUneLigne = false
  this.indicePremiereLigne = -1
  this.nombreLignesDePlusieursPoints = 0
  for (var j = 0; j <= indiceLigneEnCours; j++) {
    if (this.infoLignes[j].nombrePoints >= 2) {
      this.nombreLignesDePlusieursPoints++
      if (!auMoinsUneLigne) this.indicePremiereLigne = j
      auMoinsUneLigne = true
    }
  }
  if (!auMoinsUneLigne) {
    this.existe = false
    return
  }
  this.existe = true
}

CLieuDePoints.prototype.estDefiniParObjDs = function (listeOb) {
  return CLieuDeBase.prototype.estDefiniParObjDs.call(this, listeOb) &&
  this.pointLieGenerateur.estDefPar(listeOb)
}
