/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CLieuParVariable = require('../objets/CLieuParVariable')
const CLieuDeBase = require('../objets/CLieuDeBase')
const kernel = require('../kernel/kernel')
module.exports = CLieuParVariable

CLieuParVariable.prototype.infoHist = function () {
  return this.nom + ' : ' + kernel.getStr('LieuPt') + ' ' + kernel.getStr('de') + ' ' +
    this.pointATracer.nom + ' ' + kernel.getStr('chinfo207') + ' ' + this.variableGeneratrice.nomCalcul
}

CLieuParVariable.prototype.positionneTikz = function (infoRandom, dimfen) {
  var valeur, valeurMinimale, valeurMaximale, valeurInitiale, pas
  var vg = this.variableGeneratrice
  if (!vg.existe) {
    this.existe = false
    return
  }
  var infoLieu = this.infoLieu
  var pointATracer = this.pointATracer
  CLieuDeBase.prototype.positionneTikz.call(this, infoRandom, dimfen)
  valeurMinimale = vg.valeurMini
  valeurMaximale = vg.valeurMaxi
  valeurInitiale = vg.valeurActuelle
  pas = (valeurMaximale - valeurMinimale) / (infoLieu.nombreDePoints - 1)
  valeur = valeurMinimale - pas
  var ligneEnCours = false
  var indiceLigneEnCours = -1
  var nb = infoLieu.nombreDePoints - 1
  for (var i = 0; i < infoLieu.nombreDePoints; i++) {
    valeur = (i === 0) ? valeurMinimale : ((i === nb) ? valeurMaximale : valeur + pas)
    vg.donneValeur(valeur)
    this.listeElementsAncetres.positionne(infoRandom, dimfen)
    if (!pointATracer.existe) {
      if (ligneEnCours) {
        this.infoLignes[indiceLigneEnCours].nombrePoints =
          i - this.infoLignes[indiceLigneEnCours].indicePremierPoint
        ligneEnCours = false
      }
    } else {
      if (!(kernel.testToile(pointATracer.x, pointATracer.y))) {
        if (ligneEnCours) {
          this.infoLignes[indiceLigneEnCours].nombrePoints = i - this.infoLignes[indiceLigneEnCours].indicePremierPoint
          ligneEnCours = false
        }
      } else {
        if (ligneEnCours) {
          if (infoLieu.gestionDiscontinuite) {
            if ((Math.abs(pointATracer.x - this.xCoord[i - 1]) > dimfen.x) ||
              (Math.abs(pointATracer.y - this.yCoord[i - 1]) > dimfen.y)) {
              this.infoLignes[indiceLigneEnCours].nombrePoints = i - this.infoLignes[indiceLigneEnCours].indicePremierPoint
              ligneEnCours = false
            } else {
              this.xCoord[i] = pointATracer.x
              this.yCoord[i] = pointATracer.y
              this.infoLignes[indiceLigneEnCours].nombrePoints++
            }
          } else {
            this.xCoord[i] = pointATracer.x
            this.yCoord[i] = pointATracer.y
            this.infoLignes[indiceLigneEnCours].nombrePoints++
          }
        } else {
          this.xCoord[i] = pointATracer.x
          this.yCoord[i] = pointATracer.y
          indiceLigneEnCours++
          ligneEnCours = true
          this.infoLignes[indiceLigneEnCours].indicePremierPoint = i
          this.infoLignes[indiceLigneEnCours].nombrePoints = 1
        }
      }
    }
  }
  // On redonne à la variable génératrice sa valeur initiale
  vg.donneValeur(valeurInitiale)
  this.listeElementsAncetres.positionne(infoRandom, dimfen)
  this.nombreLignes = indiceLigneEnCours + 1
  // Si le lieu n'est formé d'aucune ligne ou s'il est formé
  // de lignes avec un seul point, il n'existe pas
  if (this.nombreLignes === 0) {
    this.existe = false
    return
  }
  var auMoinsUneLigne = false
  this.indicePremiereLigne = -1
  this.nombreLignesDePlusieursPoints = 0
  for (var j = 0; j <= indiceLigneEnCours; j++) {
    var ligneTrouvee = this.infoLignes[j].nombrePoints >= 2
    if (ligneTrouvee) {
      this.nombreLignesDePlusieursPoints++
      if (!auMoinsUneLigne) this.indicePremiereLigne = j
      auMoinsUneLigne = true
    }
  }
  this.existe = auMoinsUneLigne
}

CLieuParVariable.prototype.estDefiniParObjDs = function (listeOb) {
  return CLieuDeBase.prototype.estDefiniParObjDs.call(this, listeOb) &&
  this.variableGeneratrice.estDefPar(listeOb)
}
