/*
 * Created by yvesb on 10/10/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const CPointImage = require('../objets/CPointImage')
const CPt = require('../objets/CPt')
const CTransformation = require('../objets/CTransformation')
const kernel = require('../kernel/kernel')
module.exports = CPointImage

CPointImage.prototype.infoHist = function () {
  return this.nom + ' : ' + kernel.getStr('chinfo18') + ' ' + this.antecedent.nom + ' ' + kernel.getStr('par') +
    ' ' + this.transformation.complementInfo()
}

CPointImage.prototype.depDe4Rec = function (p) {
  if (this.elementTestePourDependDePourRec === p) return this.dependDeElementTestePourRec
  return this.memDep4Rec(CPt.prototype.depDe4Rec.call(this, p) ||
    this.antecedent.depDe4Rec(p) || this.transformation.depDe4Rec(p))
}

CPointImage.prototype.antecedentDirect = function () {
  return this.transformation
}

CPointImage.prototype.modifiableParMenu = function () {
  return !this.estElementFinal && this.transformation.imageModifiableParMenu()
}

CPointImage.prototype.appartientDroiteParDefinition = function (droite) {
  if (CPt.prototype.appartientDroiteParDefinition.call(this, droite)) return true
  var antecedent = this.antecedent
  var natTrans = this.transformation.natureTransformation()
  // Si la transormation transformme une droite en une droite paralèle, on regarde si l'antécédent appartient à une droite parallèle à droite
  if (((natTrans & CTransformation.tteTransImDtePar) !== 0) && (this.listeProprietaire.appDteParParDef(antecedent, droite))) return true
  switch (natTrans) {
    case CTransformation.homothetie :
    case CTransformation.symetrieCentrale :
      return this.transformation.centre.appartientDroiteParDefinition(droite) && antecedent.appartientDroiteParDefinition(droite)
    case CTransformation.translation :
      var trans = this.transformation
      return (this.antecedent.appartientDroiteParDefinition(droite) && this.listeProprietaire.sontParDefinitionSurParallele(trans.or, trans.ex, droite)) ||
        (trans.ex.appartientDroiteParDefinition(droite) && this.listeProprietaire.sontParDefinitionSurParallele(antecedent, trans.or, droite))
    case CTransformation.translationParVect :
      var vect = this.transformation.vect
      return (this.antecedent.appartientDroiteParDefinition(droite) && this.listeProprietaire.sontParDefinitionSurParallele(vect.point1, vect.point2, droite)) ||
        (vect.point2.appartientDroiteParDefinition(droite) && this.listeProprietaire.sontParDefinitionSurParallele(antecedent, vect.point1, droite))

    default : return false
  }
}

CPointImage.prototype.appartientCercleParDefinition = function (cercle) {
  if (CPt.prototype.appartientCercleParDefinition.call(this, cercle)) return true
  var antecedent = this.antecedent
  var natTrans = this.transformation.natureTransformation()
  var className = cercle.className
  if ((natTrans === CTransformation.rotation) && (className === 'CCercleOA' || className === 'CCercleOR' || className === 'CCercleOAB')) {
    return (this.transformation.centre === cercle.o) && antecedent.appartientCercleParDefinition(cercle)
  } else return false
}

/**
 * Fonction qui renvoie true seulement pour les objets qui sont des objets images d'un autre par une transformation
 * @returns {boolean}
 */
CPointImage.prototype.estImageParTrans = function () {
  return true
}

CPointImage.prototype.estDefiniParObjDs = function (listeOb) {
  return this.antecedent.estDefPar(listeOb) &&
  this.transformation.estDefPar(listeOb)
}
