/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const kernel = require('../kernel/kernel')
const CImplementationProto = require('../objets/CImplementationProto')
const NatCal = require('../types/NatCal')
const NatObj = require('../types/NatObj')
const AffichageDlg = require('../dialogs/AffichageDlg')

module.exports = OutilAffichageCoordLie
/**
 * Outil servant à créer une image libre sur la figure
 * @param app L'application propriétaire
 * @constructor
 */
function OutilAffichageCoordLie (app) {
  Outil.call(this, app, 'AffichageCoordLie', 32021, true)
}
OutilAffichageCoordLie.prototype = new Outil()

OutilAffichageCoordLie.prototype.select = function () {
  Outil.prototype.select.call(this)
  this.point = null
  this.obj = null
  var app = this.app
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageActif.aDesigner = NatObj.NTtPoint
  app.outilPointageActif.reset()
  app.indication('indAffLie')
}

OutilAffichageCoordLie.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  var self = this
  if (this.point === null) {
    this.point = elg
    this.ajouteClignotementDe(this.point)
    this.resetClignotement()
    app.outilPointageActif.aDesigner = NatObj.NTtPoint
    app.outilPointageActif.reset()
    app.indication('indCoord')
  } else {
    this.obj = elg
    this.ajouteClignotementDe(elg)
    new AffichageDlg(app, 'AffichageCoordLie', 1, function (rep, nbDec, taille, styleEnc, effFond, coulFond, horAlign, verAlign) {
      self.suite(rep, nbDec, taille, styleEnc, effFond, coulFond, horAlign, verAlign)
    },
    function () {
      app.activeOutilCapt()
    })
  }
}

OutilAffichageCoordLie.prototype.suite = function (rep, nbDec, taillePolice, styleEnc, effFond, coulFond, horAlign, verAlign) {
  var app = this.app
  var list = app.listePr
  this.annuleClignotement()
  var proto = app.docConsAv.getPrototype('Coord')
  proto.get(0).elementAssocie = rep
  proto.get(1).elementAssocie = this.obj
  var impProto = new CImplementationProto(list, proto)
  impProto.implemente(app.dimf, proto)
  impProto.nomProto = kernel.getStr('AffichageEq')
  var com = impProto.premierFinal(NatObj.NCommentaire)
  com.pointLie = this.point
  com.xNom = this.point.x
  com.yNom = this.point.y
  com.donneCouleur(app.getCouleur())
  com.taillePolice = taillePolice
  com.encadrement = styleEnc
  com.effacementFond = effFond
  com.couleurFond = coulFond
  com.alignementHorizontal = horAlign
  com.alignementVertical = verAlign
  com.chaineCommentaire = com.chaineCommentaire.replace(/,2/g, ',' + nbDec)
  var indImpProto = list.indexOf(impProto)
  list.positionne(false, app.dimf)
  list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
  this.saveFig()
  // Corrigé version 5.6.4
  if (app.estExercice) app.listePourConst = app.listePourConstruction()

  app.activeOutilCapt()
}

OutilAffichageCoordLie.prototype.activationValide = function () {
  return this.app.listePr.nombreObjetsCalcul(NatCal.NRepere) > 0
}

OutilAffichageCoordLie.prototype.isReadyForTouchEnd = function () {
  return false
}
