/*
 * Created by yvesb on 13/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const kernel = require('../kernel/kernel')
const CValeurAffichee = require('../objets/CValeurAffichee')
const CCommentaire = require('../objets/CCommentaire')
const AffichageValeurDlg = require('../dialogs/AffichageValeurDlg')
const AvertDlg = require('../dialogs/AvertDlg')
const CAffLiePt = require('../objets/CAffLiePt')
const Color = require('../types/Color')
const StyleEncadrement = require('../types/StyleEncadrement')
module.exports = OutilAffichageValeur
/**
 * Outil servant à créer un affichage de valeur libre sur la figure
 * @param app L'application propriétaire
 * @constructor
 */
function OutilAffichageValeur (app) {
  Outil.call(this, app, 'AffichageValeur', 32815, true)
}
OutilAffichageValeur.prototype = new Outil()

OutilAffichageValeur.prototype.select = function () {
  var app = this.app
  Outil.prototype.select.call(this)
  app.outilPointageActif = this.app.outilPointageClic
  app.outilPointageActif.reset()
  app.indication('indEndroitAff')
}

OutilAffichageValeur.prototype.traiteObjetDesigne = function (elg, point) {
  var app = this.app
  var list = app.listePr
  this.comClig = new CCommentaire(list, null, false, Color.black, point.x, point.y, 0, 0, false, null, 16, StyleEncadrement.Sans, false,
    Color.white, CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop, kernel.getStr('AffIci'))
  this.comClig.positionne(false, app.dimf)
  this.comClig.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
  this.ajouteClignotementDe(this.comClig)
  this.resetClignotement()
  var self = this
  this.affval = new CValeurAffichee(list, null, false, app.getCouleur(), point.x, point.y, 0, 0, false, null, 16, StyleEncadrement.Sans,
    false, app.doc.couleurFond, CAffLiePt.alignHorLeft, CAffLiePt.alignVerTop, null, '', '', 2)
  new AffichageValeurDlg(app, this.affval, false, function () { self.callBackOK() }, function () { self.callBackCancel() })
}

OutilAffichageValeur.prototype.callBackOK = function () {
  const app = this.app
  app.ajouteElement(this.affval)
  if (app.verifieDernierElement(1)) {
    // L'élément créé existe bien et a été ajouté à la liste
    this.affval.setReady4MathJax() // Ajouté version 6.4.1
    const self = this
    MathJax.Hub.Queue(function () { // Ajouté version 6.4.1
      self.affval.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    })
    this.annuleClignotement()
    this.saveFig()
  } else new AvertDlg(app, 'DejaCree')
  app.activeOutilCapt()
}

OutilAffichageValeur.prototype.callBackCancel = function () {
  this.app.activeOutilCapt()
}
