/*
 * Created by yvesb on 14/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const CValeurAngle = require('../objets/CValeurAngle')
const CGrandArcDeCercle = require('../objets/CGrandArcDeCercle')
const OutilArcParRapporteur = require('../outils/OutilArcParRapporteur')

module.exports = OutilArcGrandParRapporteur

/**
 * Outil de création d'un grand arc de cercle par rapporteur
 * @param app Application propriétaire
 * @param {string} toolName Le nom de l'outil
 * @param {int} toolIndex L'index de l'outil tel que dans la version Java
 * @constructor
 */
function OutilArcGrandParRapporteur (app, toolName, toolIndex) {
  if (arguments.length === 1) OutilArcParRapporteur.call(this, app, 'ArcGrandParRapporteur', 33037, true)
  else OutilArcParRapporteur.call(this, app, toolName, toolIndex)
}

OutilArcGrandParRapporteur.prototype = new OutilArcParRapporteur()

OutilArcGrandParRapporteur.prototype.ajouteArcVisuel = function (valang) {
  var app = this.app
  app.ajouteObjetVisuel(new CGrandArcDeCercle(app.listeVisuelle, null, false, app.getCouleur(), false, app.getStyleTrait(),
    this.point1, this.point2, null, valang))
}

OutilArcGrandParRapporteur.prototype.creeObjet = function () {
  var app = this.app
  var liste = app.listePr
  var svg = app.svgFigure
  var valeurAngle = this.ptMesureAngle.rendValeur()
  var indiceImpInitial = liste.longueur()

  // On arrondit au degré près en degrés
  // Modifié version 5.1.1
  // if (liste.uniteAngle == UniteAngle.Radian) valeurAngle = valeurAngle*CValeurAngle.convRadDeg;
  if (liste.uniteAngle === kernel.uniteAngle.radian) valeurAngle = valeurAngle * kernel.ConvDegRad
  if (valeurAngle !== Math.floor(valeurAngle)) valeurAngle = Math.floor(valeurAngle + 0.5)
  if (valeurAngle === -180) valeurAngle = 180
  var valang = new CValeurAngle(liste, valeurAngle)
  var ptArc = new CGrandArcDeCercle(liste, null, false, app.getCouleur(), false, app.getStyleTrait(),
    this.point1, this.point2, null, valang)
  app.ajouteElement(ptArc)

  if (app.verifieDernierElement(1)) {
    this.saveFig()
    liste.afficheTout(indiceImpInitial, svg, true, app.doc.couleurFond, app.doc.opacity)
  }
  this.deselect()
  this.select()
}

/**
 * Fonction renvoyant true si l'outil accepte qu'on crée un point par défaut lors d'un clic sur un endroit vide
 */
OutilArcGrandParRapporteur.prototype.creationPointPossible = function () {
  return true
}

OutilArcGrandParRapporteur.prototype.isWorking = function () {
  return (this.point1 !== null) && (this.point2 !== null)
}
