/*
 * Created by yvesb on 14/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const StyleTrait = require('../types/StyleTrait')
const CValeurAngle = require('../objets/CValeurAngle')
const CValeur = require('../objets/CValeur')
const CArcDeCercle = require('../objets/CArcDeCercle')
const OutilRapporteur = require('../outils/OutilRapporteur')
const Color = require('../types/Color')
const CSegment = require('../objets/CSegment')
const CPointImage = require('../objets/CPointImage')
const MotifPoint = require('../types/MotifPoint')
const CCercleOA = require('../objets/CCercleOA')
const CDroiteDirectionFixe = require('../objets/CDroiteDirectionFixe')
const CPointLieDroite = require('../objets/CPointLieDroite')
const CPointLieCercle = require('../objets/CPointLieCercle')
const CMesureAngleOriente = require('../objets/CMesureAngleOriente')
const CMesureAngleGeometrique = require('../objets/CMesureAngleGeometrique')
const CHomothetie = require('../objets/CHomothetie')
const CValeurAffichee = require('../objets/CValeurAffichee')
const CAffLiePt = require('../objets/CAffLiePt')
const StyleEncadrement = require('../types/StyleEncadrement')
const CResultatValeur = require('../objets/CResultatValeur')

module.exports = OutilArcParRapporteur

/**
 * Outil de création d'un arc de cercle par rapporteur
 * @param app L'application propriétaire
 * @param toolName String : Le nom de l'outil
 * @param toolIndex Integer : L'index de l'outil tel que dans la version Java
 * @constructor
 */
function OutilArcParRapporteur (app, toolName, toolIndex) {
  if (arguments.length === 1) OutilRapporteur.call(this, app, 'ArcParRapporteur', 33036, true)
  else OutilRapporteur.call(this, app, toolName, toolIndex)
}

OutilArcParRapporteur.prototype = new OutilRapporteur()

OutilArcParRapporteur.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var ptp = null
  var listeVisuelle = this.app.listeObjetsVisuels
  var stfc = StyleTrait.stfc(listeVisuelle)
  var stfp = StyleTrait.stfp(listeVisuelle)
  var bl = Color.black
  if (this.point2 === null) {
    ptp = this.ajouteRapporteur(this.point1) // Le pointeur renvoyé dans ptp est
    // un pointeur sur un point d'intersection d'une demi-droite avec un cercle.
    this.ptCercle = new CCercleOA(listeVisuelle, null, false, bl, false, stfc, this.point1, ptp)
    app.ajouteObjetVisuel(this.ptCercle)
    app.afficheObjetVisuels(0) // Ajout version 6.4.4
  } else {
    const inddeb = listeVisuelle.longueur()
    // Si le point dont on veut créer l'image a été désigné, le cercle précédent
    // devient fixe en remplaçant dans la liste visuelle le point souris
    // par Point2
    listeVisuelle.remplacePoint(app.mousePoint, this.point2)
    // Pour pouvoir initialiser l'abscisse du point lié, il faut mesurer
    // un angle orienté à partir d'une horizontale
    var ptdh = new CDroiteDirectionFixe(listeVisuelle, null, false, bl, true, 0, 0, true, '', 16, stfc, 1, this.point1, true, 1)
    app.ajouteObjetVisuel(ptdh)
    // On lui lie un point d'abscisse 1
    var ptld = new CPointLieDroite(listeVisuelle, null, false, bl, true, 0, 0, true, '', 16, MotifPoint.petitRond, false, true, 1, ptdh)
    app.ajouteObjetVisuel(ptld)
    // On mesure l'angle
    var ptmao = new CMesureAngleOriente(listeVisuelle, null, false, ptld, this.point1, this.point2, '')
    app.ajouteObjetVisuel(ptmao)
    app.listeObjetsVisuels.positionne(false, app.dimf)
    // On rajoute un point lié au cercle pointé par ptCercle
    // Son abscisse est son angle polaire en radians
    var ang = ptmao.rendValeur() * kernel.ConvDegRad
    if (ang < 0) ang = ang + 2 * Math.PI
    var ptPointLie = new CPointLieCercle(listeVisuelle, null, false, bl, true, 0, 0, false, '', 16,
      MotifPoint.petitRond, false, true, ang, this.ptCercle)
    app.ajouteObjetVisuel(ptPointLie)
    // On crée la mesure de l'angle orienté point2, point1, pointSouris
    this.ptMesureAngle = new CMesureAngleOriente(listeVisuelle, null, false, this.point2, this.point1, ptPointLie, '')
    app.ajouteObjetVisuel(this.ptMesureAngle)
    // On crée aussi la mesure de l'angle géométrique pour affichage sur le rapporteur
    var ptmesag = new CMesureAngleGeometrique(listeVisuelle, null, false, this.point2, this.point1, ptPointLie)
    app.ajouteObjetVisuel(ptmesag)
    // On crée une nouveau point écarté du rapporteur
    // Pour cela on crée une homothétie
    var ptHom2 = new CHomothetie(listeVisuelle, null, false, this.point1, new CValeur(listeVisuelle, 1.25))
    app.ajouteObjetVisuel(ptHom2)
    var ptimh = new CPointImage(listeVisuelle, null, false, bl, true, 0, 0, true, '', 16, MotifPoint.petitRond, false,
      ptPointLie, ptHom2)
    app.ajouteObjetVisuel(ptimh)
    // On crée l'image provisoire du point
    // D'abord une rotation
    var valang = new CValeurAngle(listeVisuelle, new CResultatValeur(listeVisuelle, this.ptMesureAngle))
    var doc = app.doc
    // On affiche la valeur liée au point précédent avec effacement de ce qu'il y a dessous
    var ptaff = new CValeurAffichee(listeVisuelle, null, false, Color.blue, 0, 0, 0, 0, false, ptimh, 16,
      StyleEncadrement.Sans, false, doc.couleurFond, CAffLiePt.alignHorCent,
      CAffLiePt.alignVerCent, ptmesag, '', '°', 0)
    app.ajouteObjetVisuel(ptaff)
    // L'arc de cercle
    // var ptArc = new CArcDeCercle(frame.cadreActif().listePr, null, false,
    //  frame.couleurActive(), false, frame.styleTraitActif(), point1, point2, null, valang);
    this.ajouteArcVisuel(valang)
    // On crée un segment pointillés (on ne sait pas si on est à l'intérieur
    // ou l'extérieur du rapporteur
    var ptseg1 = new CSegment(listeVisuelle, null, false, bl, false, stfp, this.point1, ptPointLie)
    app.ajouteObjetVisuel(ptseg1)
    app.afficheObjetVisuels(inddeb) // Ajout version 6.4.4
    // On passe la main à l'outil de désignation ad hoc
    app.outilPointageActif = app.outilPointageRapporteur
    app.outilPointageActif.reset()
    app.outilPointageRapporteur.associePointLie(ptPointLie)
  }
}

OutilArcParRapporteur.prototype.ajouteArcVisuel = function (valang) {
  var app = this.app
  app.ajouteObjetVisuel(new CArcDeCercle(app.listeVisuelle, null, false, app.getCouleur(), false, app.getStyleTrait(),
    this.point1, this.point2, null, valang))
}

OutilArcParRapporteur.prototype.creeObjet = function () {
  var app = this.app
  var liste = app.listePr
  var svg = app.svgFigure
  var valeurAngle = this.ptMesureAngle.rendValeur()
  var indiceImpInitial = liste.longueur()

  // On arrondit au degré près en degrés
  // Modifié version 5.1.1
  // if (liste.uniteAngle == UniteAngle.Radian) valeurAngle = valeurAngle*CValeurAngle.convRadDeg;
  if (liste.uniteAngle === kernel.uniteAngle.radian) valeurAngle = valeurAngle * kernel.ConvDegRad
  if (valeurAngle !== Math.floor(valeurAngle)) valeurAngle = Math.floor(valeurAngle + 0.5)
  if (valeurAngle === -180) valeurAngle = 180
  var valang = new CValeurAngle(liste, valeurAngle)
  var ptArc = new CArcDeCercle(liste, null, false, app.getCouleur(), false, app.getStyleTrait(),
    this.point1, this.point2, null, valang)
  app.ajouteElement(ptArc)

  if (app.verifieDernierElement(1)) {
    this.saveFig()
    liste.afficheTout(indiceImpInitial, svg, true, app.doc.couleurFond, app.doc.opacity)
  }
  this.deselect()
  this.select()
}

OutilArcParRapporteur.prototype.isWorking = function () {
  return (this.point1 !== null) && (this.point2 !== null)
}
