
/*
 * Created by yvesb on 27/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Color = require('../types/Color')
const MotifPoint = require('../types/MotifPoint')
const StyleTrait = require('../types/StyleTrait')
const CCentreGravite = require('../objets/CCentreGravite')
const CMilieu = require('../objets/CMilieu')
const CSegment = require('../objets/CSegment')
const OutilObjetPar3Pts = require('./OutilObjetPar3Pts')

module.exports = OutilCentreGrav

/**
 * Outil servant à créer le centre de gravité d'un triangle
 * @param app L'application propriétaire
 * @constructor
 */
function OutilCentreGrav (app) {
  OutilObjetPar3Pts.call(this, app, 'CentreGrav', 33063, true)
}

OutilCentreGrav.prototype = new OutilObjetPar3Pts()

OutilCentreGrav.prototype.objet = function (pt1, pt2, pt3) {
  var app = this.app
  return new CCentreGravite(app.listePr, null, false, app.getCouleur(), false, 0, 3, false, '', app.getTaillePoliceNom(),
    app.getStylePoint(), false, pt1, pt2, pt3)
}

OutilCentreGrav.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(list)
  var seg1 = new CSegment(list, null, false, b, false, stfp, this.point1, this.point2)
  app.ajouteObjetVisuel(seg1)
  var seg2 = new CSegment(list, null, false, b, false, stfp, this.point2, app.mousePoint)
  app.ajouteObjetVisuel(seg2)
  var seg3 = new CSegment(list, null, false, b, false, stfp, this.point1, app.mousePoint)
  app.ajouteObjetVisuel(seg3)
  var ob = this.objet(this.point1, this.point2, app.mousePoint)
  app.ajouteObjetVisuel(ob)
  var mil1 = new CMilieu(list, null, false, b, true, 0, 0, true, '', 13, MotifPoint.petitRond, false, this.point1, this.point2)
  app.ajouteObjetVisuel(mil1)
  var mil2 = new CMilieu(list, null, false, b, true, 0, 0, true, '', 13, MotifPoint.petitRond, false, this.point2, app.mousePoint)
  app.ajouteObjetVisuel(mil2)
  var mil3 = new CMilieu(list, null, false, b, true, 0, 0, true, '', 13, MotifPoint.petitRond, false, this.point1, app.mousePoint)
  app.ajouteObjetVisuel(mil3)
  var seg4 = new CSegment(list, null, false, b, false, stfp, this.point1, mil2)
  app.ajouteObjetVisuel(seg4)
  var seg5 = new CSegment(list, null, false, b, false, stfp, this.point2, mil3)
  app.ajouteObjetVisuel(seg5)
  var seg6 = new CSegment(list, null, false, b, false, stfp, app.mousePoint, mil1)
  app.ajouteObjetVisuel(seg6)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}
