/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const NatObj = require('../types/NatObj')
const NatCal = require('../types/NatCal')
const kernel = require('../kernel/kernel')
const CCalcul = require('../objets/CCalcul')
const CImplementationProto = require('../objets/CImplementationProto')
const CourbePolyDlg = require('../dialogs/CourbePolyDlg')

module.exports = OutilCourbePoly

function OutilCourbePoly (app) {
  Outil.call(this, app, 'CourbePolyDlg', 32987, true)
}
OutilCourbePoly.prototype = new Outil()

OutilCourbePoly.prototype.select = function () {
  Outil.prototype.select.call(this)
  var self = this
  new CourbePolyDlg(this.app, function (nbPoints, rep) {
    self.suite(nbPoints, rep)
  }, function () { self.app.activeOutilCapt() })
}

/**
 * Une fois que la boîte de dialogue demandant le repère et le nombre de points a été validé, cette fonction
 * est appelée avec le nombre de points demandés nbPoints et le repère rep en paramètres
 * @param nbPoints
 * @param rep
 */
OutilCourbePoly.prototype.suite = function (nbPoints, rep) {
  this.nbPoints = nbPoints
  this.rep = rep
  var app = this.app
  var list = app.listePr
  // On crée maintenant nbPoints calculs qui contiendront les coefficients directeurs souhaités pour les tangentes
  this.coef = []
  // On crée un tableau qui contiendra des pointeurs sur les points cliqués qui n'étaient pas nommés
  this.ptsSansNom = []
  for (var i = 0; i < nbPoints; i++) {
    this.coef[i] = new CCalcul(list, null, false, list.genereNomPourCalcul('mt', true), '0')
    app.ajouteElement(this.coef[i])
  }
  // Maintenant on va attendre que l'utilisateur ait cliqué sur nnPoints
  this.nbPtsCliques = 0 // Ce nombre augmentera à chaque fois qu'on clique sur un point
  this.points = [] // Ce tableau contiendra des pointeurs sur les points cliqués
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageActif.aDesigner = NatObj.NTtPoint
  app.outilPointageActif.reset()
  app.indication(kernel.getStr('indCourTan1') + '1' + kernel.getStr('indCourTan2'), '', true)
}

OutilCourbePoly.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  var list = app.listePr
  var nbPoints = this.nbPoints
  var j
  this.nbPtsCliques++
  this.points.push(elg)
  this.ajouteClignotementDe(elg)
  this.excluDeDesignation(elg)
  if (this.nbPtsCliques === 1) this.resetClignotement()
  // Si tous les points cont été désignés, on implémente la construction correspondant au nombre de points
  // puis on ouvre une boîte de dialogue permetteant de modifier les coefficients
  if (this.nbPtsCliques >= nbPoints) {
    this.annuleClignotement()
    var proto = app.docConsAv.getPrototype('PolynomeParPts' + nbPoints)
    proto.get(0).elementAssocie = this.rep
    for (j = 0; j < nbPoints; j++) proto.get(j + 1).elementAssocie = this.points[j]
    var impProto = new CImplementationProto(list, proto)
    impProto.implemente(app.dimf, proto)
    impProto.nomProto = kernel.getStr('CourbeFonc')
    var lieu = impProto.premierFinal(NatObj.NLieu)
    lieu.infoLieu.nombreDePoints = 500
    lieu.donneCouleur(app.getCouleur())
    lieu.donneStyle(app.getStyleTrait())
    var indImpProto = list.indexOf(impProto)
    list.positionne(false, app.dimf)
    list.setReady4MathJax()
    list.afficheTout(indImpProto, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity)
    app.outilCourbePoly.saveFig()
    // Corrigé version 5.6.4
    if (app.estExercice) app.listePourConst = app.listePourConstruction()
  } else app.indication(kernel.getStr('indCourTan1') + String(this.nbPtsCliques + 1) + kernel.getStr('indCourTan2'), '', true)
}

OutilCourbePoly.prototype.activationValide = function () {
  var list = this.app.listePr
  return list.nombreObjetsCalcul(NatCal.NRepere) > 0
}
