/*
 * Created by yvesb on 27/09/2016.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Color = require('../types/Color')
const MotifPoint = require('../types/MotifPoint')
const StyleTrait = require('../types/StyleTrait')
const CPointProjete = require('../objets/CPointProjete')
const CMarqueAngleGeometrique = require('../objets/CMarqueAngleGeometrique')
const CSegment = require('../objets/CSegment')
const CBissectrice = require('../objets/CBissectrice')
const OutilObjetPar3Pts = require('./OutilObjetPar3Pts')
const StyleMarqueAngle = require('../types/StyleMarqueAngle')
module.exports = OutilDtBis

/**
 * Outil servant à créer une bissectrice
 * @param app L'application propriétaire
 * @constructor
 */
function OutilDtBis (app) {
  OutilObjetPar3Pts.call(this, app, 'DtBis', 32791, true)
}

OutilDtBis.prototype = new OutilObjetPar3Pts()

OutilDtBis.prototype.objet = function (pt1, pt2, pt3) {
  var app = this.app
  return new CBissectrice(app.listePr, null, false, app.getCouleur(), false, 0, 0, false, '', app.getTaillePoliceNom(),
    app.getStyleTrait(), 0.9, pt1, pt2, pt3)
}

// On redéfinit ajouteObjetsVisuels car il y en a d'autres à part la bissectrice elle-même
OutilDtBis.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var b = Color.black
  var stfp = StyleTrait.stfp(list)
  var stfc = StyleTrait.stfc2(list)
  var seg1 = new CSegment(list, null, false, b, false, stfp, this.point1, this.point2)
  app.ajouteObjetVisuel(seg1)
  var seg2 = new CSegment(list, null, false, b, false, stfp, this.point2, app.mousePoint)
  app.ajouteObjetVisuel(seg2)
  var bis = this.objet(this.point1, this.point2, app.mousePoint)
  app.ajouteObjetVisuel(bis)
  var ptproj = new CPointProjete(list, null, false, b, true, 0, 0, true, '', 13, MotifPoint.petitRond,
    false, this.point1, bis)
  app.ajouteObjetVisuel(ptproj)
  var mar1 = new CMarqueAngleGeometrique(list, null, false, b, false,
    stfc, StyleMarqueAngle.marqueSimple, 13, this.point1, this.point2, ptproj)
  app.ajouteObjetVisuel(mar1)
  var mar2 = new CMarqueAngleGeometrique(list, null, false, b, false,
    stfc, StyleMarqueAngle.marqueSimple, 16, app.mousePoint, this.point2, ptproj)
  app.ajouteObjetVisuel(mar2)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

/**
 * Fonction renvoyant true si l'outil accepte qu'on crée un point par défaut lors d'un clic sur un endroit vide
 */
OutilDtBis.prototype.creationPointPossible = function () {
  return true
}
