/*
 * Created by yvesb on 07/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const NatObj = require('../types/NatObj')
const Outil = require('../outils/Outil')
const Nat = require('../types/Nat')
const CLieuObjetParPtLie = require('../objets/CLieuObjetParPtLie')
const LieuObjetDlg = require('../dialogs/LieuObjetDlg')
const CValeur = require('../objets/CValeur')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilLieuObjetParPtLie

function OutilLieuObjetParPtLie (app) {
  Outil.call(this, app, 'LieuObjetParPtLie', 33122, true)
}

OutilLieuObjetParPtLie.prototype = new Outil()

OutilLieuObjetParPtLie.prototype.select = function () {
  Outil.prototype.select.call(this)
  var app = this.app
  this.objetPourTrace = null
  this.pointLieGenerateur = null
  this.app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = Nat.or(NatObj.NTtObjPourLieu, NatObj.NSurface, NatObj.NImage)
  // On interdit de désigner une surface qui ne peut pas être un lieu d'objets
  var listePr = app.listePr
  var li = app.listeExclusion
  for (var i = 0; i < listePr.longueur(); i++) {
    var elb = listePr.get(i)
    if (elb.estDeNature(NatObj.NSurface) && !elb.peutGenererLieuObjet()) li.add(elb)
  }
  app.outilPointageCre.reset()
  app.indication('indLieuObj1')
}

OutilLieuObjetParPtLie.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  var list = app.listePr
  if (this.objetPourTrace === null) {
    this.objetPourTrace = elg
    // On n'exclut pas objetPourTrace d'une désignation possible car le point lié peut être égal
    // au point générateur du lieu
    this.ajouteClignotementDe(elg)
    // frame.indication(getStr("lieuParPointLie2"));
    app.listeClignotante.ajoutePointsLiesVisiblesDontDepend(list, elg)
    app.outilPointageActif = app.outilPointageObjetClignotant
    app.outilPointageActif.aDesigner = NatObj.NPointLie
    app.outilPointageActif.reset(false, true) // Deuxième paramètre true pour pouvoir désigné un objet clignotant
    this.resetClignotement()
    app.indication('indLieu2')
  } else {
    if (this.pointLieGenerateur === null) {
      this.pointLieGenerateur = elg
      this.creeObjet()
    }
  }
}

OutilLieuObjetParPtLie.prototype.actionApresDlgOK = function () {
  var app = this.app
  app.ajouteElement(this.lieu)
  if (app.verifieDernierElement(1)) {
    var self = this
    // Le lieu créé existe bien et n'a pas été déjà créé
    // Il faut appeler setReady4MathJax car le lieu peut être un lieu d'affichage LaTeX
    this.lieu.setReady4MathJax()
    // Modification version 6.4 : Il faut passer par la pile d'appels au cas où le texte à afficher commmence et finit par un $
    MathJax.Hub.Queue(function () {
      self.lieu.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    })

    this.annuleClignotement() // Pour que le centre clignotant ne soit pas enregistré comme caché
    this.saveFig()
  } else new AvertDlg(app, 'DejaCree')
  app.activeOutilCapt()
}

OutilLieuObjetParPtLie.prototype.creeObjet = function () {
  var app = this.app
  var list = app.listePr
  var self = this
  var nbTraces = new CValeur(list, 40)
  this.lieu = new CLieuObjetParPtLie(list, null, false, app.getCouleur(), false, this.objetPourTrace, nbTraces, this.pointLieGenerateur)
  new LieuObjetDlg(app, this.lieu, false, function () { self.actionApresDlgOK() }, function () { self.app.activeOutilCapt() })
}

OutilLieuObjetParPtLie.prototype.isReadyForTouchEnd = function () {
  return false
}
