/*
 * Created by yvesb on 07/03/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const NatObj = require('../types/NatObj')
const Outil = require('../outils/Outil')
const Nat = require('../types/Nat')
const CLieuObjetParVariable = require('../objets/CLieuObjetParVariable')
const ChoixVariableDlg = require('../dialogs/ChoixVariableDlg')
const LieuObjetDlg = require('../dialogs/LieuObjetDlg')
const CValeur = require('../objets/CValeur')
const NatCal = require('../types/NatCal')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilLieuObjetParVariable

function OutilLieuObjetParVariable (app) {
  Outil.call(this, app, 'LieuObjetParVariable', 33124, true)
}

OutilLieuObjetParVariable.prototype = new Outil()

OutilLieuObjetParVariable.prototype.select = function () {
  this.objetPourTrace = null
  var self = this
  Outil.prototype.select.call(this)
  new ChoixVariableDlg(this.app, function (va) { self.actionApresDlg(va) }, function () { self.app.activeOutilCapt() })
}

OutilLieuObjetParVariable.prototype.actionApresDlg = function (va) {
  var app = this.app
  app.indication('indLieuObj1')
  this.va = va // La variable choisie dans la boîte de dialogue
  var nat = Nat.or(NatObj.NTtObjPourLieu, NatObj.NSurface, NatObj.NImage)
  app.listeClignotante.ajouteObjetsParNatureDependantDe(app.listePr, nat, va, true)
  app.outilPointageActif = app.outilPointageObjetClignotant
  app.outilPointageActif.aDesigner = nat
  app.outilPointageActif.reset(false, true) // Deuxième paramètre true pour pouvoir désigné un objet clignotant
  this.resetClignotement()
}

OutilLieuObjetParVariable.prototype.traiteObjetDesigne = function (elg) {
  if (this.objetPourTrace === null) {
    this.objetPourTrace = elg
    this.creeObjet() // Défini dans les outils descendants
  }
}

OutilLieuObjetParVariable.prototype.actionApresDlgOK = function (va) {
  this.va = va
  var app = this.app
  app.ajouteElement(this.lieu)
  if (app.verifieDernierElement(1)) {
    // Le lieu créé existe bien et n'a pas été déjà créé
    this.annuleClignotement() // Pour que le centre clignotant ne soit pas enregistré comme caché
    // Il faut appeler setReady4MathJax car le lieu peut être un lieu d'affichage LaTeX
    var lieu = this.lieu
    lieu.setReady4MathJax()
    MathJax.Hub.Queue(function () {
      lieu.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    })
    this.saveFig()
  } else new AvertDlg(app, 'DejaCree')
  app.activeOutilCapt()
}

OutilLieuObjetParVariable.prototype.creeObjet = function () {
  var app = this.app
  var list = app.listePr
  var self = this
  var nbTraces = new CValeur(list, 40)
  this.lieu = new CLieuObjetParVariable(list, null, false, app.getCouleur(), false, this.objetPourTrace, nbTraces, this.va)
  new LieuObjetDlg(app, this.lieu, false, function () { self.actionApresDlgOK() }, function () { self.app.activeOutilCapt() })
}

OutilLieuObjetParVariable.prototype.isReadyForTouchEnd = function () {
  return false
}

OutilLieuObjetParVariable.prototype.activationValide = function () {
  return this.app.listePr.nombreObjetsCalcul(NatCal.NVariable) !== 0
}
