/*
 * Created by yvesb on 05/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const kernel = require('../kernel/kernel')
const OutilPar3Pts = require('./OutilPar3Pts')
const AvertDlg = require('../dialogs/AvertDlg')
const CMesureAbscisse = require('../objets/CMesureAbscisse')
const NatObj = require('../types/NatObj')
const NomMesureDlg = require('../dialogs/NomMesureDlg')

module.exports = OutilMesAbs/**
 * Outil servant à mesurer l'abscisse d'un point sur une droite relativement à deux points de la droite
 * @param app L'application propriétaire
 * @constructor
 */
function OutilMesAbs (app) {
  OutilPar3Pts.call(this, app, 'MesAbs', 32817, true, false) // Dernier paramètre false car on réactive l'outil de capture après création
}

OutilMesAbs.prototype = new OutilPar3Pts()

OutilMesAbs.prototype.select = function () {
  this.app.outilPointageActif = this.app.outilPointageCre
  OutilPar3Pts.prototype.select.call(this) // A appeler à la fin
}

OutilMesAbs.prototype.creeObjet = function () {
  var app = this.app
  var li = app.listePr
  var mes = new CMesureAbscisse(li, null, false, this.point1, this.point2, this.point3, '')
  app.ajouteElement(mes)
  var pasDejaCree = app.verifieDernierElement(1)
  if (pasDejaCree) {
    // La mesure existe bien et n'a pas été déjà créé
    this.annuleClignotement()
    var self = this
    var infoAnnul = {
      pt1: this.point1,
      pt2: this.point2,
      pt3: this.point3,
      pt1Cree: this.point1Cree,
      pt2Cree: this.point2Cree,
      pt1Nomme: this.point1Nomme,
      pt2Nomme: this.point2Nomme,
      pt3Nomme: this.point3Nomme
    }
    new NomMesureDlg(app, kernel.getStr('MesAbs') + ' ' + kernel.getStr('de') + ' ' + this.point3.nom + ' ' +
      kernel.getStr('dans') + ' (' + this.point1.nom + ',' + this.point2.nom + ')',
    function (st) { self.callBackAfterDlg(st, infoAnnul) },
    function () { self.callBackCancel(infoAnnul) })
  } else {
    new AvertDlg(app, 'DejaCree')
  }
  return pasDejaCree
}

OutilMesAbs.prototype.callBackAfterDlg = function (st) {
  var app = this.app
  var li = app.listePr
  var mes = li.get(li.longueur() - 1)
  mes.nomCalcul = st
  this.saveFig()
  // li.afficheTout(infoAnnul.indiceImpInitial, app.svgFigure, true, app.doc.couleurFond, app.doc.opacity);
}

OutilMesAbs.prototype.callBackCancel = function (infoAnnul) {
  this.app.detruitDernierElement()
  this.objetCree = false // Pour que les points éventuellemnt nommés par l'outil soient dénommés.
  this.point1 = infoAnnul.pt1
  this.point2 = infoAnnul.pt2
  this.point3 = infoAnnul.pt3
  this.point1Cree = infoAnnul.pt1Cree
  this.point2Cree = infoAnnul.pt2Cree
  this.point1Nomme = infoAnnul.pt1Nomme
  this.point2Nomme = infoAnnul.pt2Nomme
  this.point3Nomme = infoAnnul.pt3Nomme
  this.deselect()
  this.app.activeOutilCapt()
}

OutilMesAbs.prototype.actionApresPoint2 = function () {
  var app = this.app
  var list = app.listePr
  this.annuleClignotement()
  for (var i = 0; i < list.longueur(); i++) {
    var elb = list.get(i)
    if (elb.estDeNature(NatObj.NTtPoint)) {
      if (!elb.masque && kernel.aligne(this.point1.x, this.point1.y, this.point2.x, this.point2.y, elb.x, elb.y)) { this.ajouteClignotementDe(elb) }
    }
  }
  app.outilPointageActif = app.outilPointageObjetClignotant
  app.outilPointageActif.aDesigner = NatObj.NTtPoint
  app.outilPointageActif.reset(false, true) // Deuxième paramètre true pour pouvoir désigner un objet clignotant
  this.resetClignotement()
}

OutilMesAbs.prototype.nomsPointsIndisp = function () {
  return true
}
