/*
 * Created by yvesb on 24/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObjAdd')
const Nat = require('../types/Nat')
const CMesureLongueurLigne = require('../objets/CMesureLongueurLigne')
const OutilObjetPar1Objet = require('./OutilObjetPar1Objet')
const AvertDlg = require('../dialogs/AvertDlg')
const NomMesureDlg = require('../dialogs/NomMesureDlg')
module.exports = OutilMesLongLigne
/**
 * Outil servant à créer une droite horizontale
 * @param app L'application propriétaire
 * @constructor
 */
function OutilMesLongLigne (app) {
  OutilObjetPar1Objet.call(this, app, 'MesLongLigne', 33129, true, Nat.or(NatObj.NPolygone, NatObj.NLigneBrisee))
}

OutilMesLongLigne.prototype = new OutilObjetPar1Objet()

OutilMesLongLigne.prototype.indication = function () {
  return 'indPolOuLig'
}

OutilMesLongLigne.prototype.preIndication = function () {
  return 'MesLongLigne'
}

OutilMesLongLigne.prototype.creeObjet = function (elg) {
  var app = this.app
  var self = this
  this.ligne = elg
  new NomMesureDlg(app, this.tip, function (st) { self.callBackOK(st) }, function () { self.reselect() }, false)
}

OutilMesLongLigne.prototype.callBackOK = function (st) {
  var self = this
  var app = this.app
  var list = app.listePr
  var ligne = this.ligne
  var mes = new CMesureLongueurLigne(list, null, false, st, ligne)
  app.ajouteElement(mes)
  if (app.verifieDernierElement(1)) {
    // On vérifie que les points de la ligne sont bien nommés. Sinon on leur attribue un nom.
    for (var i = 0; i < ligne.colPoints.length; i++) {
      var ptp = ligne.colPoints[i].pointeurSurPoint
      if (ptp.nom === '') {
        ptp.donneNom(list.genereNomPourPoint())
        ptp.updateName(app.svgFigure, true)
      }
    }
    this.saveFig()
  } else {
    new AvertDlg(app, 'DejaCree', function () {
      self.app.activeOutilCapt()
    })
  }
}

// Pas d'objets visuels ici
OutilMesLongLigne.prototype.ajouteObjetsVisuels = function () {
}

OutilMesLongLigne.prototype.activationValide = function () {
  var list = this.app.listePr
  return list.pointeurLongueurUnite !== null && list.nombreObjetsParNatureVisibles(NatObj.NLigneOuPoly) > 0
}
