/*
 * Created by yvesb on 25/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('../outils/Outil')
const CPolygone = require('../objets/CPolygone')
const CNoeudPointeurSurPoint = require('../objets/CNoeudPointeurSurPoint')
const NatObj = require('../types/NatObj')
const CSurfacePolygone = require('../objets/CSurfacePolygone')
const StyleTrait = require('../types/StyleTrait')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilPolygone

/**
 * Outil servant à créer un polygone
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPolygone (app) {
  Outil.call(this, app, 'Polygone', 32864, true)
}

OutilPolygone.prototype = new Outil()

OutilPolygone.prototype.select = function () {
  Outil.prototype.select.call(this)
  var app = this.app
  this.listeNoeuds = []
  this.app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = NatObj.NTtPoint
  app.outilPointageCre.reset()
  app.indication('indCrPoly')
}

OutilPolygone.prototype.deselect = function () {
  this.listeNoeuds.length = 0
  Outil.prototype.deselect.call(this)
  this.app.showStopButton(false)
}

OutilPolygone.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  var list = app.listePr
  var ln = this.listeNoeuds
  ln.push(new CNoeudPointeurSurPoint(list, elg))
  this.ajouteClignotementDe(elg)
  if (ln.length === 3) app.showStopButton(true)
  if (ln.length === 1) {
    this.ajouteObjetsVisuels()
    this.resetClignotement()
    // frame.indication(getStr("polygone2"));
  } else {
    if (elg === ln[0].pointeurSurPoint) {
      if (ln.length <= 3) {
        this.deselect()
        var self = this
        new AvertDlg(app, 'ch41', function () {
          self.select()
        })
      } else this.creeObjet()
    } else {
      app.listeExclusion.retireTout()
      this.excluDeDesignation(elg)
      this.ajouteObjetsVisuels()
      this.resetClignotement()
    }
  }
}

OutilPolygone.prototype.creeObjet = function () {
  var app = this.app
  var ln = this.listeNoeuds
  var doc = app.doc
  var listePr = app.listePr
  var coul = app.getCouleur()
  var coulFond = doc.couleurFond
  var opacity = doc.opacity
  var listeNoeudsClone = []
  for (var i = 0; i < ln.length; i++) listeNoeudsClone.push(ln[i].getClone(app.listePr, app.listePr))
  var poly = new CPolygone(listePr, null, false, coul, false, app.getStyleTrait(), listeNoeudsClone)
  app.ajouteElement(poly)
  if (app.verifieDernierElement(1)) {
    this.annuleClignotement()
    poly.creeAffichage(app.svgFigure, false, coulFond, opacity)
    // Spécial pour les exercices de construction, on crée une surface remplissant le polygone
    if (app.estExercice) {
      var surf = new CSurfacePolygone(listePr, null, false, coul, false, app.getStyleRemplissage(), poly)
      app.ajouteElement(surf)
      surf.creeAffichage(app.svgFigure, false, coulFond, opacity)
    }
    this.saveFig()
    this.nbPtsCrees = 0
  } else new AvertDlg(app, 'DejaCree')
  this.reselect()
}

OutilPolygone.prototype.actionFin = function () {
  this.listeNoeuds.push(new CNoeudPointeurSurPoint(this.app.listePr, this.listeNoeuds[0].pointeurSurPoint))
  this.creeObjet()
}

OutilPolygone.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var coul = app.getCouleur()
  var listePr = app.listePr
  var ln = this.listeNoeuds
  app.supprimeObjetsVisuels()
  var listeNoeudsClone = []
  for (var i = 0; i < ln.length; i++) listeNoeudsClone.push(ln[i].getClone(listePr, listePr))
  listeNoeudsClone.push(new CNoeudPointeurSurPoint(app.listePr, app.mousePoint))
  listeNoeudsClone.push(ln[0].getClone(app.listePr, app.listePr))
  var poly = new CPolygone(listePr, null, false, coul, false, StyleTrait.stfp(listePr), listeNoeudsClone)
  app.ajouteObjetVisuel(poly)
  if (app.estExercice) {
    var surf = new CSurfacePolygone(listePr, null, false, coul, false, app.getStyleRemplissage(), poly)
    app.ajouteObjetVisuel(surf)
  }
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

OutilPolygone.prototype.isReadyForTouchEnd = function () {
  return false
}

/**
 * Fonction renvoyant true si l'outil accepte qu'on crée un point par défaut lors d'un clic sur un endroit vide
 */
OutilPolygone.prototype.creationPointPossible = function () {
  return true
}
