/*
 * Created by yvesb on 02/01/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

const Outil = require('./Outil')
const Nat = require('../types/Nat')
const NatObj = require('../types/NatObj')
const CPointLieDroite = require('../objets/CPointLieDroite')
const CPointLieCercle = require('../objets/CPointLieCercle')
const CPointLieLigne = require('../objets/CPointLieLigne')
const CPointLieLieuParPtLie = require('../objets/CPointLieLieuParPtLie')
const CPointLieLieuParVar = require('../objets/CPointLieLieuParVar')
const Point = require('../types/Point')
const Pointeur = require('../types/Pointeur')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilPtLie

/**
 * Outil servant à créer un point lié
 * @param app L'application propriétaire
 * @constructor
 */

function OutilPtLie (app) {
  Outil.call(this, app, 'PtLie', 32800, false)
}

OutilPtLie.prototype = new Outil()

OutilPtLie.prototype.select = function () {
  Outil.prototype.select.call(this)
  var app = this.app
  this.objet = null
  this.app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = Nat.or(NatObj.NTteDroite, NatObj.NTtCercle, NatObj.NPolygone,
    NatObj.NLigneBrisee, NatObj.NLieu)
  app.outilPointageCre.reset()
  app.indication('indPtLie')
}

OutilPtLie.prototype.traiteObjetDesigne = function (elg, point) {
  if (this.objet === null) {
    this.objet = elg
    this.creeObjet(elg, point)
    this.deselect()
    this.select()
  }
}

OutilPtLie.prototype.creeObjet = function (objet, point) {
  var app = this.app
  var li = app.listePr
  var pt // Le point lié à créer
  var coul = app.getCouleur()
  var stylePoint = app.getStylePoint()
  var taille = app.getTaillePoliceNom()
  if (objet.estDeNature(NatObj.NTteDroite)) { pt = new CPointLieDroite(li, null, false, coul, false, 0, 3, false, '', taille, stylePoint, false, true, 0, objet) } else
  if (objet.estDeNature(NatObj.NTtCercle)) { pt = new CPointLieCercle(li, null, false, coul, false, 0, 3, false, '', taille, stylePoint, false, true, 0, objet) } else
  if (objet.estDeNature(Nat.or(NatObj.NLigneBrisee, NatObj.NPolygone))) { pt = new CPointLieLigne(li, null, false, coul, false, 0, 3, false, '', taille, stylePoint, false, true, 0, objet) } else
  if (objet.estDeNature(NatObj.NLieu)) {
    if (objet.className === 'CLieuDePoints') { pt = new CPointLieLieuParPtLie(li, null, false, coul, false, 0, 3, false, '', taille, stylePoint, false, true, 0, objet) } else pt = new CPointLieLieuParVar(li, null, false, coul, false, 0, 3, false, '', taille, stylePoint, false, true, 0, objet)
  }
  var abs = new Pointeur()
  var pointres = new Point()
  pt.testDeplacement(app.dimf, point.x, point.y, pointres, abs)
  pt.donneAbscisse(abs.getValue())
  app.ajouteElement(pt)
  var verif = app.verifieDernierElement(1)
  if (verif) {
    pt.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    this.saveFig()
  } else new AvertDlg(app, 'DejaCree')
  return verif
}

OutilPtLie.prototype.isReadyForTouchEnd = function () {
  return false
}
