/*
 * Created by yvesb on 05/05/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const CPointParMultVect = require('../objets/CPointParMultVect')
const PtParMultVecDlg = require('../dialogs/PtParMultVecDlg')
const NatObj = require('../types/NatObj')
const CVecteur = require('../objets/CVecteur')
const StyleTrait = require('../types/StyleTrait')
const StyleFleche = require('../types/StyleFleche')
const Color = require('../types/Color')
const AvertDlg = require('../dialogs/AvertDlg')
module.exports = OutilPtParMultVec

function OutilPtParMultVec (app) {
  Outil.call(this, app, 'PtParMultVec', 32891)
}
OutilPtParMultVec.prototype = new Outil()

OutilPtParMultVec.prototype.select = function () {
  Outil.prototype.select.call(this)
  this.vect = null // Le vecteur utilisé
  var app = this.app
  var list = app.listePr
  var self = this
  this.pt = new CPointParMultVect(list, null, false, app.getCouleur(), false, 0, 0, false, '', app.getTaillePoliceNom(),
    app.getStylePoint(), false, null, null, null)
  new PtParMultVecDlg(app, this.pt, false, function () { self.actionApresDlgOK() }, function () { self.app.activeOutilCapt() })
}

OutilPtParMultVec.prototype.deselect = function () {
  this.annuleClignotement()
  this.vect = null
  this.pt = null
  Outil.prototype.deselect.call(this)
}

OutilPtParMultVec.prototype.actionApresDlgOK = function () {
  var app = this.app
  app.outilPointageActif = app.outilPointageCre
  app.outilPointageCre.aDesigner = NatObj.NVecteur
  app.outilPointageCre.reset()
  app.indication('indVect')
}

OutilPtParMultVec.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  if (this.vect === null) {
    this.vect = elg
    app.outilPointageActif.aDesigner = NatObj.NTtPoint
    app.outilPointageActif.reset()
    this.ajouteClignotementDe(elg)
    this.resetClignotement()
    app.indication('indPt')
    this.ajouteObjetsVisuels()
  } else {
    this.creeObjet(elg)
  }
}

OutilPtParMultVec.prototype.creeObjet = function (point) {
  var app = this.app
  var list = app.listePr
  var coef = this.pt.coef // A été changé par la boîte de dialogue
  this.pt.antecedent = point
  this.pt.vect = this.vect
  app.ajouteElement(this.pt)
  var verif = app.verifieDernierElement(1)
  if (verif) {
    // L'élément créé existe bien et a été ajouté à la liste
    this.pt.creeAffichage(app.svgFigure, false, app.doc.couleurFond, app.doc.opacity)
    this.saveFig()
  } else new AvertDlg(app, 'DejaCree')
  // On recrée le point pour une éventuelle autre image avec le coefficient déjà choisi
  this.pt = new CPointParMultVect(list, null, false, app.getCouleur(), false, 0, 0, false, '', app.getTaillePoliceNom(),
    app.getStylePoint(), false, null, null, coef)
}

OutilPtParMultVec.prototype.ajouteObjetsVisuels = function () {
  var app = this.app
  var list = app.listeObjetsVisuels
  var stfp = StyleTrait.stfp(list)
  var pt = new CPointParMultVect(list, null, false, Color.black, true, 0, 0, true, '', 13, 0, false, app.mousePoint,
    this.vect, this.pt.coef)
  app.ajouteObjetVisuel(pt)
  var vect = new CVecteur(list, null, false, Color.black, false, stfp, app.mousePoint, pt, StyleFleche.FlecheCourtePleine)
  app.ajouteObjetVisuel(vect)
  app.afficheObjetVisuels(0) // Ajout version 6.4.4
}

OutilPtParMultVec.prototype.activationValide = function () {
  return (this.app.listePr.nombreObjetsParNatureVisibles(NatObj.NVecteur) !== 0)
}
