/*
 * Created by yvesb on 08/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const Outil = require('./Outil')
const CListeObjets = require('../objets/CListeObjets')
const NatObj = require('../types/NatObj')

module.exports = OutilRideau
/**
 * Outil servant à démasquer des objest masqués
 * @param app L'application propriétaire
  * @constructor
 */
function OutilRideau (app) {
  Outil.call(this, app, 'Rideau', 32808, true)
}

OutilRideau.prototype = new Outil()

OutilRideau.prototype.select = function () {
  var app = this.app
  Outil.prototype.select.call(this)
  this.cursor = 'default'
  var list = this.app.listePourConst
  // Il est nécessaire de créer une liste pointant sur les objets que l'utilisateur ne va pas démasquer car
  // deselect va appeler annuleClignotement qui donne l'attribut visible à tous les objets de la liste
  this.listeOb = new CListeObjets(list.uniteAngle, '') // Contiendra des pointeurs sur les objets qu'on a choisi de démasquer
  // S'il y a plus de 1000 objets on ne donne pas la possibilité de démasquer les objets finaux de constructions masqués
  // ce qui arrive souvent pour les figures dans lesquelles on a implémenté des constructions récursives ou itératives
  var b = list.longueur() <= 1000
  for (var i = 0; i < list.longueur(); i++) {
    var el = list.get(i)
    if (el.estDeNature(NatObj.NTtObj) && el.masque && !el.estElementIntermediaire() && (b || (!el.estElementFinal))) {
      this.ajouteClignotementDe(el)
      this.listeOb.add(el)
    }
  }
  // On rend les objets masqués visibles
  list.updateEvenMasked(app.svgFigure, app.doc.couleurFond, app.doc.opacity)
  //
  app.outilPointageActif = app.outilPointageObjetClignotant
  app.outilPointageActif.aDesigner = NatObj.NTtObj
  app.outilPointageActif.reset(false, true) // Deuxième paramètre true pour pouvoir désigner un objet clignotant
  // même lorsqu'il est phase de masquage
  this.resetClignotement()
  this.cursor = 'default'
  app.indication('Rid')
}

OutilRideau.prototype.deselect = function () {
  Outil.prototype.deselect.call(this)
  this.listeOb.montreTout(false)
  this.listeOb.retireTout()
  var app = this.app
  app.listePr.update(app.svgFigure, app.doc.couleurFond, app.doc.opacity, true) // true pour que les objets masqués soient masqués
  this.saveFig()
}

OutilRideau.prototype.traiteObjetDesigne = function (elg) {
  var app = this.app
  app.listeClignotante.removeObject(elg)
  this.listeOb.removeObject(elg)
  elg.montre()
  app.listePr.update(app.svgFigure, app.doc.couleurFond, app.doc.opacity, true)
}
