/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const Nat = require('../types/Nat')
const Point = require('../types/Point')
const Pointeur = require('../types/Pointeur')
const kernel = require('../kernel/kernel')
const kernelAdd = require('../kernel/kernelAdd')
const OutilPointage = require('./OutilPointage')
const PremierDernierDlg = require('../dialogs/PremierDernierDlg')
const ChoixTypeDlg = require('../dialogs/ChoixTypeDlg')
module.exports = OutilPointageCapture

/**
 * Outil de pountage utilisé par l'outil de capture d'objet mobile
 * Hérite de OutilPointage
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPointageCapture (app) {
  OutilPointage.call(this, app)
}
OutilPointageCapture.prototype = new OutilPointage()

OutilPointageCapture.prototype.reset = function () {
  OutilPointage.prototype.reset.call(this)
  this.objetPredesigne = null
  this.cadre_Ver_Proche = false // Indique si on est en train de capturer la partie verticale du cadre d'affichage
  this.cadre_Hor_Proche = false // Indique si on est en train de capturer la partie horizontale du cadre d'affichage
  this.cadre_Ver_Capt = false // Indique si on est en train de capturer la partie verticale du cadre d'affichage
  this.cadre_Hor_Capt = false // Indique si on est en train de capturer la partie horizontale du cadre d'affichage
}

OutilPointageCapture.prototype.mousemove = function (evt) {
  this.devicemove(evt, 'mouse', kernelAdd.mousePosition)
}
OutilPointageCapture.prototype.touchmove = function (evt) {
  this.devicemove(evt, 'touch', kernelAdd.touchPosition)
}
OutilPointageCapture.prototype.devicemove = function (evt, type, fonc) {
  var point, tab, x, y
  var app = this.app
  var svg = app.svgFigure
  var doc = app.doc
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  var elc = app.elementCapture
  tab = fonc(svg, evt, app.zoomFactor)
  x = tab.x
  y = tab.y
  // if (x<0 || x>app.dimf.x || y<0 || y>app.dimf.y) return; // Devenu inutile depuis que les événements sont interceptés par un rectangle
  if (this.cadre_Ver_Capt || this.cadre_Hor_Capt) {
    var w = app.widthCadre; var h = app.heightCadre
    app.deleteCadre()
    if ((this.cadre_Ver_Capt) && (x > 20)) w = Math.round(x)
    if ((this.cadre_Hor_Capt) && (y > 20)) h = Math.round(y)
    app.createCadre(w, h)
  } else {
    if (elc === null) {
      var info = this.app.infoProx
      // point = new Point(x, y);
      var nbObjetsProches = doc.listePr.procheDePourCapture(this.aDesigner,
        tab, info, app.listeExclusion, true, type)
      if ((nbObjetsProches === 0) && app.cadre) {
        const distmin = type === 'touch' ? kernel.distMinForTouch : kernel.distMinForMouse
        this.cadre_Ver_Proche = Math.abs(x - app.widthCadre) <= distmin
        this.cadre_Hor_Proche = Math.abs(y - app.heightCadre) <= distmin
        app.outilActif.cursor = (this.cadre_Hor_Proche || this.cadre_Ver_Proche) ? 'pointer' : 'default'
      } else app.outilActif.cursor = (nbObjetsProches > 0) ? 'pointer' : 'default'
    } else {
      var dimf = doc.dimf
      var couleurFond = doc.couleurFond
      var transp = doc.opacity
      if (elc.estDeNature(NatObj.NPointMobile)) {
        if ((Math.abs(x - elc.x) >= 1) || (Math.abs(y - elc.y) >= 1)) {
          point = new Point()
          var abs = new Pointeur()
          var b = elc.testDeplacement(dimf, x, y, point, abs)
          if (b) {
            if (elc.estDeNature(NatObj.NPointCapturableNonLie)) {
              elc.placeEn(point.x, point.y)
            } else if (elc.estDeNature(NatObj.NPointLie)) {
              elc.donneAbscisse(abs.getValue())
            }
          }
        }
      } else {
        if (elc.estDeNature(NatObj.NMarqueAngle)) {
          var ray = kernel.distancePointPoint(x, y, elc.centreX, elc.centreY)
          if (ray > 3) elc.setRayon(ray)
        } else {
          if (elc.estDeNature(NatObj.NAffLieAPoint)) {
            var pt = elc.pointLie
            if (pt === null) {
              var dx = x - this.xInitSouris
              var dy = y - this.yInitSouris
              if ((dx !== 0) || (dy !== 0)) {
                elc.placeNom(this.xAffInitial + dx, this.yAffInitial + dy)
              }
            } else {
              elc.decaleNom(this.xdecalageInitialRelatifAffichage + x - this.xInitSouris,
                this.ydecalageInitialRelatifAffichage + y - this.yInitSouris)
            }
          } else {
            if (elc.estDeNature(NatObj.NLieuObjet)) {
              // Si ce lieu d'objet est capturable c'est qu'il est généré par un CCommentaire ou un CLatex
              // CLieuObjetAncetre lieu = (CLieuObjetAncetre) elementCapture;
              var aff = elc.elementAssocie
              aff.decaleNom(this.xdecalageInitialRelatifAffichage + x - this.xInitSouris,
                this.ydecalageInitialRelatifAffichage + y - this.yInitSouris)
            } else {
              elc.decaleNom(this.xdecalageInitialRelatifAffichage + x - this.xInitSouris,
                this.ydecalageInitialRelatifAffichage + y - this.yInitSouris)
            }
          }
        }
      }
      app.listeARecalculer.positionne(false, dimf)
      // Ligne suivante modifié version 6.4.8 (ajout du dernier paramètre)
      app.listeARecalculer.update(svg, couleurFond, transp, true, true)
    }
  }
}

OutilPointageCapture.prototype.mousedown = function (evt) {
  this.devicedown(evt, 'mouse', kernelAdd.mousePosition)
}

OutilPointageCapture.prototype.touchstart = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageCapture.prototype.devicedown = function (evt, type, fonc) {
  var app = this.app
  var mac
  var svg = app.svgFigure
  var doc = app.doc
  if (!doc.isActive) return
  var liste = doc.listePr
  var dimf = doc.dimf
  var couleurFond = doc.couleurFond
  var transp = doc.opacity
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  if (app.elementCapture !== null) {
    app.elementCapture = null
    app.listeARecalculer.retireTout()
    return
  }
  if (liste.macroEnCours !== null) {
    mac = liste.macroEnCours.macroEnCours()
    if ((mac.className === 'CMacroPause') && (mac.dureePause === 0)) {
      mac.passageMacroSuiv(svg, dimf, couleurFond, transp)
      return
    }
    if ((mac !== null) && (mac.className === 'CMacroApparition') && mac.executionPossible()) {
      mac.execute(svg, dimf, couleurFond, transp, true)
      // Attention mac ne pointe plus forcément sur la macro en cours
      if (!liste.macroEnCours.macroEnCours().executionEnCours) liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, transp)
      return
    }
  }
  var info = this.app.infoProx
  var point = fonc(svg, evt, app.zoomFactor)
  // if (point.x<0 || point.x>app.dimf.x || point.y<0 || point.y>app.dimf.y) return;
  this.xInitSouris = point.x
  this.yInitSouris = point.y
  var nbObjetsProches = liste.procheDePourCapture(this.aDesigner,
    point, info, app.listeExclusion, true, type)
  if (nbObjetsProches > 0) {
    if ((mac = info.infoParType[NatObj.indiceMacro].premierVoisin) !== null) {
      if (mac.executionEnCours) {
        mac.macroEnCours().termineAction(svg, dimf, couleurFond, transp)
      }
    }
    if (!liste.documentProprietaire.modeTraceActive) liste.deleteTraces()
    if (this.objetPredesigne !== null) {
      const dist = type === 'touch' ? kernel.distMinForTouch : kernel.distMinForMouse
      if (this.objetPredesigne.distancePoint(point.x, point.y, true, dist) < dist) {
        app.elementCapture = this.objetPredesigne
        this.objetPredesigne = null
        this.callBackAfterChoice()
      }
    } else {
      if (app.elementCapture === null) {
        var self = this
        if (info.premierPointVoisin !== null) { // Priorité aux points
          if (info.nombrePointsVoisins === 1) {
            app.elementCapture = info.premierPointVoisin
            this.callBackAfterChoice()
          } else {
            kernel.preventDefault(evt)
            evt.stopPropagation()
            if (info.nombreTypesProches > 1) {
              new ChoixTypeDlg(app, info, Nat.and(info.typesProches, NatObj.NPointMobile), this,
                'objetPredesigne', function () {
                  self.callBackAfterChoice()
                })
            } else {
              // A revoir il faudra proposer un dialogue de choix entre le premier ou le dernier point
              new PremierDernierDlg(app, this, 'objetPredesigne', info.premierPointVoisin,
                info.dernierPointVoisin, function () {
                  self.callBackAfterChoice()
                })
            }
          }
        } else {
          if (info.nombreTypesProches > 1) {
            new ChoixTypeDlg(app, info, info.typesProches, this,
              'objetPredesigne', function () {
                self.callBackAfterChoice()
              })
          } else {
            if (nbObjetsProches === 1) {
              app.elementCapture = info.premierVoisin()
              this.callBackAfterChoice()
            } else {
              new PremierDernierDlg(app, this, 'objetPredesigne', info.premierVoisin(),
                info.dernierVoisin(), function () {
                  self.callBackAfterChoice()
                })
            }
          }
        }
      }
    }
  } else {
    if (liste.macroEnCours !== null) {
      if (liste.macroEnCours.className === 'CMacroSuiteMacros') {
        mac = liste.macroEnCours.macroEnCours()
        if (mac.arretParClic()) {
          mac.termineAction(svg, dimf, couleurFond, transp)
          liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, transp)
        }
      } else {
        // Si la macro en cours est une macro d'apparition avec clic pour objet suivant
        // on ne la désactive que si on est au dernier objet
        mac = liste.macroEnCours.macroEnCours()
        if ((mac.className === 'CMacroApparition') && mac.executionEnCours) {
          this.liste.update(svg, couleurFond, transp)
        } else {
          if (mac.arretParClic()) mac.termineAction(svg, dimf, couleurFond, transp)
        }
      }
    } else {
      if (this.cadre_Ver_Proche) this.cadre_Ver_Capt = true
      if (this.cadre_Hor_Proche) this.cadre_Hor_Capt = true
    }
  }
}

// Fonction appelée après un clic de souris lorsqu'un dialogue de choix est apparu
// ou lorsqu'un seul objet était proche de la souris
OutilPointageCapture.prototype.callBackAfterChoice = function () {
  var app = this.app
  var doc = app.doc
  var elc = app.elementCapture
  var affEffectif
  if (elc !== null) {
    if (elc.estDeNature(Nat.or(NatObj.NAffLieAPoint, NatObj.NLieuObjet))) {
      if (elc.estDeNature(NatObj.NLieuObjet)) {
        // l'affichage générant le lieu d'objets étant souvent caché, son rectAff.x
        // et son rectAff.y peuvent être nuls.
        affEffectif = elc.elementAssocie
      } else {
        affEffectif = elc
      }
      this.xdecalageInitialRelatifAffichage = affEffectif.decX
      this.ydecalageInitialRelatifAffichage = affEffectif.decY
      this.xAffInitial = affEffectif.xNom
      this.yAffInitial = affEffectif.yNom
      /// ////////////////////////////////////
      // Supprimé version 6.0. Inutile
      // this.rectAffxInitial = aff.rectAff.x;
      // this.rectAffyInitial = aff.rectAff.y;
      /// /////////////////////////////////////
    }
    app.listeARecalculer.retireTout()
    app.listeARecalculer.ajouteObjetsDependantsSauf(app.elementCapture, doc.listePr, null)
  }
}
OutilPointageCapture.prototype.deviceup = function () {
  this.app.doc.type = '' // Pour autoriser à nouveau tous les événements souris ou touch
  var app = this.app
  var svg = this.app.svgFigure
  var doc = this.app.doc
  this.cadre_Ver_Capt = false
  this.cadre_Hor_Capt = false
  if (app.elementCapture !== null) {
    app.elementCapture = null
    // Ligne suivante modifié version 6.4.8 (ajout du dernier paramètre)
    app.listeARecalculer.update(svg, doc.couleurFond, doc.opacity, true, true)
    app.listeARecalculer.retireTout()
    app.outilCapt.saveFig()
    document.body.style.cursor = 'default'
  }
}

OutilPointageCapture.prototype.touchend = function () {
  this.deviceup()
}

OutilPointageCapture.prototype.touchcancel = function () {
  this.deviceup()
}

OutilPointageCapture.prototype.mouseup = function () {
  this.deviceup()
}
