/*
 * Created by yvesb on 11/02/2017.
 */
/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */
const NatObj = require('../types/NatObj')
const kernelAdd = require('../kernel/kernelAdd')
const OutilPointage = require('./OutilPointage')

module.exports = OutilPointageExecMac

/**
 * Outil de pointage servant exécuter des macros. Utilisé par OutilExecutionMacro
 * Hérite de OutilPointage
 * @param app L'application propriétaire
 * @constructor
 */
function OutilPointageExecMac (app) {
  OutilPointage.call(this, app)
  this.objetDesigne = null // Sera l'objet transmis à l'outil après d'éventuelles boîtes de dialogue de choix
}
OutilPointageExecMac.prototype = new OutilPointage()

OutilPointageExecMac.prototype.mousemove = function (evt) {
  this.devicemove(evt, 'mouse', kernelAdd.mousePosition)
}
OutilPointageExecMac.prototype.touchmove = function (evt) {
  this.devicemove(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageExecMac.prototype.devicemove = function (evt, type, fonc) {
  var point, nbObjetsProches
  var app = this.app
  var doc = app.doc
  var svg = app.svgFigure
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  point = fonc(svg, evt, app.zoomFactor)
  nbObjetsProches = doc.listePr.procheDe(NatObj.NMacro,
    point, app.infoProx, app.listeExclusion, false, type)
  app.outilExecutionMacro.cursor = (nbObjetsProches > 0) ? 'default' : 'crosshair'
}

OutilPointageExecMac.prototype.mousedown = function (evt) {
  this.devicedown(evt, 'mouse', kernelAdd.mousePosition)
}

OutilPointageExecMac.prototype.touchstart = function (evt) {
  this.devicedown(evt, 'touch', kernelAdd.touchPosition)
}

OutilPointageExecMac.prototype.devicedown = function (evt, type, fonc) {
  var app = this.app
  var liste = app.listePr
  var svg = app.svgFigure
  var doc = app.doc
  var dimf = app.dimf
  var couleurFond = doc.couleurFond
  var opacity = doc.opacity
  // Sur les périphériques mobiles il peut y avoir deux événements générés quand on touche l'écran : onmousedown et ontouchstart
  // if (doc.type && (doc.type != type)) return;
  if ((type === 'mouse') && (doc.type === 'touch')) return
  doc.type = type
  if (liste.macroEnCours !== null) {
    var mac = liste.macroEnCours.macroEnCours()
    if ((mac.className === 'CMacroPause') && (mac.dureePause === 0)) {
      mac.passageMacroSuiv(svg, dimf, couleurFond, opacity)
      return
    }
    if ((mac !== null) && (mac.className === 'CMacroApparition') && mac.executionPossible()) {
      mac.execute(svg, dimf, couleurFond, opacity, true)
      // Attention mac ne pointe plus forcément sur la macro en cours
      if ((liste.macroEnCours !== null) && !liste.macroEnCours.macroEnCours().executionEnCours) { liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, opacity) }
      return
    }
  }
  var point = fonc(svg, evt, app.zoomFactor)
  var info = app.infoProx
  var nbObjetsProches = liste.procheDe(NatObj.NMacro,
    point, info, app.listeExclusion, false, type)
  if (nbObjetsProches > 0) {
    mac = info.infoParType[NatObj.indiceMacro].premierVoisin
    if (mac.executionEnCours) {
      mac.macroEnCours().termineAction(svg, dimf, couleurFond, opacity)
    } else {
      if (liste.macroEnCours !== null) return
      if (mac.executionPossible()) { // Ajout version 4.8
        liste.macroEnCours = mac
        mac.setMacroLanceuse(null)
        mac.initialise()
        mac.execute(svg, dimf, couleurFond, opacity, true)
        app.outilExecutionMacro.saveFig()
      }
    }
  } else {
    if (liste.macroEnCours !== null) {
      if (liste.macroEnCours.className === 'CMacroSuiteMacros') {
        mac = liste.macroEnCours.macroEnCours()
        if (mac.arretParClic()) {
          mac.termineAction(svg, dimf, couleurFond, opacity)
          liste.macroEnCours.passageMacroSuiv(svg, dimf, couleurFond, opacity)
        }
        // passageMacroSuiv redonne éventuellement un ordre paint() suivant la nature de la macro suivante
      } else {
        // Si la macro en cours est une macro d'apparition avec clic pour objet suivant
        // on ne la désactive que si on est au dernier objet
        mac = liste.macroEnCours.macroEnCours()
        if ((mac.className === 'CMacroApparition') && mac.executionEnCours) {
          // mac.actionDansPaint(cadre);
          // cadre.paneFigure.repaint();
          this.liste.update(svg, couleurFond, opacity)
        } else {
          if (mac.arretParClic()) {
            mac.termineAction(svg, dimf, couleurFond, opacity)
          }
        }
      }
    }
  }
}
