/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

/**
 * Contient les textes utilisés dans le logiciel en langue espagnole.
 * @type {Textes}
 * @private
 */
const textesEsAdd = {
  infoabs: 'abs(x) : Valor absoluto de x',
  infointgr: 'int(x) : Parte entera de x',
  infosqrt: 'sqrt(x) : Raíz cuadrada de x',
  inforac: 'sqrt(x) : Raíz cuadrada de x',
  infosin: 'sen(x) : Seno de x',
  infocos: 'cos(x) : Coseno de x',
  infotan: 'tan(x) : Tangente de x',
  infoln: 'ln(x) : Logaritmo neperiano de x',
  infoexp: 'exp(x) : Exponencial de x',
  infoarctan: 'arctan(x) : Arco tangente de x',
  infoarccos: 'arccos(x) : Arc coseno de x',
  infoarcsin: 'arcsen(x) : Arc seno de x',
  infoch: 'cosh(x) : Coseno hiperbólico de x',
  infosh: 'senh(x) : Seno hiperbólico de x',
  infoth: 'tanh(x) : Tangente hiperbólica de x',
  infoargch: 'argcosh(x) : Argumento coseno hiperbólico de x',
  infoargsh: 'argsenh(x) : Argumento seno hiperbólico de x',
  infoargth: 'argtanh(x) : Argumento tangente hiperbólica de x',
  inforand: 'rand(x) : Número aleatorio de [0;1[',
  infoconj: 'conj(z) : Conjugado del complejo z',
  infoarg: 'arg(z) : Argumento principal del complejo z',
  infore: 're(z) : Parte real del complejo z',
  infoim: 'im(z) : Parte imaginaria del complejo z',
  infomaxi: 'máx(x,y) : Máximo entre los números x e y',
  infomini: 'mín(x,y) : Mínimo entre los números x e y',
  infogcd: 'mcd(x,y) : Máximo común divisor de x e y',
  infolcm: 'mcm(x,y) : Mínimo común múltiplo de x e y',
  infoncr: 'nCr(n,p) : Número de combinaciones de p elementos de un conjunto de n elementos',
  infonpr: 'nPr(n,p) : Número de permutaciones de p elementos de un conjunto de n elementos',
  infomod: 'mod(a,b) : Resto de la división euclidiana de a entre b',
  infosi: 'si(test,valor1,valor2) : Devuelve valor1 si el test es igual a 1 y si no valor2',
  infointegrale: 'integral(expr,var,a,b) : Valor aproximado de la integral de expr de a a b\nvar es la variable de integración',
  infoprimitive: 'primitiva(f,var,a,b): Devuelve f(b) - f(a) donde f es una fórmula de función \nvar siendo la variable de la función',
  infofact: 'fact(n) : Factorial de n (si n es un número natural)',
  infosomme: 'suma(Expr, j, inicio, fin, paso): Devuelve la suma de Expr\npara j entero variando de inicio a fin con incremento paso',
  infoproduit: 'producto(Expr,j,inicio,fin,paso): Devuelve el producto de Expr\n para j entero variando de inicio a fin con incremento de paso',
  infoLeft: 'Izquierdo(a): Miembro izquierdo de a si a es un test',
  infoRight: 'derecho(a): Miembro derecho de a si a es un test',
  infoTransp: 'transp(M) : Devuelve la traspuesta de una matriz M. \nSi M es un número, devuelve el número',
  infoDeter: 'deter(M) : Devuelve el determinante de la matriz M. \\ nSi M es un número, devuelve el número',
  OK: 'OK',
  Cancel: 'Anular',
  Oui: 'Si',
  Non: 'No',
  Dte: 'Recta',
  Ddte: 'Semirrecta',
  Seg: 'Segmento',
  Vect: 'Vector',
  Cerc: 'Circunferencia',
  Point: 'Punto',
  PointLibre: 'Punto libre',
  PointCons: 'Punto construido',
  PointLie: 'Punto ligado',
  Bary: 'Baricentro',
  PointRep: 'Punto por coordenadas',
  PointLibreEnt: 'Punto libre de coordenadas enteras',
  PointLiePoint: 'Punto ligado a un punto',
  Macro: 'Macro',
  PointParAff: 'Punto por afijo',
  Arc: 'Arco de circunferencia',
  Polygone: 'Polígono',
  Surf: 'Superficie',
  LieuPt: 'Lugar de Puntos',
  LieuPtDis: 'Lugar de puntos discreto',
  LigneBrisee: 'Poligonal',
  LieuObj: 'Lugar de objetos',
  Image: 'Imagen',
  GrapheSuiteR: 'Gráfica de sucesión real',
  GrapheSuiteComp: 'Gráfica de sucesión compleja',
  Comment: 'Visualización de texto',
  ObjetDup: 'Objeto duplicado',
  DemiPlan: 'Semiplano',
  PtIntPoly: 'Punto interior a polígono',
  PtIntCerc: 'Punto interior a circunferencia',
  EditeurFor: 'Editor de fórmula',
  PtClone: 'Punto clonado',
  DteClone: 'Recta clonada',
  DdteClone: 'Semirrecta clonada',
  SegClone: 'Segmento clonado',
  CercClone: 'Circunferencia clonada',
  Texte: 'Texto',
  desPoint: 'este punto',
  desDroite: 'esta recta',
  desSegment: 'este segmento',
  desVecteur: 'este vector',
  desDemiDroite: 'esta semirrecta',
  desCercle: 'esta circunferencia',
  desArc: 'este arco',
  desPolygone: 'este polígono',
  desLigneBrisee: 'esta poligonal',
  desSurface: 'esta superficie',
  desDemiPlan: 'este semiplano',
  desCommentaire: 'este comentario',
  desAffichageValeur: 'esta visualización de valor',
  desMacro: 'esta macro',
  desMarqueAngle: 'esta marca de ángulo',
  desMarqueSegment: 'esta marca de segmento',
  desGrapheSuiteRec: 'esta gráfica de sucesión recurrente',
  desGrapheSuiteRecComplexe: 'esta gráfica de sucesión recurrente compleja',
  desLieuPoint: 'este lugar de puntos',
  desLieuObjet: 'este lugar de objetos',
  desImage: 'esta imagen',
  desObjetDuplique: 'este objeto duplicado',
  desAPreciser: 'a precisar',
  desIntersection: 'esta intersección',
  desLatex: 'Esta visualización LaTeX',
  desEditeur: 'Este editor de fórmula',
  Annuler: 'Anular',
  Refaire: 'Rehacer',
  Creer: 'Crear',
  Capt: 'Capturar',
  CerOA: 'Circunferencia por centro y punto',
  CerOR: 'Circunferencia por centro y radio',
  ArcPetit: 'Arco menor de circunferencia',
  ArcGrand: 'Arco mayor de circunferencia',
  ArcDirect: 'Arco de circunferencia directo',
  ArcIndirect: 'Arco de circunferencia indirecto',
  ArcPetitParAng: 'Arco menor de circunferencia por ángulo',
  ArcGrandParAng: 'Arco mayor de circunferencia por ángulo',
  ArcDirectParAng: 'Arco de circunferencia directo por ángulo',
  ArcIndirectParAng: 'Arco de circunferencia indirecto por ángulo',
  Rapporteur: 'Transportador',
  ArcParRapporteur: 'Arco menor por transportador',
  ArcGrandParRapporteur: 'Arco mayor por transportador',
  LieuParPtLie: 'Lugar de puntos ligados por punto ligado',
  LieuParVariable: 'Lugar de puntos ligados por variable',
  LieuDiscretParPtLie: 'Lugar de puntos no ligados por punto ligado',
  LieuDiscretParVariable: 'Lugar de puntos no ligados por variable',
  LieuObjetParPtLie: 'Lugar de objeto por punto ligado',
  LieuObjetParVariable: 'Lugar de objeto por variable',
  MesAngNor: 'Medida de ángulo no orientado',
  Fonc: 'Función real',
  SolutionEq: 'Solución de ecuación',
  Derivee: 'Cálculo de derivada',
  DemiDt: 'Semirrecta',
  DtAB: 'Recta por dos puntos',
  DtPar: 'Paralela',
  DtPer: 'Perpendicular',
  DtBis: 'Bisectriz',
  DtMed: 'Mediatriz',
  DtHor: 'Recta horizontal',
  DtVer: 'Recta vertical',
  PtLib: 'Punto libre',
  CentreGrav: 'Centro de gravedad',
  Milieu: 'Punto medio',
  MarqueSeg: 'Marca de segmento',
  MarqueAng: 'Marca de ángulo',
  MarqueAngOr: 'Marca de ángulo orientado',
  MarqueAngOrSD: 'Marca de ángulo orientado de sentido directo',
  MarqueAngOrSI: 'Marca de ángulo orientado de sentido indirecto',
  Carre: 'Cuadrado directo',
  TriangleEq: 'Triángulo equilátero directo',
  Parallelog: 'Paralelogramo',
  PtParCoord: 'Punto por coordenadas',
  PtParAff: 'Punto por afijo',
  MarquerPt: 'Marcar punto para traza',
  DemarquerPt: 'Desmarcar punto para traza',
  Punaiser: 'Fijar un punto móvil',
  Depunaiser: 'Liberar un punto fijo',
  Rideau: 'Desenmascarar un objeto oculto',
  Gomme: 'Ocultar un objeto',
  Recalculer: 'Recalcular la figura',
  TranslationFigure: 'Trasladar la figura',
  ExecutionMacro: 'Ejecutar una macro',
  Surface: 'Superficie delimitada por circunferencia, polígono, arco o lugar de puntos',
  Surface2Lieux: 'Superficie delimitada por dos lugares de puntos',
  SurfaceLieuDroite: 'Superficie delimitada por lugar y recta',
  SurfaceLieu2Pts: 'Superficie delimitada por lugar y dos puntos',
  New: 'Nueva figura',
  PtBaseEnt: 'Punto libre de coordenadas enteras',
  PtInterieur: 'Punto interior a circunferencia o polígono',
  DtParEq: 'Recta por ecuación',
  Latex: 'Visualización LaTeX',
  LatexLiePt: 'Visualización Latex ligada a un punto',
  Commentaire: 'Visualización de texto',
  CommentaireLiePt: 'Visualización de texto ligada a un punto',
  AffichageValeur: 'Visualización de valor',
  AffichageValeurLiePt: 'Visualización de valor ligado a un punto',
  CreationLiaison: 'Creación de una ligadura entre un punto libre y un objeto',
  SuppressionLiaison: 'Supresión de una ligadura punto-objeto',
  Rot: 'Rotación',
  Hom: 'Homotecia',
  Sim: 'Semejanza',
  Sup: 'Supresión de un objeto gráfico',
  Int: 'Intersección',
  Proj: 'Proyectado ortogonal',
  Angle: 'Ángulo',
  Rapport: 'Razón',
  Rayon: 'Radio',
  SymAxiale: 'Simetría axial',
  SymCentrale: 'Simetría central',
  Trans: 'Traslación',
  ChoixCouleur: 'Color predefinido',
  ChoixStyleTrait: 'Estilo de trazo',
  ChoixEpaisseur: 'Espesor de trazo',
  ChoixOpacite: 'Transparencia para las superficies',
  ChoixStylePoint: 'Estilo de punto',
  ChoixMarqueSeg: 'Estilo de marca de segmento',
  ChoixStyleFleche: 'Estilo de flecha',
  ChoixMarqueAng: 'Estilo de marca de ángulo',
  ChoixStyleRemp: 'Estilo de relleno para las superficies',
  CouleurPerso: 'Color personalizado',
  Fermer: 'Cerrar',
  AngleRot: 'Ángulo de la rotación',
  RapportHom: 'Razón de la homotecia',
  RayCer: 'Radio de la circunferencia',
  AngArc: 'ángulo del arco',
  Avert: 'Advertencia',
  ErrSyntaxe: 'Error de sintaxis',
  ErrCalculNonExist: 'Atención : el resultado no existe',
  DejaCree: 'Un mismo objeto ya ha sido creado',
  ChoixEntre: 'Elegir entre',
  PremierObj: 'El primer objeto',
  DernierObj: 'El último objeto',
  ChoixAPrec: 'Elección a precisar',
  rotDlg: 'Ángulo de rotación',
  ChoixTypeDlg1: 'Varios tipos de objetos están<br/>próximas al lugar cliqueado<br/>Elegir entre: ',
  ChoixValeur: 'Elección de un valor',
  ChoixValeurDlg1: 'Elegir un valor',
  ChoixValeurDlg2: 'a continuación',
  ChoixValeurDlg3: 'Información sobre el valor seleccionado:',
  ChoixVariableDlg1: 'Elegir una variable',
  NewRepDlg: 'Nueva figura con referencial',
  NewRepDlg1: 'Ortonormal',
  NewRepDlg2: 'Ortogonal',
  NewRepDlg3: 'Oblicuo',
  NewRepDlg4: 'Oblicuo normado',
  NewRepDlg5: 'Cuadriculado vertical',
  NewRepDlg6: 'Cuadriculado horizontal',
  NewRepDlg7: 'Punteado',
  NewRepDlg8: 'Crear ejes en caso necesario',
  NewRepDlg12: 'Sin graduación',
  NewRepDlg13: 'Graduación simple',
  NewRepDlg14: 'Graduación para trigonometría',
  NewRepDlg15: 'Con vectores',
  testEquivalenceDlg4: 'Condición de existencia:',
  vardlg1: 'El valor mini debe ser inferior al valor maxi',
  vardlg2: 'El paso es incorrecto',
  vardlg3: 'El valor actual es incorrecto',
  lieuObjetDlg1: 'Número de posiciones<br/>Para generar el lugar: <br/>(de 2 a 100000)',
  PtIntDlg1: 'El lugar cliqueado no es interior',
  DerDlg1: 'Nombre de la derivada:',
  DerDlg2: 'Derivada de:',
  InfoFonc: 'Información sobre la función:',
  TgteDlg1: 'Tangente a la curva de',
  TgteDlg2: 'En el punto de abscisa:',
  TgteDlg3: 'Nombre del coeficiente director',
  SolEqDlg1: 'Solución aproximada de una ecuación sobre el intervalo [a;b]',
  SolEqDlg2: 'Nombre de la solución : ',
  SolEqDlg3: 'Nombre de la incógnita',
  SolEqDlg4: 'Ecuación a resolver',
  SolEqDlg5: 'Valor de a : ',
  SolEqDlg6: 'Valor de b : ',
  SolEqDlg7: 'Incertidumbre:',
  MaxDlg1: 'Abscisa del máximo de la función sobre [a;b]',
  MaxDlg2: 'Nombre del máximo:',
  MinDlg1: 'Abscisa del mínimo de la función sobre [a;b]',
  MinDlg2: 'Nombre del mínimo:',
  courbeDlg1: 'Curva a trazar sobre:',
  courbeDlg2: ']-&infin;;+&infin;[',
  courbeDlg3: '][a;b][',
  courbeDlg4: '][a;+&infin;[',
  courbeDlg5: ']-&infin;;a][',
  courbeDlg7: 'Punto ligado generador oculto',
  courbeDlg8: 'Punto generador de la curva oculto',
  courbeDlg9: 'Gestión auto de las discontinuidades',
  courbeDlg10: 'Número de puntos de la curva:',
  suiteRecDlg1: 'Nombre de la sucesión:',
  suiteRecDlg2: 'Función asociada',
  graphesrDlg1: 'Trazos de recuperación en abscisa',
  graphesrDlg2: 'Puntos unidos por segmentos',
  graphesrDlg3: 'sucesión asociada',
  saveDlg1: 'Ingresar el nombre<br/>del archivo (sin sufijo): ',
  NomCalcul: 'Nombre del cálculo',
  NomFonction: 'Nombre de la función',
  Formule: 'Fórmula',
  Calcul: 'Cálculo',
  CalculComp: 'Cálculo complejo',
  CalculReel: 'Cálculo real',
  Variable: 'Variable',
  AffVal: 'Visualización de valor',
  NomVar: 'Nombre de la variable',
  ValMin: 'Valor Mini :',
  ValMax: 'Valor maxi :',
  ValPas: 'Valor del paso :',
  ValAct: 'Valor actual :',
  DlgAss: 'Diálogo asociado :',
  Fonction: 'Función',
  FoncComp: 'Función compleja',
  TypeAPrec: 'Especificar el tipo',
  NomInvalide: 'El nombre no es válido',
  ExisteMemeNom: 'Ese nombre ya ha sido atribuido',
  ch6: 'Él o los puntos de intersección ya existen',
  ch18: 'No existe',
  ch34: 'Longitud unida de la figura',
  ch87: 'definida por',
  ch109: 'Objetos no secantes',
  ch112: 'Derivada de la función',
  ch185: 'La ecuación debe depender de la incógnita',
  ch186: 'Test de equivalencia de cálculos',
  ch187: 'Con',
  ch188: 'Sin',
  ch189: 'Reemplazamiento de valores',
  ch190: 'derivada parcial de',
  ch191: 'con respecto a la variable',
  ch192: 'Test de dependencia de la función',
  ch196: 'Test de dependencia de naturaleza del operador',
  ch200: 'Test de factorización de',
  chinfo17: 'Medida de la longitud',
  chinfo29: 'Medida del ángulo geométrico',
  chinfo30: 'Medida del ángulo orientado',
  chinfo31: 'Abscisa de',
  chinfo32: 'en el referencial',
  chinfo71: 'Área',
  chinfo72: 'del polígono',
  chinfo73: 'Longitud',
  chinfo74: 'de la poligonal',
  chinfo77: 'Valor mini:',
  chinfo78: 'Valor maxi:',
  chinfo79: 'Valor del paso:',
  chinfo80: 'Valor actual:',
  chinfo81: 'Ordenada de',
  chinfo84: 'asociada a la función',
  chinfo86: 'Número de términos:',
  chinfo93: 'Valor aproximado de la integral',
  chinfo94: 'de la función',
  chinfo95: 'Cota inferior:',
  chinfo96: 'Cota superior:',
  chinfo97: 'Valor actual:',
  chinfo98: 'Solución aproximada',
  chinfo99: 'de la ecuación',
  chinfo100: 'comprendida entre',
  et: 'y',
  chinfo102: 'Precisión:',
  chinfo107: 'del complejo',
  chinfo108: 'Afijo complejo',
  chinfo109: 'del punto',
  chinfo110: 'medido en el referencial',
  chinfo112: 'Módulo',
  chinfo115: 'del valor de',
  chinfo116: 'indiciado para los valores de',
  chinfo117: 'índice mini:',
  chinfo118: 'índice maxi:',
  chinfo153: 'Medida del coeficiente director de',
  chinfo157: 'Producto escalar del vector',
  chinfo158: 'y del vector',
  chinfo160: 'Productoria',
  chinfo164: 'Punto de afijo',
  chinfo176: 'Abscisa del máximo',
  chinfo177: 'Abscisa del mínimo',
  chinfo178: 'Abscisa en el origen',
  chinfo179: 'Ordenada en el origen',
  chinfo186: 'Segundo término:',
  chinfo180: 'Unidad sobre el eje de las abscisas',
  chinfo181: 'Unidad sobre el eje de las ordenadas',
  Valeurs: 'Valores',
  Fonctions: 'Funciones',
  ListeFonctions: 'Lista de funciones',
  avRapNul: 'La razón debe ser no nula',
  avRay: 'El radio debe ser estrictamente positivo',
  ch37: 'Uno o varios objetos que dependen de ese objeto serán también suprimidos. ¿Confirmas la supresión?',
  ch38: 'Suprimir ese objeto eliminará la intersección de dos objetos. ¿Confirmas la supresión?',
  ModifObjGraph: 'Modificación de objeto gráfico',
  ChoixTrans1: 'Varios objetos son imágenes por la misma transformación<br/>que ese objeto.<br/>Quieres que la modificación se aplique',
  ChoixTrans2: 'a todos los demás objetos imagen',
  ChoixTrans3: 'a ese objeto solo',
  RayPix: 'Radio en pixeles',
  ModifObj: 'Modificar objeto',
  ModifObjNum: 'Modificar objeto numérico',
  Palette: 'Cambiar el color o el estilo de un objeto',
  Nommer: 'Nominar un punto o una recta',
  CaptNom: 'Desplazar un nombre',
  NouvelleFigAvecRep: 'Nueva figura con referencial',
  AvertDirty: 'La figura actual no ha sido guardada.<br/>¿Quieres continuar?',
  newFig1: 'Figura con referencial',
  newFig3: 'Figura sin referencial con longitud unidad',
  newFig4: 'Figura sin referencial y sin longitud unidad',
  newFig5: 'Figura de base',
  newFig6: 'Figura con referencial modificable sin vectores',
  newFig7: 'Figura con referencial modificable con vectores',
  newFig8: 'Figura con gráficos de funciones o sucesione',
  newFig9: 'Figura de probabilidades',
  newFig10: 'Figura 3D de base',
  newFig11: 'Figura por código Base64',
  Abscisse: 'Abscisa',
  Ordonnee: 'Ordenada',
  Affixe: 'Afijo',
  Repere: 'Referencial',
  PtLie: 'Punto ligado',
  MesLong: 'Medida de longitud',
  MesAngOr: 'Medida de ángulo orientado',
  MesAbs: 'Medida de abscisa',
  dans: 'en',
  de: 'de',
  Modifier: 'Modificar',
  Supprimer: 'Suprimir',
  ObjNum: 'Objetos numéricos',
  TracerCourbe: 'Trazar curva',
  nbPtsCourbe: 'Número de puntos de la curva',
  nbPtsCourbeMsg: 'El número de puntos debe estar comprendido entre 4 y 5000',
  NomMesure: 'Nombre de la medida',
  OrAbs: 'Origen de las abscisas : ',
  OrOrd: 'Origen de las ordenadas : ',
  Unitex: 'Unidad en abscisas : ',
  Unitey: 'Unidad en ordenadas : ',
  PtLibreEnt: 'Punto libre de coordenadas enteras',
  AbsLibres: 'Abscisas libres',
  OrdLibres: 'Ordenadas libres',
  AbsMin: 'Abscisse<br/> mini',
  AbsMax: 'Abscisse<br/> maxi',
  OrdMin: 'Ordonnée<br/> mini',
  OrdMax: 'Ordonnée<br/> maxi',
  MesAbsRep: 'Medida de abscisa en un referencial',
  MesOrdRep: 'Medida de ordenada en un referencial',
  MesAffRep: 'Medida de afijo',
  MesCoefDir: 'Medida de coeficiente director',
  CoefDir: 'Coeficiente director',
  DtParCoef: 'Recta por coeficiente director',
  NewFig: 'Nueva figura',
  MesLongLigne: 'Medida de la longitud de una poligonal',
  MesAire: 'Medida de área de polígono',
  EqDroite: 'Ecuación de la recta',
  ch41: 'Un polígono debe tener al menos 3 vértices',
  ch42: 'Una poligonal debe tener al menos 3 vértices',
  UniteAngFig: '<b>Unidad de ángulo de la figura</b>',
  Degre: 'Grado',
  Radian: 'Radián',
  Nom: 'Nombre',
  NomMasque: 'Nombre oculto',
  EntreeNom: 'Entrada del nombre',
  ZoomPlus: 'Zoom +',
  ZoomMoins: 'Zoom -',
  ModeTrace: 'Interruptor del modo traza',
  Valeura: 'Valor de a',
  Valeurb: 'Valor de b',
  TransParCoord: 'Traslación de coordenadas (a,b)',
  PtPunaise: 'El punto ha sido fijado',
  PtDepunaise: 'El punto ha sido liberado',
  PtMarque: 'El punto ha sido marcado para la traza',
  PtDemarque: 'El punto ha sido desmarcado para la traza',
  NbPts: 'Número de puntos',
  LieuFerme: 'Lugar cerrado',
  GestAuto: 'Gestión auto de discontinuidades',
  Epaisseur: 'Grosor',
  Align: 'Alineación',
  Taille: 'Tamaño',
  EffFond: 'Borrado del fondo',
  EncSans: 'Sin marco',
  EncSimple: 'Marco simple',
  Enc3D: 'Marco 3D',
  CoulFond: 'Color<br/>de fondo',
  ModeAff: 'Modo de visualización',
  Valeur: 'Valor',
  EnTete: 'Encabezado:',
  SuiviDe: 'Seguido de:',
  Decimales: 'Decimales',
  AffIci: 'Visualización Aquí',
  AffLatex: 'Visualización LaTeX',
  CodeLatex: 'Código LaTeX',
  Apercu: 'Vista',
  InsVal: 'Inserción de valor',
  TexteAAff: 'Texto a visualizar',
  Vide: 'El texto está vacío',
  OpImp: 'Operación imposible',
  SupLiaisonImp1: 'La ligadura no puede ser suprimida porque ese punto ligado es un objeto final de construcción',
  SupLiaisonImp2: 'La ligadura no puede ser suprimida porque las posiciones de ese punto ligado son utilizadas por un lugar de puntos, lugar de objetos o una macro',
  PtParAbs: 'Punto por abscisa relativa a dos puntos',
  SuppressionLiaisonAff: 'Supresión de enlace visualización-punto',
  CreationLiaisonAff: 'Creación de enlace visualización-punto',
  indCapt: 'Captura un punto móvil, una marca de ángulo o una visualización',
  indPtLib: 'Cliquear para crear un punto libre',
  ind2Pt1: 'Cliquear sobre el primer punto',
  ind2Pt2: 'Cliquear sobre el segundo punto',
  ind3Pt3: 'Cliquear sobre el tercer punto',
  indInt: 'Cliquear en la intersección de los dos objetos o sobre los dos objetos  uno después del otro',
  indPtBaseEnt: 'Cliquear para crear un punto libre de coordenadas enteras',
  indProj1: 'Cliquear sobre la recta sobre la cual proyectar (o segmento o semirrecta)',
  indProj2: 'Cliquear sobre el punto a proyectar ',
  indPtInt1: 'Cliquear sobre un polígono o una circunferencia',
  indPtInt2: 'Cliquear en el interior de la superficie amarilla',
  indCaptNom: 'Deslizar un nombre de punto o de recta',
  intPtParAbs1: 'Cliquear sobre el punto de abscisa 0',
  intPtParAbs2: 'Cliquear sobre el punto de abscisa 1',
  indCrLiaison1: 'Cliquear sobre un punto libre para ligarlo a un objeto',
  indCrLiaison2: 'Cliquear sobre el objeto al cual ligar el punto',
  indSupLiaison: 'Cliquear sobre un punto ligado para transformarlo en un punto libre',
  indPun: 'Cliquear sobre un punto para fijarlo',
  indDepun: 'Cliquear sobre un punto para liberarlo',
  indMarq: 'Cliquear sobre un punto para que deje una traza',
  indDemarq: 'Cliquear sobre un punto para que no deje una traza',
  indParDtePt1: 'Cliquear sobre une recta, un segmento, una semirrecta o un punto',
  indPt: 'Cliquear sobre un punto',
  indDte: 'Cliquear sobre una recta',
  indSeg: 'Cliquear sobre un segmento',
  indPoly: 'Cliquear sobre un polígono',
  indPolOuLig: 'Cliquear sobre un polígono o una poligonal',
  indSurface: 'Cliquear sobre una circunferencia, un polígono, un arco de circunferencia o un lugar de puntos',
  indTrans1: 'Cliquear sobre el origen del vector o sobre el vector de la traslación',
  indTrans2: 'Cliquear sobre el extremo del vector de la traslación',
  indImTrans: 'Cliquear sobre un objeto para obtener su imagen',
  indCrPoly: 'Haga clic en los vértices del polígono y luego haga clic en el botón\nSTOP de la derecha o sobre el primer punto para cerrar el polígono.',
  indCrLigne: 'Cliquear sobre los vértices de la poligonal luego cliquear sobre el botón STOP de la derecha',
  indCentre: 'Cliquear sobre el centro',
  indPtCer: 'Cliquear sobre un punto de la circunferencia',
  indLieu1: 'Cliquear sobre le punto cuya traza generará el lugar',
  indLieu2: 'Cliquear sobre le punto ligado cuyas posiciones generarán el lugar',
  indPtLie: 'Cliquear sobre un objeto para crear un punto ligado a ese objeto',
  indCentSym: 'Cliquear sobre el centro de la simetría',
  indAxeSym: 'Cliquear sobre el eje de la simetría',
  indCentRot: 'Cliquear sobre el centro de la rotación',
  indCentHom: 'Cliquear sobre el centro de la homotecia',
  indCentSim: 'Cliquear sobre el centro de la semejanza',
  indLieuObj1: 'Cliquear sobre el objeto cuyas trazas generarán el lugar',
  indEndroitAff: 'Cliquear en el lugar de la visualización',
  indCrLiaisonAff1: 'Cliquear sobre una visualización para ligarla a un punto',
  indCrLiaisonAff2: 'Cliquear sobre el punto a ligar la visualización',
  indSupLiaisonAff: 'Cliquear sobre una visualización ligada para transformarla en visualización libre',
  indSurf2Lieux1: 'Cliquear sobre el primer lugar de puntos',
  indSurf2Lieux2: 'Cliquear sobre el segundo lugar de puntos',
  indSurfLieuDte1: 'Cliquear sobre el primer borde (lugar de puntos)',
  indSurfLieuDte2: 'Cliquear sobre el segundo borde (recta)',
  indSurfLieu2pts2: 'Cliquear sobre el primer punto',
  indSurfLieu2pts3: 'Cliquear sobre el segundo punto',
  indDemiPl1: 'Cliquear sobre el borde del semiplano (recta)',
  indDemiPl2: 'Cliquear sobre un punto del semiplano',
  solutionEq: 'Solución aproximada de la ecuación',
  Der: 'Función derivada',
  Max: 'Máximo de función',
  Min: 'Mínimo de función',
  InfoSuit: 'Información sobre la sucesión:',
  CourbeFonc: 'Curva de función',
  SuiteRec: 'Sucesión real recurrente u(n+1)=f[u(n)]',
  SuiteRecComplexe: 'Sucesión compleja recurrente z(n+1)=f[z(n)]',
  Lieuhf: 'El lugar no parece visible<br/>¿Confirmas su creación?',
  Curseur: 'Cursor',
  CurHor: 'Cursor horizontal',
  CurVer: 'Cursor vertical',
  CurSmall: 'Tamaño pequeño',
  CurBig: 'Tamaño grande',
  CurEnt: 'Cursor con pasos',
  CurNom: 'Nombre del cálculo que representa el cursor:',
  CurPun: 'Fijar los extremos',
  indCurseur1: 'Cliquear sobre un punto o un lugar de la figura',
  ErrMinMax: 'El valor mini debe ser inferior al valor maxi',
  Tangente: 'Tangente a curva de función',
  Add: 'Herramientas suplementarias',
  ChoixOutil: 'Elegir una herramienta',
  Fonc2Var: 'Función real de dos variables',
  Fonc3Var: 'Función real de tres variables',
  FoncComp2Var: 'Función compleja de dos variables',
  FoncComp3Var: 'Función compleja de tres variables',
  PremTerm: 'Primer término:',
  NbTerm: 'Número de términos<br>(de 2 a 100000):',
  GrapheSuiteRec: 'Gráfica de sucesión real u(n+1)=f[u(n)]',
  GrapheSuiteRecComp: 'Gráfica de sucesión compleja z(n+1)=f[z(n)]',
  OptionsFig: 'Opciones',
  CodeBase64: 'Obtener el código base64 de la figura',
  Base64: 'Código base64 de la figura:',
  FigParCode: 'Pegar aquí el código base64 de la figura',
  Base64Err: 'El código base64 no es válido',
  Open: 'Abrir archivo',
  Save: 'Guardar',
  NomFichierErr: 'El nombre del archivo no es correcto',
  FichierErr: 'Ningún archivo válido seleccionado',
  Parcourir: 'Recorrer',
  /// //////////////////////////////////////////
  PartieReelle: 'Parte real',
  PartieImaginaire: 'Parte imaginaria',
  Module: 'Módulo',
  Argument: 'Argumento principal',
  TestExistence: 'Test de existencia',
  chinfo92: 'del valor',
  Coef: 'Coeficiente:',
  Barycentre: 'Baricentro',
  indBar1: 'Cliquear sobre un punto para ponderarlo, cliquear sobre Stop para crear el baricentro',
  ErrBar: 'Creación imposible. La suma de los coeficientes es nula',
  HorsFen: 'El objeto creado está fuera de la ventana.<br/>¿Confirmas su creación?',
  unAng: 'Unidad de ángulo: ',
  niv0: 'Nivel elemental',
  niv1: 'Nivel secundaria básica',
  niv2: 'Nivel avanzado sin números complejos',
  niv3: 'Nivel avanzado con números complejos',
  SegmentParLong: 'Segmento de longitud dada',
  LongSeg: 'Longitud del segmento:',
  long: 'long',
  PolygoneReg: 'Polígono regular',
  NbSommets: 'Número de vértices: <br/> (de 3 a 100)',
  indPolReg: 'Cliquear sobre el primer vértice',
  /// ///////////////////////////////
  Animation: 'Animación de un punto ligado',
  indAnim: 'Cliquear sobre el punto parpadeante para animarlo',
  AnimCycl: 'Animación cíclica',
  Freq: 'Frecuencia (en s)',
  NbPtsPourAnim: 'Número de puntos<br/>(de 2 a 10000): ',
  InvSens: 'Invertir el sentido',
  OptionsAnim: 'Opciones de animación',
  /// /
  PtParMultVec: 'Punto M tal que Vect(AM) = kVect(u)',
  PtParMultVecDlg2: 'Valor de k',
  indVect: 'Cliquear sobre uno vector',
  indVect2: 'Cliquear sobre el segundo vector',
  PtParSommeVec: 'Punto M tal que Vect(AM) = Vect(u) + Vect(v)',
  DeriveePart: 'Derivada parcial',
  DerPartDlg1: 'Nombre de la derivada:',
  DerPartDlg2: 'Derivada parcial con respecto a:',
  /// //
  indImage: 'Cliquear para crear una imagen',
  ChoixImFond: 'Elección de la imagen de fondo',
  DelImFond: 'Quitar la imagen de fondo',
  AvertNonAnnul: 'Esta acción no se puede deshacer.<br/>¿Confirmar?',
  EditeurFormule: 'Editor de fórmula',
  EditForDlg1: 'Modifica la fórmula:',
  EditForDlg2: '(encabezamiento vacío por defecto)',
  EditForDlg3: 'Signos de multiplicación implícitos<br/>(variables a un caracter)',
  EditForDlg4: 'Visualización de la fórmula en LaTeX',
  EditForDlg5: 'Pre-código LaTeX',
  EditForDlg6: 'Visualización inicial de la fórmula',
  EditForDlg7: 'Visualización en tiempo real de la fórmula',
  Colonnes: 'Columnas',
  RenCalcDlg: 'Escribir a continuación el nuevo nombre<br/>luego cliquear sobre el botón Renombrar',
  RenommerCalcul: 'Renombrar un cálculo',
  Renommer: 'Renombrar',
  /// /
  AvertRecDeb: 'El objeto fue reclasificado hacia el comienzo',
  AvertRecFin: 'El objeto fue reclasificado hacia el final',
  AvertRecImp: 'El objeto no pudo ser reclasificado',
  indRecDeb: 'Cliquear sobre un objeto para reclasificarlo hacia el comienzo',
  indRecFin: 'Cliquear sobre un objeto para reclasificarlo hacia el final',
  ReclassDebObjGra: 'Reclasificarlo hacia el comienzo',
  ReclassFinObjGra: 'reclasificarlo hacia el final',

  NbGradx: 'Número de graduaciones sobre<br/> el eje de las abscisas :',
  NbGrady: 'Número de graduaciones sobre<br/> el eje de las ordenadas :',
  GraduationAxes: 'Graduar los ejes de un referencial',
  /// / Ajouts pour historique
  chinfo202: 'de centro',
  chinfo203: 'y origen',
  chinfo204: 'terminando en',
  chinfo205: ', ángulo',
  chinfo191: 'Arco de circunferencia directo',
  chinfo192: 'Arco de circunferencia indirecto',
  chinfo37: 'Arco de circunferencia imagen de',
  chinfo193: 'Punto de intersección de',
  chinfo194: 'con',
  chinfo227: 'que no sea',
  chinfo150: 'Centro',
  chinfo35: 'Circunferencia imagen de',
  chinfo225: 'de eje',
  chinfo195: 'Circunferencia de centro',
  par: 'por',
  chinfo36: 'Semirrecta imagen de',
  bord: 'borde',
  chinfo196: 'que pasa por',
  chinfo50: 'Recta horizontal',
  chinfo51: 'Recta vertical',
  chinfo34: 'Recta imagen de',
  chinfo152: 'Recta de coeficiente director',
  chinfo175: 'Recta de ecuación',
  chinfo6: 'Recta perpendicular a',
  chinfo88: 'Gráfico de la sucesión recurrente real  ',
  chinfo90: 'Gráfico de la sucesión recurrente compleja',
  chinfo167: 'Visualización LaTex',
  chinfo207: 'generado por',
  chinfo126: 'Macro que envía el punto ligado',
  chinfo230: 'a la posición de',
  chinfo127: 'Macro afectando el valor de la variable',
  chinfo128: 'a la variable',
  chinfo129: 'Macro revisualizando toda la figura',
  chinfo124: 'Macro de animación de punto ligado',
  chinfo119: 'Macro de aparición de ',
  chinfo130: 'Macro de bucle de animación',
  chinfo133: 'a partir de los valores de la variable',
  chinfo132: 'Macro de parpadeo de',
  MacConsItDlg1: 'Macro de construcción iterativa',
  MacConsRecDlg1: 'Macro de construcción recursiva',
  chinfo122: 'Macro de disminución',
  chinfo122Bis: 'de la variable',
  chinfo120: 'Macro que esconde los objetos ',
  chinfo121: 'Macro de incremento',
  chinfo123: 'Macro abriendo el archivo sonoro',
  chinfo170: 'Macro que activa el modo traza',
  chinfo171: 'Macro que desactiva el modo traza',
  chinfo134: 'Macro pausa de una duración igual a',
  chinfo135: 'Macro ejecutante sucesión de macros',
  chinfo136: 'Macro de traza automática de objetos por punto ligado',
  chinfo137: 'Macro de traza automática de objetos por los valores de la variable',
  chinfo165: 'marca de ángulo recto',
  chinfo66: 'Marca de ángulo no orientado',
  chinfo67: 'Marca de ángulo orientado',
  chinfo68: 'Marca de ángulo orientado en el sentido directo',
  chinfo69: 'Marca de ángulo orientado en el sentido indirecto',
  chinfo233: 'Duplicado de',
  chinfo139: 'Punto libre de coordenadas enteras',
  chinfo18: 'Punto imagen de',
  chinfo169: 'Punto interior a',
  chinfo2: 'Punto ligado a',
  chinfo16: 'Punto de abscisa',
  chinfo217: 'Punto tal que',
  chinfo218: 'vector',
  chinfo60: 'Superficie delimitada por ',
  ProtoFig: 'Protocolo de la figura',
  Const: 'Construcción',
  chinfo138: 'Comienzo de implementación de la construcción',
  rabsor: 'absor',
  rordor: 'ordor',
  ruxrep: 'uxref',
  ruyrep: 'uyref',
  rarc: 'arco',
  rcerc: 'circ',
  rcom: 'txt',
  rdd: 'sr',
  rdplan: 'splan',
  rpt: 'Pt',
  rdt: 'dt',
  redit: 'edfor',
  rgsr: 'gsrec',
  rgsc: 'gsrecc',
  rim: 'im',
  rlat: 'lat',
  rlieu: 'lug',
  rlieuOb: 'olug',
  rmac: 'mac',
  rma: 'ma',
  rmseg: 'mseg',
  raffval: 'affval',
  ObjInterm: 'Objetos intermediarios',
  liea: 'Ligado a',
  chinfo197: 'y radio',
  chinfo140: 'Punto de coordenadas',
  chinfo28: ', razón',
  ObjMas: 'Objeto oculto',
  ObjMasVis: 'Objetos ocultos visibles',
  Protocole: 'Protocolo',
  ModifBar: 'Modificación de baricentro',
  /// ////////////////
  RapportInv: 'Razón de inversión',
  ImageInv: 'imagen de punto por inversión',
  indCentInv: 'Cliquear sobre el centro de la inversión',
  indCer: 'Cliquear sobre uno circulo',
  CentreCercle: 'Crear Centro de una circunferencia',
  Divers: 'Herramientas diversas',
  ObjetDuplique: 'Crear el duplicado de un objeto',
  indDup: 'Cliquear sobre un objeto para duplicarlo',
  ObjetClone: 'Crear un clon de un objeto',
  indClone: 'Cliquear sobre un punto, un segmento, una recta, una semirrecta o una  circunferencia',
  NoObjNum: 'Ningún objeto numérico ha sido creado',
  MesProSca: 'Medida del producto escalar',
  indPtOuDte: 'Cliquear sobre un punto o una recta',
  Rap1: 'Cliquear sobre un punto para centrar el transportador',
  Rap2: 'Cliquear sobre un punto para alinear el cero del transportador',
  Rap3: 'Hacer girar el punto y luego cliquear cuando el ángulo es el correcto',
  Gom: 'Cliquear sobre un objeto para ocultarlo',
  Rid: 'Cliquear sobre un objeto parpadeante para mostrarlo',
  indSup: 'cliquear sobre un objeto a suprimir',
  /// ///////////
  LongUnit: 'Crear longitud unidad',
  AvertLu1: 'La longitud',
  AvertLu2: 'es ahora la longitud unidad de la figura',
  DispMeas: 'Visualización automática de las medidas de longitud y ángulo',
  /// ////////////////////////////
  Intitule: 'Título:',
  ClicSuiv: 'Cliquear para el objeto siguiente:',
  MacApp: 'Macro de aparición',
  indMacApp: 'Cliquear en los objetos que debe mostrar la macro, STOP para finalizar',
  indMacDisp: 'Cliquear en los objetos que la macro debe hacer desaparecer, STOP para finalizar',
  MacDisp: 'Macro de ocultamiento de objeto',
  MacAnim: 'Macro de animación',
  MacAnimTr: 'Macro de animación con traza',
  UnCyc: 'Un solo ciclo',
  RetFinCyc: 'Retorno del punto al final de la animación',
  EffFinCyc: 'Borrado de trazas al final del ciclo',
  StopClic: 'Animación detenida por un clic del ratón',
  DureeDon: 'Animación de duración (en 1/10s, 0 a 3000):',
  Info: 'Información',
  indAnimTr: 'Cliquear sobre los objetos que dejarán una traza, luego STOP',
  indTrAuto: 'Cliquear en el punto parpadeante cuyas posiciones generarán la traza',
  MacTraceAuto: 'Macro de traza auto por punto ligado',
  MacTraceAutoVa: 'Macro de traza auto por variable',
  MacAffValVar: 'Macro de afectación de valor a variable',
  AffectA: 'Afectar a',
  MacClign: 'Macro de parpadeo',
  indMacClign: 'Cliquear sobre los objetos para hacerlos parpadear, luego STOP',
  MacIncVar: 'Macro de incrementación de variable',
  MacDecVar: 'Macro de disminución de variable',
  MacModifVar: 'Macro de modificación de variable',
  ChoixMac: 'Elegir el tipo de macro a crear:',
  NewMac: 'Nueva macro',
  /// //////////////////////////////
  MacAffPtLie: 'Desplazamiento de punto ligado',
  ErrMAfPtLie: 'Creación imposible: ningún otro punto ligado al mismo objeto es visible',
  indMacAffPtLie1: 'Cliquear sobre el punto ligado que deberá ser desplazado',
  indMacAffPtLie2: 'Cliquear sobre el punto ligado hacia el cual el primer objeto debe ser desplazado',
  MacAffPtLieDlg: 'Macro rediseñando la figura',
  MacReaff: 'Macro de revisualización de la figura',
  MacActTr: 'Macro que activa el modo traza',
  MacDesactTr: 'Macro que desactiva el modo traza',
  MacPause: 'Macro de pausa',
  Jusqclic: 'Hasta clic',
  Durs: 'Durada en segundos',
  MacDispo: 'Macros disponibles :',
  MacAExec: 'Macros a ejecutar :',
  Ins: 'Insertar',
  AddDeb: 'Insertar al comienzo',
  AddFin: 'Insertar al final',
  Ret: 'Withdraw',
  RetAll: 'Withdraw all',
  ErrNoMac: 'Ningún macro escogido',
  MacSuiteMac: 'Macro de Ejecución de una sucesión de macros',
  MacJouantSon: 'Macro abriendo un archivo sonoro',
  Path: 'Ruta de acceso al archivo :',
  AddMac: 'Creación de macros',
  EdFor: 'Editor de fórmula',
  AnimVis: 'Animación visible',
  MacBoucAnim: 'Macro de bucle con animación',
  MacAvBou: 'Macro a ejecutar antes del primer bucle :',
  MacFinBou: 'Macro a ejecutar al final de cada bucle:',
  MacBoucTr: 'Macro de bucle con traza',
  ChoixOutils: 'Elección de las herramientas',
  Points: 'Puntos',
  Droites: 'Rectas',
  Segments: 'Segmentos',
  Cercles: 'Circunferencias',
  Polys: 'Polígonos ',
  Marques: 'Marcas',
  Lieux: 'Lugares',
  Transf: 'transformaciones',
  Mes: 'Medidas',
  Disp: 'visualizaciónes',
  ImageLibre: 'imagen libre',
  ImageLiee: 'imagen ligada',
  Calculs: 'Cálculos',
  Surfaces: 'Superficies',
  OutilsPer: 'Personalizar las herramientas disponibles',
  /// //////////////////////////////////////////////
  ChoixOutDlg1: 'Los ítems marcados corresponden :',
  ChoixOutDlg2: 'a las herramientas prohibidas',
  ChoixOutDlg3: 'a las herramientas permitidas',
  /// //////////////////////////////////////////////
  AvecGrad: 'Con graduaciones',
  PasCur: 'Paso del cursor :',
  PasGrad: 'Paso de graduación :',
  step: 'paso',
  stepGrid: 'pasograd',
  ErrPas: 'El paso debe dividir la diferencia valor maxi - valor mini',
  /// /////////////////////////////////////////////
  TestEq: 'Test de equivalencia',
  AComp: 'Elegir los cálculos a comparar :',
  RempVal: 'Remplazamiento de valores<br>antes de la comparación',
  EqDecFr: 'Equivalencia decimal - fracción irreducible',
  CondEx: 'Condición de existencia',
  TestFact: 'Test de factorización',
  FacComp: 'Ensaya la factorización de la elección de la izquierda por la de la derecha :',
  TestDepVar: 'Test de dependencia de función-variable',
  DepRap: 'Dependencia con respecto a :',
  TestEqNatOp: 'Test de equivalencia de tipo de operador',
  SomInd: 'Suma indicial',
  ProdInd: 'Producto indicial',
  ValSom: 'Valor a sumar',
  ValProd: 'Valor a multiplicar',
  VarInd: 'Variable indice',
  AffCad: 'Utilizar un cuadro de dimensiones dadas',
  Larg: 'Largo',
  Haut: 'Alto',
  dep: 'de',
  a: 'a',
  SaveDim: 'Guardar con las dimensiones del cuadro',
  ToolsBarHor: 'Herramientas de la barra horizontal',
  SuiteRec2: 'Sucesión recurrente real u(n+1)=f[n,u(n)]',
  SuiteRec3: 'Sucesión recurrente real u(n+2)=f[n,u(n+1),u(n)]',
  SuiteRecComplexe2: 'Sucesión recurrente compleja u(n+1)=f[n,u(n)]',
  SuiteRecComplexe3: 'Sucesión recurrente compleja u(n+2)=f[n,u(n+1),u(n)]',
  Integ: 'Valor aproximado de integral',
  IntegDlg1: 'Cálculo de un valor aproximado de integral de f sobre [a;b]<br/>por el método de Simpson (2n intervalos)',
  IntegDlg2: 'Función f',
  IntegDlg3: 'Valor de n : (200 max)',
  FusionImpConst: 'Fusionar las construcciones implementadas de la figura',
  ErrNomInd: 'Este punto debe nombrarse ya que está utilizado por una medida o un referencial',
  grec: 'griego',
  math: 'math',
  arrows: 'flechas',
  CodeOK: 'El código de la figura se ha copiado en el portapapeles',
  CopyOK: 'La figura se ha copiado en el portapapeles',
  SavePNG: 'Guardar como un archivo imagen PNG',
  SaveJPG: 'Guardar como un archivo imagen JPEG',
  SaveSVG: 'Guardar como un archivo imagen SVG',
  Export: 'Exportar la figura',
  ToolsDem: 'Utilice esta opción de herramienta para el siguiente inicio de software',
  CodeTikz: 'Obtener el código LaTex Tikz de la figura',
  TikzDlg1: 'Utilizar la longitud unidad de la figura',
  TikzDlg2: 'Imponer las dimensiones de la figura',
  TikzDlg3: 'Longitud en cm para la longitud unidad (entre 0.1 y 10 : ',
  TikzDlg4: 'Ancho en cm de la figura (entre 1 y 40) : ',
  TikzDlg5: 'Código comentado',
  Tikz: 'Código Tikz de la figura:',
  About: 'Acerca de MathGraph32',
  NivDem: 'Utilizar el nivel elegido para el próximo inicio del software',
  Copy: 'Copiar la figura en el portapapeles',
  fig0: 'Triángulo cualquiera',
  fig1: 'Triángulo rectángulo',
  fig2: 'Triángulo isósceles',
  fig3: 'Triángulo rectángulo isósceles',
  fig4: 'Triángulo equilátero',
  fig5: 'Cuadrado',
  fig6: 'Rectángulo',
  fig7: 'Paralelogramo',
  fig8: 'Rombo',
  fig9: 'Trapecio',
  fig10: 'Trapecio rectángulo',
  fig11: 'Trapecio isósceles',
  fig12: 'Hexágono regular',
  fig13: 'Pentágono regular',
  NewFigWith: 'Nueva figura con',
  reporthon: 'Ortonormal 3D referencial',
  reporthog: 'Ortogonal 3D referencial',
  cube: 'Cubo',
  boite: 'Paralelepípedo rectángulo',
  cyl: 'Cilindro recto',
  coneSimp: 'Cono simple',
  coneDb: 'Cono doble',
  tetra: 'Tetraedro',
  tetrareg: 'Tetraedro regular',
  pyrcarre: 'Pirámide de base cuadrada',
  pyrregbasecar: 'Pirámide regular de base cuadrada',
  pyrrect: 'Pirámide de base rectangular',
  prismtriang: 'Prisma de base triangular',
  prismtrap: 'Prisma de base trapezoidal',
  surf: 'Superficie de ecuación z=f(x,y)',
  cour: 'Curvas de funciones',
  courParam: 'Curva paramétrica',
  courPol: 'Curva polar',
  graSuiteunfn: 'Gráfico de sucesión un = f(n)',
  graSuitRecPrPt: 'Gráfico de sucesión u(n+1)=f[u(n)] con primer término capturable',
  graSuitRecPrTer: 'Gráfico de sucesión u(n+1)=f[u(n)] con edición del primer término',
  loiBin: 'Ley binomial',
  loiBinIntConf: 'Ley binomial: Intervalo de confianza de umbral, de 95%',
  loiBinApp: 'Ley binomial: Aproximación por ley normal',
  loiExp: 'Ley exponencial',
  loiNorm: 'Ley normal',
  loiNormeualpha: 'Ley normal: Cálculo de u_alpha',
  loiNormInv: 'Ley normal inversa',
  ChoixLang: 'Idioma utilizado',
  fr: 'Francés',
  en: 'Inglés',
  es: 'Español',
  Restart: 'MathGraph32 debe reiniciarse',
  Zoom: 'Zoom',
  Web: 'El sitio Web de MathGRaph32',
  CouFond: 'Color de fondo de la figura',
  Help: 'Ayuda',
  ToggleToolsAdd: 'Otras herramientas',
  CopierStyle: 'Seleccionar el estilo de un objeto',
  CopSty1: 'Cliquear sobre un objeto para seleccionar su estilo',
  marAngFigees: 'Marcas de ángulos fijas',
  affFiges: 'Visualizaciones fijas',
  EltsFiges: 'Elementos fijos',
  ExportHTML: 'Exportar HTML',
  Title: 'Título de la página : ',
  Aut: 'Autor : ',
  TextHtm1: 'Texto precedente a la figura',
  TextHtm2: 'Texto siguiente a la figura',
  HTML: 'Hoja de trabajo din&#225;mica MathGraph32',
  CrWith: 'Creado con',
  SavePNGWithUnity: 'Exportar con unidad en un archivo PNG',
  Unit: 'Longitud de la unidad en cm : ',
  DimFig: 'Dimensiones de la figura : ',
  CopyWithUnity: 'Copiar la figura en el portapapeles con longitud unidad',
  TaillePlus: 'Aumentar tamaño de las fuentes',
  TailleMoins: 'Diminuir tamaño de las fuentes',
  ImFondVis: 'Imagen de fondo Visible',
  AddObjMac: 'Agregar objetos a una macro',
  SupObjMac: 'Quitar objetos de una macro',
  AdImp: 'Agregado a la macro imposible : el objeto cliqueado se definió después de la macro.</br>Utilice la reclasificación de objetos.',
  indAdObj: 'Cliquear sobre los objetos para agregarlos a la macro',
  indSupObj: 'Cliquear sobre los objetos parpadeantes para quitarlos de la macro',
  SupImp: 'Debe quedar al menos un objeto',
  coefMult: 'Coeficiente multiplicador exportar imagenes (de 0.25 a 4) : ',
  ModePointsAuto: 'Alternar el modo de creación de puntos auto',
  DysDem: 'Utilizar el modo "dys" en el próximo arranque',
  start: 'Figura de inicio :',
  start0: 'Referencial cuadriculado',
  start1: 'Referencial punteado',
  start2: 'Sin referencial',
  AngUs: 'Ángulos usuales',
  MacAppParAut: 'Macro de aparición de objetos a partir de otra macro',
  MacDispParAut: 'Macro de ocultamiento de objetos a partir de otra macro',
  MacAppDispDlg1: 'Crear a partir de objetos de:',
  ConstFig: 'Construcciones de la figura disponible:',
  Implem: 'Implementar',
  NomConst: 'Nombre de la construcción : ',
  ModifConst: 'Modificación de las informaciones de una construcción',
  GestionConst: 'Gestionar las construcciones',
  SupConst: 'Suprimir una construcción de la figura',
  SrcNumDlg1: 'Elección de los objetos fuente numéricos',
  SrcNumDlg2: 'Haga clic sobre cada número de la izquierda y asóciele un objeto',
  SrcNumDlg3: 'Fuente nº.',
  SrcNumDlg4: 'Elemento asociado',
  SrcNumDlg5: 'Objeto requerido:',
  SrcNumDlg6: 'Elija un valor de los de a continuación',
  SrcNumDlg7: 'o ingrese una fórmula a continuación',
  SrcNumDlg8: 'Información sobre el elemento seleccionado:',
  SrcNumDlg9: 'Información sobre la construcción:',
  CopyCode: 'Copiar el código',
  CodeNotOK: 'El código no se pudo copiar en el portapapeles',
  ClicOn: 'Cliquear sobre ',
  ErrLong: 'Esta construcción no puede ser implementada porque la figura no tiene longitud unidad',
  ImpConstFig: 'Implementar una construcción de la figura',
  ValR: 'Un valor real',
  ValC: 'Un valor complejo',
  ImpImposs: 'La implementación de esta construcción es imposible.</br>Faltan los siguientes objetos:</br>',
  AddConst: 'Incorporar una construcción desde un archivo',
  SaveConst: 'Guardar una construcción de la figura',
  ChoixSrcNumConst: 'Elección de objetos fuentes numéricos',
  SrcDlg2: 'Elementos fuentes disponibles',
  SrcDlg3: 'Elementos fuentes elegidos',
  ChoixSrcGraphConst: 'Elección de objetos fuentes gráficos',
  ChoixFinGraphConst: 'Elección de objetos finales gráficos',
  indSrcGr: 'Cliquear sobre los objetos fuentes gráficos de la construcción a crear',
  indFinGr: 'Cliquear sobre los objetos finales gráficos de la construcción a crear',
  CreationConst: 'Creación de una construcción',
  ChoixFinNumConst: 'Elección de los objetos finales numéricos',
  FinDlg1: 'Elementos finnales disponibles :',
  ErrConst: 'Construcción rechazada : no se utiliza uno de los objetos fuente',
  AvConst1: 'Las variables fuentes elegidas como objeto final</br>han sido retiradas de los objetos fuentes',
  FinirConst: 'Finalizar la construcción en curso',
  courbeCrDlg6: 'Radio de los ganchos en píxeles',
  CourbeFoncCr: 'Curva de  funciones con gancho(s)',
  Couronne: 'Corona',
  ReInitConst: 'Reinicializar la construcción en curso',
  CourbeAvecTan: 'Curva por puntos y tangentes',
  CourbeAvecTanDlg1: 'Elija el referencial y el número de puntos <br/> por los cuales debe pasar la curva',
  CourbeAvecTanDlg2: 'Tendrá que hacer a continuación clic sobre los puntos y luego elegir <br/> los coeficientes directores de las tangentes',
  CourbeAvecTanDlg3: 'Coeficientes directores de las tangentes',
  CourbeAvecTanDlg4: 'Al punto:',
  indCourTan1: 'Haga clic sobre el punto n° ',
  indCourTan2: ' de la curva',
  Term: 'Finalizar',
  CourbePoly: 'Curva de función polinómica por puntos',
  AffichageEq: 'Visualización de ecuación de recta o circunferencia',
  indEq: 'Haga clic sobre una recta o una circunferencia',
  AffichageEqLie: 'AVisualización de ecuación de recta o circunferencia ligada a un punto',
  nbdec: 'nbdec',
  indAffLie: 'Cliquear sobre el punto en el cual la visualización se ligará',
  infConst: 'Informaciones sobre la construcción',
  NoMac: 'Ninguna macro',
  ChoixMacDem: 'Elegir una macro para el inicio',
  MacRecDlg1: 'Profundidad de la recursión : ',
  MacRecDlg2: 'Nivel de recursión : ',
  MacRecDlg3: 'Número de iteraciones por nivel : ',
  MacRecDlg4: 'Paso de la iteración (> = 1) : ',
  MacRecDlg5: 'Número de objetos fuente a reutilizar en cada iteración : ',
  MacRecDlg6: 'Crear sólo los objetos de la última generación',
  MacRecDlg7: 'Imponer un color para los objetos creados ',
  MacRecDlg8: 'Polígono rellenoi',
  MacRecDlg9: 'No unir los puntos finales',
  MacRecDlg10: 'Unir los puntos finales por un polígono',
  MacRecDlg11: 'Unir los puntos finales por una poligonal',
  errRec: 'Parámetros de recursión inválidos',
  errImpMax: 'Esta construcción no es válida: más de 200000 objetos creados',
  MacConsRec: 'Macro de construcción recursiva',
  ObjSrc: 'Objetos fuentes: ',
  MacItDlg1: 'Número de iteraciones (>=2) :',
  errIter: 'Parámetros de iteración inválidos',
  MacConsIter: 'Macro de construcción iterativa',
  AffichageCoord: 'Visualización de coordenadas',
  AffichageCoordLie: 'Visualización de coordenadas ligadas a un punto',
  indCoord: 'Cliquear sobre el punto cuyas coordenadas se quieren mostrar',
  ErrLongUn: 'CEsta construcción no puede ser implementada porque la figura no tiene longitud unidad',
  PapMil: 'Papel milimetrado',
  newFig2: 'Figura con referencial milimetrado',
  ObjFin: 'Objeto final de construcción',
  AbsOrRep: 'Abscisa en el origen del referencial',
  OrdOrRep: 'Ordenada en el origen del referencial',
  UnitexRep: 'Unidad x del referencial',
  UniteyRep: 'Unidad y del referencial',
  ObjInter: 'Objeto intermediario de construcción',
  ProtoNbMax: 'Protocolo restringido a los primeros 2000 objetos',
  ch150: '¿Desea que este punto libre fuente se convierta en un objeto final?',
  errInvFile: 'Archivo no es válido',
  TextFig: 'Texto de la figura',
  // Ce qui suit pour les exercices fais via la bibliothèque
  res: 'El ejercicio ha sido resuelto y está registrado',
  notres: 'El ejercicio no ha sido resuelto y está registrado',
  saved: 'La figura ha sido registrada',
  none: 'Ningún objeto ha sido construido',
  anom: 'Los objetos siguientes deben ser nominados: ',
  InsDynDlg1: 'Inserción de valor dinámico',
  InsDynDlg2: 'Signo + Impuesto',
  InsDynDlg3: 'Código a insertar',
  InsForDlg1: 'Inserción de fórmula',
  InsForDlg2: 'Fórmula de:',
  InsForDlg3: 'Fórmula  Simplificada',
  aconst: 'Faltan uno o más elementos de tipo: ',
  indRay1: 'Haga clic en el primer punto que da el radio',
  indRay2: 'Haga clic en el segundo punto que da el radio',
  CerOAB: 'Compás',
  VersionErr: 'Esta figura ha sido creada por una nueva versión de MathGraph32</br>Es necesario actualizar el software para abrirla',
  AffLong: 'Visualización de longitud orientado',
  proj: 'Proyectado ortogonal',
  sur: 'sobre',
  debArc: 'Cliquear sobre el punto de partida del arco',
  finArc: 'Cliquear sobre un punto que dé la dirección del punto final del arco',
  chinfo5: 'Recta paralela a',
  cloneCree: 'Se creó el clon del objeto',
  tag: 'Tag del objeto',
  changeTag: 'Cambiar el tag',
  tagErr: 'Este tag ya ha sido asignado',
  Matrice: 'Matriz',
  CalculMat: 'Cálculo matricial',
  NbLig: 'Número de filas',
  NbCol: 'Número de columnas',
  NomMat: 'Nombre de la matriz',
  infodotmult: 'dotmult(A,B) : Devuelve el producto término a término de las dos matrices A y B',
  infoInv: 'inv(A) : Inversa término a término de la matriz A',
  MatriceAleat: 'Matriz de valores enteros aleatorios',
  MatAleatComp: 'Matriz de valores distintos comprendidos entre ',
  MatriceParForm: 'Matriz por fórmula',
  formMat: 'si(i=j,1,0)',
  Det: 'Determinante',
  FracRat: 'Aproximación por fracción racional',
  infofrac: 'frac(x) : Devuelve una matriz fila cuyo primer elemento es el numerador<br>' +
  '\'de la fracción racional aproximada de x y la segunda el denominador.<br>' +
  'También se puede aplicar a una matriz de fila o columna.'
}

module.exports = textesEsAdd
