/*
 * MathGraph32 Javascript : Software for animating online dynamic mathematics figures
 * http://www.mathgraph32.org/
 * @version 5.1.2
 * @Author Yves Biton (yves.biton@sesamath.net)
 * @License: GNU AGPLv3 https://www.gnu.org/licenses/agpl-3.0.html
 */

// Constante contenant une chaîne de caractères formée des constructions chargées au départ du logiciel
const constructionsBase = 'TWF0aEdyYXBoSmF2YTEuMAAAABI+TMzNAANmcmH###8BAP8BAAAAAAAAAAADxQAAAokAAAAAAAAAFwAAAAYAC0NhcnJlRGlyZWN0AAAAAAACAAAAAgAAAAAJ#####wAAAAEAEUNFbGVtZW50R2VuZXJpcXVlAAAAAAAA#####wAAAAAAAAAAAAD##########wAAAAEACENTZWdtZW50AP####8BAAAAAAoAAAEAAQAAAAAAAAAB#####wAAAAEAFkNEcm9pdGVQZXJwZW5kaWN1bGFpcmUA#####wAAAAAADQAAAQEBAAAAAAAAAAL#####AAAAAQAJQ0NlcmNsZU9BAP####8AAAAAAQEAAAAAAAAAAf####8AAAABABBDSW50RHJvaXRlQ2VyY2xlAP####8AAAADAAAABP####8AAAABABBDUG9pbnRMaWVCaXBvaW50Af####8AAAAAAA0AAAEFAAIAAAAF#####wAAAAEADENUcmFuc2xhdGlvbgD#####AAAAAAAAAAH#####AAAAAQALQ1BvaW50SW1hZ2UB#####wAAAAAADQAAAQUAAAAABgAAAAcAAAAGAApUcmlhbmdsZUVxAAAAAAACAAAAAQAAAAAGAAAAAAAAAAAAAP####8AAAAAAAAAAAAA#####wAAAAMA#####wEAAAABAQAAAAAAAAABAAAAAwD#####AQAAAAEBAAAAAQAAAAD#####AAAAAQAQQ0ludENlcmNsZUNlcmNsZQD#####AAAAAwAAAAIAAAAFAf####8AAAAAAA0AAAEFAAEAAAAEAAAABgAUR3JhZHVhdGlvbkF4ZXNSZXBlcmUAAAAAAAMAAAAIAgAAACYAAAAAAAAAAAAaAAAAGgAAAAAAB25iZ3JhZHj#####AAAAAQAAAAAAB25iZ3JhZHn#####AAAAAf####8AAAABABNDQWJzY2lzc2VPcmlnaW5lUmVwAP####8ABWFic29yAAAAAP####8AAAABABNDT3Jkb25uZWVPcmlnaW5lUmVwAP####8ABW9yZG9yAAAAAP####8AAAABAApDVW5pdGV4UmVwAP####8ABnVuaXRleAAAAAD#####AAAAAQAKQ1VuaXRleVJlcAD#####AAZ1bml0ZXkAAAAA#####wAAAAEAEENQb2ludERhbnNSZXBlcmUA#####wAAAAAADgAAAQUAAAAAAP####8AAAABAA9DUmVzdWx0YXRWYWxldXIAAAADAAAADgAAAAQAAAANAP####8AAAAAAA4AAAEFAAAAAAD#####AAAAAQAKQ09wZXJhdGlvbgAAAAAOAAAAAwAAAA4AAAAFAAAADgAAAAQAAAANAP####8AAAAAAA4AAAEFAAAAAAAAAAAOAAAAAwAAAA8AAAAADgAAAAQAAAAOAAAABv####8AAAABAAtDSG9tb3RoZXRpZQD#####AAAABwAAAA4AAAABAAAABwD#####AAAAAAAOAAABBQAAAAAIAAAACgAAABAA#####wAAAAcAAAAOAAAAAgAAAAcA#####wAAAAAADgAAAQUAAAAACQAAAAwAAAABAP####8BAAAAAAoAAAEAAQAAAAgAAAALAAAAAQD#####AQAAAAAKAAABAAEAAAAJAAAADf####8AAAABAA9DUG9pbnRMaWVEcm9pdGUA#####wEAAAAACwABVwDAFAAAAAAAAMA0AAAAAAAABQABP9xWeJq83w4AAAAO#####wAAAAIACENNZXN1cmVYAP####8AAAAAAAAAAAAQ#####wAAAAEAB0NDYWxjdWwA#####wAFYWJzdzEAD3hDb29yZChXLE8sSSxKKQAAAA4AAAAR#####wAAAAIAEkNMaWV1T2JqZXRQYXJQdExpZQH#####AGZmZgAAABAAAAAOAAAAAQAAABAAAAAAAAAAEwD#####AAVhYnN3MgANMiphYnNvci1hYnN3MQAAAA8BAAAADwL#####AAAAAQAKQ0NvbnN0YW50ZUAAAAAAAAAAAAAADgAAAAMAAAAOAAAAEgAAAA0A#####wEAAAAACwAAAQUAAAAAAAAAAA4AAAAUAAAADgAAAAQAAAAUAf####8AZmZmAAAAFQAAAA4AAAABAAAAEAAAAAAAAAARAP####8BAAAAAAsAAVIAQCAAAAAAAADAIAAAAAAAAAUAAT#RG06BtOgfAAAAD#####8AAAACAAhDTWVzdXJlWQD#####AAAAAAAAAAAAFwAAABMA#####wAFb3JkcjEAD3lDb29yZChSLE8sSSxKKQAAAA4AAAAYAAAAFAH#####AGZmZgAAABcAAAAOAAAAAgAAABcAAAAAAAAAEwD#####AAVvcmRyMgANMipvcmRvci1vcmRyMQAAAA8BAAAADwIAAAAVQAAAAAAAAAAAAAAOAAAABAAAAA4AAAAZAAAADQD#####AQAAAAALAAABBQAAAAAAAAAADgAAAAMAAAAOAAAAGwAAABQB#####wBmZmYAAAAcAAAADgAAAAIAAAAXAAAAAP####8AAAACAAxDQ29tbWVudGFpcmUA#####wFmZmYAAAAAAAAAAABAGAAAAAAAAAAAABALAAH###8AAAABAAAAAAAAABUAAAAAAAAAAAALI1ZhbChhYnN3MSkAAAAUAf####8AZmZmAAAAHgAAAA4AAAABAAAAEAAAAAAAAAAXAP####8BZmZmAAAAAAAAAAAAQBgAAAAAAAAAAAAVCwAB####AAAAAQAAAAAAAAAVAAAAAAAAAAAACyNWYWwoYWJzdzIpAAAAFAH#####AGZmZgAAACAAAAAOAAAAAQAAABAAAAAAAAAAFwD#####AWZmZgDAIAAAAAAAAD#wAAAAAAAAAAAAFwsAAf###wAAAAIAAAABAAAAFQAAAAAAAAAAAAsjVmFsKG9yZHIxKQAAABQB#####wBmZmYAAAAiAAAADgAAAAIAAAAXAAAAAAAAABcA#####wFmZmYAwBwAAAAAAAAAAAAAAAAAAAAAABwLAAH###8AAAACAAAAAQAAABUAAAAAAAAAAAALI1ZhbChvcmRyMikAAAAUAf####8AZmZmAAAAJAAAAA4AAAACAAAAFwAAAAAAAAAGAA5NZXN1cmVMb25ndWV1cgAAAAAAAgAAAAIAAAAACQAAAAAAAUEAAAAA#####wAAAAAAAUIAAAAA##########8AAAABAAtDTWVkaWF0cmljZQD#####AQAAAAANAAABAAEAAAAAAAAAAf####8AAAABAAdDTWlsaWV1AP####8BAAAAAA0AAAEFAAAAAAAAAAAB#####wAAAAIACUNDZXJjbGVPUgD#####AQAAAAABAAAAAwAAABVAMAAAAAAAAAEAAAAEAP####8AAAACAAAABAAAAAUA#####wEAAAAADQAAAQUAAQAAAAX#####AAAAAQAJQ0xvbmd1ZXVyAf####8AAAAAAAAAAf####8AAAABAA9DVmFsZXVyQWZmaWNoZWUB#####wAAAAABAAAABhEAAAAAAAEAAAABAAAAFQAAAAAAAAAAAAAAAAEAAAAHAAAABgAIVGFuZ2VudGUAAAAAAAMAAAACAgAAAAgAAAAAAAD#####AAAAFgAAAAAAAXj#####AAAAAQAAAAAAAAAAABoAAAAaAAAAEwD#####AAF5AARmKHgp#####wAAAAEADkNBcHBlbEZvbmN0aW9uAAAAAAAAAA4AAAABAAAADQD#####AAAAAAANAAABBQAAAAACAAAADgAAAAEAAAAOAAAAA#####8AAAABAAhDRGVyaXZlZQD#####AAJmJwAAAAAAAAATAf####8AAW0ABWYnKHgpAAAAHQAAAAUAAAAOAAAAAf####8AAAABAAlDRHJvaXRlT20B#####wAAAAAACgAAAQABAAAAAgAAAAQAAAAOAAAABgAAAAYAElNlZ21lbnRQYXJMb25ndWV1cgAAAAAAAgAAAAMBAAAACwAAAAAABGxvbmf#####AAAAAQAAAAAAAAAAAAD#####AAAAGgD#####AQAAAAABAAAAAQAAAA4AAAAAAP####8AAAABAA9DUG9pbnRMaWVDZXJjbGUB#####wAAAAAAEAAAAQUAAQAAAAAAAAAAAAAAAgAAAAEB#####wAAAAAADQAAAQABAAAAAQAAAAMAAAAZAP####8BAAAAAA0AAAEFAAAAAAEAAAADAAAAGAD#####AQAAAAANAAABAAEAAAABAAAAAwAAABoA#####wEAAAAAAQAAAAUAAAAVQDAAAAAAAAABAAAABAD#####AAAABgAAAAcAAAAFAP####8BAAAAAA0AAAEFAAEAAAAIAAAAHAH#####AAAAAAEAAAAJDgAAAAAAAQAAAAEAAAAVAAAAAAAAAAAAAAAAAQAAAAAAAAAGACBHcmFkdWF0aW9uUmVwZXJlUG91clRyaWdvQXZlY1BhcwAAAAAABAAAAAgCAAAAMgAAAAAAAAAAABoAAAAaAAAAAAAHbmJncmFkeP####8AAAABAAAAAAAFZ3JhZHn#####AAAAAQAAAAAAB25iZ3JhZHn#####AAAAAf####8AAAABAApDQ2FsY0NvbnN0AP####8AA3BpMQAWMy4xNDE1OTI2NTM1ODk3OTMyMzg0NgAAABVACSH7VEQtGAAAAAkA#####wAFYWJzb3IAAAAAAAAACgD#####AAVvcmRvcgAAAAAAAAANAP####8AZmZmAAsAAAEFAAAAAAAAAAAOAAAABQAAAA4AAAAGAAAADQD#####AGZmZgALAAABBQAAAAAAAAAADwAAAAAOAAAABQAAABU#8AAAAAAAAAAAAA4AAAAGAAAAEAD#####AAAABwAAAA4AAAAEAAAABwD#####AAAAAAALAAABBQAAAAAIAAAACQAAABAA#####wAAAAcAAAAOAAAAAQAAAAcA#####wAAAAAACwAAAQUAAAAACgAAAAsAAAABAP####8BAAAAAAoAAAEAAQAAAAoAAAAMAAAAEQD#####AQAAAAALAAFXAMAYAAAAAAAAwDIAAAAAAAAFAAE#1wECwgc29wAAAA0AAAASAP####8AAAAAAAAAAAAOAAAAEwD#####AAZncmFkcGkAE3hDb29yZChXLE8sSSxKKS9waTEAAAAPAwAAAA4AAAAPAAAADgAAAAQAAAATAP####8ABXRlc3QzABVhYnMoZ3JhZHBpKTwwLjAwMDAwMDEAAAAPBP####8AAAACAAlDRm9uY3Rpb24AAAAADgAAABAAAAAVPnrX8pq8r0gAAAATAP####8AB2dyYWRwaScAEjIqYWJzb3IvcGkxLWdyYWRwaQAAAA8BAAAADwMAAAAPAgAAABVAAAAAAAAAAAAAAA4AAAAFAAAADgAAAAQAAAAOAAAAEAAAABMA#####wAFdGVzdDQAFmFicyhncmFkcGknKTwwLjAwMDAwMDEAAAAPBAAAACIAAAAADgAAABIAAAAVPnrX8pq8r0gAAAATAP####8ABXRlc3QyABhhYnMoZ3JhZHBpJysxKTwwLjAwMDAwMDEAAAAPBAAAACIAAAAADwAAAAAOAAAAEgAAABU#8AAAAAAAAAAAABU+etfymryvSAAAABMA#####wAFdGVzdDEAF2FicyhncmFkcGktMSk8MC4wMDAwMDAxAAAADwQAAAAiAAAAAA8BAAAADgAAABAAAAAVP#AAAAAAAAAAAAAVPnrX8pq8r0j#####AAAAAgAGQ0xhdGV4AP####8BAAAAAAAAAAAAAAAAQBgAAAAAAAAAAAAOCwAB####AAAAAQAAAAAAAAAVAAAAAAAAAAAAL1xJZnt0ZXN0M317MH17XElme3Rlc3QxfXtccGl9e1xWYWx7Z3JhZHBpfVxwaX19AAAAFAH#####AAAAAAAAABYAAAAOAAAAAQAAAA4AAAAA#####wAAAAEAEUNTeW1ldHJpZUNlbnRyYWxlAP####8AAAAHAAAABwD#####AQAAAAALAAABBQAAAAAOAAAAGAAAACMA#####wEAAAAAAAAAAAAAAABAGAAAAAAAAAAAABkLAAH###8AAAABAAAAAAAAABUAAAAAAAAAAAAxXElme3Rlc3Q0fXswfXtcSWZ7dGVzdDJ9ey1ccGl9e1xWYWx7Z3JhZHBpJ31ccGl9fQAAABQB#####wAAAAAAAAAaAAAADgAAAAEAAAAOAAAAAAAAAAEA#####wEAAAAACgAAAQABAAAABwAAAAwAAAARAP####8BAAAAAAsAAVIAwAAAAAAAAADANAAAAAAAAAUAAT#Fut5S+V5oAAAAHAAAABQB#####wBmZmYAAAAdAAAADwAAAAAPAgAAAA4AAAABAAAAFUAAAAAAAAAAAAAAFT#wAAAAAAAAAAAAHQAAAAAAAAAHAP####8AAAAAAAsAAAEFAAAAAB0AAAAYAAAAFAH#####AGZmZgAAAB8AAAAPAAAAAA8CAAAADgAAAAEAAAAVQAAAAAAAAAAAAAAVP#AAAAAAAAAAAAAdAAAAAAAAABAA#####wAAAAcAAAAOAAAAAgAAABAA#####wAAAAcAAAAOAAAAAwAAAA0A#####wAAAAAACwAAAQUAAAAAAAAAAA4AAAAFAAAADwAAAAAOAAAABgAAABU#8AAAAAAAAAAAAAcA#####wAAAAAACwAAAQUAAAAAIwAAACEAAAAHAP####8AAAAAAAsAAAEFAAAAACQAAAAiAAAAAQD#####AQAAAAAKAAABAAEAAAAkAAAAJQAAABEA#####wEAAAAACwABVABAIAAAAAAAAMAiAAAAAAAABQABP+KPXCj1wpcAAAAmAAAAFgD#####AAAAAAAAAAAAJwAAABMA#####wAEb3JkdAAPeUNvb3JkKFQsTyxJLEopAAAADgAAACgAAAAUAf####8AZmZmAAAAJwAAAA4AAAADAAAAJwAAAAAAAAATAP####8ABXlncmFkAAwyKm9yZG9yLW9yZHQAAAAPAQAAAA8CAAAAFUAAAAAAAAAAAAAADgAAAAYAAAAOAAAAKQAAAAcA#####wEAAAAACwAAAQUAAAAAJwAAABgAAAAUAf####8AZmZmAAAALAAAAA4AAAADAAAAJwAAAAAAAAAXAP####8BZmZmAMAcAAAAAAAAAAAAAAAAAAAAAAAnCwAB####AAAAAgAAAAEAAAAVAAAAAAAAAAAACiNWYWwob3JkdCkAAAAUAf####8AZmZmAAAALgAAAA4AAAADAAAAJwAAAAAAAAAXAP####8BZmZmAMAcAAAAAAAAP#AAAAAAAAAAAAAsCwAB####AAAAAgAAAAEAAAAVAAAAAAAAAAAACyNWYWwoeWdyYWQpAAAAFAH#####AGZmZgAAADAAAAAOAAAAAwAAACcAAAAAAAAABgAbR3JhZHVhdGlvbkF4ZXNSZXBlcmVBdmVjUGFzAAAAAAAFAAAACAIAAAAqAAAAAAAAAAAAGgAAABoAAAAAAARwYXN4#####wAAAAEAAAAAAAduYmdyYWR4#####wAAAAEAAAAAAARwYXN5#####wAAAAEAAAAAAAduYmdyYWR5#####wAAAAEAAAAJAP####8ABWFic29yAAAAAAAAAAoA#####wAFb3Jkb3IAAAAAAAAADQD#####AAAAAAALAAABBQAAAAAAAAAADgAAAAUAAAAOAAAABgAAAA0A#####wAAAAAACwAAAQUAAAAAAAAAAA8AAAAADgAAAAUAAAAVP#AAAAAAAAAAAAAOAAAABgAAABAA#####wAAAAcAAAAOAAAAAQAAAAcA#####wAAAAAACwAAAQUAAAAACAAAAAkAAAAQAP####8AAAAHAAAADgAAAAIAAAAHAP####8AAAAAAAsAAAEFAAAAAAoAAAALAAAAAQD#####AQAA#wAKAAABAAEAAAAKAAAADAAAABEA#####wEAAAAACwACVzEAwBwAAAAAAADANAAAAAAAAAUAAT#OeeeeeeeRAAAADQAAABIA#####wAAAAAAAAAAAA4AAAATAP####8ABWFic3cxABB4Q29vcmQoVzEsTyxJLEopAAAADgAAAA8AAAATAP####8ABWFic3cyAA0yKmFic29yLWFic3cxAAAADwEAAAAPAgAAABVAAAAAAAAAAAAAAA4AAAAFAAAADgAAABAAAAANAP####8AAAAAAAsAAAEFAAAAAAAAAAAOAAAABQAAAA8AAAAADgAAAAYAAAAVP#AAAAAAAAAAAAANAP####8BAAAAAAsAAAEFAAAAAAAAAAAOAAAAEQAAAA4AAAAGAAAAFAH#####AGZmZgAAAA4AAAAOAAAAAgAAAA4AAAAAAAAAFAH#####AGZmZgAAABMAAAAOAAAAAgAAAA4AAAAAAAAAEAD#####AAAABwAAAA4AAAADAAAABwD#####AAAAAAALAAABBQAAAAASAAAAFgAAABAA#####wAAAAcAAAAOAAAABAAAAAcA#####wAAAAAACwAAAQUAAAAAFwAAABgAAAABAP####8BAAD#AAoAAAEAAQAAABcAAAAZAAAAEQD#####AQAAAAALAAFSAEAcAAAAAAAAwCAAAAAAAAAFAAE#2iYIxvLViQAAABoAAAAWAP####8AAAAAAAAAAAAbAAAAEwD#####AAVvcmRyMQAPeUNvb3JkKFIsTyxJLEopAAAADgAAABwAAAATAP####8ABW9yZHIyAA0yKm9yZG9yLW9yZHIxAAAADwEAAAAPAgAAABVAAAAAAAAAAAAAAA4AAAAGAAAADgAAAB0AAAANAP####8BAAAAAAsAAAEFAAAAAAAAAAAOAAAABQAAAA4AAAAeAAAAFAH#####AGZmZgAAABsAAAAOAAAABAAAABsAAAAAAAAAFAH#####AGZmZgAAAB8AAAAOAAAABAAAABsAAAAAAAAAFwD#####AWZmZgBAAAAAAAAAAEAUAAAAAAAAAAAADgsAAf###wAAAAEAAAAAAAAAFQAAAAAAAAAAAAsjVmFsKGFic3cxKQAAABQB#####wBmZmYAAAAiAAAADgAAAAIAAAAOAAAAAAAAABcA#####wFmZmYAv#AAAAAAAABAFAAAAAAAAAAAABMLAAH###8AAAABAAAAAAAAABUAAAAAAAAAAAALI1ZhbChhYnN3MikAAAAUAf####8AZmZmAAAAJAAAAA4AAAACAAAADgAAAAAAAAAXAP####8BZmZmAMAgAAAAAAAAAAAAAAAAAAAAAAAbCwAB####AAAAAgAAAAEAAAAVAAAAAAAAAAAACyNWYWwob3JkcjEpAAAAFAH#####AGZmZgAAACYAAAAOAAAABAAAABsAAAAAAAAAFwD#####AWZmZgDAIAAAAAAAAAAAAAAAAAAAAAAAHwsAAf###wAAAAIAAAABAAAAFQAAAAAAAAAAAAsjVmFsKG9yZHIyKQAAABQB#####wBmZmYAAAAoAAAADgAAAAQAAAAbAAAAAAAAAAYAEUN1cnNldXJEaXNjcmV0SG9yAAAAAAAEAAAABgEAAAAUAAAAAAAEbWluaf####8AAAABAAAAAAAEbWF4af####8AAAABAAAAAAADcGFz#####wAAAAEAAAAAAAAAAAAA##########8AAAABABRDRHJvaXRlRGlyZWN0aW9uRml4ZQD#####AQAAAAAPAAABAAEAAAADAT#wAAAAAAAAAAAAEQH#####AAAAAAAPAAAAAAAAAAAAAABACAAAAAAAAAUAAUBhgAAAAAAAAAAABAAAABMA#####wAHbmJncmFkMAAaaW50KChtYXhpLW1pbmkpL3BhcyswLjUpKzEAAAAPAAAAACICAAAADwAAAAAPAwAAAA8BAAAADgAAAAEAAAAOAAAAAAAAAA4AAAACAAAAFT#gAAAAAAAAAAAAFT#wAAAAAAAA#####wAAAAEAC0NQb2ludENsb25lAP####8BAAAAAAsAAk8xAMA4AAAAAAAAP#AAAAAAAAABAAAAAAMAAAABAf####8AAAAAAA0AAAEAAQAAAAcAAAAFAAAAAgD#####AQAAAAAPAAABAAEAAAAHAAAACAAAABEA#####wEAAAAACwACSjEAwDYAAAAAAADALgAAAAAAAAEAAb++GRTHsP1ZAAAACQAAABAA#####wAAAAcAAAAPAwAAAA4AAAACAAAADwEAAAAOAAAAAQAAAA4AAAAAAAAABwD#####AQAAAAALAAJJMQAAAAAAAAAAAEAAAAAAAAAAAQAAAAAFAAAAC#####8AAAACAAdDUmVwZXJlAP####8AAAAAAQEAAAAHAAAADAAAAAoAAAAAAAAOAAAAAAAAABUAAAAAAAAAAAAAAA4AAAACAAAAFT#wAAAAAAAAAAAAEQD#####AQAAAAALAAJXMQDAIAAAAAAAAMA7AAAAAAAABQABP+EYMU5uRToAAAAIAAAAFAH#####AAAAAAAAAA4AAAAOAAAABgAAAA4AAAAA#####wAAAAEADUNQb2ludEJhc2VFbnQB#####wD#AAABCwACVDEAAAAAAAAAAABACAAAAAAAAAEAAQAAAA1AFAAAAAAAAAAAAAAAAAAAAAAAAAAOAAAAAAAAAA4AAAABAAAAFQAAAAAAAAAAAAAAFQAAAAAAAAAAAAAAEgD#####AAVtZXNhYgAAAA0AAAAQAAAAEwH#####AAJhYgAkaW50KG1lc2FiKjEwMDAwMDAwMDArMC41KS8xMDAwMDAwMDAwAAAADwMAAAAiAgAAAA8AAAAADwIAAAAOAAAAEQAAABVBzc1lAAAAAAAAABU#4AAAAAAAAAAAABVBzc1lAAAAAAAAABwB#####wD#AAAAAAAAAAAAAADAIAAAAAAAAAAAABAPAAAAAAABAAAAAgAAABUAAAAAAAAAAAAAAAAJAAAAEgAAAAYAF0N1cnNldXJEaXNjcmV0SG9yR3JhZHVlAAAAAAAFAAAABwEAAAAaAAAAAAAEbWluaf####8AAAABAAAAAAAEbWF4af####8AAAABAAAAAAADcGFz#####wAAAAEAAAAAAAdwYXNncmFk#####wAAAAEAAAAAAAAAAAAA#####wAAACUA#####wEAAAAADwAAAQABAAAABAE#8AAAAAAAAAAAABEB#####wAAAAAADwAAAAAAAAAAAAAAQAgAAAAAAAAFAAFAYYAAAAAAAAAAAAUAAAATAP####8AB25iZ3JhZDAAGmludCgobWF4aS1taW5pKS9wYXMrMC41KSsxAAAADwAAAAAiAgAAAA8AAAAADwMAAAAPAQAAAA4AAAABAAAADgAAAAAAAAAOAAAAAgAAABU#4AAAAAAAAAAAABU#8AAAAAAAAAAAABMA#####wAKbmJncmFkcGFzMAAeaW50KChtYXhpLW1pbmkpL3Bhc2dyYWQrMC41KSsxAAAADwAAAAAiAgAAAA8AAAAADwMAAAAPAQAAAA4AAAABAAAADgAAAAAAAAAOAAAAAwAAABU#4AAAAAAAAAAAABU#8AAAAAAAAAAAACYA#####wEAAAAACwACTzEAwDgAAAAAAAA#8AAAAAAAAAEAAAAABAAAAAEB#####wAAAAAADQAAAQABAAAACQAAAAYAAAACAP####8BAAAAAA8AAAEAAQAAAAkAAAAKAAAAEQD#####AQAAAAALAAJKMQDANgAAAAAAAMAuAAAAAAAAAQABv74ZFMew#VkAAAALAAAAEAD#####AAAACQAAAA8DAAAADgAAAAIAAAAPAQAAAA4AAAABAAAADgAAAAAAAAAHAP####8BAAAAAAsAAkkxAAAAAAAAAAAAQAAAAAAAAAABAAAAAAYAAAANAAAAJwD#####AAAAAAEBAAAACQAAAA4AAAAMAAAAAAAADgAAAAAAAAAVAAAAAAAAAAAAAAAOAAAAAgAAABU#8AAAAAAAAAAAABEA#####wEAAAAACwACVzEAwCAAAAAAAADAOwAAAAAAAAUAAT#hGDFObkU6AAAACgAAABQB#####wAAAAAAAAAQAAAADgAAAAcAAAAQAAAAAAAAABIA#####wAEYWJzdwAAAA8AAAAQAAAAEwD#####AAdhYnN3QXJyACNpbnQoYWJzdyoxMDAwMDAwMDAwKzAuNSkvMTAwMDAwMDAwMAAAAA8DAAAAIgIAAAAPAAAAAA8CAAAADgAAABIAAAAVQc3NZQAAAAAAAAAVP+AAAAAAAAAAAAAVQc3NZQAAAAAAAAAcAP####8BAAAAAAAAAAAAAAAAQBgAAAAAAAAAAAAQCwAAAAAAAQAAAAAAAAAVAAAAAAAAAAAAAAAACQAAABMAAAAUAf####8AAAAAAAAAFAAAAA4AAAAIAAAAEAAAAAAAAAAoAf####8A#wAAAQsAAlQxAAAAAAAAAAAAQAgAAAAAAAABAAEAAAAPQBQAAAAAAAAAAAAAAAAAAAAAAAAADgAAAAAAAAAOAAAAAQAAABUAAAAAAAAAAAAAABUAAAAAAAAAAAAAABIA#####wAFbWVzYWIAAAAPAAAAFgAAABMB#####wACYWIAJGludChtZXNhYioxMDAwMDAwMDAwKzAuNSkvMTAwMDAwMDAwMAAAAA8DAAAAIgIAAAAPAAAAAA8CAAAADgAAABcAAAAVQc3NZQAAAAAAAAAVP+AAAAAAAAAAAAAVQc3NZQAAAAAAAAAcAf####8A#wAAAAAAAAAAAAAAwCAAAAAAAAAAAAAWDwAAAAAAAQAAAAIAAAAVAAAAAAAAAAAAAAAACQAAABgAAAAGABFDdXJzZXVyRGlzY3JldFZlcgAAAAAABAAAAAYBAAAAFAAAAAAABG1pbmn#####AAAAAQAAAAAABG1heGn#####AAAAAQAAAAAAA3Bhc#####8AAAABAAAAAAAAAAAAAP####8AAAATAP####8AB25iZ3JhZDAAGmludCgobWF4aS1taW5pKS9wYXMrMC41KSsxAAAADwAAAAAiAgAAAA8AAAAADwMAAAAPAQAAAA4AAAABAAAADgAAAAAAAAAOAAAAAgAAABU#4AAAAAAAAAAAABU#8AAAAAAAAAAAACYA#####wEAAAAACwACTzEAwDgAAAAAAAA#8AAAAAAAAAEAAAAAAwAAABAA#####wAAAAUAAAAPAwAAAA4AAAACAAAADwEAAAAOAAAAAQAAAA4AAAAAAAAAJQD#####AQAAAAAPAAABAAEAAAADAD#wAAAAAAAAAAAAEQH#####AAAAAAAPAAAAAAAAAAAAAABACAAAAAAAAAUAAUBfAAAAAAAAAAAABwAAAAEB#####wAAAAAADQAAAQABAAAABQAAAAgAAAACAP####8BAAAAAA8AAAEAAQAAAAUAAAAJAAAAEQD#####AQAAAAALAAJKMQDANgAAAAAAAMAuAAAAAAAAAQABv74ZFMew#VkAAAAKAAAABwD#####AQAAAAALAAJJMQAAAAAAAAAAAEAAAAAAAAAAAQAAAAAIAAAABgAAACcA#####wAAAAABAQAAAAUAAAAMAAAACwAAAAAAAA4AAAAAAAAAFQAAAAAAAAAAAAAADgAAAAIAAAAVP#AAAAAAAAAAAAARAP####8BAAAAAAsAAlcxAMAgAAAAAAAAwDsAAAAAAAAFAAE#4RgxTm5FOgAAAAkAAAAUAf####8AAAAAAAAADgAAAA4AAAAEAAAADgAAAAAAAAAoAf####8A#wAAAQsAAlQxAAAAAAAAAAAAQAgAAAAAAAABAAEAAAANQBQAAAAAAAAAAAAAAAAAAAAAAAAADgAAAAAAAAAOAAAAAQAAABUAAAAAAAAAAAAAABUAAAAAAAAAAAAAABIA#####wAGbWVzYWJzAAAADQAAABAAAAATAf####8AAmFiACVpbnQobWVzYWJzKjEwMDAwMDAwMDArMC41KS8xMDAwMDAwMDAwAAAADwMAAAAiAgAAAA8AAAAADwIAAAAOAAAAEQAAABVBzc1lAAAAAAAAABU#4AAAAAAAAAAAABVBzc1lAAAAAAAAABwB#####wD#AAAAQCQAAAAAAAAAAAAAAAAAAAAAABAPAAAAAAAAAAAAAQAAABUAAAAAAAAAAAAAAAAJAAAAEgAAAAYACkN1cnNldXJIb3IAAAAAAAMAAAAFAQAAAA0AAAAAAARtaW5p#####wAAAAEAAAAAAARtYXhp#####wAAAAEAAAAAAAAAAAAA#####wAAACUA#####wEAAAAADwAAAQABAAAAAgE#8AAAAAAAAAAAABEB#####wAAAAAADwAAAMAIAAAAAAAAP#AAAAAAAAAFAABAXUAAAAAAAAAAAAMAAAAQAP####8AAAACAAAADwMAAAAOAAAAAAAAAA8BAAAADgAAAAAAAAAOAAAAAQAAAAcA#####wEAAAAADQACTzEAwBAAAAAAAABAEAAAAAAAAAUAAAAABAAAAAUAAAAQAP####8AAAACAAAADwMAAAAPAQAAABU#8AAAAAAAAAAAAA4AAAAAAAAADwEAAAAOAAAAAQAAAA4AAAAAAAAABwD#####AQAAAAANAAJJMQDAAAAAAAAAAEAIAAAAAAAABQAAAAAEAAAABwAAAAEB#####wAAAAAADQAAAQABAAAAAgAAAAQAAAARAf####8A#wAAAREAAmsxAMAAAAAAAAAAQAAAAAAAAAABAAE#3y3y3y3y3wAAAAn#####AAAAAgAPQ01lc3VyZUFic2Npc3NlAf####8AAWsAAAAGAAAACAAAAAoAAAAcAf####8A#wAAAAAAAAAAAAAAwBgAAAAAAAAAAAAKDwAAAAAAAQAAAAIAAAAVAAAAAAAAAAAAAAAAAgAAAAsAAAAGABBDdXJzZXVySG9yR3JhZHVlAAAAAAAEAAAABwEAAAAVAAAAAAAEbWluaf####8AAAABAAAAAAAEbWF4af####8AAAABAAAAAAAHcGFzZ3JhZP####8AAAABAAAAAAAAAAAAAP####8AAAAlAP####8BAAAAAA8AAAEAAQAAAAMBP#AAAAAAAAAAAAARAf####8AAAAAAA8AAADACAAAAAAAAD#wAAAAAAAABQABQFyAAAAAAAAAAAAEAAAAEAD#####AAAAAwAAAA8DAAAADgAAAAAAAAAPAQAAAA4AAAAAAAAADgAAAAEAAAAHAP####8BAAAAAA0AAk8xAMAQAAAAAAAAQBAAAAAAAAAFAAAAAAUAAAAGAAAAEAD#####AAAAAwAAAA8DAAAADwEAAAAVP#AAAAAAAAAAAAAOAAAAAAAAAA8BAAAADgAAAAEAAAAOAAAAAAAAAAcA#####wEAAAAADQACSTEAwAAAAAAAAABACAAAAAAAAAUAAAAABQAAAAgAAAABAf####8AAAAAAA0AAAEAAQAAAAMAAAAFAAAAEwD#####AAduYmdncmFkAB5pbnQoKG1heGktbWluaSkvcGFzZ3JhZCswLjUpKzEAAAAPAAAAACICAAAADwAAAAAPAwAAAA8BAAAADgAAAAEAAAAOAAAAAAAAAA4AAAACAAAAFT#gAAAAAAAAAAAAFT#wAAAAAAAAAAAAEQD#####AQAAAAARAAFXAAAAAAAAAAAAQAgAAAAAAAAFAAE#zYnYnYnYngAAAAoAAAApAP####8ABGFic3cAAAAHAAAACQAAAAwAAAATAP####8AB2Fic3dBcnIAI2ludChhYnN3KjEwMDAwMDAwMDArMC41KS8xMDAwMDAwMDAwAAAADwMAAAAiAgAAAA8AAAAADwIAAAAOAAAADQAAABVBzc1lAAAAAAAAABU#4AAAAAAAAAAAABVBzc1lAAAAAAAAABwA#####wEAAAAAAAAAAAAAAABAGAAAAAAAAAAAAAwLAAAAAAABAAAAAAAAABUAAAAAAAAAAAAAAAAJAAAADgAAABQB#####wAAAAAAAAAMAAAADgAAAAsAAAAMAAAAAAAAABQB#####wAAAAAAAAAPAAAADgAAAAsAAAAMAAAAAAAAABEB#####wD#AAABEQACazEAwAAAAAAAAABAAAAAAAAAAAEAAT#gAAAAAAAAAAAACgAAACkB#####wABawAAAAcAAAAJAAAAEgAAABwB#####wD#AAAAAAAAAAAAAADAGAAAAAAAAAAAABIPAAAAAAABAAAAAgAAABUAAAAAAAAAAAAAAAACAAAAEwAAAAYACkN1cnNldXJWZXIAAAAAAAMAAAAFAQAAAA0AAAAAAARtaW5p#####wAAAAEAAAAAAARtYXhp#####wAAAAEAAAAAAAAAAAAA#####wAAABAA#####wAAAAIAAAAPAwAAAA4AAAAAAAAADwEAAAAOAAAAAAAAAA4AAAABAAAAEAD#####AAAAAgAAAA8DAAAADwEAAAAVP#AAAAAAAAAAAAAOAAAAAAAAAA8BAAAADgAAAAEAAAAOAAAAAAAAACUA#####wEAAAAADwAAAQABAAAAAgA#8AAAAAAAAAAAABEB#####wAAAAAADwAAAMAIAAAAAAAAP#AAAAAAAAAFAAFAXgAAAAAAAAAAAAUAAAAHAP####8BAAAAAA0AAk8xAMAQAAAAAAAAQBAAAAAAAAAFAAAAAAYAAAADAAAABwD#####AQAAAAANAAJJMQDAAAAAAAAAAEAIAAAAAAAABQAAAAAGAAAABAAAAAEB#####wAAAAAADQAAAQABAAAAAgAAAAYAAAARAf####8A#wAAAREAAmsxAMAAAAAAAAAAQAAAAAAAAAABAAE#3y3y3y3y3wAAAAkAAAApAf####8AAWsAAAAHAAAACAAAAAoAAAAcAf####8A#wAAAMAgAAAAAAAAwAAAAAAAAAAAAAAKDwAAAAAAAgAAAAEAAAAVAAAAAAAAAAAAAAAAAgAAAAsAAAAGABBDdXJzZXVyVmVyR3JhZHVlAAAAAAAEAAAABwEAAAAVAAAAAAAEbWluaf####8AAAABAAAAAAAEbWF4af####8AAAABAAAAAAAHcGFzZ3JhZP####8AAAABAAAAAAAAAAAAAP####8AAAAQAP####8AAAADAAAADwMAAAAOAAAAAAAAAA8BAAAADgAAAAAAAAAOAAAAAQAAABAA#####wAAAAMAAAAPAwAAAA8BAAAAFT#wAAAAAAAAAAAADgAAAAAAAAAPAQAAAA4AAAABAAAADgAAAAAAAAATAP####8AB25iZ2dyYWQAHmludCgobWF4aS1taW5pKS9wYXNncmFkKzAuNSkrMQAAAA8AAAAAIgIAAAAPAAAAAA8DAAAADwEAAAAOAAAAAQAAAA4AAAAAAAAADgAAAAIAAAAVP+AAAAAAAAAAAAAVP#AAAAAAAAAAAAAlAP####8BAAAAAA8AAAEAAQAAAAMAP#AAAAAAAAAAAAARAf####8AAAAAAA8AAADACAAAAAAAAD#wAAAAAAAABQABQF4AAAAAAAAAAAAHAAAABwD#####AQAAAAANAAJPMQDAEAAAAAAAAEAQAAAAAAAABQAAAAAIAAAABAAAAAcA#####wEAAAAADQACSTEAwAAAAAAAAABACAAAAAAAAAUAAAAACAAAAAUAAAABAf####8AAAAAAA0AAAEAAQAAAAMAAAAIAAAAEQD#####AQAAAAARAAFXAAAAAAAAAAAAQAgAAAAAAAAFAAE#zYnYnYnYngAAAAsAAAApAP####8ABGFic3cAAAAJAAAACgAAAAwAAAATAP####8AB2Fic3dBcnIAI2ludChhYnN3KjEwMDAwMDAwMDArMC41KS8xMDAwMDAwMDAwAAAADwMAAAAiAgAAAA8AAAAADwIAAAAOAAAADQAAABVBzc1lAAAAAAAAABU#4AAAAAAAAAAAABVBzc1lAAAAAAAAABwA#####wEAAAAAwBgAAAAAAAAAAAAAAAAAAAAAAAwLAAAAAAACAAAAAQAAABUAAAAAAAAAAAAAAAAJAAAADgAAABQB#####wAAAAAAAAAMAAAADgAAAAYAAAAMAAAAAAAAABQB#####wAAAAAAAAAPAAAADgAAAAYAAAAMAAAAAAAAABEB#####wD#AAABEQACazEAwAAAAAAAAABAAAAAAAAAAAEAAT#fcEfcEfcEAAAACwAAACkB#####wABawAAAAkAAAAKAAAAEgAAABwB#####wD#AAAAQCQAAAAAAAAAAAAAAAAAAAAAABIPAAAAAAAAAAAAAQAAABUAAAAAAAAAAAAAAAACAAAAEwAAAAYAF0N1cnNldXJEaXNjcmV0VmVyR3JhZHVlAAAAAAAFAAAABwEAAAAaAAAAAAAEbWluaf####8AAAABAAAAAAAEbWF4af####8AAAABAAAAAAADcGFz#####wAAAAEAAAAAAAdwYXNncmFk#####wAAAAEAAAAAAAAAAAAA#####wAAABMA#####wAHbmJncmFkMAAaaW50KChtYXhpLW1pbmkpL3BhcyswLjUpKzEAAAAPAAAAACICAAAADwAAAAAPAwAAAA8BAAAADgAAAAEAAAAOAAAAAAAAAA4AAAACAAAAFT#gAAAAAAAAAAAAFT#wAAAAAAAAAAAAEwD#####AApuYmdyYWRwYXMwAB5pbnQoKG1heGktbWluaSkvcGFzZ3JhZCswLjUpKzEAAAAPAAAAACICAAAADwAAAAAPAwAAAA8BAAAADgAAAAEAAAAOAAAAAAAAAA4AAAADAAAAFT#gAAAAAAAAAAAAFT#wAAAAAAAAAAAAJgD#####AQAAAAALAAJPMQDAOAAAAAAAAD#wAAAAAAAAAQAAAAAEAAAAEAD#####AAAABwAAAA8DAAAADgAAAAIAAAAPAQAAAA4AAAABAAAADgAAAAAAAAAlAP####8BAAAAAA8AAAEAAQAAAAQAP#AAAAAAAAAAAAARAf####8AAAAAAA8AAAAAAAAAAAAAAEAIAAAAAAAABQABQF8AAAAAAAAAAAAJAAAAAQH#####AAAAAAANAAABAAEAAAAHAAAACgAAAAIA#####wEAAAAADwAAAQABAAAABwAAAAsAAAARAP####8BAAAAAAsAAkoxAMA2AAAAAAAAwC4AAAAAAAABAAG#vhkUx7D9WQAAAAwAAAAHAP####8BAAAAAAsAAkkxAAAAAAAAAAAAQAAAAAAAAAABAAAAAAoAAAAIAAAAJwD#####AAAAAAEBAAAABwAAAA4AAAANAAAAAAAADgAAAAAAAAAVAAAAAAAAAAAAAAAOAAAAAgAAABU#8AAAAAAAAAAAABEA#####wEAAAAACwACVzEAwCAAAAAAAADAOwAAAAAAAAUAAT#hGDFObkU6AAAACwAAABQB#####wAAAAAAAAAQAAAADgAAAAUAAAAQAAAAAAAAABIA#####wAEYWJzdwAAAA8AAAAQAAAAEwD#####AAdhYnN3QXJyACNpbnQoYWJzdyoxMDAwMDAwMDAwKzAuNSkvMTAwMDAwMDAwMAAAAA8DAAAAIgIAAAAPAAAAAA8CAAAADgAAABIAAAAVQc3NZQAAAAAAAAAVP+AAAAAAAAAAAAAVQc3NZQAAAAAAAAAcAP####8BAAAAAMAYAAAAAAAAAAAAAAAAAAAAAAAQCwAAAAAAAgAAAAEAAAAVAAAAAAAAAAAAAAAACQAAABMAAAAUAf####8AAAAAAAAAFAAAAA4AAAAGAAAAEAAAAAAAAAAoAf####8A#wAAAQsAAlQxAAAAAAAAAAAAQAgAAAAAAAABAAEAAAAPQBQAAAAAAAAAAAAAAAAAAAAAAAAADgAAAAAAAAAOAAAAAQAAABUAAAAAAAAAAAAAABUAAAAAAAAAAAAAABIA#####wAGbWVzYWJzAAAADwAAABYAAAATAf####8AAWsAJWludChtZXNhYnMqMTAwMDAwMDAwMCswLjUpLzEwMDAwMDAwMDAAAAAPAwAAACICAAAADwAAAAAPAgAAAA4AAAAXAAAAFUHNzWUAAAAAAAAAFT#gAAAAAAAAAAAAFUHNzWUAAAAAAAAAHAH#####AP8AAABAJAAAAAAAAAAAAAAAAAAAAAAAFg8AAAAAAAAAAAABAAAAFQAAAAAAAAAAAAAAAAkAAAAYAAAABgAZR3JhZHVhdGlvblJlcGVyZVBvdXJUcmlnbwAAAAAAAwAAAAkCAAAAMgAAAAAAAAAAABoAAAAaAAAAAAAHbmJncmFkeP####8AAAABAAAAAAAHbmJncmFkef####8AAAABAAAADAD#####AAZ1bml0ZXkAAAAAAAAAIQD#####AANwaTEAFjMuMTQxNTkyNjUzNTg5NzkzMjM4NDYAAAAVQAkh+1RELRgAAAAJAP####8ABWFic29yAAAAAAAAAAoA#####wAFb3Jkb3IAAAAAAAAADQD#####AGZmZgALAAABBQAAAAAAAAAADgAAAAUAAAAOAAAABgAAAA0A#####wBmZmYACwAAAQUAAAAAAAAAAA8AAAAADgAAAAUAAAAVP#AAAAAAAAAAAAAOAAAABgAAABAA#####wAAAAcAAAAOAAAABAAAAAcB#####wAAAAAACwAAAQUAAAAACAAAAAkAAAAQAP####8AAAAHAAAADgAAAAEAAAAHAP####8AAAAAAAsAAAEFAAAAAAoAAAALAAAAAQD#####AQAAAAANAAABAAEAAAAKAAAADAAAABEA#####wEAAAAACwABVwDAGAAAAAAAAMAyAAAAAAAABQABP9cBAsIHNvcAAAANAAAAEgD#####AAd4Q29vcmQxAAAAAAAAAA4AAAATAP####8ABmdyYWRwaQALeENvb3JkMS9waTEAAAAPAwAAAA4AAAAPAAAADgAAAAQAAAATAP####8ABXRlc3QzABVhYnMoZ3JhZHBpKTwwLjAwMDAwMDEAAAAPBAAAACIAAAAADgAAABAAAAAVPnrX8pq8r0gAAAATAP####8AB2dyYWRwaScAEjIqYWJzb3IvcGkxLWdyYWRwaQAAAA8BAAAADwMAAAAPAgAAABVAAAAAAAAAAAAAAA4AAAAFAAAADgAAAAQAAAAOAAAAEAAAABMA#####wAFdGVzdDQAFmFicyhncmFkcGknKTwwLjAwMDAwMDEAAAAPBAAAACIAAAAADgAAABIAAAAVPnrX8pq8r0gAAAATAP####8ABXRlc3QyABhhYnMoZ3JhZHBpJysxKTwwLjAwMDAwMDEAAAAPBAAAACIAAAAADwAAAAAOAAAAEgAAABU#8AAAAAAAAAAAABU+etfymryvSAAAABMA#####wAFdGVzdDEAF2FicyhncmFkcGktMSk8MC4wMDAwMDAxAAAADwQAAAAiAAAAAA8BAAAADgAAABAAAAAVP#AAAAAAAAAAAAAVPnrX8pq8r0gAAAAjAP####8AAAAAAAAAAAAAAAAAQBgAAAAAAAAAAAAODAAB####AAAAAQAAAAAAAAAVAAAAAAAAAAAAL1xJZnt0ZXN0M317MH17XElme3Rlc3QxfXtccGl9e1xWYWx7Z3JhZHBpfVxwaX19AAAAFAH#####AGZmZgAAABYAAAAOAAAAAQAAAA4AAAAAAAAAJAD#####AAAABwAAAAcA#####wEAAAAACwAAAQUAAAAADgAAABgAAAAjAP####8AAAAAAAAAAAAAAAAAQBgAAAAAAAAAAAAZDAAB####AAAAAQAAAAAAAAAVAAAAAAAAAAAAMVxJZnt0ZXN0NH17MH17XElme3Rlc3QyfXstXHBpfXtcVmFse2dyYWRwaSd9XHBpfX0AAAAUAf####8AZmZmAAAAGgAAAA4AAAABAAAADgAAAAAAAAABAP####8BAAAAAA0AAAEAAQAAAAcAAAAMAAAAEQD#####AQAAAAALAAFSAMAAAAAAAAAAwDQAAAAAAAAFAAE#xbreUvleaAAAABwAAAAUAf####8AZmZmAAAAHQAAAA8AAAAADwIAAAAOAAAAAQAAABVAAAAAAAAAAAAAABU#8AAAAAAAAAAAAB0AAAAAAAAABwD#####AQAAAAALAAABBQAAAAAdAAAAGAAAABQB#####wBmZmYAAAAfAAAADwAAAAAPAgAAAA4AAAABAAAAFUAAAAAAAAAAAAAAFT#wAAAAAAAAAAAAHQAAAAAAAAAQAP####8AAAAHAAAADgAAAAMAAAAQAP####8AAAAHAAAADgAAAAIAAAANAP####8AAAAAAAsAAAEFAAAAAAAAAAAOAAAABQAAAA8AAAAADgAAAAYAAAAVP#AAAAAAAAAAAAAHAP####8AAAAAAAsAAAEFAAAAACMAAAAhAAAABwD#####AAAAAAALAAABBQAAAAAkAAAAIgAAAAEA#####wEAAAAADQAAAQABAAAAJAAAACUAAAARAP####8AAAAAAAsAAVQAQCAAAAAAAADAIgAAAAAAAAUAAT#ij1wo9cKXAAAAJgAAABYA#####wAHeUNvb3JkMQAAAAAAAAAnAAAAEwD#####AARvcmR0AAd5Q29vcmQxAAAADgAAACgAAAATAP####8ABXlncmFkAAwyKm9yZG9yLW9yZHQAAAAPAQAAAA8CAAAAFUAAAAAAAAAAAAAADgAAAAYAAAAOAAAAKQAAAAcA#####wAAAAAACwAAAQUAAAAAJwAAABgAAAAUAf####8AZmZmAAAAJwAAAA4AAAACAAAAJwAAAAAAAAAUAf####8AZmZmAAAAKwAAAA4AAAACAAAAJwAAAAAAAAAXAP####8BAAAAAMAgAAAAAAAAP#AAAAAAAAAAAAAnCwAB####AAAAAgAAAAEAAAAVAAAAAAAAAAAACiNWYWwob3JkdCkAAAAUAf####8AZmZmAAAALgAAAA4AAAACAAAAJwAAAAAAAAAXAP####8BZmZmAMAgAAAAAAAAAAAAAAAAAAAAAAArCwAB####AAAAAgAAAAEAAAAVAAAAAAAAAAAACyNWYWwoeWdyYWQpAAAAFAH#####AGZmZgAAADAAAAAOAAAAAgAAACcAAAAAAAAABgAQUGFwaWVyTWlsbGltZXRyZQAAAAAABQAAAAQCAAAAFgAAAAAAAAAAABoAAAAaAAAAAAAEeG1pbv####8AAAABAAAAAAAEeG1heP####8AAAABAAAAAAAEeW1pbv####8AAAABAAAAAAAEeW1heP####8AAAABAAAADQD#####AQAA#wAQAAAAAAAAAAAAAABACAAAAAAAAAUAAAAAAAAAAA4AAAABAAAADgAAAAMAAAANAP####8AAAD#ABAAAAAAAAAAAAAAAEAIAAAAAAAABQAAAAAAAAAADgAAAAIAAAAOAAAAAwAAAAEA#####wAAAAAAEAAAAQABAAAABQAAAAYAAAARAP####8AAAAAABAAAAAAAAAAAAAAAEAIAAAAAAAABQABP6e4UzDRRF4AAAAH#####wAAAAEAFENUcmFuc2xhdGlvblBhckNvb3JkAP####8AAAAAAAAAFQAAAAAAAAAAAAAADwEAAAAOAAAABAAAAA4AAAADAAAABwD#####AQAAAAAQAAAAAAAAAAAAAABACAAAAAAAAAUAAAAACAAAAAkAAAABAP####8BAAAAABAAAAEAAQAAAAgAAAAKAAAAFAH#####AObm5gAAAAsAAAAPAAAAAA8CAAAADwEAAAAOAAAAAgAAAA4AAAABAAAAFUAkAAAAAAAAAAAAFT#wAAAAAAAAAAAACAAAAAAAAAAUAf####8ApKSkAAAACwAAAA8AAAAADwEAAAAOAAAAAgAAAA4AAAABAAAAFT#wAAAAAAAAAAAACAAAAAAAAAANAP####8BAAAAABAAAAAAAAAAAAAAAEAIAAAAAAAABQAAAAAAAAAADgAAAAEAAAAOAAAABAAAAAEA#####wEAAAAAEAAAAQABAAAABQAAAA4AAAARAP####8BAAAAABAAAAAAAAAAAAAAAEAIAAAAAAAABQABP+ZbRUi+6MgAAAAPAAAAKgD#####AAAAAAAAAA8BAAAADgAAAAIAAAAOAAAAAQAAABUAAAAAAAAAAAAAAAcA#####wEAAAAAEAAAAAAAAAAAAAAAQAgAAAAAAAAFAAAAABAAAAARAAAAAQD#####AQAAAAAQAAABAAEAAAAQAAAAEgAAABQB#####wDm5uYAAAATAAAADwAAAAAPAgAAAA8BAAAADgAAAAQAAAAOAAAAAwAAABVAJAAAAAAAAAAAABU#8AAAAAAAAAAAABAAAAAAAAAAFAH#####AL29vQAAABMAAAAPAAAAAA8BAAAADgAAAAQAAAAOAAAAAwAAABU#8AAAAAAAAAAAABAAAAAAAAAABgAIQWZmQW5nbGUAAAAAAAMAAAADAQAAAAsAAAAAAANhbmf#####AAAAAQAAAAAAAAAAAAD#####AAAAAAAAAAAAAP####8AAAATAP####8ABmFic2FuZwAIYWJzKGFuZykAAAAiAAAAAA4AAAAA#####wAAAAEACUNSb3RhdGlvbgD#####AAAAAgAAAA4AAAAAAAAABwD#####AAAAAAAQAAAAAAAAAAAAAABACAAAAAAAAAUAAAAAAQAAAAT#####AAAAAQAMQ0Jpc3NlY3RyaWNlAP####8BAAAAABAAAAEAAQAAAAEAAAACAAAABQAAABEA#####wEAAAAAEAAAAAAAAAAAAAAAQAgAAAAAAAAFAAFAcPqI+5XUWQAAAAb#####AAAAAQANQ0RlbWlEcm9pdGVPQQH#####AAAAAAAQAAABAAEAAAACAAAABf####8AAAACABdDTWFycXVlQW5nbGVHZW9tZXRyaXF1ZQH#####AAAAAAACAAAAAUBDgAAAAAAAAAAAAQAAAAIAAAAFAAAAHAH#####AAAAAAAAAAAAAAAAAEAAAAAAAAAAAAAABw0AAAAAAAEAAAABAAAAFQAAAAAAAAAAAAAAAsKwAgAAAAMAAAAGAAtNZXN1cmVBbmdsZQAAAAAAAwAAAAMAAAAACAAAAAAAAUEAAAAA#####wAAAAAAAUIAAAAA#####wAAAAAAAUMAAAAA#####wAAACwA#####wEAAAAADQAAAQABAAAAAAAAAAEAAAACAAAAEQD#####AQAAAAANAAABBQABQG8lqGWscakAAAAD#####wAAAAEAF0NNZXN1cmVBbmdsZUdlb21ldHJpcXVlAf####8AAAAAAAAAAQAAAAIAAAAuAf####8AAAAAAAEAAAABQEUGF39UkbsAAAAAAAAAAQAAAAIAAAAcAf####8AAAAAAEAIAAAAAAAAAAAAAAAAAAAAAAAEDwAAAAAAAQAAAAEAAAAVAAAAAAAAAAAAAAACwrAAAAAABQAAAAYAEk1lc3VyZUFuZ2xlT3JpZW50ZQAAAAAAAwAAAAMAAAAACAAAAAAAAAAAAAD#####AAAAAAAAAAAAAP####8AAAAAAAAAAAAA#####wAAACwA#####wEAAAAACwAAAQABAAAAAgAAAAEAAAAAAAAAEQD#####AQAAAAALAAAAAAAAAAAAAAA#4zMzMzMzMwUAAUBpPkKWDt3JAAAAA#####8AAAACABNDTWVzdXJlQW5nbGVPcmllbnRlAf####8AA2FuZwAAAAAAAAABAAAAAgAAABwB#####wAAAAAAQAgAAAAAAAA#8AAAAAAAAAAAAAQPAAAAAAABAAAAAQAAABUAAAAAAAAAAAAAAALCsAAAAAAF#####wAAAAIAE0NNYXJxdWVBbmdsZU9yaWVudGUB#####wAAAAAAAQAAAABAQluJCSuPvwAAAAAAAAABAAAAAgAAAAAGAAlNZXNMb25nT3IAAAAAAAIAAAACAAAAAAsAAAAAAAFBAAAAAP####8AAAAAAAFCAAAAAP####8AAAAlAP####8AAAAAABAAAAEAAQAAAAABP#AAAAAAAAAAAAARAP####8AAAAAABAAAlcnAAAAAAAAAAAAQAgAAAAAAAAFAAFAYMAAAAAAAAAAAAIAAAAwAP####8AAmFuAAAAAwAAAAAAAAABAAAAGwH#####AAAAAAAAAAEAAAAZAP####8AAAAAABAAAAAAAAAAAAAAAEAIAAAAAAAABQAAAAAAAAAAAf####8AAAABAA1DUG9pbnRQcm9qZXRlAP####8AAAAAABAAAlciAAAAAAAAAAAAQAgAAAAAAAAFAAAAAAEAAAACAAAAMAD#####AAJtYQAAAAMAAAAAAAAAB#####8AAAABAA5DVGVzdEV4aXN0ZW5jZQD#####AAN0bWEAAAAIAAAAHAH#####AAAAAABAAAAAAAAAAMAAAAAAAAAAAAAABhAAAAAAAAEAAAAC#####wAAAAEADUNGb25jdGlvbjNWYXIAAAAADgAAAAkAAAAPAAAAAA4AAAAEAAAADgAAAAgAAAAOAAAABAAAAAABAAAABQAAAAYACEFmZlZhbE9yAAAAAAADAAAAAQEAAAALAAAAAAABYf####8AAAABAAAAAAAAAAAAAP####8AAAAAAAAAAAAA#####wAAACUA#####wAAAAAAEAAAAQABAAAAAQE#8AAAAAAAAAAAABEA#####wAAAAAAEAACVycAAAAAAAAAAABACAAAAAAAAAUAAUBgwAAAAAAAAAAAAwAAADAA#####wACYW4AAAAEAAAAAQAAAAIAAAAZAP####8AAAAAABAAAAAAAAAAAAAAAEAIAAAAAAAABQAAAAABAAAAAgAAADIA#####wAAAAAAEAACVyIAAAAAAAAAAABACAAAAAAAAAUAAAAAAgAAAAMAAAAwAP####8AAm1hAAAABAAAAAEAAAAHAAAAMwD#####AAN0bWEAAAAIAAAAHAH#####AAAAAABAAAAAAAAAAMAAAAAAAAAAAAAABhAAAAAAAAEAAAACAAAANAAAAAAOAAAACQAAAA8AAAAADgAAAAUAAAAOAAAACAAAAA4AAAAFAAAAAAEAAAAAAAAAAQAAAAEAAAAhAP####8AAnBpABYzLjE0MTU5MjY1MzU4OTc5MzIzODQ2AAAAFUAJIftURC0Y################'
module.exports = constructionsBase
